#!/usr/bin/env python -tt
"""
Default run_file template for mcms_track_finder

Create from a template via:

    python mcms_attribution_finder.py -c rf_files/rf_model_af.py

Input:
^^^^^
None

Output:
^^^^^^
Various python data structures.

Examples:
^^^^^^^^

Notes/Warnings:
^^^^^^^^^^^^^^

Author: Mike Bauer  <mbauer@giss.nasa.gov>

Log:
2011/01 MB - File created
2012/10 MB - Updated
"""

# Standard library imports
import sys
import os
import fnmatch

# MCMS module imports: Need to have mcms in your PYTHONPATH environment variable!
from cfg import define_source
from cfg import define_figure
from cfg import define_dirs
from cfg import define_defs
from cfg import define_vars
from cfg import ensure_path
from util import FileExists
from util import organize_data
from util import query_yes_no
from util import InjestSetup
from util import Is_It_Available

# Modify these to set the paths for the data and MCMS output
import output_root
import data_root

verbose = 0
dotline = "%s" % ("-"*40)
msg = '\n%s\n%40s\n%s\n' % (dotline,"Main Settings Set",dotline)
if verbose: print msg
over_write_years = ""
months = {1: 'January', 2: 'February', 3: 'March', 4: 'April', 5: 'May',
        6: 'June', 7: 'July', 8: 'August', 9: 'September', 10: 'October',
        11: 'November', 12: 'December'}
# ----------------------------------------------------------------------------
#                               Main Settings
# ----------------------------------------------------------------------------

# ----------------------------------------------------------------------------
# Pick data source see files in /cfg
#   See cfg/mcms_source_base_defs.py
# ----------------------------------------------------------------------------
pick_source = define_source()
msg = pick_source.make_pick("REPLACE")
if verbose: print msg
model = pick_source.selection
model_def = pick_source.selections[model][0]

# ----------------------------------------------------------------------------
# Import the base directories for the data and MCMS output
#   See rf_files/output_root.py
#       rf_files/data_root.py
# ----------------------------------------------------------------------------
if hasattr(output_root,'%s_root' % model):
    oroot = getattr(output_root,'%s_root' % model)
else:
    oroot = output_root.root
if hasattr(data_root,'%s_root' % model):
    droot = getattr(data_root,'%s_root' % model)
else:
    droot = data_root.root
# Can manually override these here
#oroot = ""
#droot = ""

# ----------------------------------------------------------------------------
# Full path to the root directory where pick specific output will be stored.
#   If multiple data sets under same model (say GCM runs) add a subdirectory
#   to the model root. Can be worked with ensembles and experiments within the
#   same model.
#
# Example:
#   ensembles = {"control_run" : [""],
#       "Scenario_A1B_run" : ["E3abaoM20","E3abboM20","E3abcoM20","E3abdoM20"],
#       "Scenario_A1B_iceMelt_run" : ["E3FIBaoM20","E3FIBboM20"]}
#   blobs = {"control_run" : "gissm_20",
#           "Scenario_A1B_run" : "gissm_20",
#           "Scenario_A1B_iceMelt_run" : "gissm_20"}
#   experiment = "Scenario_A1B_iceMelt_run"
#   ensemble = ensembles[experiment][0]
#   blob = blobs[experiment]
#
# Blob represents the sometimes long descriptive name some GCMs add to their
# output files (e.g., for the file naming pattern
#   cam_b30.14_37kaDVT_T85.cam2.h1.0531-01-01-PSL.nc
# the blob would be
#   cam_b30.14_37kaDVT_T85.cam2.h1
# which tells mcms that the changeable part of the file name pattern is
#   .0531-01-01-PSL.nc
# ----------------------------------------------------------------------------
ensemble = ""
experiment = ""
blob = ""
if ensemble:
    subdir = "%s/%s/" % (experiment,ensemble)
else:
    if experiment:
        subdir = "%s/" % (experiment)
    else:
        subdir = ""
out_path = "%s%s%s/" % (oroot,model,subdir)
out_path = ensure_path(subdir,out_path,check_exists=0)

# ----------------------------------------------------------------------------
# Full path to the directory where the SLP data files are stored.
# ----------------------------------------------------------------------------
slp_path = "%s%s/%s" % (droot,model,subdir)
slp_path = ensure_path(subdir,slp_path)

# ----------------------------------------------------------------------------
# Directory to be created for storing temporary model specific files.
# ----------------------------------------------------------------------------
if subdir:
    shared_path = "%s%s_files/" % (oroot,model)
else:
    shared_path = "%s_files/" % (out_path.replace("%s/" % model,model))
shared_path = ensure_path(subdir,shared_path)

# --------------------------------------------------------------------------
# Names associated with the netcdf files for the source
#   See cfg/mcms_ncvars_base_defs.py
# --------------------------------------------------------------------------
source_vars = define_vars()
msg = source_vars.make_pick(model)
if verbose: print msg
nc_vars = source_vars.selection
break_fname = nc_vars['break_fname']
month_sort_order = nc_vars['month_sort_order']
the_calendar = nc_vars['calendar']
jd_fake = nc_vars['jd_fake']
get_time_dat = "time_dat"

# --------------------------------------------------------------------------
# Fast way to do a single year ... by splitting the year into pieces &
#   work on each subset with a processor. This is generally for debugging
#   or testing something.
# Example:
#   fast_run = 1
#   fast_year = 2000
# --------------------------------------------------------------------------
fast_run = 0
fast_year = 0
if verbose and fast_run: print "Fast run mode for %d" % fast_year

# --------------------------------------------------------------------------
# Skip some some timesteps when reading the SLP data (convert a 3 hourly dataset
#   into a 6 hourly one). This is generally for debugging or testing something.
# Example:
#   skip_hours = ["03","09","15","21"]
# --------------------------------------------------------------------------
skip_hours = []

# --------------------------------------------------------------------------
# Over write the default of doing all years located by mcms_setup.py
# Example:
#   over_write_years = [2000,2010]
# --------------------------------------------------------------------------
over_write_years = []
if fast_run:
    over_write_years = [fast_year,fast_year]

# --------------------------------------------------------------------------
# Get some definitions. Note must have run mcms_setup.py already!
# --------------------------------------------------------------------------
fnc_out,inputs = InjestSetup("%ss_datf.pz" % (shared_path),0)
msg = "OUTPUT PATH (Default): %s"
if out_path != fnc_out[inputs.index("out_path")]:
    msg = "OUTPUT PATH (Modified): %s"
if verbose: print msg % out_path
if verbose: print "SHARED PATH: %s" % shared_path

# --------------------------------------------------------------------------
# Create needed directories if they don't exist.
# --------------------------------------------------------------------------
pick_dirs = define_dirs()
msg = pick_dirs.make_pick("standard")
dirs_set = pick_dirs.selection
need_dirs = ["%s%s" % (out_path,x) for x in dirs_set]
need_dirs.append("%s%s" % (out_path,"figs/maps/global/"))
need_dirs.append("%s%s" % (out_path,"figs/maps/global/annual/"))
need_dirs.append("%s%s" % (out_path,"figs/pdfs/global/"))
need_dirs.append("%s%s" % (out_path,"figs/pdfs/global/annual/"))
for ndir in need_dirs:
    if not os.path.exists(ndir):
        try:
            os.mkdir(ndir)
            if verbose:
                print "\tMade: %s" % ndir
        except:
            sys.exit("ERROR: creating %s" % ndir)

# --------------------------------------------------------------------------
# Updates if needed
# --------------------------------------------------------------------------
msg = "SLP PATH (Default): %s"
check_time = 0
if slp_path != fnc_out[inputs.index("slp_path")]:
    msg = "SLP PATH (Modified): %s"
    check_time = 1
if not os.path.exists(slp_path):
    sys.exit("ERROR: slp_path not found.")
if verbose: print msg % slp_path
msg = "SOURCE YEARS (Modified): %s"
if not over_write_years:
    over_write_years = fnc_out[inputs.index("super_years")]
    over_write_years = [int(over_write_years[0]),int(over_write_years[1])]
    msg = "SOURCE YEARS (Default): %s"
super_years = over_write_years
# Delete the rest
del fnc_out

# --------------------------------------------------------------------------
# Alter default behavior found in mcms_defs.py
# --------------------------------------------------------------------------
# Pick data source see defs files in /cfg
pick_defs = define_defs()
msg = pick_defs.make_pick(model)
if verbose: print msg
defs_set = pick_defs.selection


# ----------------------------------------------------------------------------
#                           Extra Settings Set
# ----------------------------------------------------------------------------
msg = '\n%s\n%40s\n%s\n' % (dotline,"Extra Settings Set",dotline)
if verbose: print msg

# What sort of figures (png faster, eps/pdf better quality)
pick_figure = define_figure()
msg = pick_figure.make_pick("png")
if verbose: print msg
fig_format  = pick_figure.selection

# Set for debugging as multiprocessing doesn't always report why a program hangs.
single_processor = 0
if fast_run:
    single_processor = 0
msg = "Warning Running Single Processor Mode!"
if verbose and single_processor: print msg

# Manually set the number of processors for multiprocessing.
processor_cnt = 0
if verbose and processors: print "Manual processor count: %d" % processor_cnt

#
# Override values from mcms_setup.py
#

# Match RAIBLE et. all 2008
#defs_set['max_cyclone_speed']= 42.0
#defs_set['age_limit'] = 72.0

# Breakdown, mods to print step by step of how MCMS ATT works
#  breakdown = 1 # plot SLP contours and centers only
#  breakdown = 2 # plot selected contours and centers
breakdown = 0
# Special version to plot a match to
#   Simmonds. Size Changes over the Life of Sea Level Cyclones in the NCEP Reanalysis.
#    Monthly Weather Review (2000) vol. 128 pp. 4118-4125
simmonds = 0
sim_year = 1996
if simmonds or breakdown:
    super_years = [sim_year,sim_year]
msgb = "Running in Breakdown Mode! Sim_year %d"
msgs = "Running in Simmonds Mode! Sim_year %d"
if verbose:
    if breakdown: print msgb % sim_year
    if simmonds: print msgbs % sim_year

# Set this to allow plotting of contour data (for debugging or otherwise)
plot_contours = 0
#plot_contours = 1
if simmonds or breakdown:
 plot_contours = 1
msg = "Plotting Contours"
if verbose and plot_contours: print msg

# Save ATT etc using the projection grid (large, high resolution)
save_hi_res = 0
msg = "Saving on high resolution grid"
if verbose and save_hi_res: print msg

# Save ATT etc using the data source grid
save_source_res = 1
if simmonds:
    # Turn on for b4 breakdown
    save_source_res = 0
if breakdown:
    # Turn on for b4 breakdown
    save_source_res = 1
msg = "Saving on data source grid"
if verbose and save_source_res: print msg

# Set contour levels and interval
clevs=[940,1040,2]
if breakdown:
    clevs=[960,1040,2]

# Use a linear contour intervals or non-linear where the interval
# falls in half from what clevs[2] uses above 980 hPa.
linear_intervals = 1
msg = "Using Contours: %s (linear %r)"
if verbose:
    print msg % (str(clevs), linear_intervals)

projs = ['laea','stere','aeqd']
pmap = projs[0]
mproj_sh = 'sp'+pmap
mproj_nh = 'np'+pmap
msg = "Using Map Projection: %s"
if verbose:
    print msg % (pmap)

# Limit contours to those whose circumference is less than
#   some set amount. Here we use the circumference of a
#   small circle on a sphere with the diameter of a set
#   See make_circle_v5.py
#   zonal wave number 4 centered on a latitude of 45 degrees.
#contour_limit = 21000.0
# zonal wave number 5 centered on a latitude of 45 degrees.
#contour_limit = 17000.0
# zonal wave number 6 centered on a latitude of 45 degrees.
contour_limit = 14000.0

# Smooth/interpolate data field by
#   hemisphere to higher grid density
nxx = 180
nyy = 180

# Set the density of the gridids for the final fill, you might
#   want this to be higher than nxx,nyy because the spacing of
#   the grids are fixed distance and thus when the fill is moved
#   to a fixed lon/lat grid at high latitudes you can get skipped
#   fill grids.
gxx = nyy*2
gyy = nxx*2

#   Set boundaries for map (set to zero to use defs default)
lat_bound = 0

if simmonds or breakdown:
    if not breakdown:
        lat_bound = -30.0
    gxx = nyy*1
    gyy = nxx*1

if plot_contours:
    res = "c"
else:
    res = None

# Use to plot discarded centers on plots
add_dumped = 0
if simmonds or breakdown:
    add_dumped = 1
if not plot_contours:
    add_dumped = 0
msg = "Plotting Dumped centers"
if verbose and add_dumped:
    print msg

# Create progress files which can used by python util/done.py out_path
#   to check the progress of a run if you cannot use the progress_bar
#   (e.g., remote logging or background process).
progress = 0
if fast_run:
    progress = 1
if progress:
    file_list = os.listdir(out_path)
    file_list.sort()
    pfiles = [x for x in file_list if x.find("progress") != -1]
    for p in pfiles:
        os.remove(out_path+p)

# --------------------------------------------------------------------------
# Define some files and important values
# --------------------------------------------------------------------------
start_year = int(super_years[0])
end_year = int(super_years[1])
years = range(start_year,end_year+1)
nyears = end_year-start_year+1
clim_tag = "%04d-%04d" % (start_year,end_year)
header = "mcms_%s_XXXX" % (model)
tail = "_tracks.txt"
cut_tail = len(tail)
centers_file = "%s%s%s" % (out_path,header,tail)
dumped_centers_file = "%s%s_dumped_centers.txt" % (out_path,header)
if verbose:
    print "CENTERS_FILE: %s" % centers_file
    if add_dumped: print "DUMPED_CENTERS_FILE: %s" % dumped_centers_file
if verbose: print msg % repr(super_years)