#!/usr/bin/env python -tt
"""
Default run_file template for mcms_check

Input:
^^^^^
None

Output:
^^^^^^
Various python data structures.

Examples:
^^^^^^^^

Notes/Warnings:
^^^^^^^^^^^^^^

Author: Mike Bauer  <mbauer@giss.nasa.gov>

Log:
2011/01 MB - File created
"""

# Standard library imports
import sys
import os
import cPickle

# MCMS module imports
import util.mcms_utilities
import cfg.mcms_source_base_defs
import cfg.mcms_figure_base_defs
import cfg.mcms_season_base_defs
import cfg.mcms_m2s_base_defs
import cfg.mcms_m2y_base_defs
import cfg.mcms_hemisphere_base_defs
import cfg.mcms_intensity_base_defs
import cfg.mcms_statgroup_base_defs
import cfg.mcms_mapgroup_base_defs
import cfg.mcms_lifegroup_base_defs

# ----------------------------------------------------------------------------
verbose = 0

# Pick data source see files in /cfg
pick_source = cfg.mcms_source_base_defs.define_source()
msg = pick_source.make_pick("nra2")
if verbose: print msg
model = pick_source.selection

# Pick whether track or attribute information used
#  tracks: track info included in file
#  att: attribute info included in file
# The default is to use the type of mcms data file to
#   determine what is done. You can override this by
#   setting tracks or att to 0 or 1.
#
tails = ["_att.txt","_tracks.txt","_centers.txt","_dumped_centers.txt"]
tail = tails[0]
# Length of file ending to replace if using year_loop
cut_tail = len(tail)
#
tracks = ""
att = ""
if tail.find("tracks") != -1:
    tracks = 1
if tail.find("att") != -1:
    att = 1
    # Note att files can contain track info so if you want
    # track statistics for an att file manually set tracks
    # to 1 here.
    tracks = 1

# Halt program on error or just warn?
exit_on_error = 1

# Plot map on error (requires matplotlib, also doubles or more memory
# footprint)
plot_on_error = 0

# No Warnings: Turn on/off error checking completely (for debugging)
error_check = 0

# Stop before making plots. (plots require matplotlib and
#   GMT (The Generic Mapping Tools)
skip_plots = 0

# Skip to making plots, that is skip compile_stats
skip_to_plots = 1
if skip_plots and skip_to_plots:
    sys.exit("Having skip_plots and skip_to_plots means doing nothing.")

# Make only summary plots (for whole records) and skip
#   the year-by-by year plots
summary_plots_only = 1

# Skip reading files from compile_stats and read pre-compiled
#   files instead, for rapid tweaking of plots in make_plot.
#   NOTE this applies to both the histogram and maps so if only
#   wish to tweak one of these use comments as needed.
recover_stat_files = 1

# Fixed Size (replace ATT+ATTS with a 5x5 grid centered on the center
#   gridID (tests the impact of ATT+ATTS on averaging).
fixed_size = 0

# Set for debugging as multiprocessing doesn't always report why
#   a program hangs.
single_processor = 0

# Find the STD maps using compile_stats
save_std = 0
if save_std:
    # On this run of compile_stats don't save the histogram
    #   data or mean files, instead read the mean files
    #   previously saved (from compile_stats) and find
    #   the standard deviation maps using the long term
    #   mean of the whole record.
    save_stat_files = 0
    save_mean_files = 0
    sys.exit("STD not avaiable yet!")
else:
    save_stat_files = 1
    save_mean_files = 1

# What sort of figures (png faster, eps/pdf better quality)
pick_figure = cfg.mcms_figure_base_defs.define_figure()
msg = pick_figure.make_pick("pdf")
if verbose: print msg
fig_format  = pick_figure.selection

#fig_format = ".png"
#fig_format = ".eps"
#fig_format = ".pdf"

# Store some maps and netcdf files for latter use
store_netcdf = 1

# Don't make plots (usually used with store_netcdf, for
#   saving the data for making plots rather than making them).
plots_no_draw = 1

# Check for graphics programs
#   The psrose plotting program from the GMT (The Generic Mapping Tools,
#   http://gmt.soest.hawaii.edu/) project provides a much better wind
#   rose plot than that from the matplotlib plotting package that is
#   generally used by the MCMS project.
psrose = util.mcms_utilities.Is_It_Available("psrose")
if not psrose:
    msg = "Warning: GMT (The Generic Mapping Tools) not found. "
    msg += "Some plots will not be made."
    print msg
try:
    import matplotlib
except ImportError:
    print "Warning: matplotlib not available. Disabling plotting altogether."
    skip_plots = 1

# Values only taken at center rather than over area (for mean maps)
point_source = 0
if not att:
    point_source = 1

if skip_plots and skip_to_plots:
    sys.exit("Stop: Can't have skip_plots and skip_to_plots")

# Skip high latitudes for angle, separation, area. skip_high_lat is the
#   absolute latitude above which no values retained so set to 90 to
#   disable. Note for bearing this is hard coded to 85 no matter what the
#   skip_high_lat value.
skip_high_lat = 90

# Deal with Seasons
# FIX:
# NOTE: For now djf, ndjfma etc only valid for summary (climatology as we don't
#   carry the Dec Year X over to the JAN, FEB of Year X + 1 so only totals valid.
#   could be done with m2s and the same read forward/backward used w/ tracks option.
#   or could pass those back to main and into next year as needed.
use_season = "3mo"
pick_season = cfg.mcms_season_base_defs.define_season()
msg = pick_season.make_pick(use_season)
if verbose: print msg
seasons = pick_season.selection

pick_m2s = cfg.mcms_m2s_base_defs.define_m2s()
msg = pick_m2s.make_pick(use_season)
if verbose: print msg
m2s = pick_m2s.selection

pick_m2y = cfg.mcms_m2y_base_defs.define_m2y()
msg = pick_m2y.make_pick(use_season)
if verbose: print msg
m2y = pick_m2y.selection

# Select break down by hemisphere
pick_hemisphere = cfg.mcms_hemisphere_base_defs.define_hemisphere()
msg = pick_hemisphere.make_pick("hemi")
if verbose: print msg
hemis = pick_hemisphere.selection

# Select by system intensity
pick_intensity = cfg.mcms_intensity_base_defs.define_intensity()
msg = pick_intensity.make_pick("3inten")
if verbose: print msg
intensities = pick_intensity.selection

# --------------------------------------------------------------------------
# Alter default behavior found in either defs_vX.py or setup_vX.py
# --------------------------------------------------------------------------

# The default behavior is to read SLP data from the
# directory slp_path defined in setup_vX.py.
# Here you can elect to override this behavior.
over_write_slp_path = ""
if over_write_slp_path and not over_write_slp_path.endswith("/"):
    sys.exit("paths must end in / see %s" % over_write_slp_path)

# The default behavior is to save results
# in the directory out_path defined in
# setup_vX.py. Here you can elect to override
# this behavior.
over_write_out_path = ""
if over_write_out_path and not over_write_out_path.endswith("/"):
    sys.exit("paths must end in / see %s" % over_write_out_path)

# This next set of lines should be copied from setup_vX.py
# Full path to the root directory where pick specific output will be stored.
# Note it's possible that all of these directories are identical.
# Uncomment to use same structure for all models
base_dir = "/Volumes/scratch/output/"
result_directories = dict(zip(pick_source.selections.keys(),
    [base_dir for x in range(len(pick_source.selections))]))
try:
    result_directory = result_directories[model]
    if not os.path.exists(result_directory):
        sys.exit("ERROR: result_directory not found.")
except:
    sys.exit("ERROR: model not listed in result_directories.")

# Directory to be created for storing temporary pick specific files.
shared_path = "%s%s_files/" % (result_directory,model)
if shared_path and not shared_path.endswith("/"):
    sys.exit("paths must end in / see %s" % shared_path)

# The default behavior is to run over all the
# years found by setup_vX.py. Here you can
# elect to override this behavior.
over_write_years = []
if model == "era40":
    over_write_years = [1961,1990]
#over_write_years = [1979,1979]
#over_write_years = [1985,2008]
#over_write_years = [1991,1992]

# See recipe inline_1 in inline.py
# Get some definitions. Note must have run setup_vx.py already!
sf_file = "%ss_dat.p" % (shared_path)
try:
    fnc_out = cPickle.load(open(sf_file))
except:
    sys.exit("\n\tWARNING: Error reading or finding %s." % (sf_file))
# Parse fnc_out
inputs = ("im","jm","maxid","lats","lons","timestep","dx","dy","dlon",
    "dlat","start_lat","start_lon","dlon_sq","dlat_sq","two_dlat",
    "model_flag","eq_grid","tropical_n","tropical_s","bot","mid","top",
    "row_start","row_end","tropical_n_alt","tropical_s_alt","bot_alt",
    "top_alt","lon_shift","lat_flip","the_calendar","found_years",
    "super_years","dim_lat","dim_lon","dim_time","var_lat","var_lon",
    "var_time","var_slp","var_topo","var_land_sea_mask","file_seperater",
    "no_topo","no_mask","slp_path","model","out_path","shared_path",
    "lat_edges","lon_edges","land_gridids","troubled_centers","faux_grids")
# Use what wanted
if not over_write_years:
    over_write_years = fnc_out[inputs.index("super_years")]
    over_write_years = [int(over_write_years[0]),int(over_write_years[1])]
if not over_write_out_path:
    over_write_out_path = fnc_out[inputs.index("out_path")]
if not over_write_slp_path:
    over_write_slp_path = fnc_out[inputs.index("slp_path")]
row_end = fnc_out[inputs.index("row_end")]
row_start = fnc_out[inputs.index("row_start")]
maxid = fnc_out[inputs.index("maxid")]
tropical_n_alt = fnc_out[inputs.index("tropical_n_alt")]
tropical_s_alt = fnc_out[inputs.index("tropical_s_alt")]
eq_grid = fnc_out[inputs.index("eq_grid")]
im = fnc_out[inputs.index("im")]
jm = fnc_out[inputs.index("jm")]
lons = fnc_out[inputs.index("lons")]
lats = fnc_out[inputs.index("lats")]
timestep = fnc_out[inputs.index("timestep")]
if model not in ["nra","nra2"]:
    # For needed for what_do/mcms_read
    land_gridids = fnc_out[inputs.index("land_gridids")]
# Delete the rest
del fnc_out

start_year = over_write_years[0]
end_year = over_write_years[1]+1

# See recipe inline_3 in inline.py
# Standard subdirectories to out_path:
standard_subdirs = ['','comps/','pdfs/','stats/','stats/tmp/','netcdfs/']
# Store figs by hemisphere and season to limit the number of
#   files in each.
fig_dirs = ["%s/%s/%s/" % (x,y,z) for x in
        ['figs/pdfs','figs/pdfs2d','figs/comps','figs/maps','figs/tseries']
        for y in hemis for z in seasons]
standard_subdirs.extend(fig_dirs)
need_dirs = ("%s%s" % (over_write_out_path,x) for x in standard_subdirs)
for nd in need_dirs:
    if not os.path.exists(nd):
        os.makedirs(nd)
        print "\tDirectory %s Created." % (nd)

# Set up histogram/timeseries statistics
hgroups = cfg.mcms_statgroup_base_defs.define_statgroup(att,tracks)

# Setup map statistics
mgroups = cfg.mcms_mapgroup_base_defs.define_mapgroup(att,tracks,hemis)

# Setup life cycle statistics
lgroups = cfg.mcms_lifegroup_base_defs.define_lifegroup(tracks,timestep,
    hgroups[0],hgroups[2])

