#!/usr/bin/env python -tt
"""
Lists of statistics to collect in mcms_check.py

Input:
^^^^^
None

Output:
^^^^^^
Various python data structures.
		
Examples:
^^^^^^^^

Notes/Warnings: 
^^^^^^^^^^^^^^

Author: Mike Bauer  <mbauer@giss.nasa.gov>

Log:
2011/01 MB - File created
"""

# Standard library imports
import sys
import os

# MCMS module imports

# ----------------------------------------------------------------------------
def define_lifegroup(tracks,timestep,stat_group,per_step_group):
    # compile_stats will create a data file storing each usi (putative cyclone)
    #   such that all the wanted data is laid out in the following format.
    #
    #   Many of these statistics are arranged about the minimum lifetime SLP (MLS)
    #   for that cyclone/track. We can imagine hypothetical cyclone whose life
    #   history is perfectly symmetric about its MLS. That is, the central SLP
    #   begins and ends at the same value, which is also the maximum lifetime 
    #   SLP for the cyclone. If cyclone development is likewise symmetric then
    #   the amount of time between the start of the cyclone and the MLS and that
    #   between the MLS and end of the cyclone are the same.
    #   
    # usi - Unique cyclone/track identifier.
    # span - Total track length (# of timesteps, includes the MLS timestep))
    #   span = 1 + spanb + spane
    # spanb - Start to MLS span (excludes the MLS timestep).
    #   spanb = 0 if the MLS is at the beginning of the track.
    #   spanb = span-1 if the MLS is at the end of the track
    # spane - MLS to end span (excludes the MLS timestep).
    #   spane = span-1 if the MLS is at the beginning of the track.
    #   spane = 0 if the MLS is at the end of the track
    # asymmetry - Measure time asymmetry about the MLS. 
    #   asymmetry = (spane-spanb)/(span-1)
    #             = -1 when MLS is at the beginning of the track
    #             =  0 when MLS is at the middle of the track
    #             =  1 when MLS is at the end of the track
    # amplitude - Maximum SLP difference between the beginning/end of the track
    #   and the MLS.
    #   amplitude = ampb IF ampb >= ampe ELSE ampe
    # ampb - SLP at start of track minus that at the MLS.
    #   ampb = 0 if MLS is at the beginning of the track, amplitude = ampe
    # ampe - SLP at end of track minus that at the MLS.
    #   ampe = 0 if MLS is at the end of the track, amplitude = ampb
    # amp-asymmetry - Measure SLP amplitude asymmetry about the MLS.
    #   amp-asymmetry = (ampe - ampb)/amplitude
    #                 = -1 if MLS is at the beginning of the track
    #                 < 0 if ampb > ampe, MLS not at the beginning of the track
    #                 = 0 if ampb = ampe
    #                 > 0 if ampe > ampb, MLS not at the end of the track
    #                 = 1 MLS is at the end of the track
    # aged-indices - array indices of the track data to follow so that they
    #   can be counted and summed in MLS relative arrays.
    # usi
    # span spanb spane asymmetry amplitude ampb ampe amp-asymmetry
    # aged_indices
    # track sequences based on stat_group.
    #   note that missing values are denoted with -1 to preserve their aged
    #   relative position and to allow for co-occurrence statistics such as
    #   correlation.
    #   Also, some quantities (e.g.,"track-length") are single
    #   value rather than sequences.
    # Example.
    stat_header = ['span','spanb','spane','asymmetry','amplitude',
            'ampb','ampe','amp-asymmetry','aged-indices']
    stat_holder = stat_header[:]
    stat_body = [x for x in stat_group if x not in per_step_group]
    stat_holder.extend(stat_body)
    stat_header_fmt = dict.fromkeys(stat_header,"%s: %s\n")
    stat_body_fmt = dict.fromkeys(stat_body,"%s: %s\n")

    # Maximum number of timesteps to collect aged statistics.
    #   aged_centered_index is an array of length 1 + 2*max_age
    #   Note because python is zero indexed min_loc can be set to
    #       max_age      
    #   Default 14 days
    max_age = int(14*timestep)
    min_loc = max_age
    age_len = 1+(2*min_loc)    
    if not tracks:
        # If by centers than can't know lifetime statistics.
        max_age = 1
        min_loc = 1
        age_len = 1
    return (stat_header,stat_header_fmt,stat_body,stat_body_fmt,
        max_age,min_loc,age_len)
    
#---Start of main code block. This show how to use the above code
if __name__=='__main__':
    verbose = 1
    att = 1
    tracks = 0
    timestep = 6
    stat_group = ['area-total', 'att-area', 'att-aveedge', 'att-depth', 
        'att-maxedge', 'att-minedge', 'att-perimeter', 'atts-area',
        'atts-aveedge', 'atts-depth', 'atts-maxedge', 'atts-minedge',
        'atts-perimeter', 'bearing', 'cnts', 'cnts-diffs', 'continued-usi', 
        'depth-total', 'dissimilarity', 'empty', 'entangled', 'fill', 
        'gridids', 'intensity', 'laplacian', 'lats', 'lons', 'lost-usi', 
        'natt', 'natts', 'new-usi', 'perimeter-total', 'sep', 'sep-alt', 
        'slp', 'speed', 'track-ave', 'track-length']
    per_step_group= ['cnts-diffs', 'cnts', 'empty', 'new-usi', 'lost-usi', 
        'continued-usi', 'track-ave']
    
    lgroups = define_lifegroup(tracks,timestep,stat_group,per_step_group)
    if verbose:
        print lgroups