#!/usr/bin/env python -tt
"""
Below Replace NAME with the word in the file name mcms_NAME_base_defs.py

Base NAME definitions for the MCMS project.

Provides name structure for NAME used by the MCMS project.

Input:
^^^^^
None

Output:
^^^^^^
Various python data structures.

Examples:
^^^^^^^^

Notes/Warnings:
^^^^^^^^^^^^^^
Users need not alter this file to add new NAME for their use,
rather they can create modify the NAME_defs.py file instead.

Author: Mike Bauer  <mbauer@giss.nasa.gov>

Log:
2011/01 MB - File created
"""

# Standard library imports
import sys
import os

# MCMS module imports
import util.pick_factory

# ----------------------------------------------------------------------------

# Define a simple function to break the SLP file names into year, month
#   etc.
def break_fname_year(fname,blob=""):
    # example: slp.1979.nc
    bf = fname.split(".")
    yyyy = int(bf[1])
    mm = ""
    dd = ""
    # how to re-make name
    pattern = "slp.year.nc"
    return (yyyy,mm,dd,pattern)
def break_fname_monthword_year(fname,blob=""):
    # example: slp.MAY.3794.nc
    bf = fname.split(".")
    yyyy = int(bf[2])
    mm = bf[1]
    dd = ""
    # how to re-make name
    pattern = "slp.month.year.nc"
    return (yyyy,mm,dd,pattern)
def break_fname_monthwordyear(fname,blob=""):
    # example: SLPDEC3781.E3oM20.nc
    yyyy = int(fname[6:10])
    mm = fname[3:6]
    dd = ""
    # how to re-make name
    pattern = "SLPmonthyear.nc"
    return (yyyy,mm,dd,pattern)
def break_fname_yearmonthday(fname,blob=""):
    # example: cam_b30.12_2kaDVTn_T85.cam2.h1.0322-03-09-PSL.nc
    blob_size = len(blob)
    keep = fname.split(blob)[1]
    yyyy = int(keep[1:5])
    mm = int(keep[6:8])
    dd = int(keep[9:11])
    # how to re-make name
    #"%s.%04d-%02d-%02d-PSL.nc"
    pattern = "blob.year-month-day-PSL.nc"
    return (yyyy,mm,dd,pattern)
def break_fname_yearmonthday_tight(fname,blob="."):
    # example: slp.20110125.nc
    keep = fname.split(blob)[1]
    yyyy = int(keep[0:4])
    mm = int(keep[4:6])
    dd = int(keep[6:8])
    # how to re-make name
    pattern = "slp.yearmonthday.nc"
    return (yyyy,mm,dd,pattern)

# Pseudo-code to unpack the slp data filename ... where slp is
#   the same as 'var_slp' and then year, month, day etc. can be
#   replaced with the correct value as the data are accessed.
#   This should/must be the same as the 'pattern' used by 'break_fname'
#   
file_seperater = "slp.year.nc" 

# The type of calendar the data uses CF Metadata convention
#
# gregorian or standard, proleptic_gregorian, noleap or 365_day, 
#   all_leap or 366_day, 360_day, julian, none
calendar = 'standard'

# MCMS calculates the Julian date, which doesn't apply to data from time periods
#   far removed from the modern era (e.g., paelo runs). In this case, set jd_fake
#   to 1 and a timestep based fake Julian date is used instead.
jd_fake = 0

# Make faux_grids based on point-registered data (see mcms_setup.py).
#   0 - Do nothing.
#   1 - Just use lon/lat to define grid centers and edges.
#   2 - Same as above but create polar cap which is half-width.
#   3 - Interpolate to make a grid.
#
#   See the pixel_registered, point_registered, gridline_registered discussion
#   in mcms_setup.py
faux_grids = 2

# Does the data have a polar cap? That is, the polar most grid is a 1/2 grid 
#   spacing?
polar_cap = 0

# Is the equator on a grid edge or inside a grid? Note might have to examine
#   the lat and lat_edges from mcms_setup.py to see if this happens because
#   of the changes due to faux_grids
eq_grid_on_edge = 0

# If the file names have months in words rather than numbers, the following
#   lookup table is needed to properly sort the files by year and month.
# If the file name doesn't have months as words then set to {}
month_sort_order_lower = {
        'jan':0,'feb':1,'mar':2,'apr':3,'may':4,'jun':5,'jul':6,'aug':7,
        'sep':8,'oct':9,'nov':10,'dec':11}
month_sort_order_upper = {
        'JAN':0,'FEB':1,'MAR':2,'APR':3,'MAY':4,'JUN':5,'JUL':6,'AUG':7,
        'SEP':8,'OCT':9,'NOV':10,'DEC':11}

# Python dictionary defining the long name and short name for the SLP
#   sources.
#
#   {"key" : ["Long Name/Title","Short Name"]}
#
#   Keys are short unique words that are used to refer to the wanted selection.
#   Long Names are longer phrases describing each option, these are sometimes
#       used as titles for plots and such.
#   Short Names much like the keys (often the same) a short identifier used
#       for file naming and such.
#
base_vars = {'dim_lat' : "lat",'dim_lon' : "lon", 'dim_time' : "time",
    'var_lat' : "lat", 'var_lon' : "lon", 'var_time' : "time",
    'var_slp' : "slp", 'var_topo' : "hgt", 'var_land_sea_mask' : "sftlf",
    'break_fname' : break_fname_year, 'calendar' : calendar, 'jd_fake' : jd_fake,
    'month_sort_order' : {},'faux_grids' : faux_grids, "polar_cap" : polar_cap,
    'eq_grid_on_edge' : eq_grid_on_edge, 'file_seperater' : file_seperater }

nra_vars = base_vars.copy()
nra_vars["var_slp"] = "slp"

nra2_vars = base_vars.copy()
nra2_vars["var_slp"] = "mslp"

erai_vars = base_vars.copy()
erai_vars["var_slp"] = "msl"
erai_vars["dim_lat"] = "latitude"
erai_vars["dim_lon"] = "longitude"
erai_vars["dim_time"] = "time"
erai_vars["var_lat"] = "latitude"
erai_vars["var_lon"] = "longitude"
erai_vars["var_time"] = "time"
erai_vars["var_land_sea_mask"] = "lsm"

era40_vars = erai_vars.copy()

merrac_vars = base_vars.copy()
merrac_vars["var_slp"] = "slp"
merrac_vars["dim_lat"] = "latitude"
merrac_vars["dim_lon"] = "longitude"
merrac_vars["dim_time"] = "time"
merrac_vars["var_lat"] = "latitude"
merrac_vars["var_lon"] = "longitude"
merrac_vars["var_time"] = "time"
merrac_vars['var_topo'] = "phis"
merrac_vars["var_land_sea_mask"] = "FRLAND"
merrac_vars["faux_grids"] = 1
merrac_vars['eq_grid_on_edge'] = 1
merrac_vars["jd_fake"] = 0
merrac_vars['break_fname'] = break_fname_yearmonthday_tight
merrac_vars['file_seperater'] = "slp.yearmonthday.nc"

merran_vars = merrac_vars.copy()
merrac_vars['eq_grid_on_edge'] = 0

ccsm_vars = base_vars.copy()
ccsm_vars["var_slp"] = "PSL"
ccsm_vars["dim_lat"] = "lat"
ccsm_vars["dim_lon"] = "lon"
ccsm_vars["dim_time"] = "time"
ccsm_vars["var_lat"] = "lat"
ccsm_vars["var_lon"] = "lon"
ccsm_vars["var_time"] = "time"
ccsm_vars['var_topo'] = "Z3"
ccsm_vars["var_land_sea_mask"] = "landfrac"
ccsm_vars["faux_grids"] = 1
ccsm_vars['eq_grid_on_edge'] = 1
ccsm_vars["jd_fake"] = 1
ccsm_vars['break_fname'] = break_fname_yearmonthday
ccsm_vars['calendar'] = "noleap"
ccsm_vars['file_seperater'] = "blob.year-month-day-PSL.nc"

gissm_vars = base_vars.copy()
gissm_vars["jd_fake"] = 1
gissm_vars['break_fname'] = break_fname_monthword_year
gissm_vars['calendar'] = "noleap"
gissm_vars['month_sort_order'] = month_sort_order_upper 
gissm_vars['var_topo'] = "topo"
gissm_vars['polar_cap'] = 1
#gissm_vars["faux_grids"] = 1
gissm_vars['eq_grid_on_edge'] = 1
gissm_vars['file_seperater'] = "slp.month.year.nc"

ipsl_vars = base_vars.copy()
ipsl_vars["var_slp"] = "psl"
ipsl_vars['var_topo'] = "orog"
ipsl_vars['calendar'] = "noleap"

selections = {"test" : ["Test/Debug netcdf variables",nra_vars],
            "nra" : ["NRA netcdf variables",nra_vars],
            "nra2" : ["NRA2 netcdf variables",nra2_vars],
            "erai" : ["ERAI netcdf variables",erai_vars],
            "era40" : ["ERA40 netcdf variables",era40_vars],
            "merrac" : ["MERRA netcdf variables",merrac_vars],
            "merran" : ["MERRA netcdf variables",merran_vars],
            "ccsm" : ["CCSM netcdf variables",ccsm_vars],
            "gissm" : ["GISSM netcdf variables",gissm_vars],
            "ipsl" : ["IPSL netcdf variables",ipsl_vars]
            }
NAME = __file__.split("_")[1]
path = os.path.abspath(os.path.dirname(__file__))
fname = "%s/%s.py" % (path,"%s_defs" % NAME)

def define_vars():
    return util.pick_factory.define_something(fname,selections,NAME)

#---Start of main code block. This show how to use the above code
if __name__=='__main__':

    verbose = True
    pick_vars = define_vars()
    msg = pick_vars.make_pick("nra2")
    if verbose: print msg
    nc_vars = pick_vars.selection
