#!/usr/bin/env python -tt
""" This module processes a continuous data set (e.g., SLP)
    by applying a contouring algorithm and using that output
    to define a set of contour objects (putative cyclones) of
    closed isobars around low pressure areas. These are then
    stored for for future linking to tracked cyclones.

    Inputs:

        field : 2d (lat,lon) numpy array of data to be processed.
        lons  : 1d numpy array of longitudes.
        lats  : 1d numpy array of latitudes.
        levels : 1d numpy array of the contours to be checked.
        m_map : basemap instance of a map projection.
        hemi :hemisphere being done
                hemi = 0 global (requires global map projection)
                     = 1 Northern Hemisphere
                     = 2 Southern Hemisphere
        bounding_lat : Equatorward most contours allowed (degrees)
        spanning_grids_e : * see Note 1
        spanning_grids_w : * see Note 1
        date_line : * see Note 2

    Note 1: The contouring program currently being used
            matplotlib version 0.99.0
            matplotlib-basemap 0.99.4
            has the limitation that contour lines crossing
            the international dateline are broken into
            separate parts. Thus steps are needed to
            be sure contours are reassembled for our
            use. spanning_grids_ e and w are the gridids
            for field that are w/in a user set closeness
            to the dateline (we use 10 grids on each side.
            This is for the contour fill.
    Note 2: Same issue as Note 1 but only need the griddids
            one on each side of the dateline. This is for
            the contour merge.

    2010/06 MB - File created.
    2010/07 MB - Fixed High Pressure Bug and Polar Gap problem.
    2010/07 MB - Added Polar and Tropical screening, Fixed ATT save bug.
    2011/09 MB - Added ckeys = sorted(c_objects.keys()) to the
                    "Assign centers to c_objects" this fixed cases where a
                    high wraping around a enclosed low led to the incorrect
                    discard of contours.
"""

# Standard library imports
import sys
import os

# MCMS module imports

# Globals
results = []

def dsplit(a, n):
    k, m = len(a)//n, len(a) % n
    return (a[i * k + min(i, m):(i + 1) * k + min(i + 1, m)] for i in xrange(n))

def find_attritubes(af_runfile,af_loop_year,fast):
    # --------------------------------------------------------------------------
    # Setup Section
    # --------------------------------------------------------------------------

    # Standard library imports
    import cPickle

    # Third party module imports
    import numpy
    import matplotlib.pyplot as plt
    from mpl_toolkits.basemap import Basemap, shiftgrid, addcyclic


    # MCMS module imports
    from util import defs
    from util import InjestSetup
    from util import acquire_centers_att
    from util import PullData
    from util import att_make_sm
    from util import isolate_contour_objects
    from util import att_assign_cobjects
    from util import att_discard_contours
    from util import att_view_progress
    from util import fill_att
    from util import fill_atts
    from util import att_find_sep

    # Import run_file
    __import__(af_runfile)
    af = sys.modules[af_runfile]

    if af.plot_contours:
        #from util import plotmap
        from util import att_plot_contours

    if af.breakdown:
        from att_breakdown import att_breakdown_0
        from att_breakdown import att_breakdown_1
        from att_breakdown import att_breakdown_2

    if af.save_hi_res:
        from util import att_save_hi_res

    if af.save_source_res:
       from util import att_save_source_res

    # Fetch definitions and impose those set in cf.defs_set.
    defs = defs(**af.defs_set)

    # Pre-bind for speed
    NA = numpy.array

    if af.verbose:
       from util import PrintColumns
       print_col = PrintColumns
    else:
        print_col = 0

    # Get some definitions. Note must have run mcms_setup.py already!
    fnc_out,inputs = InjestSetup("%ss_datf.pz" % (af.shared_path),0)
    # Keep Wanted values
    im = fnc_out[inputs.index("im")]
    jm = fnc_out[inputs.index("jm")]
    maxid = fnc_out[inputs.index("maxid")]
    lons = fnc_out[inputs.index("lons")]
    lats = fnc_out[inputs.index("lats")]
    timestep = fnc_out[inputs.index("timestep")]
    dx = fnc_out[inputs.index("dx")]
    dy = fnc_out[inputs.index("dy")]
    dlon = fnc_out[inputs.index("dlon")]
    dlat = fnc_out[inputs.index("dlat")]
    start_lat = fnc_out[inputs.index("start_lat")]
    start_lon = fnc_out[inputs.index("start_lon")]
    dlon_sq = fnc_out[inputs.index("dlon_sq")]
    dlat_sq = fnc_out[inputs.index("dlat_sq")]
    two_dlat = fnc_out[inputs.index("two_dlat")]
    eq_grid = fnc_out[inputs.index("eq_grid")]
    tropical_n = fnc_out[inputs.index("tropical_n")]
    tropical_s = fnc_out[inputs.index("tropical_s")]
    bot = fnc_out[inputs.index("bot")]
    mid = fnc_out[inputs.index("mid")]
    top = fnc_out[inputs.index("top")]
    row_start = fnc_out[inputs.index("row_start")]
    row_end = fnc_out[inputs.index("row_end")]
    tropical_n_alt = fnc_out[inputs.index("tropical_n_alt")]
    tropical_s_alt = fnc_out[inputs.index("tropical_s_alt")]
    bot_alt = fnc_out[inputs.index("bot_alt")]
    top_alt = fnc_out[inputs.index("top_alt")]
    lon_shift = fnc_out[inputs.index("lon_shift")]
    lat_flip = fnc_out[inputs.index("lat_flip")]
    the_calendar = fnc_out[inputs.index("the_calendar")]
    found_years = fnc_out[inputs.index("found_years")]
    dim_lat = fnc_out[inputs.index("dim_lat")]
    dim_lon = fnc_out[inputs.index("dim_lon")]
    dim_time = fnc_out[inputs.index("dim_time")]
    var_lat = fnc_out[inputs.index("var_lat")]
    var_lon = fnc_out[inputs.index("var_lon")]
    var_time = fnc_out[inputs.index("var_time")]
    var_slp = fnc_out[inputs.index("var_slp")]
    var_topo = fnc_out[inputs.index("var_topo")]
    var_land_sea_mask = fnc_out[inputs.index("var_land_sea_mask")]
    file_seperator = fnc_out[inputs.index("file_seperater")]
    no_topo = fnc_out[inputs.index("no_topo")]
    no_mask = fnc_out[inputs.index("no_mask")]
    slp_path = fnc_out[inputs.index("slp_path")]
    pbot = fnc_out[inputs.index("pbot")]
    ptop = fnc_out[inputs.index("ptop")]
    faux_grids = fnc_out[inputs.index("faux_grids")]
    eq_grid_on_edge = fnc_out[inputs.index("eq_grid_on_edge")]
    high_res = fnc_out[inputs.index("high_res")]
    row_start_index = row_start.index
    if af.model not in ["nra","nra2"]:
        af.tropical_end = row_end[tropical_n_alt]
        af.tropical_start = row_start[tropical_s_alt]
        af.maxid = maxid
        af.land_gridids = []
    del fnc_out,inputs

    if af.save_source_res:
        # Import a bunch of model grid specific information
        #   Note must have run mcms_setup.py already!
        fnc_out,inputs = InjestSetup("%scf_dat.pz" % (af.shared_path),1)
        # Keep Wanted values
        ijdict = fnc_out[inputs.index("ijdict")]
        # Drop the remainder
        del fnc_out,inputs

    if af.linear_intervals:
        # Regular fixed intervals
        levels = numpy.arange(af.clevs[0], af.clevs[1]+af.clevs[2],af.clevs[2])
    else:
        # None non-linear interval
        l1 = numpy.arange(af.clevs[0],af.clevs[1]+af.clevs[2],af.clevs[2])
        l2 = numpy.arange(980, l1[-1],af.clevs[2]*0.5)
        levels = list(l1)
        levels.extend(list(l2))
        levels = dict((x,1) for x in levels)
        levels = levels.keys()
        levels.sort()
        levels = NA(levels)
    af.levels = levels
    if af.verbose:
        print "\tLevels:"
        print_col(list(levels),indent_tag="\t\t",fmt="% 7.2f",cols=5,width=5)

    if not af.save_hi_res:
        if not af.save_source_res:
            print "\n\nI'm Not Saving Anything!\n\n"

    # Retrieve time_dat
    fnc_out,inputs = InjestSetup("%s%s.pz" % (af.shared_path,af.get_time_dat),4)
    super_uci_stamps = fnc_out[inputs.index("super_uci_stamps")]
    super_time_data = fnc_out[inputs.index("super_time_data")]
    super_years = fnc_out[inputs.index("super_years")]
    timestep = fnc_out[inputs.index("timestep")]
    super_lookup = fnc_out[inputs.index("super_lookup")]
    del fnc_out

    # If the data files are not annual create a virtual year to hold all the
    #   data files
    #
    # What files contain data for cf_loop_year?
    # Watch out for cases like this slp.19820101 which could match 1982 and 2010
    #
    fnames = [x for x in super_time_data
            if super_time_data[x]['uci_stamps'][0][:4].find(str(af_loop_year)) != -1]
    fnames = sorted(fnames)

    # Ensure fnames sorted properly
    if af.month_sort_order:
        fnames = sorted(fnames,
                key = lambda fnames: (af.break_fname(fnames)[0],
                af.month_sort_order[af.break_fname(fnames)[1]]))

    jd_fake = super_time_data[fnames[0]]['jd_fake']
    date_stamps = []
    adates = []
    uci_stamps = []
    tsteps = 0
    for fname in fnames:
        date_stamps.extend(super_time_data[fname]['date_stamps'])
        adates.extend(super_time_data[fname]['adates'])
        uci_stamps.extend(super_time_data[fname]['uci_stamps'])
        tsteps += super_time_data[fname]['tsteps']
print add skipdates (see rf_merrac_tf.py)
    if jd_fake:
        # adates are tsteps for each file... ensure correct
        years = range(super_years[0],super_years[-1]+1)
        jd_offset = years.index(cf_loop_year)*tsteps
        adates = []
        ttsteps = jd_offset
        for fname in fnames:
            adates_tmp = ["%010d" % (x+ttsteps)
                    for x in range(super_time_data[fname]['tsteps'])]
            adates.extend(adates_tmp)
            ttsteps += super_time_data[fname]['tsteps']
    annual_file = 0
    if len(fnames) == 1:
        annual_file = 1

    if af.lat_bound:
        lat_bound = af.lat_bound
    else:
        lat_bound = -1*defs.tropical_boundary
    mproj_sh = af.mproj_sh
    mproj_nh = af.mproj_nh
    clevs = af.clevs
    linear_intervals = af.linear_intervals
    res = af.res
    gxx = af.gxx
    gyy = af.gyy
    nxx = af.nxx
    nyy = af.nyy
    af.jm = jm
    af.im = im
    af.lats = lats
    af.lons = lons
    af.row_start_index = row_start.index
    af.row_start = row_start

    if fast[0]:
        do_list = list(dsplit(range(tsteps),fast[0]))
        run_steps = do_list[fast[1]]

    # Create polar Basemap instance.
    if af.breakdown:
        # setup lambert azimuthal equal area basemap.
        # lat_ts is latitude of true scale.
        # lon_0,lat_0 is central point.
        res = "l"
        golden_ratio = 1.61803399
        wide = 9000000
        tall = wide/golden_ratio
        # NATL
        wide = 8000000
        tall = wide/golden_ratio
        m_map_nh = Basemap(width=wide,height=tall,resolution="i",projection='laea',
            lat_ts=50,lat_0=50,lon_0=-40.0,boundinglat=-1*lat_bound,
            rsphere=6378137.0)#area_thresh=20000)
    else:
        m_map_nh = Basemap(projection=mproj_nh,lat_0=lats[-1],boundinglat=-1*lat_bound,
                lon_0=lons[0],resolution=res,rsphere=6378137.0,area_thresh=500000)
    m_map_sh = Basemap(projection=mproj_sh,lat_0=lats[0],boundinglat=lat_bound,
            lon_0=lons[0],resolution=res,rsphere=6378137.0,area_thresh=500000)

    make_sm = 0
    smooth_grid_x = smooth_grid_y = smooth_grid_area = bins = 0
    smooth_grid_row_start = smooth_grid_row_start_index = 0
    tropical_grids = deep_tropical_grids = polar_grids = 0
    sm_file = "%ssm_%04d_%04d_dat.p" % (af.shared_path,gyy,gxx)
    if os.path.exists(sm_file):
        try:
            fnc_out = cPickle.load(open(sm_file))
            (smooth_grid_x,smooth_grid_y,smooth_grid_area,
             smooth_grid_row_start,smooth_grid_row_end,bins,
             tropical_grids,deep_tropical_grids,polar_grids) = fnc_out
            smooth_grid_row_start_index = smooth_grid_row_start.index
            af.bins = bins
            # Save memory
            del fnc_out
        except:
            sys.exit("\n\tWARNING: Error reading %s." % (sm_file))
    else:
        make_sm = 1

    #---------------------------------------------------------------------------
    # Read all centers into memory
    #---------------------------------------------------------------------------
    centers,ids,in_file,readit = acquire_centers_att(af,af_loop_year)
    if af.verbose: print "\tCenters Read",len(centers)

    if af.add_dumped:
        pass
        if af.verbose: print "\tDumped Centers Read",len(dumped_centers)

    a_tail = ""
    if af.save_hi_res:
        a_tail = "atthi"
        # Open data file to save results
        att_file_hi = in_file.replace("tracks",a_tail)
        if fast[0]:
            att_file_hi = att_file_hi.replace(".txt","_%04d.txt" % (fast[1]))
        try:
            att_save_hi = open(att_file_hi,"w")
        except:
            sys.exit("\n\tWARNING: Error Creating %s." % (att_file_hi))
    if af.save_source_res:
        a_tail = "att"
        # Open data file to save results
        att_file = in_file.replace("tracks",a_tail)
        if fast[0]:
            att_file = att_file.replace(".txt","_04%d.txt" % (fast[1]))
        try:
            att_save = open(att_file,"w")
        except:
            sys.exit("\n\tWARNING: Error Creating %s." % (att_file))

    if af.progress:
        # Open file for progress
        cfmt = "%scontours_%s_%04d_%04d_%04d_%04d_progress.txt"
        prog_file = cfmt % (af.out_path,af.model,af.clevs[0],af.clevs[1],af.clevs[2],af_loop_year)
        if fast[0]:
            prog_file = cfmt % (af.out_path,af.model,af.clevs[0],af.clevs[1],af.clevs[2],fast[1])
        # Open with a buffer size of zero to make updates rapid
        try:
            prog_save = open(prog_file,"w",0)
        except:
            sys.exit("\n\tWARNING: Error Creating %s." % (prog_file))

        # List tsteps and a_tail in progress file.
        prog_save.write("%s\n" % (a_tail))
        if fast[0]:
            prog_save.write("%d\n" % (len(run_steps)))
        else:
            prog_save.write("%d\n" % (tsteps))

    if af.verbose: print "\n\n%s" % ("="*40)
    last_center = ""
    last_dumped_center = ""
    af.hemis = hemis = ["nh","sh"]
    m_map = [m_map_nh,m_map_sh]

    #---------------------------------------------------------------------------
    # Main Loop over Time
    #---------------------------------------------------------------------------
    fname = super_lookup[uci_stamps[0]]
    fname_ext = slp_path+fname+".nc"
    if not annual_file:
        tsteps_sub = 0
        slp,tt,tu = PullData(fname_ext,af_loop_year,var_slp,
            var_time,lat_flip,lon_shift,defs.read_scale,'all')

    last_center = ""
    step = -1
    for uci_starter in uci_stamps:
        current_centers = []
        found_none = 1
        if len(centers) < 1:
            more = 0
        else:
            more = 1
        while more:
            # See if overflow from last read useful or read new record
            if last_center:
                center = last_center
                last_center = ""
            else:
                center = centers.pop()
                if not centers: # No more centers
                    more = 0
            # Store if center falls on wanted date.
            if center.startswith(uci_starter,0,10):
                current_centers.append(center)
                found_none = 0
            else:
                # Store for next time
                last_center = center
                more = 0
        if not centers:
            # Get the very last center.
            if last_center.startswith(uci_starter,0,10):
                current_centers.append(last_center)
                found_none = 0
        if found_none:
            # No centers found
            msg = "\tError: No Centers Found! %s" % uci_starter
            sys.exit(msg)
        dumped_canidates = []
        dumped_center_places = {}
        current_dumped_centers = []
        if af.add_dumped:
            pass
            #dumped_canidates_nh = []
            #dumped_canidates_sh = []
            #if len(dumped_centers) < 1:
                #more = 0
            #else:
                #more = 1
            #while more:
                ## See if overflow from last read useful or read new record
                #if last_dumped_center:
                    #center = last_dumped_center
                    #last_dumped_center = ""
                #else:
                    #dumped_center = dumped_centers.pop()
                    #if len(dumped_centers) < 1: # No more centers
                        #more = 0
                ## Store if center falls on wanted date.
                #if dumped_center.startswith(date_stamp,0,10):
                    #current_dumped_centers.append(dumped_center)
                    #found_none = 0
                #else:
                    ## Store for next time
                    #last_dumped_center = dumped_center
                    #more = 0
            ##print "Current Dumped Centers",len(current_dumped_centers)
            ## Locate Dumped Centres
            #for pnt in current_dumped_centers:
                #clat = 90.0 - (readit3.center_holder[pnt][ids['CoLat']]*0.01)
                #clon = readit3.center_holder[pnt][ids['Lon']]*0.01
                #cslp = readit3.center_holder[pnt][ids["GridSLP"]]*0.001
                #h = (0 if clat >= 0 else 1)
                #cx,cy = m_map[h](clon,clat)
                #dumped_center_places[pnt] = (cx,cy,h,cslp,clat,clon)
                #if h:
                    #dumped_canidates_sh.append(pnt)
                #else:
                    #dumped_canidates_nh.append(pnt)
            #dumped_canidates = [dumped_canidates_nh,dumped_canidates_sh]

        step += 1
        adate = adates[step]
        date_stamp = date_stamps[step]
        if fast[0]:
            if step < run_steps[0]:
                continue
            elif step > run_steps[-1]:
                return

        if annual_file:
            slp = PullData(fname_ext,af_loop_year,var_slp,var_time,lat_flip,
                    lon_shift,defs.read_scale,step)
            field = slp[0,:,:].copy()
        else:
            #if fast[0]:
            #    sys.exit("figure this out")
            new_fname = super_lookup[uci_starter]
            if new_fname != fname:
                fname = new_fname
                fname_ext = af.slp_path+fname+".nc"
                tsteps_sub = 0
                slp,tt,tu = PullData(fname_ext,af_loop_year,var_slp,
                    var_time,lat_flip,lon_shift,defs.read_scale,'all')
            field = slp[tsteps_sub,:,:].copy()
            tsteps_sub += 1

        #if step > 3: break
        #if step != 2:
        #    print "skipping"
        #    if step > 2:
        #        sys.exit()
        #    else:
        #        continue
        #current_centers = ['20110101001093719937']

        # Add cyclic longitude to data and longitudes
        z,x = addcyclic(field,lons)
        # Shift data and longitudes to start at start_lon
        # also converts 0-360 to +-180 format
        z,x = shiftgrid(180.0,z,x,start=False)
        # Smooth Field
        smooth_field_nh,xx_nh,yy_nh = m_map_nh.transform_scalar(z,x,lats,
                nxx,nyy,returnxy=True,masked=True)
        # Make a LineCollection of contours
        contours_nh = m_map_nh.contour(xx_nh,yy_nh,smooth_field_nh,levels).collections
        # Smooth Field
        smooth_field_sh,xx_sh,yy_sh = m_map_sh.transform_scalar(z,x,lats,
                nxx,nyy,returnxy=True,masked=True)
        # Make a LineCollection of contours
        contours_sh = m_map_sh.contour(xx_sh,yy_sh,smooth_field_sh,levels).collections

        # Stuff for convenience
        xx = [xx_nh,xx_sh]
        yy = [yy_nh,yy_sh]
        lat_bounds = [lat_bound*-1,lat_bound]
        smooth_field = [smooth_field_nh,smooth_field_sh]
        contours = [contours_nh,contours_sh]
        af.lat_bounds = lat_bounds

        if step == 0 and make_sm:
            fnc_out = att_make_sm(z,x,lats,af,m_map_nh,m_map_sh)
            # Save output
            smooth_grid_x,smooth_grid_y,smooth_grid_area,smooth_grid_row_start,\
                    smooth_grid_row_end,bins,tropical_grids,deep_tropical_grids, \
                    polar_grids = fnc_out
            cPickle.dump(fnc_out,open(sm_file, "wb",-1))
            make_sm = 0
            smooth_grid_row_start_index = smooth_grid_row_start.index
            af.bins = bins

        # Call the contouring routines
        c_objects,c_lengths,c_paths = isolate_contour_objects(contours,m_map,af)
        # Plot what we've done so far (normally commented out)
        #if af.plot_contours:
            #att_view_progress(af,m_map,xx,yy,smooth_field,[],[],[],[],"",c_objects,
                    #[],[],[],[],[],do_this=0)
            #sys.exit("\nStop Here")

        # Convert centers into map coordinates (vertices)
        center_places = {}
        canidates_nh = []
        canidates_sh = []
        for center in current_centers:
            clat = 90.0 - (readit.center_holder[center][ids['CoLat']]*0.01)
            clon = readit.center_holder[center][ids['Lon']]*0.01
            #clon = ((clon+180.0)%360)-180.0
            cslp = readit.center_holder[center][ids["GridSLP"]]*0.001
            h = (0 if clat >= 0 else 1)
            cx,cy = m_map[h](clon,clat)
            center_places[center] = (cx,cy,h,cslp,clat,clon)
            if h:
                canidates_sh.append(center)
            else:
                canidates_nh.append(center)
        canidates = [canidates_nh,canidates_sh]

        if af.breakdown:
            att_breakdown_0(af,m_map,xx,yy,smooth_field,canidates,center_places,
                    dumped_canidates,dumped_center_places,date_stamp)

        # Assign centers to c_objects
        unique_contours,shared_contours = att_assign_cobjects(af,
                c_objects,c_lengths,c_paths,center_places,canidates)

        # Discard Contours with more than 75% of vertices are
        #   below 30 degrees or any below 15 degrees.
        msg = att_discard_contours(af,c_objects,c_lengths,c_paths,
                unique_contours,shared_contours,m_map)

        # Assign empty centers
        empty_centers = dict.fromkeys([x for x in current_centers
            if x not in unique_contours],1)
        empty_centers = empty_centers.keys()

        if af.verbose:
            print "\t\tUnique Contours (ATT): %d" % len(unique_contours)
            msg = "\t\t\tCenter (%s) has %d ATT Contours:"
            for center in unique_contours:
                print msg % (center,len(unique_contours[center]))
                print_col(unique_contours[center],indent_tag="\t\t\t\t",
                          fmt="%12s",cols=3,width=12,sort_me=0)
            print "\t\tEmpty Centers (%d):" % (len(empty_centers))
            print_col(empty_centers,indent_tag="\t\t\t\t",
                          fmt="%22s",cols=2,width=25,sort_me=0)
        if af.breakdown:
            att_breakdown_1(af,m_map,xx,yy,smooth_field,canidates,center_places,
                    dumped_canidates,dumped_center_places,date_stamp,
                    unique_contours,c_objects,shared_contours)

        # Plot what we've done so far (normally commented out)
        #if af.plot_contours:
        #    att_view_progress(af,m_map,xx,yy,smooth_field,canidates,center_places,
        #        dumped_canidates,dumped_center_places,uci_starter,c_objects,
        #        unique_contours,shared_contours,[],[],empty_centers,do_this=1)

        # Fill ATT or unique_contours: Find the highest pressure
        #   unique_contour for each center and fill it.
        center_att,center_att_radius,center_att_depth,center_att_area, \
                center_att_perimeter = fill_att(af,unique_contours,c_objects,
                        c_lengths,c_paths,center_places,m_map,
                        smooth_grid_row_start,smooth_grid_row_start_index,
                        smooth_grid_area,uci_starter)

        # Fill ATTS or shared_contours: Find the highest pressure
        #   shared_contour for each center and fill it, excluding
        #   any ATT points.
        center_atts,center_atts_radius,center_atts_depth,center_atts_area, \
                center_atts_perimeter,center_entangled = fill_atts(af,
                        shared_contours,c_objects,c_lengths,c_paths,
                        center_places,m_map,smooth_grid_row_start,
                        smooth_grid_row_start_index,smooth_grid_area,
                        center_att)

        # Plot what we've done so far (normally commented out)
        #if af.plot_contours:
        #    att_view_progress(af,m_map,xx,yy,smooth_field,canidates,center_places,
        #        dumped_canidates,dumped_center_places,uci_starter,c_objects,
        #        unique_contours,shared_contours,center_att,center_atts,
        #        empty_centers,do_this=2)

        # Find the distance to nearest center for each center.
        center_separation = att_find_sep(af,center_places,center_entangled,canidates)

        if af.save_hi_res:
            # Save full resolution data to file
            att_save_hi_res(af,readit,current_centers,empty_centers,center_att,
                    center_att_area,center_att_perimeter,center_att_radius,
                    center_att_depth,center_separation,center_atts,
                    center_atts_area,center_atts_perimeter,center_atts_radius,
                    center_atts_depth,center_entangled,att_save_hi)
        if af.save_source_res:
            # Collapse c_objects to gridIDs of the SLP source grid
            att_save_source_res(af,readit,current_centers,empty_centers,center_att,
                    center_att_area,center_att_perimeter,center_att_radius,
                    center_att_depth,center_separation,center_atts,
                    center_atts_area,center_atts_perimeter,center_atts_radius,
                    center_atts_depth,center_entangled,center_places,m_map,
                    ijdict,att_save)

        if af.breakdown:
            att_breakdown_2(af,m_map,xx,yy,smooth_field,canidates,center_places,
                    dumped_canidates,dumped_center_places,date_stamp,
                    unique_contours,c_objects,shared_contours,center_att,
                    center_atts)

        # Uncomment to plot consolidated contours (gridIDs)
        #if af.plot_contours:
        #    #att_plot_contours()
        #    if af.simmonds or af.breakdown:
        #        return

        # Even if not plotting the contour call creates an object that is
        #   retained each loop and will cause large memory usage if not
        #   terminated with this call.
        plt.clf()
        plt.close('all')

        if af.progress:
            # Update progress file
            if fast[0]:
                prog_save.write("%d\n" % (step-run_steps[0]))
            else:
                prog_save.write("%d\n" % (step))

    if af.save_hi_res:
        try:
            att_save_hi.close()
        except:
            sys.exit("\n\tWARNING: Error Closing %s." % (att_file_hi))
    if af.save_source_res:
        try:
            att_save.close()
        except:
            sys.exit("\n\tWARNING: Error Closing %s." % (att_file))
    if af.progress:
        try:
            prog_save.close()
        except:
            sys.exit("\n\tWARNING: Error Closing %s." % (prog_file))

    return af_loop_year

def main(argv):

    # MCMS module imports
    from extras import ProgressBarAlt
    from extras import ProgressBar
    from util import PrintColumns
    from util import FileExists
    from util import ParseRunFile

    def CompletedCallbackPbar(step):
        """
        Reports the process of a multiprocessing session.
        """
        # Assumes the calling program has global ProgressBar defined.
        global results
        if step == 1: sys.exit()
        results.append(step)
        mp_pbar.update_time(len(results),which_one=step)
        print(mp_pbar)
        return

    # --------------------------------------------------------------------------
    # Parse Request
    # --------------------------------------------------------------------------

    # Define the base run_file template
    here_path = os.path.abspath(os.path.dirname(__file__))
    run_file_template = "%s/cfg/mcms_attribute_finder_template.py" % (here_path)
    if not FileExists(run_file_template,__name__): return 1

    # --------------------------------------------------------------------------
    # Parse Request
    # --------------------------------------------------------------------------
    rf = ParseRunFile(run_file_template,argv,here_path)
    if rf == 1:
        return rf

    # --------------------------------------------------------------------------
    # Execute the request
    # --------------------------------------------------------------------------
    fast = [0,0]
    if rf.verbose:
        rf.single_processor = 1
    if rf.single_processor:
        # This is a single processor version or for debugging
        if rf.verbose:
            pbara = rf.years
        else:
            pbara = ProgressBarAlt(rf.years,"\tFinding Centers ... ",60)
        for loop_year in pbara:
            msg = find_attritubes(rf.getthis,loop_year,fast)
            if msg == 1: return 1
            results.append(msg)
    elif rf.fast_run:
        # Test requires python 2.6 or higher
        #
        # Test for python version with parallel processing capabilities.
        PROCESSORS =  1
        if sys.version_info >= (2, 6):
            import multiprocessing as mp
            PROCESSORS = mp.cpu_count()
        else:
            print "Parallelism requires python 2.6 or higher."
            return 1
        # Manually set processors count
        if rf.processor_cnt:
            PROCESSORS = rf.processor_cnt
        max_processors = (rf.end_year-rf.start_year)
        if PROCESSORS > max_processors:
            PROCESSORS = max_processors
        if PROCESSORS > mp.cpu_count():
            PROCESSORS = mp.cpu_count()
        msg = "\n\tStarting %s (process id %s) with %d processors (%d-%d)"
        print msg % ('Attribute_Finder (Fast)',os.getpid(),
                PROCESSORS,rf.start_year,rf.end_year)

        # ----------------------------------------------------------------------
        # Start Main Loop over super_years
        # ----------------------------------------------------------------------
        # Spawn n=PROCESSORS processes
        pool = mp.Pool(PROCESSORS)
        args_used = []
        mp_pbar = ProgressBar(PROCESSORS,"Finished")
        mp_pbar.fill_char = '-'
        checkin = CompletedCallbackPbar
        for i in xrange(PROCESSORS):
            fast = [PROCESSORS,i]
            # Object holding objects
            args_used.append((rf.getthis,rf.start_year,fast))
            pool.apply_async(find_attritubes,args_used[-1],callback=checkin)
        pool.close()
        pool.join()
    else:
        # Test requires python 2.6 or higher
        #
        # Test for python version with parallel processing capabilities.
        PROCESSORS =  1
        if sys.version_info >= (2, 6):
            import multiprocessing as mp
            PROCESSORS = mp.cpu_count()
        else:
            print "Parallelism requires python 2.6 or higher."
            return 1
        # Manually set processors count
        if rf.processor_cnt:
            PROCESSORS = rf.processor_cnt
        # Don't breakup by less than 1 year per processor
        max_processors = (rf.end_year-rf.start_year)
        if PROCESSORS > max_processors:
            PROCESSORS = max_processors
        if PROCESSORS > mp.cpu_count():
            PROCESSORS = mp.cpu_count()

        msg = "\n\tStarting %s (process id %s) with %d processors (%d-%d)"
        print msg % ('Attribute_Finder',os.getpid(),
                PROCESSORS,rf.start_year,rf.end_year)

        # ----------------------------------------------------------------------
        # Start Main Loop over super_years
        # ----------------------------------------------------------------------
        # Spawn n=PROCESSORS processes
        pool = mp.Pool(PROCESSORS)
        args_used = []
        mp_pbar = ProgressBar(
            len(range(rf.start_year,rf.end_year+1)),"Finished")
        mp_pbar.fill_char = '-'
        checkin = CompletedCallbackPbar
        for loop_year in rf.years:
            # Object holding objects
            args_used.append((rf.getthis,loop_year,fast))
            pool.apply_async(find_attritubes,args_used[-1],callback=checkin)
        pool.close()
        pool.join()

    if rf.fast_run:
        # Reassemble fast_run data file
         import shutil
         tail = ["atthi","att"]
         tail1 = ["atthi.txt","att.txt"]
         saver = [rf.save_hi_res,rf.save_source_res]
         for ss in range(2):
             if saver[ss]:
                 file_list = os.listdir(rf.out_path)
                 file_list.sort()
                 pfiles = [x for x in file_list if x.find(tail[ss]) != -1]
                 pfiles = [x for x in pfiles if not x.endswith(tail1[ss])]
                 pfiles = [x for x in pfiles if x.find(str(rf.start_year)) != -1]
                 pfiles.sort()
                 if len(pfiles) != PROCESSORS:
                     sys.exit("Warning Missing Files? pfiles (%d/%d): %s"
                             % (len(pfiles),PROCESSORS,repr(pfiles)))
                 base_file = rf.out_path + pfiles[0].replace("_0000","")
                 if os.path.exists(base_file):
                     sys.exit("Warning %s exists!" % base_file)
                 try:
                     b_save = open(base_file,"a")
                 except:
                     sys.exit("\n\tWARNING: Error Creating %s." % (base_file))
                 for i in xrange(PROCESSORS):
                     try:
                         f = open(rf.out_path + pfiles[i],'r')
                     except:
                         sys.exit("\n\tWARNING: Error Reading %s." % (rf.out_path + pfiles[i]))
                     try:
                         shutil.copyfileobj(f,b_save)
                     except:
                         sys.exit("\n\tWARNING: Error Concatenating %s to %s."
                                 % (rf.out_path + pfiles[i],base_file))
                     try:
                         f.close()
                     except:
                         sys.exit("\n\tWARNING: Error Closing %s." % (rf.out_path + pfiles[i]))
                 try:
                     b_save.close()
                 except:
                     sys.exit("\n\tWARNING: Error Closing %s." % (base_file))
                 print "\n\n Finished Concatenating to %s" % base_file
                 print "\n\n Helpful hint"
                 print "\n\n rm -f %s" % (" ".join(["%s%s" % (rf.out_path,x) for x in pfiles]))
                 print "\n\n"
    else:
        print "\n\n"
        PrintColumns(results,indent_tag="\t\t",fmt="% 5d",cols=5,width=5)
        if abs(rf.years[0]-rf.years[-1]) > 1:
            # Check at all years where finished.
            a = set(results)
            b = set([x for x in xrange(rf.start_year,rf.end_year+1)])
            c = a.symmetric_difference(b)
            if len(c):
                c = sorted(list(c))
                msg = "\tWarning results and years differ.\n\t\tDiffering: %s"
                print msg % (repr(c))

    if rf.progress:
        # Remove progress files
        file_list = os.listdir(rf.out_path)
        file_list.sort()
        pfiles = [x for x in file_list if x.find("progress") != -1]
        for pf in pfiles:
            os.remove(os.path.join(rf.out_path,pf))

#---Start of main code block.
if __name__=='__main__':
    sys.exit( main(sys.argv) )


