#!/usr/bin/env python -tt
""" This module extracts cyclone centers from a series of sea level
    pressure fields. The is module contains the main logic of this process.

    Options/Arguments:


    Returns/Creates:
    centers_file -- ASCII file of candidate centers found and kept.
    dumped_centers_file -- ASCII file of candidate centers found and
                           discarded.

    Examples:

    Notes: See bottom of this document for extended notes that are denoted
       in the code. For parallel execution see bottom.

       If using python version 2.6 and greater we can make use of multiple
       CPUs or multi-core CPUs in a symmetric multi-processing (SMP)
       or shared memory environment.

    Memory Use: Depending on resolution and options expect at least 200MB per
            instance.

    Run Time: With the NCEP Reanalysis I get about 140 timesteps per minute per
          instance. Thus a year takes roughly 10 minutes. This is on a
          Mac Pro with 2.26 GHz processors, 12 GB of RAM, and a 3 disk RAID.

    Author: Mike Bauer  <mbauer@giss.nasa.gov>

    Log:
    2006/12  MB - File created.
    lost track of dates of updates and fixes... sorry
    2008/10  MB - Added input checks, docstring.
    2008/11  MB - Added Polar_Fix to keep N pole from having 40 centers.
    2009/11  MB - Fixed problem on Julian Days for GCMs using noleap calendars.
    2009/11  MB - Reduced memory leak issue my removing year loop.
    2010/06  MB - Fixed Center Sort bug and moved to v5, removed polar_fix.
    2011/01  MB - Updates and fixes
    2012/05  MB - Tweeks for MERRA, and gzip setup data files.
"""

from __future__ import division
# Alter python 2 division where division of two *integer* values gives an
#   *integer* result (i.e., 1/2 = 0 but 1.0/2 = 0.5 or 1/2.0 = 0.5) to the
#   more rational python 3+ version where all division returns floats
#   (i.e., 1/2 = 0.5). Adds a new integer division operator so that 1//2 = 0

# Standard library imports
import sys
import os

# Third party module imports

# Globals
results = []

# Error Messages
emsg1 = "\n\tWarning Missing Center(s) Check: %d\n\t\tlen(kept_centers): %5d"
emsg1 += "\n\t\tlen(kept_pool)   : %5d\n\t\tlen(center_pool) : %5d"
emsg2 = emsg1.replace("Warning Missing Center(s) Check: %d",
    "%s\n\tDone with Non-overlapping Centers" % ("*"*60))
emsg2 += "\n\t%s\n"  % ("*"*60)
emsg2a = emsg2.replace("Non-overlapping","Singletons")
emsg3 = emsg1.replace("\n\t\tlen(kept_pool)   : %5d",
    "\n\t\tlen(kept_pool)   : %5d\n\t\tlen(discard_pool): %5d")
emsg4 = emsg3.replace("Warning Missing Center(s) Check: %d",
    "%s\n\tDone with Reducible Centers" % ("*"*60))
emsg4 += "\n\t%s\n"  % ("*"*60)
emsg5 = emsg4.replace("Reducible","Irreducible")
emsg6 = "%sWarning %s: %s not updated correctly"
emsg7 = "%sWarning %s: Singleton\n%ssingleton(s): %s"
emsg8 = "%sWarning %s: %s"

# Verbose Messages
gfmt0 = "%s%s %s"
gfmt = "%sUpdated %s:"
gfmt1 = gfmt+"\n%sin: %d\n%sout: %d"
gfmt2 = "%sTesting overlap_set[%7d]: %s\n%scase_set: %s"
gfmt2 += "\n%smembers: %s\n%sirreducible: %d"
gfmt3 = gfmt+" %s"
gfmt4 = gfmt3+"\n%s%s: %s"
gfmt5 = "%s%s: %s"
gfmt6 = "%sIrreducible Prune (Iteration %d)\n"+gfmt5+"\n"+gfmt5

# Baseline tab indent
base_line_tab_all = 1
deal_with_reducible_bt = base_line_tab_all+2
deal_with_irreducible_bt = base_line_tab_all+2
tabs = ["%s" % ("\t"*(base_line_tab_all+x)) for x in xrange(10)]


def dsplit(a, n):
    k, m = len(a)// n, len(a) % n
    return (a[i * k + min(i, m):(i + 1) * k + min(i + 1, m)] for i in xrange(n))

def find_centers(cf_runfile,cf_loop_year,cf_fast):
    """Supervisor for the generation of statistics."""
    # --------------------------------------------------------------------------
    # Setup Section
    # --------------------------------------------------------------------------

    # Standard library imports

    # Third party module imports
    import numpy

    # MCMS module imports
    from util import defs
    from util import g2l
    from util import gcd
    from util import shift_gridid
    from util import inflate_gdict
    from util import inflate_gdict_limited
    from util import InjestSetup
    from util import print_center
    from util import form_lon
    from util import unform_lon
    from util import form_colat
    from util import unform_colat
    from util import form_uci
    from util import form_center
    from util import form_center_alt
    from util import form_overlap_sets
    from util import deal_with_touching
    from util import correct_overlap_asymmetry
    from util import trim_singletons
    from util import deal_with_reducible
    from util import deal_with_irreducible
    from util import fit_poles
    from util import PullData

    # Import run_file
    __import__(cf_runfile)
    cf = sys.modules[cf_runfile]

    # Conditional modules
    make_plots = (1 if (cf.save_plots or cf.plot_on_error) else 0)

    if make_plots:
        from util import plotmap
        from util import plotmap_polar
        from util import Save_NetCDF
        from util import Error_Plot
        from util import Error_Plot_Polar
    if cf.save_stats:
        from util import Save_NetCDF
    if cf.smooth:
        import scipy.ndimage as ndimage
        from scipy.ndimage.filters import minimum_filter
        from scipy.ndimage import generate_binary_structure
    if cf.use_projection:
        from mpl_toolkits.basemap import Basemap, shiftgrid, addcyclic
        from scipy.ndimage.filters import minimum_filter,gaussian_laplace
        from scipy.ndimage import generate_binary_structure
        from util import l2g
        l2g2 = l2g.l2g2

    # Fetch definitions and impose those set in cf.defs_set.
    defs = defs(**cf.defs_set)
    inv_accuracy = 1.0/defs.accuracy

    if defs.plim_filter:
        defs.plim_filter = int(defs.plim_filter*defs.read_scale*defs.accuracy)

    # Predefined data storage
    center_data_type = numpy.dtype(defs.center_data)

    # Pre-bind for speed
    N_less = numpy.less; N_greater = numpy.greater
    N_average = numpy.average; N_take = numpy.take
    N_ones = numpy.ones; N_size = numpy.size; N_array = numpy.array
    N_multiply = numpy.multiply; N_sometrue = numpy.sometrue
    N_subtract = numpy.subtract; N_add = numpy.add
    N_where = numpy.where; N_Divide  = numpy.true_divide
    if cf.smooth:
        GFilter =  ndimage.gaussian_filter
        N_put = numpy.put
        N_take = numpy.take
    if cf.use_projection:
        FLap = gaussian_laplace

    # Quit on error else just send message to logfile?
    def do_this(do_msg,do_msg2,do_date_stamp):
        print do_msg
        print do_msg2
        print "\t\tDate_Stamp:"+date_stamp

    if cf.save_plots:
        fplot = plotmap(missing=-10000000000.0,color_scheme="hot_r",lon_0=0.0)
    if make_plots:
        plot = plotmap(clevs=[980,1020,2],cints=[960.0,1013.0],
            color_scheme="bone",lon_0=0.0)
        fplot = plotmap(missing=-10000000000.0,color_scheme="hot_r",lon_0=0.0)

    # Get some definitions. Note must have run mcms_setup.py already!
    if (defs.troubled_filter or cf.screen_tropical_land or cf.smooth or cf.use_projection):
        fnc_out,inputs = InjestSetup("%ss_dat.pz" % (cf.shared_path),0)
    else:
        fnc_out,inputs = InjestSetup("%ss_datf.pz" % (cf.shared_path),0)
    # Keep Wanted values
    im = fnc_out[inputs.index("im")]
    jm = fnc_out[inputs.index("jm")]
    maxid = fnc_out[inputs.index("maxid")]
    lons = fnc_out[inputs.index("lons")]
    lats = fnc_out[inputs.index("lats")]
    timestep = fnc_out[inputs.index("timestep")]
    dx = fnc_out[inputs.index("dx")]
    dy = fnc_out[inputs.index("dy")]
    dlon = fnc_out[inputs.index("dlon")]
    dlat = fnc_out[inputs.index("dlat")]
    start_lat = fnc_out[inputs.index("start_lat")]
    start_lon = fnc_out[inputs.index("start_lon")]
    dlon_sq = fnc_out[inputs.index("dlon_sq")]
    dlat_sq = fnc_out[inputs.index("dlat_sq")]
    two_dlat = fnc_out[inputs.index("two_dlat")]
    eq_grid = fnc_out[inputs.index("eq_grid")]
    tropical_n = fnc_out[inputs.index("tropical_n")]
    tropical_s = fnc_out[inputs.index("tropical_s")]
    bot = fnc_out[inputs.index("bot")]
    mid = fnc_out[inputs.index("mid")]
    top = fnc_out[inputs.index("top")]
    row_start = fnc_out[inputs.index("row_start")]
    row_end = fnc_out[inputs.index("row_end")]
    tropical_n_alt = fnc_out[inputs.index("tropical_n_alt")]
    tropical_s_alt = fnc_out[inputs.index("tropical_s_alt")]
    bot_alt = fnc_out[inputs.index("bot_alt")]
    top_alt = fnc_out[inputs.index("top_alt")]
    lon_shift = fnc_out[inputs.index("lon_shift")]
    lat_flip = fnc_out[inputs.index("lat_flip")]
    the_calendar = fnc_out[inputs.index("the_calendar")]
    found_years = fnc_out[inputs.index("found_years")]
    dim_lat = fnc_out[inputs.index("dim_lat")]
    dim_lon = fnc_out[inputs.index("dim_lon")]
    dim_time = fnc_out[inputs.index("dim_time")]
    var_lat = fnc_out[inputs.index("var_lat")]
    var_lon = fnc_out[inputs.index("var_lon")]
    var_time = fnc_out[inputs.index("var_time")]
    var_slp = fnc_out[inputs.index("var_slp")]
    var_topo = fnc_out[inputs.index("var_topo")]
    var_land_sea_mask = fnc_out[inputs.index("var_land_sea_mask")]
    file_seperator = fnc_out[inputs.index("file_seperater")]
    no_topo = fnc_out[inputs.index("no_topo")]
    no_mask = fnc_out[inputs.index("no_mask")]
    pbot = fnc_out[inputs.index("pbot")]
    ptop = fnc_out[inputs.index("ptop")]
    faux_grids = fnc_out[inputs.index("faux_grids")]
    eq_grid_on_edge = fnc_out[inputs.index("eq_grid_on_edge")]
    high_res = fnc_out[inputs.index("high_res")]
    row_start_index = row_start.index
    halfway = fnc_out[inputs.index("halfway")]
    no_poles = fnc_out[inputs.index("no_poles")]
    if hasattr(cf, 'no_poles'):
        no_poles = cf.no_poles
    polar = fnc_out[inputs.index("polar")]
    if faux_grids and make_plots:
        # Reset polar most latitudes to poles for plotting
        lats[0] = -90.0
        lats[-1] = 90.0
    if cf.screen_tropical_land:
        land_gridids = fnc_out[inputs.index("tropical_land_gridids")]
    else:
        land_gridids = [[],[]]
    if defs.troubled_filter:
        troubled_centers = fnc_out[inputs.index("troubled_centers")]
        if cf.screen_tropical_land:
            troubled_centers[0] = [x for x in troubled_centers[0] if x not in land_gridids[0]]
            troubled_centers[1] = [x for x in troubled_centers[1] if x not in land_gridids[1]]
        if defs.tropical_filter:
            troubled_centers[0] = [x for x in troubled_centers[0] if x > top]
            troubled_centers[1] = [x for x in troubled_centers[1] if x <= bot]
    else:
        troubled_centers = [[],[]]
    if hasattr(cf, 'high_res'):
        high_res = cf.high_res
    if cf.smooth or cf.use_projection:
        smask = fnc_out[inputs.index("smask")]
    super_mask = [[],[]]
    if cf.screen_tropical_land:
        super_mask[0].extend(land_gridids[0])
        super_mask[1].extend(land_gridids[1])
    if defs.tropical_filter:
        super_mask[0].extend(range(row_start[eq_grid],top+1))
        super_mask[1].extend(range(bot,row_start[eq_grid]+1))
    if cf.drop_troubled:
        super_mask[0].extend([x for x in troubled_centers[0]])
        super_mask[1].extend([x for x in troubled_centers[1]])
    if cf.use_hotspot:
        super_mask[0].extend([x for x in cf.discard_hot_spots if x > (tropical_n+int(5.0/dy))*im])
        super_mask[1].extend([x for x in cf.discard_hot_spots if x < (tropical_s-int(5.0/dy))*im])
        troubled_centers[0].extend([x for x in cf.screen_hot_spots if x not in troubled_centers[0] and
             x > (tropical_n+int(5.0/dy))*im])
        troubled_centers[1].extend([x for x in cf.screen_hot_spots if x not in troubled_centers[1] and
             x < (tropical_s-int(5.0/dy))*im])
    if no_poles:
        super_mask[0].extend([x for x in no_poles if x >= row_start[eq_grid]])
        super_mask[1].extend([x for x in no_poles if x <= row_start[eq_grid]])
    super_mask[0] = sorted(list(set(super_mask[0])))
    super_mask[1] = sorted(list(set(super_mask[1])))
    del land_gridids
    # Drop the remainder
    del fnc_out,inputs

    # Import a bunch of model grid specific information
    #   Note must have run mcms_setup.py already!
    fnc_out,inputs = InjestSetup("%scf_dat.pz" % (cf.shared_path),1)
    # Keep Wanted values
    use_all_lons = fnc_out[inputs.index("use_all_lons")]
    search_radius = fnc_out[inputs.index("search_radius")]
    regional_nys = fnc_out[inputs.index("regional_nys")]
    rdict = fnc_out[inputs.index("rdict")]
    ldict = fnc_out[inputs.index("ldict")]
    ijdict = fnc_out[inputs.index("ijdict")]
    lapp_cutoff = fnc_out[inputs.index("lapp_cutoff")]
    hpg_cutoff = fnc_out[inputs.index("hpg_cutoff")]
    # Drop the remainer
    del fnc_out,inputs
    if cf.lapp_cutoff:
        # Over ride
        lapp_cutoff = cf.lapp_cutoff
    if cf.hpg_cutoff:
        # Over ride
        hpg_cutoff = cf.hpg_cutoff
    if hasattr(cf, 'pure_candidates'):
        pure_candidates = cf.pure_candidates
        if pure_candidates:
            from util import Save_NetCDF
    else:
        pure_candidates = 0

    # Retrieve time_dat
    fnc_out,inputs = InjestSetup("%s%s.pz" % (cf.shared_path,cf.get_time_dat),4)
    super_uci_stamps = fnc_out[inputs.index("super_uci_stamps")]
    #super_tsteps = fnc_out[inputs.index("super_tsteps")]
    super_time_data = fnc_out[inputs.index("super_time_data")]
    super_years = fnc_out[inputs.index("super_years")]
    timestep = fnc_out[inputs.index("timestep")]
    super_lookup = fnc_out[inputs.index("super_lookup")]
    del fnc_out

    # If the data files are not annual create a virtual year to hold all the
    #   data files
    #
    # What files contain data for cf_loop_year?
    # Watch out for cases like this slp.19820101 which could match 1982 and 2010
    #
    fnames = [x for x in super_time_data
            if super_time_data[x]['uci_stamps'][0][:4].find(str(cf_loop_year)) != -1]
    fnames = sorted(fnames)

    # Ensure fnames sorted properly
    skip_hours = []
    if hasattr(cf, 'skip_hours'):
        skip_hours = cf.skip_hours

    if cf.month_sort_order:
        fnames = sorted(fnames,
                key = lambda fnames: (cf.break_fname(fnames)[0],
                cf.month_sort_order[cf.break_fname(fnames)[1]]))

    jd_fake = super_time_data[fnames[0]]['jd_fake']
    date_stamps = []
    adates = []
    uci_stamps = []
    tsteps = 0
    for fname in fnames:
        date_stamps.extend(super_time_data[fname]['date_stamps'])
        adates.extend(super_time_data[fname]['adates'])
        uci_stamps.extend(super_time_data[fname]['uci_stamps'])
        tsteps += super_time_data[fname]['tsteps']
    if jd_fake:
        # adates are tsteps for each file... ensure correct
        years = range(super_years[0],super_years[-1]+1)
        jd_offset = years.index(cf_loop_year)*tsteps
        adates = []
        ttsteps = jd_offset
        for fname in fnames:
            adates_tmp = ["%010d" % (x+ttsteps)
                    for x in range(super_time_data[fname]['tsteps'])]
            adates.extend(adates_tmp)
            ttsteps += super_time_data[fname]['tsteps']
    annual_file = 0
    if len(fnames) == 1:
        annual_file = 1

    # Summary Stats: Checks on operation to report and catch oddities.
    known_flags = {0 : "Passed all filters",
                   1 : "Failed concavity/Laplacian filter",
                   2 : "Failed regional minimum filter",
                   3 : "Failed troubled center filter"}
    eps_flags = { 0 : "Raw center count",
                  1 : "Dropped pre-screening",
                  2 : "Not even a candidate center"}
    flag_files = {0 : "passed",
                  1 : "lap",
                  2 : "reg",
                  3 : "troub"}
    troubflag = 3
    regflag = 2
    lapflag = 1
    passedflag = 0
    flag_colors = { 0 : "black",
                    1 : "blue",
                    2 : "yellow",
                    3 : "red",
                    4 : "green",
                    9 : "orange"
                    }
    total_time_steps = 0

    if cf_fast[0]:
        do_list = list(dsplit(range(tsteps),cf_fast[0]))
        run_steps = do_list[cf_fast[1]]

    if cf.smooth or cf.use_projection:
        # Make stencil for locating minima
        #
        # Create a 5-pnt neighbor stencil
        #       X
        #     X 0 X
        #       X
        neighbors_5pnt = generate_binary_structure(2,1)
        # Create a 9-pnt neighbor stencil
        #     X X X
        #     X 0 X
        #     X X X
        neighbors_9pnt = generate_binary_structure(2,2)
        neighbors = neighbors_5pnt
        #neighbors = neighbors_9pnt

        # Stencil for discrete Laplacian (5-pnt)
        lap_stencil = numpy.array([[0, 1, 0],[1, -4, 1], [0, 1, 0]])

        # Discard polar minima
        # polar cut off
        ncut = 85.0
        scut = -1.0*ncut
        nidx=(numpy.abs(lats-ncut)).argmin()
        sidx=(numpy.abs(lats-scut)).argmin()
        drop_polar = range(1+sidx)
        drop_polar.extend(range(nidx,jm))

    # Used flags
    nflags = len(known_flags)
    c_halo = numpy.array([x  for y in xrange(3) for x in xrange(im)],
            dtype=numpy.int32)

    # If not a hi-res model faster to make the whole range of values upfront
    #   otherwise make as needed. The method below uses more memory but is
    #   about 30% faster.
    if not high_res:
        gdict = inflate_gdict(im,jm)
        gdict_outer = {}
        rdict_exp = {}
        for gridid in xrange(im*jm):
            if not cf.use_projection:
                if gridid in polar:
                    gdict_outer[gridid] = [shift_gridid(
                        gridid,dc,dr,c_halo,halfway,im,jm)
                        for dr in [3,0,-3] for dc in [-3,0,3] ]
            row = gridid//im
            col = gridid%im
            rdict_exp[gridid] = [shift_gridid(x,col,0,c_halo,halfway,im,jm)
                    for x in rdict[row]]
        if cf.limit_sh:
            # Remove most of the NH gridids
            tmp = dict([(x,gdict[x]) for x in xrange((tropical_n+int(5.0/dy))*im)])
            gdict = tmp
            if not cf.use_projection:
                tmp = dict([(x,gdict_outer[x]) for x in xrange((tropical_n+int(5.0/dy))*im)])
                gdict_outer = tmp
            tmp = dict([(x,rdict_exp[x]) for x in xrange((tropical_n+int(5.0/dy))*im)])
            rdict_exp = tmp
        if cf.limit_nh:
            # Remove most of the SH gridids
            tmp = dict([(x,gdict[x]) for x in xrange((tropical_s-int(5.0/dy))*im,maxid)])
            gdict = tmp
            if not cf.use_projection:
                tmp = dict([(x,gdict_outer[x]) for x in xrange((tropical_s-int(5.0/dy))*im,maxid)])
                gdict_outer = tmp
            tmp = dict([(x,rdict_exp[x]) for x in xrange((tropical_s-int(5.0/dy))*im,maxid)])
            rdict_exp = tmp
    if cf.limit_sh:
        # Remove most of the NH gridids
        troubled_centers[0] = []
        super_mask[0] = []
        smask[0] = []
    if cf.limit_nh:
        # Remove most of the SH gridids
        troubled_centers[1] = []
        super_mask[1] = []
        smask[1] = []

    total_centers_used = 0
    total_centers_cnt = [0]*nflags # make size of total flag count
    esp_cnt = [0,0,0]

    if pure_candidates:
        pure_file = "%snetcdfs/%s_cf_pure_canidates_%04d.nc" % (
                cf.out_path,cf.model,cf_loop_year)
        pcandidates = numpy.zeros((jm*im),dtype=numpy.float)

    if cf.save_plots or cf.save_stats:
        # Files for histogram
        flag_cnt = numpy.zeros((jm*im,nflags),dtype=numpy.float)

    # Define some files
    header = "mcms_%s_%04d_" % (cf.model,cf_loop_year)
    centers_file = "%s%scenters.txt" % (cf.out_path,header)
    dumped_centers_file = "%s%sdumped_centers.txt" % (cf.out_path,header)
    if cf_fast[0]:
        centers_file = centers_file.replace(".txt","_%04d.txt" % (cf_fast[1]))

    # Open files for storage.
    try:
        centers_save = open(centers_file,"w")
    except:
        print "\n\tWARNING: Error Opening %s." % (centers_file)
        return 1
    if defs.keep_discards:
        try:
            dumped_centers_save = open(dumped_centers_file,"w")
        except:
            print "\n\tWARNING: Error Opening %s." % (dumped_centers_file)
            return 1
    else:
        dumped_centers_file = ""

    # Open file for save progress state (see python util/done.py out_path)
    #  This is useful if running in background or remotely so
    #  you can check the progress without the normal progress_bar.
    if cf.progress:
        cfmt = "%scenters_%s_%s_progress.txt"
        if cf_fast[0]:
            prog_file = cfmt % (cf.out_path,cf.model,cf_fast[1])
        else:
            prog_file = cfmt % (cf.out_path,cf.model,cf_loop_year)
        # Open with a buffer size of zero to make updates rapid
        try:
            prog_save = open(prog_file,"w",0)
        except:
            print "\n\tWARNING: Error Creating %s." % (prog_file)
            return 1
        # List tsteps and a_tail in progress file.
        prog_save.write("%s\n" % ("centers"))
        if cf_fast[0]:
            prog_save.write("%d\n" % (len(run_steps)))
        else:
            prog_save.write("%d\n" % (tsteps))

    if cf.use_projection:
        # Number of grids for projection (per hemisphere).
        nxx = 180 # ~86 km
        nyy = 180 # ~86 km

        # Make nxx,nyy such that the projected grid size is some multiple
        #   of the latitude spacing (111 km) to keep grid overlap simple.

        #nxx = int(140*dy) # ~111 km or 1 degree latitude
        #nyy = int(140*dy) # ~11 km

        # Type of interpolation to use with projection
        use_orders = {0: "nearest-neighbor", 1: "bilinear", 3: "cubic spline"}
        use_order = 3
        # Map projection to use
        projs = ['laea','stere','aeqd']
        # For hemispheres
        pmap = projs[0]
        mproj_sh = 'sp'+pmap
        mproj_nh = 'np'+pmap
        lat_bound = -1*defs.tropical_boundary

        # Create polar Basemap instance.
        # set res to None if not printing
        res = None
        res = "l"
        if res:
            m_map_nh = Basemap(projection=mproj_nh,lat_0=lats[-1],
                    boundinglat=-1*lat_bound,lon_0=lons[0],resolution=res,
                    rsphere=6378137.0,area_thresh=50000)
            m_map_sh = Basemap(projection=mproj_sh,lat_0=lats[0],
                    boundinglat=lat_bound,lon_0=lons[0],resolution=res,
                    rsphere=6378137.0,area_thresh=50000)
        else:
            m_map_nh = Basemap(projection=mproj_nh,lat_0=lats[-1],
                    boundinglat=-1*lat_bound,lon_0=lons[0],resolution=None)
            m_map_sh = Basemap(projection=mproj_sh,lat_0=lats[0],
                    boundinglat=lat_bound,lon_0=lons[0],resolution=None)

        fine_check = list(numpy.arange(-0.5,0.51,0.1))
        coarse_check = [-0.5,0,0.5]

    # --------------------------------------------------------------------------
    # Main Program Logic
    # --------------------------------------------------------------------------
    # Process the SLP field one time step at a time

    #CUT
    #tsteps = 40; print "\ntsteps",tsteps
    #tsteps = 1; print "\ntsteps",tsteps
    #tsteps = 16; print "\ntsteps",tsteps

    fname = super_lookup[uci_stamps[0]]
    fname_ext = cf.slp_path+fname+".nc"
    if not annual_file:
        tsteps_sub = 0
        slp,tt,tu = PullData(fname_ext,cf_loop_year,var_slp,
            var_time,lat_flip,lon_shift,defs.read_scale,'all')
    last_center_cnt = -1

    real_diff = 500

    if high_res:
        # Make some to start the process
        gdict_outer = {}
        gdict = {}
        rdict_exp = {}
        for row in rdict:
            rdict_exp[row_start[row]] = rdict[row]
        rdict_exp[maxid-1] = [shift_gridid(x,im,0,c_halo,halfway,im,jm)
                for x in rdict[jm-1]]

    # ***********************
    # Start Loop over Time
    # ***********************
    if cf.single_processor:
        from extras import ProgressBarAlt
        pbara = ProgressBarAlt(xrange(0,tsteps),"\tFinding Centers.... ",60)
    else:
        pbara = xrange(0,tsteps)
    for step in pbara:

        # Update progress file
        if cf.progress:
            if cf_fast[0]:
                prog_save.write("%d\n" % (step-run_steps[0]))
            else:
                prog_save.write("%d\n" % (step))
        if make_plots:
            temp_centers = {}
            temp_discards = {}

        adate = adates[step]
        uci_stamp = uci_stamps[step]
        date_stamp = date_stamps[step]

        #print "Doing",date_stamp
        #if date_stamp != "1980 01 08 00": continue
        #if int(adate) <= 12161: continue
        #if int(adate) < 2455879250: continue
        #if date_stamp != "2009 12 19 12": continue

        if date_stamp[-2:] in skip_hours: continue

        if cf_fast[0]:
            if step < int(run_steps[0]):
                continue
            elif step > int(run_steps[-1]):
                return

        #skip_fast = [2455567750,2455591000,2455613500,2455637000,2455660000,2455682500,2455705500,2455729000,
        #        2455751500,2455774000,2455796500,2455819250,2455842750,2455865500,2455888000,2455911000]
        #if int(adate) < skip_fast[cf_fast[1]]:
        #    continue

        # Get SLP field, make 1d integer array. To allow for exact comparisons
        # impose a fixed accuracy (significant digits) via defs.accuracy.
        if annual_file:
            slp = PullData(fname_ext,cf_loop_year,var_slp,var_time,lat_flip,
                    lon_shift,defs.read_scale,step)
            slp_step = N_multiply(slp[0,:,:].copy(),defs.accuracy)
        else:
            new_fname = super_lookup[uci_stamp]
            if new_fname != fname:
                fname = new_fname
                fname_ext = cf.slp_path+fname+".nc"
                tsteps_sub = 0
                slp,tt,tu = PullData(fname_ext,cf_loop_year,var_slp,
                    var_time,lat_flip,lon_shift,defs.read_scale,'all')
            if cf.model == "ccsm" and cf_loop_year in [301,531] and step == 0:
                # First step of this dataset isn't SLP but SP
                tsteps_sub +=1
                continue
            slp_step = N_multiply(slp[tsteps_sub,:,:].copy(),defs.accuracy)
            tsteps_sub += 1
        if cf.smooth and not cf.use_projection:
            slp_step_smooth = GFilter(slp_step,sigma=1,order=0,mode='wrap')
            slp_step_smooth.shape = im*jm
            slp_step.shape = im*jm
            N_put(slp_step_smooth,smask,N_take(slp_step,smask))
            slpint_smooth = slp_step_smooth.astype(numpy.int)
            # Uncomment to save example of what smoothing does
            #save_it = Save_NetCDF(slp_step_smooth,lons,lats,"smooth.nc",0)
            #save_it = Save_NetCDF(slp_step,lons,lats,"raw.nc",0)
            #sys.exit("\nStop Here")
        elif cf.use_projection:
            if cf.smooth:
                # Smooth then project
                slp_step_smooth = GFilter(slp_step,sigma=1,order=0,mode='wrap')
                slp_step_smooth.shape = im*jm
                slp_step.shape = im*jm
                N_put(slp_step_smooth,smask,N_take(slp_step,smask))
                slp_step = slp_step_smooth
                slp_step.shape = (jm,im)
            # Add cyclic longitude to data and longitudes
            nz,nx = addcyclic(slp_step,lons)
            # Shift data and longitudes to start at start_lon
            #   also converts 0-360 to +-180 format
            nz,nx = shiftgrid(180.0,nz,nx,start=False)
            # Interpolate
            smooth_field_nh,xx_nh,yy_nh = m_map_nh.transform_scalar(nz,nx,lats,nxx,nyy,
                    returnxy=True,masked=False,order=use_order)
            smooth_field_sh,xx_sh,yy_sh = m_map_sh.transform_scalar(nz,nx,lats,nxx,nyy,
                    returnxy=True,masked=False,order=use_order)
            # Get lat/lons of nyy by nxx evenly space grid.
            lons_nh, lats_nh = m_map_nh.makegrid(nxx,nyy)
            lons_sh, lats_sh = m_map_sh.makegrid(nxx,nyy)
            m_map = [m_map_nh,m_map_sh]
            xx = [xx_nh,xx_sh]
            yy = [yy_nh,yy_sh]
            plons = [lons_nh,lons_sh]
            plats = [lats_nh,lats_sh]
            smooth_field = [smooth_field_nh,smooth_field_sh]
            lat_bounds = [lat_bound*-1,lat_bound]
            ## Note: in the case of equal area grids it is possible that the projected
            ##   grids at high latitude contain overlap or contain more than one
            ##   grid from the lon/lat data. This leads to issues when centers are
            ##   found on the projected grid.
            smooth_grid_x_nh = list(xx_nh[0])
            smooth_grid_y_nh = [x[0] for x in yy_nh]
            smooth_grid_dx = smooth_grid_x_nh[1] - smooth_grid_x_nh[0]
            smooth_grid_dy = smooth_grid_y_nh[1] - smooth_grid_y_nh[0]
            # Fine the SLP gradient and Laplacian fields
            laplacian = [0,1]
            gradient = [0,1]
            # Need to divide by the square of the grid spacing, which
            #   puts it in units of hPa/m**2. However,  lapp_cutoff is scaled
            #   in units of hPa/deg lat**2, so scale by m per lat
            smooth_grid_dx_lat = smooth_grid_dx*0.00000900900901
            smooth_grid_dx_km = smooth_grid_dx*0.001
            for h in [0,1]:
                lap = FLap(smooth_field[h]*inv_accuracy,sigma=1.0,mode='nearest')
                lap = N_Divide(lap,4.0*smooth_grid_dx_lat**2)
                laplacian[h] = lap
                #
                grad = numpy.gradient(smooth_field[h]*inv_accuracy)
                # Scale to hPa/km
                grad = N_Divide(grad,2.0*smooth_grid_dx_km)
                # Make magnitude
                grad = numpy.sqrt(grad[0]*grad[0] + grad[1]*grad[1])
                gradient[h] = grad
            ## Uncomment to make a plot
            ## Set res = "l" where m_map created (above)
            #import matplotlib.pyplot as plt
            #hemis = ["nh","sh"]
            #for h in [0,1]:
            #    circles = [lat_bounds[h]]
            #    #pname = "%sfigs/other/slp_%s_%s%s" % (cf.out_path,hemis[h],adate,".png")
            #    #fig = plt.figure(figsize=(8,8),frameon=True)
            #    #ax = fig.add_subplot(1,1,1)
            #    #image3 = m_map[h].imshow(smooth_field[h]*inv_accuracy,interpolation='nearest',cmap=plt.cm.spectral)
            #    #m_map[h].drawparallels(circles,labels=[0,0,0,0])
            #    #m_map[h].drawmeridians([0.0],labels=[0,0,0,0])
            #    #m_map[h].drawmeridians([180.0],labels=[0,0,0,0])
            #    #m_map[h].drawcoastlines(linewidth=1.)
            #    #image3.set_clim(940.0,1040.0)
            #    #aa = fig.colorbar(image3,orientation='horizontal',extend="both",
            #            #spacing='uniform',ticks=None,fraction=0.021,pad=0.03,aspect=40)
            #    #fig.savefig(pname,dpi=144,facecolor='w',edgecolor='w',
            #        #orientation='landscape',bbox_inches='tight',
            #        #pad_inches=0.03)
            #    #plt.close('all')
            #    pname = "%sfigs/other/grad_%s_%s%s" % (cf.out_path,hemis[h],adate,".png")
            #    fig = plt.figure(figsize=(8,8),frameon=True)
            #    ax = fig.add_subplot(1,1,1)
            #    image3 = m_map[h].imshow(gradient[h],interpolation='nearest',cmap=plt.cm.hot_r)
            #    m_map[h].drawparallels(circles,labels=[0,0,0,0])
            #    m_map[h].drawmeridians([0.0],labels=[0,0,0,0])
            #    m_map[h].drawmeridians([180.0],labels=[0,0,0,0])
            #    m_map[h].drawcoastlines(linewidth=1.)
            #    image3.set_clim(0.0,0.07)
            #    aa = fig.colorbar(image3,orientation='horizontal',extend="both",
            #            spacing='uniform',ticks=None,fraction=0.021,pad=0.03,aspect=40)
            #    fig.savefig(pname,dpi=144,facecolor='w',edgecolor='w',
            #        orientation='landscape',bbox_inches='tight',
            #        pad_inches=0.03)
            #    plt.close('all')
            #    #pname = "%sfigs/other/lap_%s_%s%s" % (cf.out_path,hemis[h],adate,".png")
            #    #fig = plt.figure(figsize=(8,8),frameon=True)
            #    #ax = fig.add_subplot(1,1,1)
            #    ##image3 = m_map[h].imshow(laplacian[h],interpolation='nearest',cmap=plt.cm.hot_r)
            #    ##image3 = m_map[h].imshow(laplacian[h],interpolation='nearest',cmap=plt.cm.seismic)
            #    #image3 = m_map[h].imshow(laplacian[h],interpolation='nearest',cmap=plt.cm.spectral)
            #    #m_map[h].drawparallels(circles,labels=[0,0,0,0])
            #    #m_map[h].drawmeridians([0.0],labels=[0,0,0,0])
            #    #m_map[h].drawmeridians([180.0],labels=[0,0,0,0])
            #    #m_map[h].drawcoastlines(linewidth=1.)
            #    #image3.set_clim(-0.5,0.5)
            #    #aa = fig.colorbar(image3,orientation='horizontal',extend="both",
            #            #spacing='uniform',ticks=None,fraction=0.021,pad=0.03,aspect=40)
            #    #fig.savefig(pname,dpi=144,facecolor='w',edgecolor='w',
            #        #orientation='landscape',bbox_inches='tight',
            #        #pad_inches=0.03)
            #    #plt.close('all')
            #sys.exit("\nStop Here")

        slp_step.shape = im*jm
        slpint = slp_step.astype(numpy.int)
        if not cf.smooth:
            slpint_smooth = slpint

        esp_cnt[0] += im*jm # Total number processes grid cells
        # Check four corners of slpint
        #print slpint[0],slpint[143],slpint[-144],slpint[-1]

        if cf.smooth and not cf.use_projection:
            slpint_smooth.shape = (jm,im)
            # Find local minima in slpint_smooth
            local_min = numpy.nonzero(slpint_smooth == minimum_filter(
                slpint_smooth,footprint=neighbors,size=1,mode='wrap'))
            slpint_smooth.shape = jm*im
            # Discard polar minima
            tmpmin = [(x,y) for x,y in zip(local_min[1],local_min[0]) if y not in drop_polar]
            local_min = ([x[1] for x in tmpmin],[x[0] for x in tmpmin])
            # Special polar smooth and minima filter
            slp_step.shape = (jm,im)
            slp_step_smoothp = GFilter(slp_step,sigma=1,order=0,mode='nearest')
            slp_step.shape = jm*im
            plocal_min = numpy.nonzero(slp_step_smoothp == minimum_filter(
                slp_step_smoothp,footprint=neighbors,size=1,mode='wrap'))
            tmpmin = [(x,y) for x,y in zip(plocal_min[1],plocal_min[0]) if y in drop_polar]
            local_min[0].extend([x[1] for x in tmpmin])
            local_min[1].extend([x[0] for x in tmpmin])
            # Apply previous filters and create initial center list
            centers = {}
            for x,y in zip(local_min[1],local_min[0]):
                gridid = y*im + x
                if defs.plim_filter:
                    if slpint[gridid] > defs.plim_filter: continue
                if gridid in super_mask[0]: continue
                if gridid in super_mask[1]: continue
                centers[gridid] = slpint[gridid]
        elif cf.use_projection:
            centers = {}
            center_drop_grad =  {}
            center_drop_highslp = {}
            center_lap = {}
            centers = {}
            centers_9pnt = {}
            for h in [0,1]:
                # Find local minima (5 pnt)
                local_min = numpy.nonzero(smooth_field[h] == minimum_filter(
                    smooth_field[h],footprint=neighbors,size=1,mode='wrap'))
                # Find local minima (9 pnt)
                local_min_9pnt = numpy.nonzero(smooth_field[h] == minimum_filter(
                    smooth_field[h],footprint=neighbors_9pnt,size=1,mode='wrap'))
                # Apply previous filters and create initial center list
                cindex = -1
                for x,y in zip(xx[h][local_min],yy[h][local_min]):
                    cindex += 1
                    # Get central and 4 corner lon,lat values
                    center_lon,center_lat = m_map[h](x,y,inverse=True)
                    if abs(center_lat) < 15: continue
                    if h:
                        if center_lat > 0: continue
                    else:
                        if center_lat < 0: continue
                    # Check for additional lon/lat grids on the interior of
                    #   this grid.
                    if abs(center_lat) < 70.0:
                        find_grid = coarse_check
                    else:
                        find_grid = fine_check
                    # Find the closest gridID(s) to this center in the original grid
                    for pdx in find_grid:
                        for pdy in find_grid:
                            center_lon,center_lat = m_map[h](
                                    x+smooth_grid_dx*pdx,
                                    y+smooth_grid_dy*pdy,inverse=True)
                            center_lon = round(center_lon,4)
                            center_lat = round(center_lat,4)
                            center_lon = (center_lon + 360.0)%360.0
                            if center_lat > 90.0:
                                center_lat = 90.0
                            if center_lat < -90.0:
                                center_lat = -90.0
                            if center_lon < 0.0:
                                center_lon = 360.0 + center_lon
                            if center_lon > 360.0:
                                center_lon = center_lon - 360.0
                            gridid = l2g2(center_lon,center_lat,lons,lats,im)[0]
                            if gridid in centers: continue
                            if gridid in smask[h]: continue
                            if gridid in super_mask[h]: continue
                            centers[gridid] = slpint[gridid]
                            # Apply Concavity Test:
                            if laplacian[h][local_min[0][cindex]][local_min[1][cindex]] >= lapp_cutoff:
                                center_lap[gridid] = int(1000.0*
                                        laplacian[h][local_min[0][cindex]][local_min[1][cindex]])

                # Apply previous filters and create initial center list
                cindex = -1
                for x,y in zip(xx[h][local_min_9pnt],yy[h][local_min_9pnt]):
                    cindex += 1
                    # Get central and 4 corner lon,lat values
                    center_lon,center_lat = m_map[h](x,y,inverse=True)
                    if abs(center_lat) < 15: continue
                    if h:
                        if center_lat > 0: continue
                    else:
                        if center_lat < 0: continue
                    # Check for additional lon/lat grids on the interior of
                    #   this grid.
                    if abs(center_lat) < 70.0:
                        find_grid = coarse_check
                    else:
                        find_grid = fine_check
                    # Find the closest gridID(s) to this center in the original grid
                    for pdx in find_grid:
                        for pdy in find_grid:
                            center_lon,center_lat = m_map[h](
                                    x+smooth_grid_dx*pdx,
                                    y+smooth_grid_dy*pdy,inverse=True)
                            center_lon = round(center_lon,4)
                            center_lat = round(center_lat,4)
                            center_lon = (center_lon + 360.0)%360.0
                            if center_lat > 90.0:
                                center_lat = 90.0
                            if center_lat < -90.0:
                                center_lat = -90.0
                            if center_lon < 0.0:
                                center_lon = 360.0 + center_lon
                            if center_lon > 360.0:
                                center_lon = center_lon - 360.0
                            gridid = l2g2(center_lon,center_lat,lons,lats,im)[0]
                            if gridid in centers_9pnt: continue
                            if gridid not in smask[h]: continue
                            if gridid in super_mask[h]: continue
                            centers_9pnt[gridid] = slpint[gridid]
                            # Apply some other testsi
                            if gridid in troubled_centers[h]:
                                # Filter to see if center SLP too high
                                if slpint[gridid] > 1020000:
                                    center_drop_highslp[gridid] = 1
                                #  SLP Gradient Check (use projected data)
                                if gradient[h][local_min_9pnt[0][cindex]][local_min_9pnt[1][cindex]] > hpg_cutoff:
                                    center_drop_grad[gridid] = 1
                            # Concavity Test:
                            if laplacian[h][local_min_9pnt[0][cindex]][local_min_9pnt[1][cindex]] >= lapp_cutoff:
                                center_lap[gridid] = int(1000.0*
                                        laplacian[h][local_min_9pnt[0][cindex]][local_min_9pnt[1][cindex]])
                # Merge 5pnt and 9pnt centers
                centers.update(centers_9pnt)

                    ## Find the closest gridID(s) to this center in the original grid
                    #for pdx in find_grid:
                        #for pdy in find_grid:
                            #center_lon,center_lat = m_map[h](
                                    #x+smooth_grid_dx*pdx,
                                    #y+smooth_grid_dy*pdy,inverse=True)
                            #center_lon = round(center_lon,4)
                            #center_lat = round(center_lat,4)
                            #center_lon = (center_lon + 360.0)%360.0
                            #if center_lat > 90.0:
                                #center_lat = 90.0
                            #if center_lat < -90.0:
                                #center_lat = -90.0
                            #if center_lon < 0.0:
                                #center_lon = 360.0 + center_lon
                            #if center_lon > 360.0:
                                #center_lon = center_lon - 360.0
                            #gridid = l2g2(center_lon,center_lat,lons,lats,im)[0]
                            #if gridid in centers: continue
                            #if defs.plim_filter:
                                #if slpint[gridid] > defs.plim_filter: continue
                            #if defs.tropical_filter:
                                #if bot <= gridid <= top: continue
                            #if gridid in land_gridids: continue
                            #if cf.drop_troubled:
                                #if gridid in troubled_centers: continue
                            #if cf.use_hotspot == 2:
                                #if gridid in cf.hot_spots: continue
                            #centers[gridid] = slpint[gridid]
                            ## Apply some other tests
                            #if defs.troubled_filter and gridid in troubled_centers:
                                ## Filter to see if center SLP too high
                                #if slpint[gridid] > 1020000:
                                    #center_drop_highslp[gridid] = 1
                                ##  SLP Gradient Check (use projected data)
                                #if gradient[h][local_min[0][cindex]][local_min[1][cindex]] > hpg_cutoff:
                                    #center_drop_grad[gridid] = 1
                            ## Concavity Test:
                            #if laplacian[h][local_min[0][cindex]][local_min[1][cindex]] >= lapp_cutoff:
                                #center_lap[gridid] = int(1000.0*
                                        #laplacian[h][local_min[0][cindex]][local_min[1][cindex]])
        else:
            # If searching for high pressure reverse pressure field so
            # that highs are lows.
            if defs.find_highs:
                slpint_smooth = slpint_smooth*-1

            # Screen SLP field by defs.plim.
            if defs.plim_filter:
                tmp1 = N_less(slpint_smooth,defs.plim_filter)
            else:
                tmp1 = N_ones(N_size(slpint_smooth))

            # Screen SLP field in tropics
            if defs.tropical_filter:
                tmp1[bot:top] = 0

            # Limit center finding by hemisphere (for speed)
            if cf.limit_nh:
                tmp1[:tropical_s*im] = 0
            if cf.limit_sh:
                tmp1[tropical_n*im:] = 0

            # Apply previous filters and create initial center list
            centers = {}
            for gridid in xrange(maxid):
                if tmp1[gridid]:
                    if gridid in super_mask[0]: continue
                    if gridid in super_mask[1]: continue
                    centers[gridid] = slpint[gridid]
        # Total number processes grid cells dropped by pre-processing
        esp_cnt[1] += im*jm - len(centers)
        #----------------------------------------------------------------------
        # Stage 1: Cycle over initial center list, apply more filters and checks
        #   See doc/working_with_grids.pdf
        #----------------------------------------------------------------------
        if not centers:
            err_num = 1
            fmt = "\n\tFail Check %d: No initial centers for this timestep."
            smsg = fmt % (err_num)
            msg = "\t\tlen(centers): %d" % (len(centers))
            if cf.plot_on_error:
                msg1 = fmt % (err_num)
                center_loc = []
                discard_loc = []
                fmt = "%sfigs/other/error_%d_%s%s"
                Error_Plot(fmt % (cf.out_path,err_num,adate,cf.fig_format),
                           plot,slp_step*inv_accuracy,lons,lats,center_loc,
                           discard_loc,smsg)
            do_this(msg,smsg,date_stamp)
            if cf.exit_on_error: return 1

        total_time_steps += 1 # Got to initial center list

        # Pre bind for some speedup
        kept_centers = {}
        #kept_centers = []
        #kept_centers_append = kept_centers.append

        #print "\tStage 1 Grid Pool CNT:",len(centers)

        if high_res:
            # Create gdict and gdict_outer based on centers
            need = [x for x in centers.keys() if x not in gdict]
            g = inflate_gdict_limited(need,c_halo,im,jm)
            gdict.update(g)
            if not cf.use_projection:
                need = [x for x in centers.keys() if x not in gdict_outer]
                for n in need:
                    gdict_outer[n] = [shift_gridid(n,dc,dr,c_halo,halfway,im,jm)
                            for dr in [3,0,-3] for dc in [-3,0,3]]

        if pure_candidates and (cf.smooth or cf.use_projection):
            for center in centers:
                pcandidates[center] += 1
            continue
        # ***********************
        # Start Loop over Centers
        # ***********************
        for center in centers:
            # Centers to be kept fall out the bottom of this loop.

            # Retrieve pre-calculated 8 neighboring gridIDs
            if not cf.use_projection:
                if center in polar or high_res:
                    upm,upc,upp,cnm,cnt,cnp,dnm,dnc,dnp = gdict_outer[center][:]
                else:
                    upm,upc,upp,cnm,cnt,cnp,dnm,dnc,dnp = gdict[center][:]
            else:
                upm,upc,upp,cnm,cnt,cnp,dnm,dnc,dnp = gdict[center][:]

            #import util.print_gdict
            #PrintGdict = util.print_gdict.PrintGdict
            #print center
            #print PrintGdict(gdict[center])
            #if center in gdict_outer:
            #    print PrintGdict(gdict_outer[center])

            if not cf.smooth and not cf.use_projection:
                # Screen for along longitude maxima
                if N_less(slpint_smooth[cnm],slpint_smooth[center]) and \
                    N_greater(slpint_smooth[center],slpint_smooth[cnp]):
                    #if verbose_high: print "\t\t\tAlong Longitude Maxima"
                    esp_cnt[2] += 1
                    continue

                # Screen for along longitude strong non-minima (strong slant)
                if N_less(slpint_smooth[cnm],slpint_smooth[center]) and \
                    N_greater(slpint_smooth[cnp],slpint_smooth[center]):
                    if N_greater(abs(slpint_smooth[cnm]-slpint_smooth[cnp]),real_diff):
                        #if verbose_high: print "\t\t\tAlong Longitude non-minima"
                        esp_cnt[2] += 1
                        continue

                # Screen for along longitude strong non-minima (strong slant)
                if N_less(slpint_smooth[cnp],slpint_smooth[center]) and \
                    N_greater(slpint_smooth[cnm],slpint_smooth[center]):
                    if N_greater(abs(slpint_smooth[cnm]-slpint_smooth[cnp]),real_diff):
                        #if verbose_high: print "\t\t\tAlong Longitude non-minima"
                        esp_cnt[2] += 1
                        continue

                # Screen for along latitude maxima
                if N_less(slpint_smooth[dnc],slpint_smooth[center]) and \
                    N_greater(slpint_smooth[center],slpint_smooth[upc]):
                    #if verbose_high: print "\t\t\tAlong latitude maxima"
                    esp_cnt[2] += 1
                    continue # drop center

                # Screen for along latitude strong non-minima (strong slant)
                if N_less(slpint_smooth[dnc],slpint_smooth[center]) and \
                    N_greater(slpint_smooth[upc],slpint_smooth[center]):
                    if N_greater(abs(slpint_smooth[dnc]-slpint_smooth[upc]),real_diff):
                        #if verbose_high: print "\t\t\tAlong latitude non-minima"
                        esp_cnt[2] += 1
                        continue

                # Screen for along latitude strong non-minima (strong slant)
                if N_less(slpint_smooth[upc],slpint_smooth[center]) and \
                    N_greater(slpint_smooth[dnc],slpint_smooth[center]):
                    if N_greater(abs(slpint_smooth[dnc]-slpint_smooth[upc]),real_diff):
                        #if verbose_high: print "\t\t\tAlong latitude non-minima"
                        esp_cnt[2] += 1
                        continue

                # Screen for diagonal maxima
                if N_less(slpint_smooth[upm],slpint_smooth[center]) and \
                    N_greater(slpint_smooth[center],slpint_smooth[dnp]):
                    #if verbose_high: print "\t\t\tDiagonal maxima"
                    esp_cnt[2] += 1
                    continue # drop center
                else:
                    if N_less(slpint_smooth[dnm],slpint_smooth[center]) and \
                        N_greater(slpint_smooth[center],slpint_smooth[upp]):
                        #if verbose_high: print "\t\t\t*Diagonal maxima"
                        esp_cnt[2] += 1
                        continue # drop center

                # Screen for diagonal non-minima (strong slant)
                if N_less(slpint_smooth[upm],slpint_smooth[center]) and \
                    N_greater(slpint_smooth[dnp],slpint_smooth[center]):
                    if N_greater(abs(slpint_smooth[upm]-slpint_smooth[dnp]),real_diff):
                        #if verbose_high: print "\t\t\tAlong latitude non-minima"
                        esp_cnt[2] += 1
                        continue
                else:
                    if N_less(slpint_smooth[dnm],slpint_smooth[center]) and \
                     N_greater(slpint_smooth[upp],slpint_smooth[center]):
                        if N_greater(abs(slpint_smooth[dnm]-slpint_smooth[upp]),real_diff):
                            #if verbose_high: print "\t\t\tAlong latitude non-minima"
                            esp_cnt[2] += 1
                            continue

                # Screen for diagonal non-minima (strong slant)
                if N_less(slpint_smooth[dnp],slpint_smooth[center]) and \
                    N_greater(slpint_smooth[upm],slpint_smooth[center]):
                    if N_greater(abs(slpint_smooth[dnp]-slpint_smooth[upm]),real_diff):
                        #if verbose_high: print "\t\t\tAlong latitude non-minima"
                        esp_cnt[2] += 1
                        continue
                else:
                    if N_less(slpint_smooth[upp],slpint_smooth[center]) and \
                     N_greater(slpint_smooth[dnm],slpint_smooth[center]):
                        if N_greater(abs(slpint_smooth[upp]-slpint_smooth[dnm]),real_diff):
                            #if verbose_high: print "\t\t\tAlong latitude non-minima"
                            esp_cnt[2] += 1
                            continue
            if pure_candidates:
                pcandidates[center] += 1
                continue

            # To force map of candidates, uncomment and comment filters below
            #   and uncomment the Error_Plot.

            #if verbose_high: print "\t\t\tKeep"
            # Okay if got here at least a local minima
            total_centers_used += 1
            if cf.use_projection:
                if center in center_drop_highslp:
                    if defs.keep_discards:
                        msg = form_center(uci_stamp,adate,ijdict[center][3],
                            ijdict[center][2],center,centers[center],0,
                            0,troubflag,None,None,None,None,0)
                        dumped_centers_save.writelines(msg)
                    total_centers_cnt[troubflag] += 1
                    if cf.save_plots or cf.save_stats:
                        flag_cnt[center,troubflag] += 1
                    if make_plots:
                        temp_discards[center] = form_center(uci_stamp,
                            adate,ijdict[center][3],ijdict[center][2],
                            center,centers[center],0,0,
                            troubflag,None,None,None,None,1)
                    # Remove center
                    if center in center_drop_grad:
                        del center_drop_grad[center]
                    continue
                if center in center_drop_grad:
                    if defs.keep_discards:
                        msg = form_center(uci_stamp,adate,ijdict[center][3],
                            ijdict[center][2],center,centers[center],0,
                            0,troubflag,None,None,None,None,0)
                        dumped_centers_save.writelines(msg)
                    total_centers_cnt[troubflag] += 1
                    if cf.save_plots or cf.save_stats:
                        flag_cnt[center,troubflag] += 1
                    if make_plots:
                        temp_discards[center] = form_center(uci_stamp,
                            adate,ijdict[center][3],ijdict[center][2],
                            center,centers[center],0,0,
                            troubflag,None,None,None,None,1)
                    continue
                if center not in center_lap:
                    if defs.keep_discards:
                        msg = form_center(uci_stamp,adate,ijdict[center][3],
                            ijdict[center][2],center,centers[center],0,
                            0,lapflag,None,None,None,None,0)
                        dumped_centers_save.writelines(msg)
                    total_centers_cnt[lapflag] += 1
                    if cf.save_plots or cf.save_stats:
                        flag_cnt[center,lapflag] += 1
                    if make_plots:
                        temp_discards[center] = form_center(uci_stamp,
                            adate,ijdict[center][3],ijdict[center][2],
                            center,centers[center],0,0,
                            lapflag,None,None,None,None,1)
                    continue
                center_laplacian = center_lap[center]
                if pure_candidates:
                    pcandidates[center] += 1
                    continue
            else:
                # DISCOVERY MODE: Turn off (via commenting from DISCOVERY START
                # to  DISCOVERY END and uncommenting the next line down. Allows
                # for center finding without Concavity Test.
                center_laplacian = 0

                # DISCOVERY START
                h = (0 if center//im <= eq_grid else 1)
                if center in troubled_centers[h]:
                    # Derived SLP Test: SLPs from grids whose surface elevation is
                    # above sea level or who have immediate neighbors with that
                    # quality are tested with more caution due to the indirect
                    # methods used to determine SLP.

                    # Filter to see if center SLP too high implying that SLP
                    # reduction error in play. The value 1020 is based on
                    # examination of 2D histograms of center SLP and the
                    # horizontal SLP gradient from NCEP Reanalysis suggesting
                    # that high SLP centers over land mostly occur over very
                    # high topography (Greenland & Antarctica) and are suspect.
                    if slpint[center] > 1020000:
                        if defs.keep_discards:
                            msg = form_center(uci_stamp,adate,ijdict[center][3],
                                ijdict[center][2],center,centers[center],0,
                                center_laplacian,troubflag,None,None,None,None,0)
                            dumped_centers_save.writelines(msg)
                        total_centers_cnt[troubflag] += 1
                        if cf.save_plots or cf.save_stats:
                            flag_cnt[center,troubflag] += 1
                        if make_plots:
                            temp_discards[center] = form_center(uci_stamp,
                                adate,ijdict[center][3],ijdict[center][2],
                                center,centers[center],0,center_laplacian,
                                troubflag,None,None,None,None,1)
                        continue # drop center

                    # Ring Symmetry Test: Normally the SLP around a cyclone
                    #   increases radially in a fairly smooth and symmetrical
                    #   way. Errors in SLP reduction can create large departures
                    #   from this and we flag these as potentially non-physical
                    #   results. Found to be the result of large relief
                    #   (absolute topography changes) around the center rather
                    #   than just high topography.
                    if high_res:
                        ring = N_take(slpint,gdict_outer[center][:])
                    else:
                        ring = N_take(slpint,gdict[center][:])
                    ring_slp_diff = [abs(x - ring[4]) for x in ring]
                    tmp = ring_slp_diff[:]
                    tmp.sort()
                    # Find HPG (horizontal pressure gradient hPa/km)
                    clon = ijdict[center][2]
                    clat = ijdict[center][3]
                    bit = ring_slp_diff.index(tmp[-1])
                    if high_res:
                        bite_high = gdict_outer[center][bit]
                    else:
                        bite_high = gdict[center][bit]
                    # Find Horizontal Pressure Gradient
                    rlon = ijdict[bite_high][2]
                    rlat = ijdict[bite_high][3]
                    distx = gcd(clon,clat,rlon,rlat)
                    # Take the steepest gradient for comparison.
                    hpg_high = tmp[-1]/distx
                    # Holton says the horizontal pressure gradient is on the
                    # order of 0.01 hPa/km.The NCEP Reanalysis suggests that
                    # overland HPG above this value are often in areas of
                    # high relief and/or high topography.
                    # Find the Absolute HPG

                    # Save HPG for post analysis uncomment save lap below
                    #lap_save.writelines("%f " % (abs(hpg_high)*inv_accuracy))
                    if abs(hpg_high)*inv_accuracy > hpg_cutoff: # make hPa/km
                        if defs.keep_discards:
                            msg = form_center(uci_stamp,adate,ijdict[center][3],
                                ijdict[center][2],center,centers[center],0,
                                center_laplacian,troubflag,None,None,None,None,0)
                            dumped_centers_save.writelines(msg)
                        total_centers_cnt[troubflag] += 1
                        if cf.save_plots or cf.save_stats:
                            flag_cnt[center,troubflag] += 1
                        if make_plots:
                            temp_discards[center] = form_center(uci_stamp,
                                adate,ijdict[center][3],ijdict[center][2],
                                center,centers[center],0,center_laplacian,
                                troubflag,None,None,None,None,1)
                        continue # drop center

                # Due to contraction of grid spacing with latitude this part of the
                #   troubled_filter and the laplacian filter are not applied to
                #   near-polar grids
                if center not in polar:
                    # Concavity Test: See Note 1

                    # Find 9-pnt average SLP for each of the 9-pnts around center.
                    if high_res:
                        all_nine = gdict_outer[center][:]
                        nine_pnt_aves = []
                        nine_pnt_aves_append = nine_pnt_aves.append # prebind
                        for eachone in all_nine:
                            if high_res:
                                if eachone not in gdict_outer:
                                    gdict_outer[eachone] = [
                                        shift_gridid(eachone,dc,dr,c_halo,halfway,im,jm)
                                        for dr in [3,0,-3] for dc in [-3,0,3]]
                            nine_pnt_aves_append(N_average(N_take(
                                slpint,gdict_outer[eachone][:])))
                    else:
                        all_nine = gdict[center][:]
                        nine_pnt_aves = []
                        nine_pnt_aves_append = nine_pnt_aves.append # prebind
                        for eachone in all_nine:
                            if high_res:
                                if eachone not in gdict:
                                    gdict[eachone] = [
                                        shift_gridid(eachone,dc,dr,c_halo,halfway,im,jm)
                                        for dr in [1,0,-1] for dc in [-1,0,1]]
                            nine_pnt_aves_append(N_average(N_take(
                                slpint,gdict[eachone][:])))

                    # Find 9-pnt Laplacian of the averaged SLPs. See Note 2
                    center_laplacian = 0
                    row = center//im
                    if ldict[row][0]: # non-polar (90 degrees)

                        # Scale pressures back to hPa
                        nine_pnt_aves = [x*inv_accuracy for x in nine_pnt_aves]

                        # 1/a^2sin^2(lat) * d^2P/dlon^2
                        termA = ldict[row][1] * (nine_pnt_aves[3] -
                                                    2.0*nine_pnt_aves[4] +
                                                    nine_pnt_aves[5]) / dlon_sq
                        # 1/a^2 * d^2P/dlat^2
                        termB = defs.inv_earth_radius_sq * ((nine_pnt_aves[1] -
                                                             2.0*nine_pnt_aves[4] +
                                                             nine_pnt_aves[7])/
                                                            dlat_sq)
                        # cot(lat)/a^2 * dP/dlat
                        termC = ldict[row][2] * (nine_pnt_aves[1] -
                                                    nine_pnt_aves[7]) / two_dlat

                        lapp = defs.two_deg_lat*(termA + termB + termC) # hPa/lat^2

                        # Scale for saving
                        center_laplacian = int(lapp*1000.0)

                        if lapp < lapp_cutoff:
                            if defs.keep_discards:
                                msg = form_center(uci_stamp,adate,ijdict[center][3],
                                    ijdict[center][2],center,centers[center],0,
                                    center_laplacian,lapflag,None,None,None,None,0)
                                dumped_centers_save.writelines(msg)
                            total_centers_cnt[lapflag] += 1
                            if cf.save_plots or cf.save_stats:
                                flag_cnt[center,lapflag] += 1
                            if make_plots:
                                temp_discards[center] = form_center(uci_stamp,adate,
                                    ijdict[center][3],ijdict[center][2],center,
                                    centers[center],0,center_laplacian,lapflag,None,
                                    None,None,None,1)
                            continue # drop center
                    # DISCOVERY END

            # Create a rdict entry if needed
            if high_res:
                if center not in rdict_exp:
                    row = center//im
                    col = center%im
                    rdict_exp[center] = [shift_gridid(x,col,0,c_halo,halfway,im,jm)
                            for x in rdict[row]]

            # Calculate regional_average SLP
            slp_ave = int(N_average(N_take(slpint,rdict_exp[center])))

            # Store possible centers
            kept_centers[center] = form_center(uci_stamp,adate,
                ijdict[center][3],ijdict[center][2],center,centers[center],
                slp_ave,center_laplacian,0,None,None,None,None,1)

        if not cf.use_projection:
            #
            # Deal with Special Case of polar grids
            #
            fnc_out = fit_poles(0,1,slpint,row_start,row_end,0)
            if fnc_out:
                # SH
                slp_ave = int(N_average(N_take(slpint,rdict_exp[0])))
                kept_centers[0] = form_center(uci_stamp,adate,
                    ijdict[0][3],ijdict[0][2],0,slpint[0],
                    slp_ave,0,0,None,None,None,None,1)
                total_centers_used += 1
            fnc_out = fit_poles(jm-1,-1,slpint,row_start,row_end,0)
            if fnc_out:
                # NH
                slp_ave = int(N_average(N_take(slpint,rdict_exp[maxid-1])))
                kept_centers[maxid-1] = form_center(uci_stamp,adate,
                    ijdict[maxid-1][3],ijdict[maxid-1][2],maxid-1,slpint[maxid-1],
                    slp_ave,0,0,None,None,None,None,1)
                total_centers_used += 1

        ## Make debugging plot
        #title = "Kept Centers [%d]" % len(centers)
        #discard_loc = []
        #center_loc = []
        #d_colors = []
        #c_colors = []
        #for center in kept_centers:
        #    center_loc.append((unform_lon(kept_centers[center]['Lon']),
        #    unform_colat(kept_centers[center]['CoLat'])))
        #Error_Plot("%sfigs/other/initial_%s%s" % (cf.out_path,adate,".png"),
        #    plot,slp_step*inv_accuracy,lons,lats,center_loc,discard_loc,title,
        #            c_colors,d_colors)
        #print "Made","%sfigs/other/initial_%s%s" % (cf.out_path,adate,".png")
        #sys.exit("Stop Here")

        #if cf.smooth:
        #    # Put back original SLP field
        #    del slp_step_smooth
        #    del slpint_smooth

        # ***********************
        # End Loop over Centers
        # ***********************
        if not kept_centers:
            err_num = 2
            fmt = "\n\tFail Check %d: No kept centers for this timestep."
            smsg = fmt % (err_num)
            msg = "\t\tlen(kept_centers): %d" % (len(kept_centers))
            if make_plots:
                center_loc = []
                discard_loc = []
                for c in temp_discards:
                    discard_loc.append((unform_lon(temp_discards[c]['Lon']),
                        unform_colat(temp_discards[c]['CoLat'])))
                fmt = "%sfigs/other/error_%d_%s%s"
                Error_Plot(fmt % (cf.out_path,err_num,adate,cf.fig_format),
                            plot,slp_step*inv_accuracy,lons,lats,center_loc,
                            discard_loc,smsg)
            do_this(msg,smsg,date_stamp)
            if cf.exit_on_error: return 1

        #-----------------------------------------------------------------------
        # Stage 2: Cycle over centers that passed Stage 1. Reduce the center
        #    list to contain only a single center within defs.critical_radius of
        #    each other.
        #    See doc/over_lap_rules.pdf
        #-----------------------------------------------------------------------
        local_verbose = 0
        if local_verbose:
            from util import PrintColumns
            msg1 = "\n\n\tStage 2 Center CNT: %d from %d candidates"
            print msg1 % (len(kept_centers),total_centers_used)
            msg1 = "\t\t% 3d %s"
            #for e in flags_used:
            #    print msg1 % (total_centers_cnt[e],known_flags[e])

        # Use this to turn of troubled filter in overlap reduction.
        tcc = troubled_centers
        center_pool = set(kept_centers.keys())
        kept_pool = set([])
        discard_pool = set([])
        if local_verbose:
            print
            print step,date_stamp
            print "%s" % ("*"*20)
            print "\nCenter Pool (%d): " % len(center_pool)
            PrintColumns(center_pool,indent_tag="\t\t",fmt="%5d ",cols=10,width=10)
            print

        # Deal with centers that directly touch. An issue with very high
        #   resolution data.
        if local_verbose:
            print "\nRunning deal_with_touching"
        if high_res:
            # Create gdict based on centers
            need = [x for x in center_pool if x not in gdict]
            g = inflate_gdict_limited(need,c_halo,im,jm)
            gdict.update(g)

        overlap_set = form_overlap_sets(center_pool,gdict)
        if overlap_set == 1:
            print "Doing",date_stamp,step
            if cf.exit_on_error: return 1
        n_c = len(center_pool)
        rt = deal_with_touching(overlap_set,center_pool,kept_pool,discard_pool,
                kept_centers,tcc,deal_with_reducible_bt,local_verbose)
        if rt == 1:
            print "Doing",date_stamp,step
            if cf.exit_on_error: return 1
#fix    # Update to discard touching (at this point not being logged in discards,
        kept_centers = dict((k, v) for k, v in kept_centers.iteritems()
                if k in kept_pool)
        center_pool = set(kept_centers.keys())

        total_centers_used = total_centers_used - (n_c - len(center_pool))
        kept_pool = set([])
        discard_pool = set([])
        if local_verbose:
            print "\n*Center Pool (%d): " % len(center_pool)
            PrintColumns(center_pool,indent_tag="\t\t",fmt="%5d ",cols=10,width=10)
            print
        if local_verbose:
            print emsg2 % (len(kept_centers),len(kept_pool),len(center_pool))
        ## Uncomment to make a plot showing the touching for this center
        ##   Requires make_plots = 1 to import matplotlib etc.
        #for center in overlap_set:
        #    #if not overlap_set[center]: continue # uncomment to skip singletons
        #    title = "Overlap_Set[%d]" % center
        #    discard_loc = []
        #    d_colors = []
        #    c_colors = ['red']
        #    center_loc = [(unform_lon(kept_centers[center]['Lon']),
        #        unform_colat(kept_centers[center]['CoLat']))]
        #    for c in overlap_set[center]:
        #        if c == center: continue
        #        discard_loc.append((unform_lon(kept_centers[c]['Lon']),
        #            unform_colat(kept_centers[c]['CoLat'])))
        #        d_colors.append(flag_colors[2])
        #    Error_Plot("%sfigs/other/touching_%07d_%s%s" % (cf.out_path,center,
        #        adate,cf.fig_format),plot,slp_step*inv_accuracy,lons,lats,
        #        center_loc,discard_loc,title,c_colors,d_colors)

        # Create overlap sets for each center using rdict
        if local_verbose:
            print "\nRunning form_overlap_sets"
        if high_res:
            need = [x for x in center_pool if x not in rdict_exp]
            for n in need:
                row = n//im
                col = n%im
                rdict_exp[n] = [shift_gridid(x,col,0,c_halo,halfway,im,jm)
                        for x in rdict[row]]
        overlap_set = form_overlap_sets(center_pool,rdict_exp)
        if overlap_set == 1:
            print "Doing",date_stamp,step
            if cf.exit_on_error: return 1
        ## Uncomment to make a plot showing the overlap_set for this center
        ##   Requires make_plots = 1 to import matplotlib etc.
        #for center in overlap_set:
        #    #if not overlap_set[center]: continue # uncomment to skip singletons
        #    title = "Overlap_Set[%d]" % center
        #    discard_loc = []
        #    d_colors = []
        #    c_colors = ['red']
        #    center_loc = [(unform_lon(kept_centers[center]['Lon']),
        #        unform_colat(kept_centers[center]['CoLat']))]
        #    for c in overlap_set[center]:
        #        if c == center: continue
        #        discard_loc.append((unform_lon(kept_centers[c]['Lon']),
        #            unform_colat(kept_centers[c]['CoLat'])))
        #        d_colors.append(flag_colors[2])
        #    Error_Plot("%sfigs/other/overlap_%07d_%s%s" % (cf.out_path,center,
        #        adate,cf.fig_format),plot,slp_step*inv_accuracy,lons,lats,
        #        center_loc,discard_loc,title,c_colors,d_colors)

        # Deal with asymmetric overlap
        if local_verbose:
            print "\nRunning correct_overlap_asymmetry"
        rt = correct_overlap_asymmetry(overlap_set,center_pool,local_verbose)
        if rt == 1:
            print "Doing",date_stamp,step
            if cf.exit_on_error: return 1
        # Check for errors
        if len(center_pool)+len(kept_pool) != len(kept_centers):
            print emsg1 % (1,len(kept_centers),len(kept_pool),len(center_pool))
            print "Doing",date_stamp,step
            if cf.exit_on_error: return 1
        if local_verbose:
            print emsg2 % (len(kept_centers),len(kept_pool),len(center_pool))
        ## Uncomment to make a plot showing the overlap_set for this center
        ##   Requires make_plots = 1 to import matplotlib etc.
        #if not rt:
        #    continue
        #for center in overlap_set:
        #    if center not in rt: continue
        #    #if not overlap_set[center]: continue # uncomment to skip singletons
        #    title = "Overlap_Set[%d]" % center
        #    discard_loc = []
        #    d_colors = []
        #    c_colors = ['red']
        #    center_loc = [(unform_lon(kept_centers[center]['Lon']),
        #        unform_colat(kept_centers[center]['CoLat']))]
        #    for c in overlap_set[center]:
        #        if c == center: continue
        #        discard_loc.append((unform_lon(kept_centers[c]['Lon']),
        #            unform_colat(kept_centers[c]['CoLat'])))
        #        d_colors.append(flag_colors[2])
        #    Error_Plot("%sfigs/other/aoverlap_%07d_%s%s" % (cf.out_path,center,
        #        adate,cf.fig_format),plot,slp_step*inv_accuracy,lons,lats,
        #        center_loc,discard_loc,title,c_colors,d_colors)

        # Deal with singletons (centers with no overlap)
        if local_verbose:
            print "\nRunning trim_singletons"
        rt = trim_singletons(overlap_set,center_pool,kept_pool,local_verbose)
        if rt == 1:
            print "Doing",date_stamp,step
            if cf.exit_on_error: return 1
        # Check for errors
        if len(center_pool)+len(kept_pool) != len(kept_centers):
            print emsg1 % (2,len(kept_centers),len(kept_pool),len(center_pool))
            print "Doing",date_stamp,step
            if cf.exit_on_error: return 1
        if local_verbose:
            print emsg2a % (len(kept_centers),len(kept_pool),len(center_pool))
        ## Uncomment to make a plot showing the overlap_set for centers that
        ##   have overlaps
        #for center in overlap_set:
        #    title = "Overlap_Set[%d]" % center
        #    discard_loc = []
        #    d_colors = []
        #    c_colors = ['red']
        #    center_loc = [(unform_lon(kept_centers[center]['Lon']),
        #        unform_colat(kept_centers[center]['CoLat']))]
        #    for c in rdict_exp[center]:
        #        if c in overlap_set[center]: continue
        #        discard_loc.append((ijdict[c][2],ijdict[c][3]))
        #        d_colors.append(flag_colors[1])
        #    for c in overlap_set[center]:
        #        if c == center: continue
        #        discard_loc.append((unform_lon(kept_centers[c]['Lon']),
        #            unform_colat(kept_centers[c]['CoLat'])))
        #        d_colors.append(flag_colors[2])
        #    Error_Plot("%sfigs/other/has_overlap_%07d_%s%s" % (cf.out_path,
        #        center,adate,cf.fig_format),plot,slp_step*inv_accuracy,lons,
        #        lats,center_loc,discard_loc,title,c_colors,d_colors)

        # All the remaining centers in center_pool have some sort of overlap
        #   with another center.
        # There are two types of overlap:
        #   1) Reducible: The overlap_sets of these centers area equivalent,
        #       which is to say that they all contain the same centers. For
        #       example Center A *only* overlaps Center B and Center B *only*
        #       overlaps Center A. In this case we can just apply our selection
        #       rules to select the one to keep and the one to discard.
        #   2) Irreducible: At least one center has an overlap_set that is not
        #       equivalent to those of the other overlapping centers. For
        #       example, Center A *only* overlaps Center B, but Center B
        #       overlaps Center A *and* Center C, and Center C *only* overlaps
        #       Center B. In this case we cannot simply apply our selection
        #       rules. We attempt to unravel this problem iteratively so see if
        #       it can be broken down in to reducible parts and then solved.
        if local_verbose:
            print "\nRunning deal_with_reducible"
        rt = deal_with_reducible(overlap_set,center_pool,kept_pool,
            discard_pool,kept_centers,tcc,deal_with_reducible_bt,local_verbose)
        if rt == 1:
            print "Doing",date_stamp,step
            if cf.exit_on_error: return 1
        # Check for errors
        if len(center_pool)+len(kept_pool)+len(discard_pool) != len(kept_centers):
            print emsg3 % (3,len(kept_centers),len(kept_pool),
                len(discard_pool),len(center_pool))
            print "Doing",date_stamp,step
            if cf.exit_on_error: return 1
        if local_verbose:
            print emsg4 % (len(kept_centers),len(kept_pool),len(discard_pool),
                len(center_pool))
        ## Uncomment to make a plot showing the overlap_set for centers that
        ##   have irreducible overlaps
        #from itertools import chain
        #for center in overlap_set:
        #    cf.fig_format = ".png"
        #    title = "Overlap_Set[%d]" % center
        #    case_set = [overlap_set[c] for c in overlap_set[center]]
        #    members = set(chain(*case_set))
        #    discard_loc = []
        #    d_colors = []
        #    c_colors = ['red']
        #    center_loc = [(unform_lon(kept_centers[center]['Lon']),
        #        unform_colat(kept_centers[center]['CoLat']))]
        #    for c in rdict_exp[center]:
        #        if c in overlap_set[center]: continue
        #        discard_loc.append((ijdict[c][2],ijdict[c][3]))
        #        d_colors.append(flag_colors[1])
        #    for c in overlap_set[center]:
        #        if c == center: continue
        #        discard_loc.append((unform_lon(kept_centers[c]['Lon']),
        #            unform_colat(kept_centers[c]['CoLat'])))
        #        d_colors.append(flag_colors[2])
        #    for c in members:
        #        if c in overlap_set[center]: continue
        #        discard_loc.append((unform_lon(kept_centers[c]['Lon']),
        #            unform_colat(kept_centers[c]['CoLat'])))
        #        d_colors.append(flag_colors[3])
        #    Error_Plot("%sfigs/other/irr_overlap_%07d_%s%s" % (cf.out_path,
        #        center,adate,cf.fig_format),plot,slp_step*inv_accuracy,lons,
        #        lats,center_loc,discard_loc,title,c_colors,d_colors)

        if center_pool:
            if local_verbose:
                print "\nRunning deal_with_irreducible"
            rt = deal_with_irreducible(overlap_set,center_pool,kept_pool,
                discard_pool,kept_centers,tcc,deal_with_irreducible_bt,
                local_verbose)
            if rt == 1:
                print "Doing",date_stamp,step
                if cf.exit_on_error: return 1

        # Check for errors
        if len(center_pool)+len(kept_pool)+len(discard_pool) != len(kept_centers):
            print emsg3 % (4,len(kept_centers),len(kept_pool),
                len(discard_pool),len(center_pool))
            print "Doing",date_stamp,step
            if cf.exit_on_error: return 1
        if local_verbose:
            print emsg5 % (len(kept_centers),len(kept_pool),len(discard_pool),
                len(center_pool))

        ## Uncomment to make a plot retained and discarded centers
        #title = "Kept Centers [%d]" % len(kept_pool)
        #discard_loc = []
        #center_loc = []
        #d_colors = []
        #c_colors = []
        #for center in kept_pool:
        #    center_loc.append((unform_lon(kept_centers[center]['Lon']),
        #        unform_colat(kept_centers[center]['CoLat'])))
        #for c in discard_pool:
        #    discard_loc.append((unform_lon(kept_centers[c]['Lon']),
        #        unform_colat(kept_centers[c]['CoLat'])))
        #    d_colors.append(flag_colors[2])
        #Error_Plot("%sfigs/other/kept_%s%s" % (cf.out_path,adate,cf.fig_format),
        #    plot,slp_step*inv_accuracy,lons,lats,center_loc,discard_loc,title,
        #    c_colors,d_colors)

        # Check for errors
        if len(discard_pool)+len(kept_pool) != len(kept_centers):
            print "\n\tWarning Missing Center(s)"
            print "\t\tStarting Center Pool (%d): " % len(kept_centers)
            print "\t\tNew Keep Centers (%d)" % len(kept_pool)
            print "\t\tNew Center Pool (%d): " % len(center_pool)
            print "Doing",date_stamp,step
            if cf.exit_on_error: return 1
        if center_pool:
            print "\n\tWarning center_pool unemptied"
            print center_pool
            print "Doing",date_stamp,step
            if cf.exit_on_error: return 1
        # Double Check
        for center in kept_pool:
            region = rdict_exp[center]
            overlap_set = set([g for g in kept_pool if g in region])
            if len(overlap_set) > 1:
                print "\n\tWarning Recheck of Overlap for center %d" % center
                print "\toverlap_set",overlap_set
                print "Doing",date_stamp
                print "Doing",date_stamp,step
                if cf.exit_on_error: return 1

        #-----------------------------------------------------------------------
        # Stage 3: Cycle over centers, refining the center location by way of
        #    fractional grid positions based on fitting a parabolic function
        #   to the local slp field, and then save to file.
        #-----------------------------------------------------------------------

        # Find fractional grid positions based on fitting a parabolic function
        # to the local slp field.
        for center in kept_centers:
            # Skip discards if not saving
            if not defs.keep_discards and center in discard_pool:
                total_centers_cnt[regflag] += 1
                continue

            upm,upc,upp,cnm,cnt,cnp,dnm,dnc,dnp = gdict[center][:]

            # Fit the E-W SLP Field.
            numerator = N_subtract(slpint[cnm],slpint[cnp])
            denominator = N_subtract(N_add(slpint[cnm],
                                           slpint[cnp]),2*slpint[center])
            if denominator == 0:
                factor = 0.0 # use original grid
            else:
                factor = 0.5 * numerator/denominator
                # This happens because of adjacent equality
                if factor > 0.5:
                    factor = 0.5
                if factor < -0.5:
                    factor = -0.5
            final_x = factor + ijdict[center][0]
            if final_x < 0.0: # wrap around
                final_x = (im-1) + final_x
            new_lon = g2l(final_x,start_lon,start_lat,dlon,dlat,jm,"lon",
                "free",False,True,faux_grids)

            # Fit the N-S SLP Field.
            numerator = N_subtract(slpint[dnc],slpint[upc])
            denominator = N_subtract(N_add(slpint[dnc],slpint[upc]),
                2*slpint[center])
            if denominator == 0:
                factor = 0.0 # use original grid
            else:
                factor = 0.5 * numerator/denominator
                # This happens because of adjacent equality
                if factor > 0.5:
                    factor = 0.5
                if factor < -0.5:
                    factor = -0.5
            final_y = factor + ijdict[center][1]
            if final_y > jm-1:
                final_y = jm-1
            if final_y < 0.0:
                final_y = 0
            new_lat = g2l(final_y,start_lon,start_lat,dlon,dlat,jm,"lat",
                "free",False,True,faux_grids)

            # Save to file.
            save_flag = 0
            if center in discard_pool: save_flag = regflag
            # msg = defs.center_fmt % (date_stamp,adate,
            #         int((90.0-new_lat)*100),int(new_lon*100),center,
            #         kept_centers[center][1],kept_centers[center][4],kept_centers[center][5],save_flag,0,0,
            #         uci_stamp,int(round((90.0-new_lat)*100)),
            #         int(round(new_lon*100)),defs.usi_template)
            #
            kept_centers[center]['Lon'] = form_lon(new_lon)
            kept_centers[center]['CoLat'] = form_colat(new_lat)
            # Don't use new_lat,new_lon due to rounding can result in
            #   duplicate uci (see write_mcms)
            kept_centers[center]['UCI'] = form_uci(uci_stamp,ijdict[center][3],
                ijdict[center][2])
            if kept_centers[center]['UCI'] == 1:
                print "Warning: Saving Center"
                if cf.exit_on_error: return 1
            kept_centers[center]['Flags'] = save_flag

            msg = form_center_alt(kept_centers[center])
            if msg == 1:
                print "Warning: Saving Center"
                if cf.exit_on_error: return 1

            if cf.save_plots or cf.save_stats:
                flag_cnt[center,save_flag] += 1
            total_centers_cnt[save_flag] += 1
            # To save both kept and discarded to the same file, treating the
            #   save_flag != 0 centers at alternatives for tracking use
            #   the following.
            # centers_save.writelines(msg)
            #if make_plots:
            #if save_flag == 0:
            #    temp_centers[center] = kept_centers[center]
            #else:
            #    temp_discards[center] = kept_centers[center]

            # To save kept an discards to separate files use the following.
            if save_flag == 0:
                # Save these centers to disk as potential cyclones
                if make_plots:
                    temp_centers[center] = kept_centers[center]
                centers_save.writelines(msg)
            else:
                # Save discarded centers to disk
                if make_plots:
                    temp_discards[center] = kept_centers[center]
                dumped_centers_save.writelines(msg)

        # Check that all Stored UCI are unique
        used_uci = len(set([kept_centers[c]['UCI'] for c in kept_centers]))
        if used_uci != len(kept_centers):
            # Check if repeat
            done = {}
            for c in kept_centers:
                seen = [x for x in kept_centers if kept_centers[x]['UCI'] == kept_centers[c]['UCI']]
                if len(seen) > 1:
                    print "\n\tWarning Duplicate UCI"
                    for x in seen:
                        if x not in done:
                            print
                            print_center(kept_centers[x])
                            done[x] = 1
            # Else Skip
            b = set([kept_centers[c]['UCI'] for c in kept_centers])
            c = [kept_centers[c]['UCI'] for c in kept_centers]
            if [x for x in c if x not in b] or [x for x in b if x not in c]:
                print "\n\tWarning Non_unique UCI",used_uci,len(kept_centers)
                print b
                print
                print c
                print [x for x in c if x not in b],[x for x in b if x not in c]
                print "Doing",date_stamp,step
            print
            if cf.exit_on_error: return 1

        ## Uncomment to make a plot retained and discarded centers
        ## REQUIRES plot_on_error to be true/1
        #title = "Final Centers %s UTC: %d from %d candidates" % (
                #date_stamp,len(kept_pool),len(kept_centers))
        #discard_loc = []
        #center_loc = []
        #d_colors = []
        #c_colors = []
        #for c in temp_centers:
            #center_loc.append((unform_lon(temp_centers[c]['Lon']),
                #unform_colat(temp_centers[c]['CoLat'])))
        #for c in temp_discards:
            #discard_loc.append((unform_lon(temp_discards[c]['Lon']),
                #unform_colat(temp_discards[c]['CoLat'])))
            #d_colors.append(flag_colors[temp_discards[c]['Flags']])
        ##Error_Plot("%sfigs/other/final_%s%s" % (cf.out_path,adate,
        ##    cf.fig_format),plot,slp_step*inv_accuracy,lons,lats,center_loc,
        ##    discard_loc,title,c_colors,d_colors)
        #Error_Plot_Polar("%sfigs/other/final_global_%s%s" % (cf.out_path,adate,
            #cf.fig_format),plotmap_polar,slp_step*inv_accuracy,lons,lats,
            #eq_grid,eq_grid_on_edge,center_loc,discard_loc,title,c_colors,
            #d_colors,cf.limit_nh,cf.limit_sh)
        ##if tsteps == 1: sys.exit("Stop Here")

    # ***********************
    # End Loop over Time
    # ***********************

    # -------------------------------------------------------------------------
    # Clean up
    # -------------------------------------------------------------------------

    # Close open files
    try:
        centers_save.close()
    except:
        print "\n\tWARNING: Error Closing %s." % (centers_file)
        return 1
    if defs.keep_log:
        log_file.close()
        sys.stdout = screenout # redirect stdout back to screen
    if defs.keep_discards:
        try:
            dumped_centers_save.close()
        except:
            print "\n\tWARNING: Error Closing %s." % (dumped_centers_file)
            return 1
    if cf_fast[0]:
        del PullData,slpint,slp_step,slp
        del adates,uci_stamps,date_stamps
        return cf_loop_year

    if pure_candidates:
        save_it = Save_NetCDF(pcandidates,lons,lats,pure_file,0,
                name="pure candidates",
                long_name="Total Number of Pure Candidate Centers",units='Counts')
        return cf_loop_year

    #
    # FINAL check to be sure all timesteps run and all centers accounted for.
    #
    if total_centers_used != sum(total_centers_cnt):
        msg = "%04d Total Count Error:\n\ttotal_centers_used = %d"
        msg += "\n\ttotal_centers_cnt = %s sum(%d)"
        print msg % (cf_loop_year,total_centers_used,repr(total_centers_cnt),
                        sum(total_centers_cnt))
        if cf.exit_on_error: return 1

    # Last minute check that reasonable count.
    if total_centers_cnt[0] < 10*tsteps:
        err_num = 6
        smsg = "\n\tFail Check %d: Final Center CNT < 10*tsteps" % (err_num)
        msg = "\t\tFinal Center CNT: %d < 10*tsteps %d "
        msg = msg % (total_centers_cnt[0],10*tsteps)
        date_stamp = "Full Record"
        do_this(msg,smsg,date_stamp)
        if cf.exit_on_error: return 1

    #
    # Create Report File and Plots if requested.
    #
    base_file = centers_file.replace("centers.txt","cf_base_report.txt")
    base_file = base_file.replace(cf.out_path,"%sstats/" % (cf.out_path))
    try:
        base_save = open(base_file,"w")
    except:
        print "\n\tWARNING: Error Creating %s." % (base_file)
        return 1
    base_save.writelines("%04d\n" % (cf_loop_year))
    base_save.writelines("Total Time Steps: %d\n" % (tsteps))
    base_save.writelines("Based Grid Count (im*jm): %d\n" % (im*jm))
    base_save.writelines("Total Possible Center CNT: %d\n" % (esp_cnt[0]))
    msg1 = "Centers dropped by preprocessing: %d (%6.2f%%) or %d/step\n"
    base_save.writelines(msg1 % (esp_cnt[1],
        100.0*(float(esp_cnt[1])/float(esp_cnt[0])),float(esp_cnt[1])/float(tsteps)))
    msg1 = "Centers dropped by non-minima: %d (%6.2f%%) or %d/step\n"
    base_save.writelines(msg1 % (esp_cnt[2],
        100.0*(float(esp_cnt[2])/float(esp_cnt[0])),float(esp_cnt[2])/float(tsteps)))
    msg1 = "Raw candidate centers: %d (%6.2f%%) or %d/step\n"
    base_save.writelines(msg1 % (total_centers_used,
        100.0*(float(total_centers_used)/float(esp_cnt[0])),float(total_centers_used)/float(tsteps)))
    msg1 = "Kept candidate centers: %d (%6.2f%%) or %d/step\n"
    base_save.writelines(msg1 % (total_centers_cnt[0],
        100.0*(float(total_centers_cnt[0])/float(esp_cnt[0])),float(total_centers_cnt[0])/float(tsteps)))
    base_save.close()


    report_file = centers_file.replace("centers.txt","centers_report.txt")
    report_file = report_file.replace(cf.out_path,"%sstats/" % (cf.out_path))
    try:
        report_save = open(report_file,"w")
    except:
        print "\n\tWARNING: Error Creating %s." % (report_file)
        return 1

    report_save.writelines("%04d\n" % (cf_loop_year))
    msg1 = "Final Center CNT: %d (%6.2f%%) from %d candidates where\n"
    if total_centers_used > 0:
        msg = msg1 % (total_centers_cnt[0],
                    100.0*(total_centers_cnt[0]/total_centers_used),
                    total_centers_used)
    else:
        msg = "total_centers_used = 0"
    report_save.writelines(msg)
    msg1 = "\t% 6d\t(%6.2f%%)\t%s\n"
    for e in flag_files:
        if total_centers_used > 0:
            msg = msg1 % (total_centers_cnt[e],
                        100.0*(total_centers_cnt[e]/total_centers_used),
                        known_flags[e])
        else:
            msg = "total_centers_used = 0"
        report_save.writelines(msg)
    report_save.close()

    if cf.save_plots or cf.save_stats:
        # Make frequency plots/data files by year
        for flag in flag_files:
            # Just counts
            # FIX error with numpy/matplot lib and missing so just set to 0 now
            #comp_out = N_where(flag_cnt[:,flag] < 1.,-10000000000.0,
            #    flag_cnt[:,flag])
            comp_out = N_where(flag_cnt[:,flag] < 1.,0.0,flag_cnt[:,flag])
            fmt = "%sfigs/maps/global/annual/%s_cf_%s_%04d%s"
            pname = fmt % (cf.out_path,cf.model,flag_files[flag],
                cf_loop_year,cf.fig_format)
            #if cf.save_plots: # Uncomment for annual plots
            #    fplot.create_fig()
            #    fplot.add_field(lons,lats,comp_out,ptype='pcolor',)
            #    fplot.finish(pname)
            #    #print "\tMade figure %s" % (pname)
            if cf.save_stats:
                pname = pname.replace(cf.fig_format,".nc")
                pname = pname.replace("figs/maps/global/annual","netcdfs")
                save_it = Save_NetCDF(flag_cnt[:,flag],lons,lats,pname,0,
                    name=flag_files[flag],long_name=known_flags[flag],
                    units='Counts')
                #print "\tCreated file %s" % (pname)
                del save_it
            del comp_out

    # Save memory when Pull_Data called in loop stores a copy of slp
    # and thus doubles the memory footprint of the code.
    del PullData,slpint,slp_step,slp
    del adates,uci_stamps,date_stamps
    if make_plots:
        del plotmap,fplot,plot
    if cf.progress:
        try:
            prog_save.close()
        except:
            print "\n\tWARNING: Error Closing %s." % (prog_file)
            return 1

    # Exit
    return cf_loop_year

def main(argv):

    # MCMS module imports
    from extras import ProgressBarAlt
    from extras import ProgressBar
    from util import summary_report
    from util import PrintColumns
    from util import FileExists
    from util import ParseRunFile

    # Use to remake summery stats without rerunning the center_finder
    recover_saved = 0
    if recover_saved:
        print "\n\tWarning Recovering Saved Rather than Running!\n"

    def CompletedCallbackPbar(step):
        """
        Reports the process of a multiprocessing session.
        """
        # Assumes the calling program has global ProgressBar defined.
        global results
        if step == 1: sys.exit()
        results.append(step)
        mp_pbar.update_time(len(results),which_one=step)
        print(mp_pbar)
        return

    # --------------------------------------------------------------------------
    # Parse Request
    # --------------------------------------------------------------------------

    # Define the base run_file template
    here_path = os.path.abspath(os.path.dirname(__file__))
    run_file_template = "%s/cfg/mcms_center_finder_template.py" % (here_path)
    if not FileExists(run_file_template,__name__): return 1

    # --------------------------------------------------------------------------
    # Parse Request
    # --------------------------------------------------------------------------
    rf = ParseRunFile(run_file_template,argv,here_path)
    if rf == 1:
        return rf
    years = range(int(rf.start_year),int(rf.end_year)+1)
    fast = [0,0]
    # --------------------------------------------------------------------------
    # Execute the request
    # --------------------------------------------------------------------------
    if not recover_saved:
        if rf.single_processor:
            # This is a single processor version or for debugging
            pbara = ProgressBarAlt
            for loop_year in pbara(years,"\tFinding Centers ... ",60):
                msg = find_centers(rf.getthis,loop_year,fast)
                if msg == 1: sys.exit()
                results.append(msg)
        elif rf.fast_run:
            PROCESSORS = 1
            if sys.version_info >= (2, 6):
                import multiprocessing as mp
                PROCESSORS = mp.cpu_count()
            else:
                print "Parallelism requires python 2.6 or higher."
                return 1
            # Manually set processors count
            if rf.processor_cnt:
                PROCESSORS = rf.processor_cnt
            if PROCESSORS > mp.cpu_count():
                PROCESSORS = mp.cpu_count()
            msg = "\n\tStarting %s FAST (process id %s) with %d processors"
            print msg % ('Processing Data',os.getpid(),PROCESSORS)
            # ----------------------------------------------------------------------
            # Start Main Loop over super_years
            # ----------------------------------------------------------------------
            # Spawn n=PROCESSORS processes
            pool = mp.Pool(PROCESSORS)
            args_used = []
            mp_pbar = ProgressBar(PROCESSORS,"Finished")
            mp_pbar.fill_char = '-'
            checkin = CompletedCallbackPbar
            for i in xrange(PROCESSORS):
                fast = [PROCESSORS,i]
                if i != 15: continue # mikemike
                args_used.append((rf.getthis,rf.start_year,fast))
                pool.apply_async(find_centers,args_used[-1],callback=checkin)
            pool.close()
            pool.join()
        else:
            # ----------------------------------------------------------------------
            # Note on parallel processing
            # ----------------------------------------------------------------------

            # If you are running on a machine with multiple CPUs or multicore CPUs
            #   we can take advantage of that and speed things up a lot.

            # The form of parallelism used here is dumb brute force in that
            # I simply launch processes that work independently on part of
            # the problem and put it all back together when I'm done. This
            # is not MPI message passing or OpenMP shared memory.

            # Test requires python 2.6 or higher
            #
            # Test for python version with parallel processing capabilities.
            PROCESSORS =  1
            if sys.version_info >= (2, 6):
                import multiprocessing as mp
                PROCESSORS = mp.cpu_count()
            else:
                print "Parallelism requires python 2.6 or higher."
                return 1

            # Manually set processors count
            if rf.processor_cnt:
                PROCESSORS = rf.processor_cnt
            # Don't breakup by less than 1 year per processor
            max_processors = rf.nyears
            if PROCESSORS > max_processors:
                PROCESSORS = max_processors
            if PROCESSORS > mp.cpu_count():
                PROCESSORS = mp.cpu_count()

            msg = "\n\tStarting %s (process id %s) with %d processors (%d-%d)"
            print msg % ('Processing Data',os.getpid(),
                PROCESSORS,rf.start_year,rf.end_year)

            # ----------------------------------------------------------------------
            # Start Main Loop over super_years
            # ----------------------------------------------------------------------
            # Spawn n=PROCESSORS processes
            pool = mp.Pool(PROCESSORS)
            args_used = []
            mp_pbar = ProgressBar(rf.nyears,"Finished")
            mp_pbar.fill_char = '-'
            checkin = CompletedCallbackPbar
            for loop_year in xrange(rf.start_year,rf.end_year+1):
                # Object holding objects
                args_used.append((rf.getthis,loop_year,fast))
                pool.apply_async(find_centers,args_used[-1],callback=checkin)
            pool.close()
            pool.join()

        if not rf.fast_run:
            print "\n\n"
            PrintColumns(results,indent_tag="\t\t",fmt="% 5d",cols=5,width=5)
            if abs(years[0]-years[-1]) > 1:
                # Check at all years where finished.
                a = set(results)
                b = set([x for x in xrange(rf.start_year,rf.end_year+1)])
                c = a.symmetric_difference(b)
                if len(c):
                    c = sorted(list(c))
                    msg = "\tWarning results and years differ.\n\t\tDiffering: %s"
                    print msg % (repr(c))
                    if rf.exit_on_error: return 1

    print "\n\tSaving Final Reports and Files..."

    #import sys; sys.exit("Stop Early")

    if rf.pure_candidates:
        import numpy
        import netCDF4 as NetCDF
        from util import Save_NetCDF
        fmt = "%snetcdfs/%s_cf_pure_canidates_%s%s"
        stag = "%04d-%04d" % (int(rf.start_year),int(rf.end_year))
        nfiles = [fmt % (rf.out_path,rf.model, "%04d" % loop_year,".nc")
                for loop_year in xrange(rf.start_year,rf.end_year+1)]
        if len(nfiles) == 1: return
        first = 1
        for nfile in nfiles:
            cdf = NetCDF.Dataset(nfile,'r',format='NETCDF3_CLASSIC')
            data_raw = cdf.variables['pure candidates']
            data_in = data_raw[:] # put into a numpy array
            lons = cdf.variables["lon"][:]
            lats = cdf.variables["lat"][:]
            shape = data_in.shape
            if first:
                first = 0
                data_out = numpy.ma.zeros(shape,dtype=numpy.float32)
                for j in xrange(shape[0]):
                    for i in xrange(shape[1]):
                        if data_in[j,i] > 0:
                            data_out[j,i] = data_in[j,i]
                #data_out = numpy.ma.array(data_in[:],dtype=numpy.float32,copy=1)
            else:
                for j in xrange(shape[0]):
                    for i in xrange(shape[1]):
                        if data_in[j,i] > 0:
                            if data_out[j,i] >= 0:
                                data_out[j,i] = data_out[j,i] + data_in[j,i]
                #data_out = numpy.ma.add(data_out,data_in)
            cdf.close()
            del data_raw,data_in
        pure_file = "%snetcdfs/%s_cf_pure_canidates_%s.nc" % (rf.out_path,rf.model,stag)
        save_it = Save_NetCDF(data_out,lons,lats,pure_file,0,
                name="pure candidates",
                long_name="Total Number of Pure Candidate Centers",units='Counts')
        return

    if rf.fast_run:
        #PROCESSORS = 16
        # Reassemble fast_run data file
        import shutil
        if recover_saved:
            if sys.version_info >= (2, 6):
                import multiprocessing as mp
                PROCESSORS = mp.cpu_count()
                # Manually set processors count
                if rf.processor_cnt:
                    PROCESSORS = rf.processor_cnt
            else:
                print "Parallelism requires python 2.6 or higher."
                return 1
            # Manually set processors count
            if rf.processor_cnt:
                PROCESSORS = rf.processor_cnt
            if PROCESSORS > mp.cpu_count():
                PROCESSORS = mp.cpu_count()
        file_list = os.listdir(rf.out_path)
        file_list.sort()
        pfiles = [x for x in file_list
                if (x.find("centers") != -1 and x.endswith(".txt"))]
        pfiles = [x for x in pfiles if not x.find("progress") != -1]
        pfiles.sort()
        if len(pfiles) != PROCESSORS:
            sys.exit("Warning Missing Files? pfiles (%d/%d): %s"
                    % (len(pfiles),PROCESSORS,repr(pfiles)))
        base_file = rf.out_path + pfiles[0].replace("_0000","")
        if os.path.exists(base_file):
            sys.exit("Warning %s exists!" % base_file)
        try:
            b_save = open(base_file,"a")
        except:
            sys.exit("\n\tWARNING: Error Creating %s." % (base_file))
        for i in xrange(PROCESSORS):
            try:
                f = open(rf.out_path + pfiles[i],'r')
            except:
                sys.exit("\n\tWARNING: Error Reading %s." % (rf.out_path + pfiles[i]))
            try:
                shutil.copyfileobj(f,b_save)
            except:
                sys.exit("\n\tWARNING: Error Concatenating %s to %s."
                        % (rf.out_path + pfiles[i],base_file))
            try:
                f.close()
            except:
                sys.exit("\n\tWARNING: Error Closing %s." % (rf.out_path + pfiles[i]))
        try:
            b_save.close()
        except:
            sys.exit("\n\tWARNING: Error Closing %s." % (base_file))
        print "\n\n Finished Concatenating to %s" % base_file
        print "\n\n Helpful hint"
        print "\n\n rm -f %s" % (" ".join(["%s%s" % (rf.out_path,x) for x in pfiles]))
        print "\n\n"
        # Make at least the "passed" statistic report
        from util import Save_NetCDF
        import numpy
        from util import acquire_centers_year
        stag = "%04d-%04d" % (int(rf.start_year),int(rf.start_year))
        rf.tail = "_centers.txt"
        rf.cut_tail = len(rf.tail)
        fmt = "%snetcdfs/%s_cf_%s_fast_%s%s"
        ofile = fmt % (rf.out_path,rf.model,"passed",stag,".nc")
        if os.path.exists(ofile):
            os.remove(ofile)
        lons = numpy.array(rf.flons)
        lats = numpy.array(rf.flats)
        im = len(lons)
        jm = len(lats)
        maxid = im*jm
        cnts = numpy.zeros(maxid)
        readit,centers,ee,cc = acquire_centers_year(rf,rf.start_year)
        ids = readit.ids
        for center in centers:
           cgrid = readit.center_holder[center][ids['GridID']]
           cnts[cgrid] += 1
        save_it = Save_NetCDF(cnts,lons,lats,ofile,0,
                name="passed",long_name="Centers",
                units='Counts')
        return
    # --------------------------------------------------------------------------
    # Create a report for the whole time record
    # --------------------------------------------------------------------------
    total_cnt = summary_report(rf.out_path,rf.model,rf.super_years)

    # Sum the stats files over the whole period and make a frequency of the
    #   total number of candidate centers (passed+discards)
    if rf.save_stats:
        import numpy
        import netCDF4 as NetCDF
        from util import plotmap
        from util import Save_NetCDF

        # See flag_files in find_centers
        flag_files = {0 : "passed", 1 : "lap", 2 : "reg",
            3 : "troub"}
        known_flags = {0 : "Passed all filters",
            1 : "Failed concavity/Laplacian filter",
            2 : "Failed regional minimum filter",
            3 : "Failed troubled center filter"}
        fmt = "%snetcdfs/%s_cf_%s_%s%s"
        fmt1 = "%sfigs/maps/global/annual/%s_cf_%s_%s%s"
        stag = "%04d-%04d" % (int(rf.start_year),int(rf.end_year))
        one = 1
        flag_store = {}
        pbara = ProgressBarAlt
        for flag in pbara(flag_files,"\tSaving Statistics ... ",60):
        #for flag in flag_files:
            nfiles = [fmt % (rf.out_path,rf.model,flag_files[flag],
                    "%04d" % loop_year,".nc") for loop_year in
                    xrange(rf.start_year,rf.end_year+1)]
            sfiles = " ".join(nfiles)
            ofile = fmt % (rf.out_path,rf.model,flag_files[flag],stag,".nc")
            if os.path.exists(ofile):
                os.remove(ofile)
            first = 1
            for nfile in nfiles:
                cdf = NetCDF.Dataset(nfile,'r', format='NETCDF3_CLASSIC')
                data_raw = cdf.variables[flag_files[flag]]
                data_in = data_raw[:] # put into a numpy array
                lons = cdf.variables["lon"][:]
                lats = cdf.variables["lat"][:]
                if first:
                    first = 0
                    data_out = numpy.ma.array(data_in[:],
                        dtype=numpy.float32,copy=1)
                    if one:
                        one = 0
                        candidates = numpy.ma.array(data_in[:],
                            dtype=numpy.float32,copy=1)
                else:
                    data_out = numpy.ma.add(data_out,data_in)
                    candidates = numpy.ma.add(candidates,data_in)
                cdf.close()
                del data_raw,data_in
            # Store summary file
            flag_store[flag] = data_out
            #save_it = Save_NetCDF(data_out,lons,lats,ofile,0,
            #    name=flag_files[flag],long_name=known_flags[flag],
            #    units='Counts')
            del data_out
            #pname = fmt1 % (rf.out_path,rf.model,flag_files[flag],
            #    stag,rf.fig_format)
            #fplot = plotmap(missing=-10000000000.0,color_scheme="hot_r",lon_0=0.0)
            #fplot.create_fig()
            #fplot.add_field(lons,lats,data_out,ptype='pcolor',)
            #fplot.finish(pname)
        ofile = fmt % (rf.out_path,rf.model,"candidates",stag,".nc")
        save_it = Save_NetCDF(candidates,lons,lats,ofile,0,
                name="candidates",long_name="Total Number of Candidate Centers",
                units='Counts')
        NMW = numpy.ma.masked_where
        NW = numpy.where
        ND = numpy.true_divide
        candidates.mask = numpy.ma.nomask
        bot = numpy.array(candidates)
        pbara = ProgressBarAlt
        for flag in pbara(flag_files,"\tSaving More Statistics ... ",60):
        #for flag in flag_files:
            ofile = fmt % (rf.out_path,rf.model,flag_files[flag],stag,".nc")
            ffile = fmt % (rf.out_path,
                rf.model,"freq_"+flag_files[flag],stag,".nc")
            data_out = flag_store[flag]
            # Scale by the number of total centers (passed+discarded)
            data_out.mask = numpy.ma.nomask
            top = numpy.array(data_out)
            # Faster than freq = NW(bot > 0, ND(top,bot), 0), and no
            #   divide by zero warning.
            freq = top/(bot+(bot==0.0))
            # numpy.where and numpy.divide don't work to avoid division by zero
            #   in this case, numpy.ma.divide does, we can suppress the warning
            #   however.
            #numpy.seterr(divide='ignore', invalid='ignore')
            #freq = NW(bot > 0, ND(top,bot), 0)
            freq = NMW(freq <= 0,freq)
            save_it = Save_NetCDF(freq,lons,lats,ffile,0,
                name=flag_files[flag],long_name=known_flags[flag],
                units='Frequency of Occurrence Among Candidate Centers')
            save_it = Save_NetCDF(top,lons,lats,ofile,0,
                name=flag_files[flag],long_name=known_flags[flag],
                units='Counts')
            if rf.save_plots:
                pname = fmt1 % (rf.out_path,rf.model,flag_files[flag],
                    stag,rf.fig_format)
                fplot = plotmap(missing=0.0,color_scheme="hot_r",lon_0=0.0)
                fplot.create_fig()
                fplot.add_field(lons,lats,freq,ptype='pcolor',)
                fplot.finish(pname)
                pname = fmt1 % (rf.out_path,rf.model,"freq_"+flag_files[flag],
                    stag,rf.fig_format)
                fplot = plotmap(missing=0.0,color_scheme="hot_r",lon_0=0.0)
                fplot.create_fig()
                fplot.add_field(lons,lats,top,ptype='pcolor',)
                fplot.finish(pname)
        del candidates
    return

if __name__ == "__main__":
    sys.exit( main(sys.argv) )
