#!/usr/bin/env python -tt
""" This module assigns an intensity to a cyclone.

    Options/Arguments:

    Returns/Creates:

    Examples:

    Notes: 

     Cyclone Intensity: Based on central SLP.
        1) The 1/3, 1/3, 1/3 rule: take SLP source distribution and partitioning
           it into 3 equal parts with the lower third being for "weak" cyclones,
           the middle third for "moderate" cyclones and the upper third being for
           "strong" cyclones.
              * Can be applied on a center-by-center basis in which case a cyclone
                track may span multiple intensity categories over time. The
                alternative approach applies a single intensity category to an
                entire cyclone track, often based on the minimum lifetime SLP of
                the track.
              * This is an adaptive measure so results will differ based on the
                distribution of the SLPs used. Thus a strong cyclone might differ
                when the SLP source covers the entire record, a single hemisphere,
                a single season, or just the month the track resides in. This is
                an issue when looking for changes/trends in cyclone intensity as
                this is a sliding scale.
              * Method of application (choice of SLP pool):
                 1) Whole record and hemisphere (i.e, 1979-2008 NH).
                 2) Whole record, season and hemisphere (i.e., 1979-2008 DJF NH).
                 3) By year, hemisphere and season (i.e, 1979 DJF NH).
                 4) By month and hemisphere (i.e., Jan 1979 NH).

        2) Fixed SLP Thresholds: Advantage being a fixed criteria that climate
           change could be seen relative too. For example, use the same criteria
           for a control and experiment to see if sifts intensity.

        3) Multivariate method used in Bauer and Del Genio (2005); based on ideas
           from Zielinski (2002).

              I = (1030.0 - SLP() + Deepinging_Rate + Pressure_Gradient

              * The deepening rate is taken over the 24 hours preceding
                the SLP minimum (units of hPa per 24 hours). A few
                cyclones are acquired at or near their minimum lifetime
                SLP (due to the break-up of a preexisting system). In
                these cases the SLP tendency of the succeeding 24 hours
                was used instead (although this takes some liberties with
                the idea of cyclone development being symmetrical about
                the point of peak intensity.). The pressure gradient
                represents the maximum value within a 1500 km radius of
                the SLP minima (units of hPa per 1000 km).

    Author: Mike Bauer  <mbauer@giss.nasa.gov>

    Log:
    2006/12  MB - File created.
    2010/07  MB - Moved to v5.
    2011/11  MB - Added option to store percentile rather than 1,2,3 for 
                    intensity... this allows for more granuality in sorting
                    by intensity and it is easy to find the 33,66 quartiles
                    for str,mod,wea anyway.
    2012/06  MB - Added option to read the boundaries from another run and use
                    them to define intensity (e.g., use control run values to
                    evaluate an experimental run).
"""

# Standard library imports
import sys
import os
import math

# Third party module imports
import extras.progress_bar

# MCMS module imports


#def test_spilt(slps,lower_third,upper_third):
#    sall = len(slps)
#    low = len([x for x in slps if x <= lower_third])
#    mid = len([x for x in slps if lower_third < x < upper_third])
#    hi  = len([x for x in slps if x >= upper_third])
#    if sall != low+mid+hi:
#        sys.exit("Percentile Error")

def plot_pdf(plt,PDF,slps,en_stats,bins_centers,bins_left_edge,title,pname,scale=0.001):

    if pname.find(".pdf") != -1:
        dpi = 144
    else:
        dpi = 140
    # en_stats = [mean,median,std,lower_third,upper_third,slps.min,slps.max,slps.size]
    fmter = "Count: %d\nMin/Max: %d/%d\nMean/Median: %.2f/%.2f\nSTD: %.2f"             
    slps1 = [x*scale for x in slps]
    stitle = fmter % (en_stats[7],int(en_stats[5]*scale),int(en_stats[6]*scale),
                      en_stats[0]*scale,en_stats[1]*scale,en_stats[2]*scale)
    stdensity = PDF([x*scale for x in bins_centers],en_stats[0]*scale,en_stats[2]*scale)
    fig = plt.figure()
    ax = fig.add_subplot(111)
    n, bins, patches = plt.hist(slps1,[x*scale for x in bins_left_edge],normed=True,facecolor='grey',alpha=0.75)
    ax.plot([x*scale for x in bins_centers],stdensity, 'r--', linewidth=1)
    ax.set_xlabel('SLP (hPa)')
    ax.set_ylabel('Normalized Count (i.e., a PDF)')
    ax.set_title(title)
    plt.suptitle(stitle,fontsize=6,x=0.73,y=0.98,horizontalalignment='left')
    ax.grid(True)
    midpnt_x = (en_stats[3]*scale+en_stats[5]*scale)*0.5
    mid_pnt_y = plt.ylim()[1] * 0.5
    yup = mid_pnt_y*0.01
    ax.text(midpnt_x,mid_pnt_y+yup,"Strong",horizontalalignment='center',fontsize='small')
    # Arrow from lower_third to slps.min 
    ax.annotate("",xy=(en_stats[3]*scale,mid_pnt_y),xytext=(en_stats[5]*scale,mid_pnt_y),
               xycoords='data',arrowprops=dict(arrowstyle="<->"))
    midpnt_x = (en_stats[4]*scale+en_stats[6]*scale)*0.5 + 0
    ax.text(midpnt_x,mid_pnt_y+yup,"Weak",horizontalalignment='center',fontsize='small')
    # Arrow from upper_third to slps.max
    ax.annotate("",xy=(en_stats[4]*scale,mid_pnt_y),xytext=(en_stats[6]*scale,mid_pnt_y),
                xycoords='data',arrowprops=dict(arrowstyle="<->"))
    plt.savefig(pname,dpi=140,facecolor='w',edgecolor='w',orientation='landscape')
    
    # Print data
    print "+++++"
    print pname
    import numpy
    print "normalized"
    bins_width = abs(bins_left_edge[0]-bins_left_edge[1])
    #print bins_width
    hist, bins = numpy.histogram(slps1,bins=[x*scale for x in bins_left_edge],normed=True)
    for i in xrange(len(hist)):
        print "%20g %20g %20g" % (bins_left_edge[i]*0.001,bins_centers[i]*0.001,hist[i]*bins_width*0.001)
    #print sum(hist)*bins_width*0.001
    #print "raw"
    #hist, bins = numpy.histogram(slps1,bins=[x*scale for x in bins_left_edge],normed=False)
    #for i in xrange(len(hist)):
    #    print "%20g %20g %20g" % (bins_left_edge[i]*0.001,bins_centers[i]*0.001,hist[i])
    print "----"
    
    ## Make cumulative distribtuion
    #pname = pname.replace("hist","histc")
    #fig = plt.figure()
    #ax = fig.add_subplot(111)
    #n, bins, patches = plt.hist(slps1,[x*scale for x in bins_left_edge],histtype='step',
    #                            normed=True,cumulative=True,facecolor='grey',alpha=0.75)
    #ax.set_xlabel('SLP (hPa)')
    #ax.set_ylabel('Normalized Cumulative Count')
    #ax.set_title(title)
    #plt.suptitle(stitle, fontsize=6,x=0.73,y=0.98,horizontalalignment='left')
    #ax.grid(True)
    #midpnt_x = (en_stats[3]*scale+en_stats[5]*scale)*0.5    
    #ax.text(midpnt_x,0.335,"Strong",horizontalalignment='center',fontsize='small')
    #ax.annotate("",xy=(en_stats[3]*scale,0.333),xytext=(en_stats[5]*scale,0.333),
    #           xycoords='data',arrowprops=dict(arrowstyle="<->"))
    #midpnt_x = (en_stats[4]*scale+en_stats[6]*scale)*0.5 + 0
    #ax.text(midpnt_x,0.669,"Weak",horizontalalignment='center',fontsize='small')
    #ax.annotate("",xy=(en_stats[4]*scale,0.666),xytext=(en_stats[6]*scale,0.666),
    #            xycoords='data',arrowprops=dict(arrowstyle="<->"))
    #plt.savefig(pname,dpi=140,facecolor='w',edgecolor='w',orientation='landscape')
    #plt.close('all')

#def percentile(N, percent, key=lambda x:x):
#    """
#    Find the percentile of a list of values.

#    @parameter N - is a list of values.
#    @parameter percent - a float value from 0.0 to 1.0.
#    @parameter key - optional key function to compute value from each element of N.

#    @return - the percentile of the values
#    """
#    if not N:
#        return None
#    k = (len(N)-1) * percent
#    f = math.floor(k)
#    c = math.ceil(k)
#    if f == c:
#        return key(N[int(k)])
#    d0 = key(N[int(f)]) * (c-k)
#    d1 = key(N[int(c)]) * (k-f)
#    return d0+d1

def setup_bins(bins_left_edge,bins_width,verbose=0):
    """Set up bins for histograms"""
    # Make a few extras
    bins_width = abs(bins_left_edge[0]-bins_left_edge[1])
    bins_centers = bins_left_edge + 0.5*bins_width
    bins_right_edge = bins_left_edge + bins_width
    if verbose:
        # To print out bins
        fmt = "Bin % 4d: % 7.2f <= % 7.2f < % 7.2f"
        for bin in range(len(bins_left_edge)):
            print fmt % (bin,bins_left_edge[bin],bins_centers[bin],
                         bins_right_edge[bin])
        print "Total Bin Cnt: %d" % (len(bins_centers))
    return (bins_centers,bins_right_edge)

def main(defs_set, what_do, over_write_years, over_write_out_path,
        shared_path, over_write_slp_path, model, skip_to_plots, make_plots,
        land_sea, zmean, fig_format, alter_files, continuous,the_calendar,
        jd_fake,super_uci_stamps,super_time_data,super_lookup,
        break_fname,month_sort_order,timestep,use_other_bounds,out_path_alt,
        model_alt,clim_tag_alt,ls_tag_alt,slp_pools,used_pool,fixed,use_fixed,
        basin,basin_name,which_hemi):

    # Some generally useful calendar tools.
    months = {1: 'January', 2: 'February', 3: 'March', 4: 'April', 5: 'May',
              6: 'June', 7: 'July', 8: 'August', 9: 'September', 10: 'October',
              11: 'November', 12: 'December'}
    months_inv = {'February': 2, 'October': 10, 'March': 3, 'August': 8, 'May': 5,
                  'January': 1, 'June': 6, 'September': 9, 'April': 4,
                  'December': 12, 'July': 7, 'November': 11}
    months_short = {1: 'JAN', 2: 'FEB', 3: 'MAR', 4: 'APR', 5: 'MAY',
              6: 'JUN', 7: 'JUL', 8: 'AUG', 9: 'SEP', 10: 'OCT',
              11: 'NOV', 12: 'DEC'}
    months_short_inv = {'FEB': 2, 'OCT': 10, 'MAR': 3, 'AUG': 8, 'MAY': 5,
                  'JAN': 1, 'JUN': 6, 'SEP': 9, 'APR': 4,
                  'DEC': 12, 'JUL': 7, 'NOV': 11}
    
    months_Length = {'February': 28, 'October': 31, 'March': 31, 'August': 31,
                     'May': 31, 'January': 31, 'June': 30, 'September': 30,
                     'April': 30, 'December': 31, 'July': 31, 'November': 30}
    months_Length_inv = {1: 31, 2: 28, 3: 31, 4: 30, 5: 31, 6: 30,
                         7: 31, 8: 31, 9: 30, 10: 31, 11: 30, 12: 31}
    mm2season = { 1  : 0, 2  : 0, 3  : 1,
                  4  : 1, 5  : 1, 6  : 2,
                  7  : 2, 8  : 2, 9  : 3,
                  10 : 3, 11 : 3, 12 : 0}

    # Some division for the data
    seasons          = ["djf","mam","jja","son"]
    hemispheres      = ["nh","sh"]
    surfaces         = ["ocean","land"]
    system_types     = ["1st","last","pmin"]
    intensities      = ["wea", "mod", "str"]

    # -------------------------------------------------------------------------
    # Setup
    # -------------------------------------------------------------------------
    
    # Standard library imports
    import sys
    import cPickle

    # Third party module imports
    import extras.progress_bar
    import numpy
    from statlib import stats, pstat
    import scipy.stats
    if zmean:
        import netcdftime
        import netCDF4 as NetCDF
    if make_plots or zmean:
        from datetime import datetime, timedelta
        from dateutil.relativedelta import relativedelta
        import matplotlib.pyplot as plt
        import matplotlib.mlab as mlab
        import matplotlib.dates as mdates
        from matplotlib import dates    
        
    # MCMS module imports
    import util.mcms_utilities
    import util.defs
    import util.read_mcms
    import util.save_netcdf
    import util.plot_map

    # pre-bind
    Read_MCMS = util.read_mcms.Read_MCMS
    Pull_Data = util.mcms_utilities.PullData
    Bin = numpy.histogram
    NA = numpy.array
    NRA = numpy.arange
    NZ = numpy.zeros
    NMean = numpy.mean
    NSTD = numpy.std
    NM = numpy.multiply
    Median = numpy.median
    Percentile = scipy.stats.scoreatpercentile
    Skew = scipy.stats.skew
    Kurtosis = scipy.stats.kurtosis
    LSR = scipy.stats.linregress
    if make_plots:
        PDF = mlab.normpdf
    print_col = util.mcms_utilities.WrapList
    InjestSetup = util.mcms_utilities.InjestSetup

    testtmp = Read_MCMS(**what_do)
    ids = testtmp.ids
    # Used to know the relative position on the intensity flag
    tmp = [testtmp.center_table[x] for x in ids if ids[x] < ids['Intensity']]
    tmp = [int(x.split(':')[1][:2]) for x in tmp]
    # All fields up to 'Intensity' plus spaces between
    intensity_skip = sum(tmp) + len(tmp) - 1

    # Fetch definitions.
    defs = util.defs.defs(**defs_set)
    fnc_out,inputs = InjestSetup("%ss_datf.pz" % (shared_path),0)
    # Keep Wanted values
    im = fnc_out[inputs.index("im")]
    jm = fnc_out[inputs.index("jm")]
    maxid = fnc_out[inputs.index("maxid")]
    lons = fnc_out[inputs.index("lons")]
    lats = fnc_out[inputs.index("lats")]
    timestep = fnc_out[inputs.index("timestep")]
    dx = fnc_out[inputs.index("dx")]
    dy = fnc_out[inputs.index("dy")]
    dlon = fnc_out[inputs.index("dlon")]
    dlat = fnc_out[inputs.index("dlat")]
    start_lat = fnc_out[inputs.index("start_lat")]
    start_lon = fnc_out[inputs.index("start_lon")]
    dlon_sq = fnc_out[inputs.index("dlon_sq")]
    dlat_sq = fnc_out[inputs.index("dlat_sq")]
    two_dlat = fnc_out[inputs.index("two_dlat")]
    eq_grid = fnc_out[inputs.index("eq_grid")]
    tropical_n = fnc_out[inputs.index("tropical_n")]
    tropical_s = fnc_out[inputs.index("tropical_s")]
    bot = fnc_out[inputs.index("bot")]
    mid = fnc_out[inputs.index("mid")]
    top = fnc_out[inputs.index("top")]
    row_start = fnc_out[inputs.index("row_start")]
    row_end = fnc_out[inputs.index("row_end")]
    tropical_n_alt = fnc_out[inputs.index("tropical_n_alt")]
    tropical_s_alt = fnc_out[inputs.index("tropical_s_alt")]
    bot_alt = fnc_out[inputs.index("bot_alt")]
    top_alt = fnc_out[inputs.index("top_alt")]
    lon_shift = fnc_out[inputs.index("lon_shift")]
    lat_flip = fnc_out[inputs.index("lat_flip")]
    the_calendar = fnc_out[inputs.index("the_calendar")]
    found_years = fnc_out[inputs.index("found_years")]
    super_years = fnc_out[inputs.index("super_years")]
    dim_lat = fnc_out[inputs.index("dim_lat")]
    dim_lon = fnc_out[inputs.index("dim_lon")]
    dim_time = fnc_out[inputs.index("dim_time")]
    var_lat = fnc_out[inputs.index("var_lat")]
    var_lon = fnc_out[inputs.index("var_lon")]
    var_time = fnc_out[inputs.index("var_time")]
    var_slp = fnc_out[inputs.index("var_slp")]
    var_topo = fnc_out[inputs.index("var_topo")]
    var_land_sea_mask = fnc_out[inputs.index("var_land_sea_mask")]
    file_seperater = fnc_out[inputs.index("file_seperater")]
    no_topo = fnc_out[inputs.index("no_topo")]
    no_mask = fnc_out[inputs.index("no_mask")]
    pbot = fnc_out[inputs.index("pbot")]
    ptop = fnc_out[inputs.index("ptop")]
    faux_grids = fnc_out[inputs.index("faux_grids")]
    eq_grid_on_edge = fnc_out[inputs.index("eq_grid_on_edge")]
    row_start_index = row_start.index
    troubled_centers = fnc_out[inputs.index("troubled_centers")]
    out_path = fnc_out[inputs.index("out_path")]
    # Drop the remainder
    del fnc_out,inputs    

    ## Get some definitions. Note must have run setup_vx.py already!
    #sf_file = "%ss_dat.p" % (shared_path)
    #try:
    #    fnc_out = cPickle.load(open(sf_file))
    #    (im,jm,maxid,lats,lons,timestep,dx,dy,dlon,dlat,start_lat,start_lon,
    #            dlon_sq,dlat_sq,two_dlat,model_flag,eq_grid,tropical_n,tropical_s,
    #            bot,mid,top,row_start,row_end,tropical_n_alt,tropical_s_alt,
    #            bot_alt,top_alt,lon_shift,lat_flip,the_calendar,found_years,
    #            super_years,dim_lat,dim_lon,dim_time,var_lat,var_lon,var_time,
    #            var_slp,var_topo,var_land_sea_mask,file_seperater,no_topo,
    #            no_mask,slp_path,model,out_path,shared_path,lat_edges,lon_edges,
    #            land_gridids,troubled_centers,faux_grids,polar_s,polar_n,
    #            pbot,ptop) = fnc_out
    #    # Save memory
    #    del troubled_centers
    #    if not land_sea:
    #        del land_gridids
    #    del lat_edges
    #    del lon_edges
    #    del fnc_out
    #except:
    #    sys.exit("\n\tWARNING: Error reading or finding %s." % (sf_file))

    # Update over_write values
    if over_write_years:
        super_years = over_write_years
    if over_write_slp_path:
        slp_path = over_write_slp_path
    if over_write_out_path:
        out_path = over_write_out_path
   

    # Years to check
    years = range(int(super_years[0]),int(super_years[-1])+1)
    clim_tag = "%04d-%04d" % (int(super_years[0]),int(super_years[-1]))

    if make_plots:
        dstart = datetime(years[0],1,1,0,0)
        dend = datetime(years[-1],1,1,0,0)
        if the_calendar in ['standard','gregorian']:
            delta = timedelta(days=365)
        elif the_calendar in ["noleap",'365_day']:
            delta = timedelta(days=365)
        else:
            sys.exit("error in calendar")
        x_dat_years = dates.drange(dstart, dend, delta)
        dend = datetime(years[-1],12,1,0,0)
        # Below doesn't work due to ambiguity of adding a month to a date
        #delta = timedelta(days=30)
        #x_dat_months = dates.drange(dstart, dend, delta)
        # This works but is less pretty
        delta = relativedelta(months=+1)
        x_dat_months = []
        d = dstart
        while d <= dend:
            d += delta
            x_dat_months.append(dates.date2num(d))

    # The histogram can be used to efficiently calculate the mean
    # and standard deviation of very large data sets without the
    # need to store individual values.
    #
    # Setup bins for these calculations.
    if zmean:
        bin_width = 5.0*defs.accuracy;
        bins_left_edge = NRA(int(-100*defs.accuracy),int(50*defs.accuracy)+bin_width,bin_width)
    else:
        bin_width = 5.0*defs.accuracy;
        bins_left_edge = NRA(int(930.0*defs.accuracy),int(1040.0*defs.accuracy),bin_width)
    (bins_centers,bins_right_edge) = setup_bins(bins_left_edge,
                                                bin_width,verbose=0)
    nbins = len(bins_left_edge)-1
    bins_left_edge = NA(bins_left_edge)

    # Add Tag if removing zonal mean
    if zmean:
        zm_tag = "_zanom"
        zm_tag_title = "ZAnom"
    else:
        zm_tag = ""
        zm_tag_title = ""

    # Add Tag if screening by land or sea
    if land_sea == 1:
        ls_tag = "_land"
        ls_tag_title = "Land Only"
    elif  land_sea == 2:
        ls_tag = "_ocean"
        ls_tag_title = "Ocean Only"
    else:
        ls_tag = ""
        ls_tag_title = ""

    if basin_name:
        b_tag = "_%s" % basin_name
        b_tag_title = basin_name
    else:
        b_tag = ""
        b_tag_title = ""


    # Stats file
    stats_file = "%sstats/mcms_%s_%s_intensity_stats%s%s%s.txt" % (out_path,model,
            clim_tag,zm_tag,ls_tag,b_tag)
    stats_save = open(stats_file,"w")

    # --------------------------------------------------------------------------
    # Main Program Logic
    # --------------------------------------------------------------------------
    big_table = {}

    if zmean:
        # Create an array of zonal means by month
        the_zmeans = NZ((len(years),12,jm),dtype=numpy.float)
        the_zmean_annual = NZ((len(years),jm),dtype=numpy.float)
        the_zmeans_sclim = NZ((len(seasons),jm),dtype=numpy.float)
        the_zmeans_season = NZ((len(years),len(seasons),jm),dtype=numpy.float)

    # Create a lookup table
    ilook = {}
    i = 0
    for year in years:
        j = {}
        for y in range(1,13):
            j[y] = i
            i += 1
        ilook[year] = j
    
    #skip_to_plots = 0
    if not skip_to_plots:
        iyear = 0
        readin = {}
        for loop_year in extras.progress_bar.ProgressBarAlt(years,"\tReading Centers.... ",60):
            
            #print "\tIngesting ",loop_year

            # Adjust in_file names
            tag = str(what_do["in_file"][-cut_tail-4:-cut_tail])
            what_do["in_file"] = what_do["in_file"].replace(tag,str(loop_year))

            # Instantiate read_mcms
            readit = Read_MCMS(**what_do)
            readit.check_time()
            readit.check_place()

            if zmean:
                # If the data files are not annual create a virtual year to hold all the 
                #   data files
                #
                # What files contain data for cf_loop_year?
                # Watch out for cases like this slp.19820101 which could match 1982 and 2010
                fnames = [x for x in super_time_data 
                        if super_time_data[x]['uci_stamps'][0][:4].find(str(loop_year)) != -1]
                # Ensure fnames sorted properly
                fnames = sorted(fnames,
                        key = lambda fnames: (break_fname(fnames)[0],
                            month_sort_order[break_fname(fnames)[1]]))
                date_stamps = []
                adates = []
                uci_stamps = []
                tsteps = 0
                for fname in fnames:
                    date_stamps.extend(super_time_data[fname]['date_stamps'])
                    adates.extend(super_time_data[fname]['adates'])
                    uci_stamps.extend(super_time_data[fname]['uci_stamps'])
                    tsteps += super_time_data[fname]['tsteps']
                if jd_fake:
                    # adates are tsteps for each file... ensure correct
                    years = range(super_years[0],super_years[-1]+1) 
                    jd_offset = years.index(loop_year)*tsteps
                    adates = []
                    ttsteps = jd_offset
                    for fname in fnames:
                        adates_tmp = ["%010d" % (x+ttsteps) 
                                for x in range(super_time_data[fname]['tsteps'])]
                        adates.extend(adates_tmp)
                        ttsteps += super_time_data[fname]['tsteps']
                annual_file = 0
                if len(fnames) == 1:
                    annual_file = 1
                tstepsperday = float(24.0/timestep)

                if annual_file:
                    slp,tt,tu = Pull_Data(fname_ext,loop_year,var_slp,
                        var_time,lat_flip,lon_shift,defs.read_scale,'all')
                    slp_months = [d.month for d in dtimes]
                    # Find the 12 monthly mean maps
                    for month in range(1,13):
                        this_month = [step for step in xrange(0,tsteps) if slp_months[step] == month]
                        # Find the zonal average of the monthly mean maps
                        this_sum = NMean(slp[this_month[0]:this_month[-1]+1,:,:],2)
                        this_sum = NMean(this_sum,0)
                        the_zmeans[iyear,month-1,:] = NM(this_sum,defs.accuracy)
                else:
                    for fname in fnames:
                        fname_ext = slp_path+fname+".nc"
                        slp_months = list(set([int(x.split()[1]) for x in super_time_data[fname]['date_stamps']]))
                        if len(slp_months) != 1:
                            sys.exit("Multiple months in file?")
                        month = slp_months[0]
                        slp,tt,tu = Pull_Data(fname_ext,loop_year,var_slp,
                            var_time,lat_flip,lon_shift,defs.read_scale,'all')
                        # Find the zonal average of the monthly mean maps
                        this_sum = NMean(slp[:,:,:],2)
                        this_sum = NMean(this_sum,0)
                        the_zmeans[iyear,month-1,:] = NM(this_sum,defs.accuracy)
 
            # Read center files
            readit.fetch_centers()
            centers = readit.center_holder.keys()
            readin[loop_year] = len(centers)

            # Create a lookup table with keyed with center_uci and storing
            # an array [year,month,season,hemi,gridslp]
            for center in centers:
                # Trim for land/sea if asked
                if land_sea == 1: # land only
                    if int(readit.center_holder[center][ids['GridID']]) not in readit.land_gridids:
                        continue
                elif land_sea == 2: # sea only
                    if int(readit.center_holder[center][ids['GridID']]) in readit.land_gridids:
                        continue
                if basin:
                    if int(readit.center_holder[center][ids['GridID']]) not in basin:
                        continue
                hemi = (0 if 9000-int(readit.center_holder[center][ids['CoLat']]) >= 0 else 1)
                gridslp = int(readit.center_holder[center][ids['GridSLP']])
                if zmean:
                    center_gridid = int(readit.center_holder[center][ids['GridID']])
                    # Find row for the location
                    row_guess = [x for x in row_start
                                 if x <= center_gridid]
                    row_guess = row_start_index(row_guess[-1])
                    big_table[center] = [hemi,gridslp,row_guess]
                else:
                    big_table[center] = [hemi,gridslp]
            iyear += 1

        if zmean:
            # Find the climatological zonal average (by Year)
            the_zmeans_annual = NMean(the_zmeans,1)
            # Find the climatological zonal average
            the_zmeans_clim = NMean(the_zmeans,1)
            the_zmeans_clim = NMean(the_zmeans_clim,0)
            # Find climatological seasonal zonal average
            the_zmeans_mclim = NMean(the_zmeans,0)
            the_zmeans_sclim[0,:] = NM(the_zmeans_mclim[11,:]+the_zmeans_mclim[0,:]+the_zmeans_mclim[1,:],0.333)
            the_zmeans_sclim[1,:] = NM(the_zmeans_mclim[2,:]+the_zmeans_mclim[3,:]+the_zmeans_mclim[4,:],0.333)
            the_zmeans_sclim[2,:] = NM(the_zmeans_mclim[5,:]+the_zmeans_mclim[6,:]+the_zmeans_mclim[7,:],0.333)
            the_zmeans_sclim[3,:] = NM(the_zmeans_mclim[8,:]+the_zmeans_mclim[9,:]+the_zmeans_mclim[10,:],0.333)
            # Find year to year seasonal zonal average
            iyear = 0
            for year in years:
                the_zmeans_season[iyear,0,:] = NM(the_zmeans[iyear,11,:]+the_zmeans[iyear,0,:]+the_zmeans[iyear,1,:],0.333)
                the_zmeans_season[iyear,1,:] = NM(the_zmeans[iyear,2,:]+the_zmeans[iyear,3,:]+the_zmeans[iyear,4,:],0.333)
                the_zmeans_season[iyear,2,:] = NM(the_zmeans[iyear,5,:]+the_zmeans[iyear,6,:]+the_zmeans[iyear,7,:],0.333)
                the_zmeans_season[iyear,3,:] = NM(the_zmeans[iyear,8,:]+the_zmeans[iyear,9,:]+the_zmeans[iyear,10,:],0.333)
                iyear += 1
    
        print "\tCenters Read (%d): " % sum(readin.values())
        print_col(["%d (%d)" % (loop_year,readin[loop_year]) for loop_year in years],"\t\t",80) 
        centers = big_table.keys()

        # Save tmp files in case want to tweak figures.
        fmt1 = "%sstats/tmp/refine_data_%s_%s%s%s%s.p"
        if zmean:
            rfile = fmt1 % (out_path,model,clim_tag,zm_tag,ls_tag,b_tag)
            fnc = (big_table,centers,the_zmeans_annual,the_zmeans_clim,
                   the_zmeans_mclim,the_zmeans_sclim,the_zmeans_season,the_zmeans)
        else:
            rfile = fmt1 % (out_path,model,clim_tag,zm_tag,ls_tag,b_tag)
            fnc = (big_table,centers)
        try:
            cPickle.dump(fnc,open(rfile,"wb",-1))
            print "\n\tMade %s" % rfile
        except:
            sys.exit("\n\tWARNING: Error Creating %s." % (rfile))
    else:
        # unPickle objects:
        fmt1 = "%sstats/tmp/refine_data_%s_%s%s%s%s.p"
        if zmean:
            rfile = fmt1 % (out_path,model,clim_tag,zm_tag,ls_tag,b_tag)
        else:
            rfile = fmt1 % (out_path,model,clim_tag,zm_tag,ls_tag,b_tag)
        try:
            fnc = cPickle.load(open(rfile))
            if zmean:
                (big_table,centers,the_zmeans_annual,the_zmeans_clim,
                    the_zmeans_mclim,the_zmeans_sclim,the_zmeans_season,the_zmeans) = fnc
            else:
                (big_table,centers) = fnc
        except:
            sys.exit("\n\tWARNING: Error Reading %s." % (rfile))
    
    if use_other_bounds:
        # Read/Replace the thirds
        # Save tmp files in case want to tweak figures.
        fmt2 = "%sstats/tmp/refine_thirds_%s_%s%s%s%s.p"
        tfile = fmt2 % (out_path_alt,model_alt,clim_tag_alt,zm_tag_alt,ls_tag_alt,b_tag)
        try:
            thirds = cPickle.load(open(tfile))
        except:
            sys.exit("\n\tWARNING: Error Reading %s." % (tfile))

    #if use_other_bounds:
    #    # unPickle objects:
    #    fmt1 = "%sstats/tmp/refine_data_%s_%s%s%s.p"
    #    if zmean:
    #        rfile = fmt1 % (out_path_alt,model_alt,clim_tag_alt,zm_tag_alt,ls_tag_alt)
    #    else:
    #        rfile = fmt1 % (out_path_alt,model_alt,clim_tag_alt,zm_tag_alt,ls_tag_alt)
    #    try:
    #        fnc = cPickle.load(open(rfile))
    #        if zmean:
    #            (big_table_alt,centers_alt,the_zmeans_annual_alt,the_zmeans_clim_alt,
    #                the_zmeans_mclim_alt,the_zmeans_sclim_alt,the_zmeans_season_alt,the_zmeans_alt) = fnc
    #        else:
    #            (big_table_alt,centers_alt) = fnc
    #    except:
    #        sys.exit("\n\tWARNING: Error Reading %s." % (rfile))
    #    import sys; sys.exit("Stop Here")

    #slp_pools = ["whole_hemi","whole_hemi_season","year_hemi_season","year_hemi_month"]
    list_month_season = []
    for loop_year in years:
        for loop_month in range(1,13):
            list_month_season.append(mm2season[loop_month])
    by_month = NZ((len(years),12,2,2),dtype=numpy.float)

    # Create Table for partitioning Centers.
    # thirds (year_month,hemi,(lower_third,upper_third),slp_pool)
    if not use_other_bounds:
#fix something needs attention here. keep adding, the problem is that thirds
# is year indexed so the two files need the same number of years.
        thirds = NZ((len(years)*12,2,2,len(slp_pools)),dtype=numpy.float)
    # tstats(year_month,hemi,(mean,median,STD,min,max,cnt,Skew,Kurtosis),slp_pool)
    tstats = NZ((len(years)*12,2,8,len(slp_pools)),dtype=numpy.float)
    #used_pool = 3
    
    if continuous:
        # rather than thirds use whole number percentiles
        #   wpercent (year_month,hemi,(1..99 percentiles),slp_pool)
        wpercent =  NZ((len(years)*12,2,99,len(slp_pools)),dtype=numpy.float)

#tmp
    #make_plots = 0

    # SLP Pool: Whole record and hemisphere (i.e, 1979-2008 NH)
    print "\tStarting: Whole record and hemisphere (i.e, 1979-2008 NH)"
    for hemi in range(2):
        this_hemi = [x for x in centers if big_table[x][0] == hemi]
        if which_hemi:
            if hemi != which_hemi[1]: continue
        if zmean:
            slps = [big_table[x][1]-the_zmeans_clim[big_table[x][2]] for x in this_hemi]
        else:
            slps = [big_table[x][1] for x in this_hemi]
        slps = NA(slps)
        (n,bins) = Bin(slps,bins_left_edge)
        mean = NMean(slps)
        median = Median(slps)
        std = NSTD(slps)
        lower_third = int(round(Percentile(slps,33.3)))
        upper_third = int(round(Percentile(slps,66.6)))
        if use_fixed:
            # Average over everything
            lower_third = NMean(fixed[:,hemi,1,0])
            upper_third = NMean(fixed[:,hemi,0,0])
            if zmean:
                lower_third = NMean(fixed[:,hemi,0,0])
                upper_third = NMean(fixed[:,hemi,1,0])
            #print "A hemi %d lower %f upper %f" % (hemi,lower_third,upper_third)
        en_stats = [mean,median,std,lower_third,upper_third]
        #test_spilt(slps,lower_third,upper_third)
        thirds[:,hemi,1,0] = lower_third
        thirds[:,hemi,0,0] = upper_third
        if zmean:
            thirds[:,hemi,0,0] = lower_third
            thirds[:,hemi,1,0] = upper_third
        if continuous:
            for per in xrange(1,100):
                wpercent[:,hemi,per-1,0] = int(round(Percentile(slps,float(per))))      
        tstats[:,hemi,0,0] = mean
        tstats[:,hemi,1,0] = median
        tstats[:,hemi,2,0] = std
        tstats[:,hemi,3,0] = slps.min()
        tstats[:,hemi,4,0] = slps.max()
        tstats[:,hemi,5,0] = slps.size
        en_stats.append(tstats[0,hemi,3,0])
        en_stats.append(tstats[0,hemi,4,0])
        en_stats.append(tstats[0,hemi,5,0])
        tstats[:,hemi,6,0] = Skew(slps)
        tstats[:,hemi,7,0] = Kurtosis(slps)
        if make_plots:
            pname = "%sfigs/pdfs/%s/%s/%s_slp_hist_%s_%s_%s%s%s%s%s" % (
                out_path,hemispheres[hemi],"annual",model,clim_tag,
                hemispheres[hemi],"annual",ls_tag,zm_tag,b_tag,fig_format)
            title = '%s %s %s %s %s %s %s' % (model.upper(),clim_tag,
                                      hemispheres[hemi].upper(),
                                      "Annual",ls_tag_title,zm_tag_title,b_tag_title)
            plot_pdf(plt,PDF,slps,en_stats,bins_centers,bins_left_edge,title,pname)
            print "\t\tMade",pname
        # SLP Pool: Whole record, season and hemisphere (i.e., 1979-2008 DJF NH).
        for season in range(len(seasons)):
            this_season = [x for x in this_hemi if mm2season[int(x[4:6])] == season]
            if zmean:
                slps = [big_table[x][1]-the_zmeans_sclim[season,big_table[x][2]] for x in this_hemi]
            else:
                slps = [big_table[x][1] for x in this_season]
            slps = NA(slps)
            (n,bins) = Bin(slps,bins_left_edge)
            mean = NMean(slps)
            median = Median(slps)
            std = NSTD(slps)
            lower_third = int(round(Percentile(slps,33.3)))
            upper_third = int(round(Percentile(slps,66.6)))
            if use_fixed:
                # Just the season
                lower_third = fixed[season,hemi,1,1]
                upper_third = fixed[season,hemi,0,1]
                if zmean:
                    lower_third = fixed[season,hemi,0,1]
                    upper_third = fixed[season,hemi,1,1]
                #print "B hemi %d season %d lower %f upper %f" % (hemi,season,lower_third,upper_third)
            if continuous:
                these_pers = []
                for per in xrange(1,100):
                    these_pers.append(int(round(Percentile(slps,float(per)))))            
            en_stats = [mean,median,std,lower_third,upper_third]
            #test_spilt(slps,lower_third,upper_third) 
            for x in range(len(years)*12):
                if list_month_season[x] == season:
                    thirds[x,hemi,1,1] = lower_third
                    thirds[x,hemi,0,1] = upper_third
                    if zmean:
                        thirds[x,hemi,0,1] = lower_third
                        thirds[x,hemi,1,1] = upper_third
                    if continuous:
                        for per in xrange(1,100):
                            wpercent[x,hemi,per-1,1] = these_pers[per-1]
                    tstats[x,hemi,0,1] = mean
                    tstats[x,hemi,1,1] = median
                    tstats[x,hemi,2,1] = std
                    tstats[x,hemi,3,1] = slps.min()
                    tstats[x,hemi,4,1] = slps.max()
                    tstats[x,hemi,5,1] = slps.size
                    en_stats.append(tstats[x,hemi,3,1])
                    en_stats.append(tstats[x,hemi,4,1])
                    en_stats.append(tstats[x,hemi,5,1])
                    tstats[x,hemi,6,1] = Skew(slps)
                    tstats[x,hemi,7,1] = Kurtosis(slps)
            if make_plots:
                pname = "%sfigs/pdfs/%s/%s/%s_slp_hist_%s_%s_%s%s%s%s%s" % (
                    out_path,hemispheres[hemi],seasons[season],model,clim_tag,
                    hemispheres[hemi],seasons[season],ls_tag,zm_tag,b_tag,fig_format)
                title = '%s %s %s %s %s %s %s' % (model.upper(),clim_tag,
                                            hemispheres[hemi].upper(),
                                            seasons[season].upper(),
                                            ls_tag_title,zm_tag_title,b_tag_title)
                plot_pdf(plt,PDF,slps,en_stats,bins_centers,bins_left_edge,title,pname)
                print "\t\tMade",pname
    print "\tStarting: By year, hemisphere and annual (i.e, 1979 NH) and (i.e., 1979 DJF NH)"
    iyear = 0
    for year in years:
        # SLP Pool: By year, hemisphere and annual (i.e, 1979 NH).
        this_years = [x for x in centers if x.startswith(str(year))]
        for hemi in range(2):
            this_hemi = [x for x in this_years if big_table[x][0] == hemi]
            if which_hemi:
                if hemi != which_hemi[1]: continue
            #if zmean:
            #    slps = [big_table[x][1]-the_zmeans_annual[iyear,big_table[x][2]] for x in this_hemi]
            #else:
            #    slps = [big_table[x][1] for x in this_hemi]
            #slps = NA(slps)
            #(n,bins) = Bin(slps,bins_left_edge)
            #mean = NMean(slps)
            #median = Median(slps)
            #std = NSTD(slps)
            # Find Lower 1/3 and Upper 1/3 Boundry
            #lower_third = int(round(Percentile(slps,33.3)))
            #upper_third = int(round(Percentile(slps,66.6)))
            #en_stats = [mean,median,std,lower_third,upper_third]
            #test_spilt(slps,lower_third,upper_third)
            #if make_plots:
            #    en_stats.append(slps.min())
            #    en_stats.append(slps.max())
            #    en_stats.append(slps.size)
            #    pname = "%sfigs/pdfs/%s_slp_hist_%s_%s_%s%s%s%s" % (out_path,model,str(year),
            #                                                    hemispheres[hemi],"annual",
            #                                                    ls_tag,zm_tag,fig_format)
            #    title = '%s %s %s %s %s %s' % (model.upper(),str(year),hemispheres[hemi].upper(),
            #                             "Annual",ls_tag_title,zm_tag_title)
            #    plot_pdf(plt,PDF,slps,en_stats,bins_centers,bins_left_edge,title,pname)
            #    print "\t\tMade",pname

            # SLP Pool: By year, season and hemisphere (i.e., 1979 DJF NH).
            for season in range(len(seasons)):
                this_season = [x for x in this_hemi if mm2season[int(x[4:6])] == season]
                if zmean:
                    slps = [big_table[x][1]-the_zmeans_season[iyear,season,big_table[x][2]] for x in this_season]
                else:
                    slps = [big_table[x][1] for x in this_season]
                slps = NA(slps)
                (n,bins) = Bin(slps,bins_left_edge)
                mean = NMean(slps)
                median = Median(slps)
                std = NSTD(slps)
                # Find Lower 1/3 and Upper 1/3 Boundry
                lower_third = int(round(Percentile(slps,33.3)))
                upper_third = int(round(Percentile(slps,66.6)))
                if use_fixed:
                    lower_third = fixed[season,hemi,1,2]
                    upper_third = fixed[season,hemi,0,2]
                    if zmean:
                        lower_third = fixed[season,hemi,0,2]
                        upper_third = fixed[season,hemi,1,2]
                    #print "C hemi %d season %d lower %f upper %f" % (hemi,season,lower_third,upper_third)
                en_stats = [mean,median,std,lower_third,upper_third]
                #test_spilt(slps,lower_third,upper_third)
                if continuous:
                    these_pers = []
                    for per in xrange(1,100):
                        these_pers.append(int(round(Percentile(slps,float(per)))))                
                i = 0
                for x in years:
                    for y in range(12):
                        if x == year:
                            if list_month_season[i] == season:
                                thirds[i,hemi,1,2] = lower_third
                                thirds[i,hemi,0,2] = upper_third
                                if zmean:
                                    thirds[i,hemi,0,2] = lower_third
                                    thirds[i,hemi,1,2] = upper_third
                                if continuous:
                                    for per in xrange(1,100):
                                        wpercent[i,hemi,per-1,2] = these_pers[per-1]                                
                                tstats[i,hemi,0,2] = mean
                                tstats[i,hemi,1,2] = median
                                tstats[i,hemi,2,2] = std
                                tstats[i,hemi,3,2] = slps.min()
                                tstats[i,hemi,4,2] = slps.max()
                                tstats[i,hemi,5,2] = slps.size
                                en_stats.append(tstats[i,hemi,3,2])
                                en_stats.append(tstats[i,hemi,4,2])
                                en_stats.append(tstats[i,hemi,5,2])
                                tstats[i,hemi,6,2] = Skew(slps)
                                tstats[i,hemi,7,2] = Kurtosis(slps)
                        i += 1
                #if make_plots:
                #    pname = "%sfigs/pdfs/%s_slp_hist_%s_%s_%s%s%s%s" % (out_path,model,str(year),
                #                                                    hemispheres[hemi],
                #                                                    seasons[season],
                #                                                    ls_tag,zm_tag,
                #                                                    fig_format)
                #    title = '%s %s %s %s %s %s' % (model.upper(),str(year),
                #                                hemispheres[hemi].upper(),
                #                                seasons[season].upper(),ls_tag_title,zm_tag_title)
                #    plot_pdf(plt,PDF,slps,en_stats,bins_centers,bins_left_edge,title,pname)
                #    print "\t\tMade",pname

            # SLP Pool: By year, month and hemisphere (i.e., 1979 January NH).
            for month in range(1,13):
                this_month  = [x for x in this_hemi if int(x[4:6]) == month]
                if zmean:
                    slps = [big_table[x][1]-the_zmeans[iyear,month-1,big_table[x][2]] for x in this_month]
                else:
                    slps = [big_table[x][1] for x in this_month]
                slps = NA(slps)
                (n,bins) = Bin(slps,bins_left_edge)
                mean = NMean(slps)
                median = Median(slps)
                std = NSTD(slps)
                # Find Lower 1/3 and Upper 1/3 Boundary
                lower_third = int(round(Percentile(slps,33.3)))
                upper_third = int(round(Percentile(slps,66.6)))
                if use_fixed:
                    # By month
                    lower_third = fixed[month-1,hemi,1,3]
                    upper_third = fixed[month-1,hemi,0,3]
                    if zmean:
                        lower_third = fixed[month-1,hemi,0,3]
                        upper_third = fixed[month-1,hemi,1,3]
                    #print "D month - 1 %d hemi %d lower %f upper %f" % (month-1,hemi,lower_third,upper_third)
                en_stats = [mean,median,std,lower_third,upper_third]
                #test_spilt(slps,lower_third,upper_third)
                if continuous:
                    these_pers = []
                    for per in xrange(1,100):
                        these_pers.append(int(round(Percentile(slps,float(per)))))                
                i = 0
                j = 0
                for x in years:
                    for y in range(1,13):
                        if x == year:
                            if y == month:
                                thirds[i,hemi,1,3] = lower_third
                                thirds[i,hemi,0,3] = upper_third
                                if zmean:
                                    thirds[i,hemi,0,3] = lower_third
                                    thirds[i,hemi,1,3] = upper_third
                                if continuous:
                                    for per in xrange(1,100):
                                        wpercent[i,hemi,per-1,3] = these_pers[per-1]                                
                                by_month[j,y-1,hemi,0] = lower_third
                                by_month[j,y-1,hemi,1] = upper_third
                                tstats[i,hemi,0,3] = mean
                                tstats[i,hemi,1,3] = median
                                tstats[i,hemi,2,3] = std
                                tstats[i,hemi,3,3] = slps.min()
                                tstats[i,hemi,4,3] = slps.max()
                                tstats[i,hemi,5,3] = slps.size
                                en_stats.append(tstats[i,hemi,3,3])
                                en_stats.append(tstats[i,hemi,4,3])
                                en_stats.append(tstats[i,hemi,5,3])
                                tstats[i,hemi,6,3] = Skew(slps)
                                tstats[i,hemi,7,3] = Kurtosis(slps)
                        i += 1
                    j += 1
                #if make_plots:
                #    pname = "%sfigs/pdfs/%s_slp_hist_%s_%s_%s%s%s%s" % (out_path,model,str(year),
                #                                                    hemispheres[hemi],
                #                                                    months[month],
                #                                                    ls_tag,zm_tag,
                #                                                    fig_format)
                #    title = '%s %s %s %s %s %s' % (model.upper(),str(year),
                #                                hemispheres[hemi].upper(),
                #                                months[month].upper(),ls_tag_title,zm_tag_title)
                #    plot_pdf(plt,PDF,slps,en_stats,bins_centers,bins_left_edge,title,pname)
                #    print "\t\tMade",pname
        iyear += 1

    if len(years) > 10 and make_plots:
        print "\tStarting Time Series"
        # Only do if a decade of data or more
        golden_ratio = 1.61803399*2.0
        width = 10
        height = width/golden_ratio
        third_lines = ['-','--']
        #        hemi_colors = ['r','b']
        hemi_colors = ['k','k']
        season_marker = 'o'
        year_season_marker = '*'
        month_marker = '^'
        for month in range(12):
            for hemi in range(2):
                if which_hemi:
                    if hemi != which_hemi[1]: continue
                pname = "%s/figs/pdfs/%s/%s_slp_hist_pressed_%s_%s_%s%s%s%s%s" % (
                    out_path,hemispheres[hemi],model,clim_tag,months[month+1],
                    hemispheres[hemi],ls_tag,zm_tag,b_tag,fig_format)
                title = '%s %s %s %s %s %s %s' % (model.upper(),clim_tag,hemispheres[hemi].upper(),
                                            months[month+1].upper(),ls_tag_title,zm_tag_title,b_tag_title)
                fig = plt.figure(figsize=(width,height))
                ax = fig.add_subplot(111)
                ax.plot(x_dat_years,thirds[:len(years),hemi,0,0]*0.001,
                        third_lines[0]+hemi_colors[hemi],linewidth=3,
                        label='Climatology-Lower')
                ax.plot(x_dat_years,thirds[:len(years),hemi,1,0]*0.001,
                        third_lines[1]+hemi_colors[hemi],linewidth=3,
                        label='Climatology-Upper')
                ax.plot(x_dat_years,by_month[:,month,hemi,0]*0.001,
                        third_lines[0]+hemi_colors[hemi]+month_marker,linewidth=1,
                        markersize=4,label='Year-Monthly-Lower')
                ax.plot(x_dat_years,by_month[:,month,hemi,1]*0.001,
                        third_lines[1]+hemi_colors[hemi]+month_marker,linewidth=1,
                        markersize=4,label='Year-Monthly-Upper')
                pick = [thirds[month,hemi,0,1]*0.001 for x in x_dat_years]
                ax.plot(x_dat_years,pick,
                        third_lines[0]+hemi_colors[hemi],linewidth=0.5,
                        markersize=4,label='Climatology-Season-Lower')
                pick = [thirds[month,hemi,1,1]*0.001 for x in x_dat_years]
                ax.plot(x_dat_years,pick,
                        third_lines[1]+hemi_colors[hemi],linewidth=0.5,
                        markersize=4,label='Climatology-Season-Upper-')
                ax.set_xlabel('Time (Years)')
                ax.set_ylabel('SLP (hPa)')
                ax.xaxis.set_major_formatter(dates.DateFormatter('%Y'))
                ax.set_title(title)
                ax.grid(True)
                plt.savefig(pname,dpi=140,facecolor='w',edgecolor='w',orientation='landscape')
                print "\t\tMade",pname
                plt.close('all')

#tmp
    # Save tmp files in case want to tweak figures and data save.
    #fmt1 = "%sstats/tmp/tmp.p"
    #cPickle.dump((x_dat_years,x_dat_months,thirds,tstats),open(fmt1 % (out_path),"wb",-1))
    # unPickle objects:
    #(x_dat_years,x_dat_months,thirds,tstats) = cPickle.load(open(fmt1 % (out_path)))

    if make_plots and not use_fixed:
        golden_ratio = 1.61803399*2.0
        width = 40.0
        # Just Annual Cycle Version
        #width = 10
        height = width/golden_ratio

        pname = "%sfigs/tseries/global/%s_slp_bounds_time_%s%s%s%s%s" % (out_path,model,clim_tag,
                                                        ls_tag,zm_tag,b_tag,fig_format)
        title = '%s %s' % (model.upper(),clim_tag)
        fig = plt.figure()
        fig = plt.figure(figsize=(width,height))
        ax = fig.add_subplot(111)
        third_lines = ['-','--']
        hemi_colors = ['r','b']
        season_marker = 'o'
        year_season_marker = '*'
        month_marker = '^'

        for hemi in range(2):
            if which_hemi:
                if hemi != which_hemi[1]: continue
            # Uncomment a Set to plot. I find Set A the most interesting.

            ## Set B: Just Annual Cycle Version (uses "whole_hemi" and "whole_hemi_season" slp_pool)
            #ax.plot([x+1 for x in range(12)],thirds[0:12,hemi,0,0]*0.001,
            #        third_lines[0]+hemi_colors[hemi],linewidth=3)
            #ax.plot([x+1 for x in range(12)],thirds[0:12,hemi,1,0]*0.001,
            #        third_lines[1]+hemi_colors[hemi],linewidth=3)
            #ax.plot([x+1 for x in range(12)],thirds[0:12,hemi,0,1]*0.001,
            #        third_lines[0]+hemi_colors[hemi]+season_marker,linewidth=1)
            #ax.plot([x+1 for x in range(12)],thirds[0:12,hemi,1,1]*0.001,
            #        third_lines[1]+hemi_colors[hemi]+season_marker,linewidth=1)

            # Set A: plots climatological as lines (uses "whole_hemi" slp_pool)
            ax.plot(x_dat_months,thirds[:,hemi,0,0]*0.001,
                    third_lines[0]+hemi_colors[hemi],linewidth=3,
                    label='Climatology-Lower')
            ax.plot(x_dat_months,thirds[:,hemi,1,0]*0.001,
                    third_lines[1]+hemi_colors[hemi],linewidth=3,
                    label='Climatology-Upper')

            ## Set C: plots seasonal thirds (uses "whole_hemi_season","year_hemi_season"
            ## slp_pools)
            #ax.plot(x_dat_months,thirds[:,hemi,0,1]*0.001,
            #        third_lines[0]+hemi_colors[hemi]+season_marker,linewidth=1,
            #        markersize=12,label='Climatology-Season-Lower')
            #ax.plot(x_dat_months,thirds[:,hemi,1,1]*0.001,
            #        third_lines[1]+hemi_colors[hemi]+season_marker,linewidth=1,
            #        markersize=12,label='Climatology-Season-Upper-')
            #ax.plot(x_dat_months,thirds[:,hemi,0,2]*0.001,
            #        third_lines[0]+hemi_colors[hemi]+year_season_marker,linewidth=1,
            #        markersize=12,label='Year-Season-Lower')
            #ax.plot(x_dat_months,thirds[:,hemi,1,2]*0.001,
            #        third_lines[1]+hemi_colors[hemi]+year_season_marker,linewidth=1,
            #        markersize=12,label='Year-Season-Upper')

            # Set A: plots monthly thirds with lines and symbols
            # (uses "year_hemi_month" slp_pool)
            ax.plot(x_dat_months,thirds[:,hemi,0,used_pool]*0.001,
                    third_lines[0]+hemi_colors[hemi]+month_marker,linewidth=1,
                    markersize=12,label='Year-Monthly-Lower')
            ax.plot(x_dat_months,thirds[:,hemi,1,used_pool]*0.001,
                    third_lines[1]+hemi_colors[hemi]+month_marker,linewidth=1,
                    markersize=12,label='Year-Monthly-Upper')
            ax.xaxis.set_major_formatter(dates.DateFormatter('%Y'))

        ax.set_xlabel('Time (Months)')
        ax.set_ylabel('SLP (hPa)')
        ax.set_title(title)
        ax.grid(True)
        plt.savefig(pname,dpi=140,facecolor='w',edgecolor='w',orientation='landscape')
        plt.close('all')
        print "\t\tMade",pname

    if not use_other_bounds:
        # Save tmp files in case want to tweak figures.
        fmt2 = "%sstats/tmp/refine_thirds_%s_%s%s%s%s.p"
        tfile = fmt2 % (out_path,model,clim_tag,zm_tag,ls_tag,b_tag)
        try:
            cPickle.dump(thirds,open(tfile,"wb",-1))
            print "\n\tMade %s" % tfile
        except:
            sys.exit("\n\tWARNING: Error Creating %s." % (tfile))
    
    if use_fixed:
        thirds = fixed

    if alter_files:   

        # Alter Files to include new intensity values.
        tag = str(what_do["in_file"][-cut_tail-4:-cut_tail])
        center_file = what_do["in_file"]
        iyear = 0
        for loop_year in years:
            # Adjust in_file names
            center_file = center_file.replace(tag,str(loop_year))
            tag = str(loop_year)
            print "\t\tAltering:",center_file
            # Create a temporary write file
            tmp_file = center_file.replace(".txt","new.txt")
            # Open files for read and write
            try:
                centers_read = open(center_file,"r")
            except:
                sys.exit("\n\tWARNING: Error Opening %s." % (center_file))
            try:
                centers_write = open(tmp_file,"w")
            except:
                sys.exit("\n\tWARNING: Error Creating %s." % (tmp_file))
            for line in centers_read:
                # Process line
                parts = line.split(None)
                flag = int(parts[0])
                len_start = len(line)
                ids_add_on = 0
                iskip = intensity_skip
                if flag == -999:
                    # Deal with Empty Centers (att files) 
                    ids_add_on = 1
                    iskip += 5
                if flag > 0 or flag == -999:
                    # Assign Intensity to Center
                    year = int(parts[ids['YYYY']+ids_add_on])
                    month = int(parts[ids['MM']+ids_add_on])
                    gridslp = int(parts[ids['GridSLP']+ids_add_on])
                    hemi = (0 if 9000-int(parts[ids['CoLat']+ids_add_on]) >= 0 else 1)
                    ival = ilook[year][month]
                    # Assigned Intensity Classification
                    if zmean:
                        up_third = int(thirds[ival,hemi,1,used_pool])
                        low_third = int(thirds[ival,hemi,0,used_pool])                        
                    else:
                        up_third = int(thirds[ival,hemi,0,used_pool])
                        low_third = int(thirds[ival,hemi,1,used_pool])
                    if zmean:
                        gridslp = (gridslp - 
                                   the_zmeans[iyear,month-1,
                                              big_table[parts[ids['UCI']+ids_add_on]][2]])
                    # Use the 1/3, 1/3, 1/3 rule for intensity.
                    if gridslp < low_third:
                        intensity = 3 # Strong
                    elif gridslp > up_third:
                        intensity = 1 # Weak
                    else:
                        intensity =  2 # Moderate
                    if continuous:
                        # Store percentile rather than 1,2,3
                        intensity = 0
                        for per in xrange(1,100):
                            if gridslp < wpercent[ival,hemi,per-1,used_pool]: 
                                intensity = per
                                break
                        # Move so intensity goes from 1-99
                        if intensity >= 99: intensity = 99
                    # Modify line for output
                    #mikemike
                    #print line
                    #print line[:iskip]
                    #print line[iskip+3:]
                    line = line[:iskip]+" %02d" % (intensity)+line[iskip+3:]
                    #print line
                    #sys.exit("Stop Here")
                    # Check nothing wrong
                if len_start != len(line):
                    sys.exit("Length Error: Stopping (%d,%d)" % (len_start,len(line)))
                # Save to line to file
                centers_write.writelines(line)
            # Close Files
            centers_read.close()
            centers_write.close()
            iyear += 1
            # If made it here, then safe to rename and thus replace original file.
            # Warning I highly recommend you work with a copy and save the original
            # files for save keeping.
            os.rename(tmp_file,center_file)
    else:
        print "\tNote data files are not being altered."
 
    # Save Intensity Info for review and such
    # SEofEst = Standard Error of the Estimate
    # 2tailP = two-tailed probability
    terms = ("Year","Month","Hemi","Mean","Median","STD",
             "Min","Max","Lower 1/3","Upper 1/3","Cnt","Skew",
             "Kurtosis","Slope","Intercept","R",
              "2tailP","SEofEst")
    tag = ''.join(["%12s " % (x) for x in terms])
    fmts = "%12d %12d %12s %12d %12d %12d %12d %12d %12d %12d %12d %12f %12f %12f %12f %12f %12f %12f"
    hemis = ("NH","SH")

    # SLP Pool: Whole record and hemisphere (i.e, 1979-2008 NH)
    stats_save.writelines("# SLP Pool: Whole record and hemisphere (i.e, %s NH)\n" % (clim_tag))
    stats_save.writelines(tag+"\n")
    fmts = "%12s %12s %12s %12d %12d %12d %12d %12d %12d %12d %12d %12f %12f %12s %12s %12s %12s %12s"
    for hemi in range(2):
        if which_hemi:
            if hemi != which_hemi[1]: continue
        line = fmts % (clim_tag,"Annual",hemis[hemi],
                       int(tstats[0,hemi,0,0]),
                       int(tstats[0,hemi,1,0]),
                       int(tstats[0,hemi,2,0]),
                       int(tstats[0,hemi,3,0]),
                       int(tstats[0,hemi,4,0]),
                       int(thirds[0,hemi,1,0]),
                       int(thirds[0,hemi,0,0]),
                       int(tstats[0,hemi,5,0]),
                       tstats[0,hemi,6,0],
                       tstats[0,hemi,7,0],"-","-", "-", "-", "-")
        stats_save.writelines(line+"\n")
    # SLP Pool: Whole record, season and hemisphere (i.e., 1979-2008 DJF NH)
    stats_save.writelines("# SLP Pool: Whole record, season and hemisphere (i.e., %s DJF NH)\n" % (clim_tag))
    stats_save.writelines(tag+"\n")
    for hemi in range(2):
        if which_hemi:
            if hemi != which_hemi[1]: continue        
        for seas in range(len(seasons)):
            # remap to get correct values
            if seas == 0:
                tseas = 0
            elif seas == 1:
                tseas = 3
            elif seas == 2:
                tseas = 8
            elif seas == 3:
                tseas = 11
            line = fmts % (clim_tag,seasons[seas].upper(),hemis[hemi],
                           int(tstats[tseas,hemi,0,1]),
                           int(tstats[tseas,hemi,1,1]),
                           int(tstats[tseas,hemi,2,1]),
                           int(tstats[tseas,hemi,3,1]),
                           int(tstats[tseas,hemi,4,1]),
                           int(thirds[tseas,hemi,1,1]),
                           int(thirds[tseas,hemi,0,1]),
                           int(tstats[tseas,hemi,5,1]),
                           tstats[tseas,hemi,6,1],
                           tstats[tseas,hemi,7,1],"-","-", "-", "-", "-")
            stats_save.writelines(line+"\n")
    # SLP Pool: By year, hemisphere and annual (i.e, 1979 NH).
    # SLP Pool: By year, season and hemisphere (i.e., 1979 DJF NH).

    # SLP Pool: By year, month and hemisphere (i.e., 1979 January NH).
    fmts = "%12d %12d %12s %12d %12d %12d %12d %12d %12d %12d %12d %12f %12f %12s %12s %12s %12s %12s"
    stats_save.writelines("# SLP Pool: By year, month and hemisphere (i.e., 1979 January NH)\n")
    stats_save.writelines(tag+"\n")
    for loop_year in years:
        intensity_file = "%sstats/mcms_%s_%04d%s%s_intensity.txt" % (out_path,model,int(loop_year),zm_tag,b_tag)
        intensity_save = open(intensity_file,"w")
        intensity_save.writelines(tag+"\n")
        for hemi in range(2):
            if which_hemi:
                if hemi != which_hemi[1]: continue
            for month in range(1,13):
                ival = ilook[loop_year][month]
                line = fmts % (loop_year,month,hemis[hemi],
                               int(tstats[ival,hemi,0,used_pool]),
                               int(tstats[ival,hemi,1,used_pool]),
                               int(tstats[ival,hemi,2,used_pool]),
                               int(tstats[ival,hemi,3,used_pool]),
                               int(tstats[ival,hemi,4,used_pool]),
                               int(thirds[ival,hemi,1,used_pool]),
                               int(thirds[ival,hemi,0,used_pool]),
                               int(tstats[ival,hemi,5,used_pool]),
                               tstats[ival,hemi,6,used_pool],
                               tstats[ival,hemi,7,used_pool],"-","-", "-", "-", "-")
                intensity_save.writelines(line+"\n")
                line = fmts % (loop_year,month,hemis[hemi],
                               int(tstats[ival,hemi,0,3]),
                               int(tstats[ival,hemi,1,3]),
                               int(tstats[ival,hemi,2,3]),
                               int(tstats[ival,hemi,3,3]),
                               int(tstats[ival,hemi,4,3]),
                               int(thirds[ival,hemi,1,3]),
                               int(thirds[ival,hemi,0,3]),
                               int(tstats[ival,hemi,5,3]),
                               tstats[ival,hemi,6,3],
                               tstats[ival,hemi,7,3],"-","-", "-", "-", "-")
                stats_save.writelines(line+"\n")
        intensity_save.close()
        print "\t\tMade",intensity_file

    # Test for Trends in boundaries see by thirds sorted by month and hemisphere over years
    if len(years) > 10 and not use_fixed:
       # Only do if a decade of data or more
        y_vals = NZ((len(years),12,2,3),dtype=numpy.float)
        iyear = 0
        for loop_year in years:
            for hemi in range(2):
                if which_hemi:
                    if hemi != which_hemi[1]: continue
                for month in range(1,13):
                    ival = ilook[loop_year][month]
                    # Strong end of the distribution
                    y_vals[iyear,month-1,hemi,0] = thirds[ival,hemi,0,3]
                    # Weak end of the distribution
                    y_vals[iyear,month-1,hemi,1] = thirds[ival,hemi,1,3]
                    # Intermediate/Middle of the distribution
                    y_vals[iyear,month-1,hemi,2] = thirds[ival,hemi,0,3] - thirds[ival,hemi,1,3]
            iyear += 1

        fmts = "%12s %12d %12s %12d %12d %12d %12d %12d %12s %12s %12d %12f %12f %12.3f %12.3f %12.3f %12.3f %12.3f"
        stats_save.writelines("# Trend Analysis with SLP Pool: By year, month and hemisphere (i.e., 1979 January NH)\n")
        stats_save.writelines(tag+"\n")
        whats = ["Strong","Weak","Middle"]
        for hemi in range(2):
            if which_hemi:
                if hemi != which_hemi[1]: continue
            for month in range(1,13):
                for whaty in range(3):
                    y = y_vals[:,month-1,hemi,whaty]
                    x = range(len(years))
                    y = NA(y)
                    mean = NMean(y)
                    median = Median(y)
                    std = NSTD(y)
                    # Use Linear Regession
                    fnc = LSR(x,y)
                    line = fmts % (clim_tag,month,hemis[hemi],int(mean),int(median),
                                   int(std),int(y.min()),int(y.max()),whats[whaty],"",y.size,
                                   int(Skew(y)),int(Kurtosis(y)),fnc[0],fnc[1],fnc[2],fnc[3],fnc[4])
                    stats_save.writelines(line+"\n")

    print "\t\tMade",stats_file
    stats_save.close()
    return "Done"

def make_box(lat_bnds,lon_bnds,lons,lats,im):
    # GridIDs within the lat and lon bounds
    import util.l2g    
    mbox = {}
    for lat in lat_bnds:
        for lon in lon_bnds:
            gridid = util.l2g.l2g2(lon,lat,lons,lats,im)[0]
            mbox[gridid] = 1
    mbox = mbox.keys()
    return mbox

#---Start of main code block.
if __name__=='__main__':

    import cPickle
    import util.mcms_utilities
    import cfg.mcms_ncvars_base_defs
    import cfg.mcms_source_base_defs
    import cfg.mcms_figure_base_defs
    import cfg.mcms_defs_base_defs
    import cfg.mcms_directories_base_defs
    InjestSetup = util.mcms_utilities.InjestSetup
    verbose = 0

    # --------------------------------------------------------------------------
    # Select options for this run.
    # --------------------------------------------------------------------------
    picks = {0 : "NCEP/NCAR Reanalysis 1",
             1 : "NCEP-DOE Reanalysis 2",
             2 : "NASA GISS GCM ModelE",
             3 : "GFDL GCM",
             4 : "ERA-Interim Reanalysis",
             5 : "ERA-40 Reanalysis",
             6 : "CCSM",
             7 : "NASA GISS GCM ModelE M20",
             }
    pick = 7
    if pick not in picks:
        sys.exit("ERROR: pick not listed in picks.")

    # This next set of lines should be copied from setup_vX.py
    # Short names by which pick will be labeled.
    models = ["nra","nra2","giss","gfdl","erai","era40","ccsm","gissm"]
    try:
        model = models[pick]
    except:
        sys.exit("ERROR: pick not listed in models.")

    # Methods of collecting slp statistics and choosing how to use them to
    #   define intensity.
    slp_pools = ["whole_hemi","whole_hemi_season","year_hemi_season","year_hemi_month"]
    used_pool = 3
    
    # Use the refine_data_xxxx.p files of another run/model to set the 
    #   boundaries for this model.... rather than calculate new ones.
    use_other_bounds = 0
    out_path_alt = ""
    model_alt = ""
    clim_tag_alt = ""
    ls_tag_alt = ""
    if use_other_bounds:
        out_path_alt = "/Volumes/scratch/output/gissm/control_run/"
        model_alt = "gissm"
        clim_tag_alt = "3781-3800"
        zm_tag_alt = ""
        #zm_tag_alt = "_zanom"
        ls_tag_alt = ""
        #ls_tag_alt = "_land"
        #ls_tag_alt = "_ocean"
#fix something needs attention here. The problem here is that thirds is year
#        indexed which means both runs need the same number of years of data
#        for this to work.

    # Fixed: Use fixed bounds rather than calculated ones.
    use_fixed = 1 # See below!

    # Alter files: Alter and Save intensity to input files... otherwise just
    #   fine stats and make plots (if asked for).
    alter_files = 1

    # Save/plot Stats
    make_plots = 0

    # Use previously stored center data.
    skip_to_plots = 0

    # Remove zonal mean
    zmean = 0

    # continuous. Rather than store intensity by str,mod,wea instead store
    #   the percentile (i.e., 5 for 5%, 97 for 97%).
    continuous = 0

    # Separate by land_sea screen # land_sea = 0 All, land_sea = 1 Land, land_sea = 2 Ocean
    land_sea = 0

    # What sort of figures
    fig_format = ".png"
    #fig_format = ".eps"
    #fig_format = ".pdf"

    # Length of file ending to replace if using year_loop
    tails = ["_att.txt","_tracks.txt","_centers.txt","_dumped_centers.txt"]
    tail = tails[1]
    cut_tail = len(tail)

    # --------------------------------------------------------------------------
    # Define all modules to be imported.
    # --------------------------------------------------------------------------

    # # Basic standard Python modules to import.
    # imports = []
    # imports.append("import stats, sys, numpy, cPickle")
    # # Note if can't use scipy see commented code for alternatives.
    # #imports.append("from scipy.stats.mstats import mquantiles")
    # imports.append("import scipy.stats")
    # 
    # if zmean:
    #     imports.append("import netcdftime")
    #     imports.append("import netCDF4 as NetCDF")
    # if make_plots or zmean:
    #     imports.append("from datetime import datetime, timedelta")
    #     imports.append("from dateutil.relativedelta import relativedelta")
    #     imports.append("import matplotlib.pyplot as plt")
    #     imports.append("import matplotlib.mlab as mlab")
    #     imports.append("import matplotlib.dates as mdates")
    #     imports.append("from matplotlib import dates")
    # 
    # # My modules to import w/ version number appended.
    # my_base = ["defs","read_mcms","print_col"]
    # if make_plots:
    #     my_base.append("save_netcdf")
    #     my_base.append("plot_map")
    # for x in my_base:
    #     tmp = "import %s%s as %s" % (x,vnum,x)
    #     imports.append(tmp)
    # 
    # # To save a double copy of the data being retained by pull_data it is
    # # necessary to reimport and delete pull_data_vX.py inside each loop.
    # import_read =  "import %s%s as %s" % ("pull_data",vnum,"pull_data")

    # --------------------------------------------------------------------------
    # Alter default behavior found in either defs_vX.py or setup_vX.py
    # --------------------------------------------------------------------------

    # The default behavior is to read SLP data from the
    # directory slp_path defined in setup_vX.py.
    # Here you can elect to override this behavior.
    over_write_slp_path = ""
    ensembles = {"control_run" : [""],
            "Scenario_A1B_run" : ["E3abaoM20","E3abboM20","E3abcoM20","E3abdoM20","E3abeoM20"],
            "Scenario_A1B_iceMelt_run" : ["E3FIBaoM20","E3FIBboM20","E3FIBcoM20","E3FIBdoM20","E3FIBeoM20"]
            }
    experiment = "Scenario_A1B_run"
    ensemble = ensembles[experiment][0]
    subdir = "%s/%s/" % (experiment,ensemble)
    if not ensemble:
        subdir = subdir.replace("//","/")
    over_write_slp_path = "/Volumes/scratch/data/%s/%s" % (model,subdir)

    # The default behavior is to save results
    # in the directory out_path defined in
    # setup_vX.py. Here you can elect to override
    # this behavior.
    over_write_out_path = ""
    over_write_out_path = "/Volumes/scratch/output/%s/%s" % (model,subdir)
    if not over_write_out_path:
        # This next set of lines should be copied from setup_vX.py
        # Full path to the root directory where pick specific output will be stored.
        # Note it's possible that all of these directories are identical.
        # Uncomment to use unique structure for each model
        #result_directories = ["/Volumes/scratch/output/",] 
        # Uncomment to use same structure for all models
        result_directories = ["/Volumes/Scratch/output/" for x in range(len(picks))]
        # Uncomment to make current directory for all models
        #result_directories = [os.getcwd() for x in picks]
        try:
            result_directory = result_directories[pick]
            if not os.path.exists(result_directory):
                sys.exit("ERROR: result_directory not found.")
        except:
            sys.exit("ERROR: pick not listed in result_directories.")

        # Directory to be created for storing temporary pick specific files.
        shared_path = "%s%s_files/" % (result_directory,model)
    else:
        shared_path = "%s%s_files/" % ("/Volumes/scratch/output/",model)
 
    # The default behavior is to run over all the
    # years found by setup_vX.py. Here you can
    # elect to override this behavior.
    over_write_years = []
    #over_write_years = [1984,1984]
    #over_write_years = [1979,1979]

    # If command line python mcms_xxx.py 1
    #   suppress query_yes_no for time_dat
    no_ask = 0
    if len(sys.argv) > 1:
        no_ask = 1

    break_fname = ""
    month_sort_order = ""
    the_calendar =  ""
    jd_fake = ""
    timestep = ""
    if over_write_slp_path:
        # Evoke mcms_organize_data.py because the time_dat.p file created b
        #   mcms_setup.py may not correctly represent the data
        source_vars = cfg.mcms_ncvars_base_defs.define_vars()
        msg = source_vars.make_pick(model)
        nc_vars = source_vars.selection
            
        break_fname = nc_vars['break_fname']
        month_sort_order = nc_vars['month_sort_order']
        the_calendar = nc_vars['calendar']
        jd_fake = nc_vars['jd_fake']

        # Name for new time_dat file
        #get_time_dat  = "time_dat_control_run"
        #get_time_dat  = "time_dat_Scenario_A1B_run_E3abaoM20"
        if ensemble:
            get_time_dat  = "time_dat_%s_%s" % (experiment,ensemble)
        else:
            get_time_dat  = "time_dat_%s" % (experiment)

        # Test to see if already get_time_dat defined
        gf = shared_path+get_time_dat+".pz"
        over = 1
        if no_ask: over = 0
        if util.mcms_utilities.FileExists(gf,__name__,verbose=False) and not no_ask:
            # Warn about overwriting
            msg = "\n\tWarning: Overwrite [%s]?"
            print msg % gf
            answer = util.mcms_utilities.query_yes_no("\t\tContinue?")
            if not answer: over = 0
        if over:
            # Run mcms_organize_data... create time_dat.p
            import util.mcms_organize_data
            organize_data = util.mcms_organize_data.organize_data

            # Assume that mcms_ncvars_base_defs.py correctly defines
            #   file_seperater
            #   month_sort_order
            #   break_fname
            #   calendar
            #   jd_fake
            # If not put the new ones here and pass along
            #break_fname = nc_vars['break_fname']
            #month_sort_order = nc_vars['month_sort_order']
            #the_calendar = nc_vars['calendar']
            #jd_fake = nc_vars['jd_fake']

            msg = organize_data(over_write_slp_path,break_fname,nc_vars['var_slp'],
                    month_sort_order,the_calendar,nc_vars['var_time'],jd_fake,
                    shared_path,base_name=get_time_dat,verbose=0)  
            print msg
        # Update super_years to match
        fnc_out,inputs = util.mcms_utilities.InjestSetup(gf,4)
        super_years = fnc_out[inputs.index("super_years")]
        super_uci_stamps = fnc_out[inputs.index("super_uci_stamps")]
        super_time_data = fnc_out[inputs.index("super_time_data")]
        timestep = fnc_out[inputs.index("timestep")]
        super_lookup = fnc_out[inputs.index("super_lookup")]
        over_write_years = super_years
        del fnc_out
    
    season_selects = {0 : ["annual"],
            1 : ["annual","djf","mam","jja","son"],
            2 : ["annual","ndjfma","mjjaso"],
            3 : ["annual","ndjf","mamjjaso"]
            }
    season_select = 1
    seasons = season_selects[season_select]
    mm2season = { 0 : {1 : 0 , 2 : 0, 3 : 0, 4 : 0, 5 : 0, 6 : 0, 7 : 0, 8 : 0,
        9 : 0, 10 : 0, 11 : 0, 12 : 0},
        1 : { 1 : 1, 2 : 1, 3 : 2, 4 : 2, 5 : 2, 6 : 3, 7 : 3, 8 : 3, 9 : 4,
            10 : 4, 11 : 4, 12 : 1},
        2 : { 1 : 1, 2 : 1, 3 : 1, 4 : 1, 5 : 2, 6 : 2, 7 : 2, 8 : 2, 9 : 2,
            10 : 2, 11 : 1, 12 : 1},
        3 : { 1 : 1, 2 : 1, 3 : 2, 4 : 2, 5 : 2, 6 : 2, 7 : 2, 8 : 2, 9 : 2,
            10 : 2, 11 : 1, 12 : 1}}
    # How season months relate to the current year "0"
    mm2year = { 0 : {1 : 0 , 2 : 0, 3 : 0, 4 : 0, 5 : 0, 6 : 0, 7 : 0, 8 : 0,
        9 : 0, 10 : 0, 11 : 0, 12 : 0},
                1 : {1 : 0 , 2 : 0, 3 : 0, 4 : 0, 5 : 0, 6 : 0, 7 : 0, 8 : 0,
        9 : 0, 10 : 0, 11 : 0, 12 : -1},
                2 : {1 : 0 , 2 : 0, 3 : 0, 4 : 0, 5 : 0, 6 : 0, 7 : 0, 8 : 0,
        9 : 0, 10 : 0, 11 : -1, 12 : -1},
                3 : {1 : 0 , 2 : 0, 3 : 0, 4 : 0, 5 : 0, 6 : 0, 7 : 0, 8 : 0,
        9 : 0, 10 : 0, 11 : -1, 12 : -1}}
    m2y = mm2year[season_select]
    m2s = mm2season[season_select]

    # FIX:
    # NOTE: For now djf, ndjfma etc only valid for summary (climatology as we don't
    #   carry the Dec Year X over to the JAN, FEB of Year X + 1 so only totals valid.
    #   could be done with m2s and the same read forward/backward used w/ tracks option.
    #   or could pass those back to main and into next year as needed.

    # Select by system intensity
    intensity_selects = { 0 : ["all"],
            1 : ["all","wea", "mod", "str"]
            }
    intensity_select = 1
    intensities = intensity_selects[intensity_select]

    # --------------------------------------------------------------------------
    # Alter default behavior found in mcms_defs.py
    # --------------------------------------------------------------------------
    # Pick data source see defs files in /cfg
    pick_defs = cfg.mcms_defs_base_defs.define_defs()
    msg = pick_defs.make_pick(model)
    if verbose: print msg
    defs_set = pick_defs.selection

    # --------------------------------------------------------------------------
    # Run main()
    # --------------------------------------------------------------------------

    msg = "\n\t====\tAssigning Intensity\t===="
    print msg

    fnc_out,inputs = InjestSetup("%ss_dat.pz" % (shared_path),0)
    # Keep Wanted values
    im = fnc_out[inputs.index("im")]
    jm = fnc_out[inputs.index("jm")]
    maxid = fnc_out[inputs.index("maxid")]
    lons = fnc_out[inputs.index("lons")]
    lats = fnc_out[inputs.index("lats")]
    timestep = fnc_out[inputs.index("timestep")]
    dx = fnc_out[inputs.index("dx")]
    dy = fnc_out[inputs.index("dy")]
    dlon = fnc_out[inputs.index("dlon")]
    dlat = fnc_out[inputs.index("dlat")]
    start_lat = fnc_out[inputs.index("start_lat")]
    start_lon = fnc_out[inputs.index("start_lon")]
    dlon_sq = fnc_out[inputs.index("dlon_sq")]
    dlat_sq = fnc_out[inputs.index("dlat_sq")]
    two_dlat = fnc_out[inputs.index("two_dlat")]
    eq_grid = fnc_out[inputs.index("eq_grid")]
    tropical_n = fnc_out[inputs.index("tropical_n")]
    tropical_s = fnc_out[inputs.index("tropical_s")]
    bot = fnc_out[inputs.index("bot")]
    mid = fnc_out[inputs.index("mid")]
    top = fnc_out[inputs.index("top")]
    row_start = fnc_out[inputs.index("row_start")]
    row_end = fnc_out[inputs.index("row_end")]
    tropical_n_alt = fnc_out[inputs.index("tropical_n_alt")]
    tropical_s_alt = fnc_out[inputs.index("tropical_s_alt")]
    bot_alt = fnc_out[inputs.index("bot_alt")]
    top_alt = fnc_out[inputs.index("top_alt")]
    lon_shift = fnc_out[inputs.index("lon_shift")]
    lat_flip = fnc_out[inputs.index("lat_flip")]
    the_calendar = fnc_out[inputs.index("the_calendar")]
    found_years = fnc_out[inputs.index("found_years")]
    super_years = fnc_out[inputs.index("super_years")]
    dim_lat = fnc_out[inputs.index("dim_lat")]
    dim_lon = fnc_out[inputs.index("dim_lon")]
    dim_time = fnc_out[inputs.index("dim_time")]
    var_lat = fnc_out[inputs.index("var_lat")]
    var_lon = fnc_out[inputs.index("var_lon")]
    var_time = fnc_out[inputs.index("var_time")]
    var_slp = fnc_out[inputs.index("var_slp")]
    var_topo = fnc_out[inputs.index("var_topo")]
    var_land_sea_mask = fnc_out[inputs.index("var_land_sea_mask")]
    file_seperater = fnc_out[inputs.index("file_seperater")]
    no_topo = fnc_out[inputs.index("no_topo")]
    no_mask = fnc_out[inputs.index("no_mask")]
    pbot = fnc_out[inputs.index("pbot")]
    ptop = fnc_out[inputs.index("ptop")]
    faux_grids = fnc_out[inputs.index("faux_grids")]
    eq_grid_on_edge = fnc_out[inputs.index("eq_grid_on_edge")]
    row_start_index = row_start.index
    land_gridids = fnc_out[inputs.index("land_gridids")]
    out_path = fnc_out[inputs.index("out_path")]
    # Drop the remainder
    del fnc_out,inputs    
    if over_write_years:
        super_years = over_write_years
    if over_write_out_path:
        out_path = over_write_out_path
    if over_write_slp_path:
        slp_path = over_write_slp_path

    #super_years = [3781,3782]
    # Special case where only a defined basin is used to report statistics, plots
    #   etc.
    basin_filter = 0
    basin = []
    basin_name = ""
    which_hemi = []
    if basin_filter:
        import numpy
        which_hemi = ["nh",0]
        alter_files = 0
        # Four ocean basins from Bender et al. 2011
        to_poles = 1
        if to_poles:
            north_bnds = list(numpy.arange(25.0,90.25,0.25))
            south_bnds = list(numpy.arange(-90.0,-25.25,0.25))
            pacific_bnds = list(numpy.arange(120.0,240.25,0.25))
            atlantic_bnds = list(numpy.arange(280.0,360.25,0.25))
            #north_bnds = list(numpy.arange(25.0,80.25,0.25))
            #south_bnds = list(numpy.arange(-80.0,-25.25,0.25))
            #north_bnds = list(numpy.arange(40.0,80.25,0.25))
            #south_bnds = list(numpy.arange(-80.0,-40.25,0.25))
        else:
            north_bnds = list(numpy.arange(25.0,65.25,0.25))
            south_bnds = list(numpy.arange(-65.0,-25.25,0.25))
            pacific_bnds = list(numpy.arange(120.0,240.25,0.25))
            atlantic_bnds = list(numpy.arange(280.0,360.25,0.25))
        # North Pacific 25-65N, 120-240E
        #NPAC = make_box(north_bnds,pacific_bnds,lons,lats,im)
        # North Atlantic 25-65N, 280-360E
        NATL = make_box(north_bnds,atlantic_bnds,lons,lats,im)
        # South Pacific 25-65S, 120-240E
        #SPAC = make_box(south_bnds,pacific_bnds,lons,lats,im)
        # South Atlantic 25-65N, 280-360E
        #SATL = make_box(south_bnds,atlantic_bnds,lons,lats,im)
        #basins = {"NPAC" : NPAC,
        #        "NALT" : NATL,
        #        "SPAC" : SPAC,
        #        "SALT" : SATL
        #        }
        basin = NATL
        basin_name = "NATL"

    fixed = []
    if use_fixed:
        # Use this line from the intensity_stats.txt file:
        #
        # Note that thirds has these dimentions
        #   hemis = [nh,sh]
        #   thresholds = [lower_third,upper_third]
        #   (len(years)*12,len(hemis),len(thresholds),len(slp_pools))
        #
        # This has to be hand fit to the data being done right know, a better solution
        # would be to read and parse the intensity_stats.txt file directly.
        #   Also doesn't work right now for continous
        import numpy
        years = range(int(super_years[0]),int(super_years[-1])+1)
        fixed = numpy.zeros((len(years)*12,2,2,len(slp_pools)),dtype=numpy.float)
        # Set all year based data to the same.
        lower = 0        
        upper = 1
        # From SLP Pool: Whole record, season and hemisphere
        lower_thres_season = {"nh" : [1003025,1005635,1005126,1003025],
                "sh": [988448,987923,987758,988448]}
        upper_thres_season = {"nh" : [991886,998027,996773,991886],
                "sh": [980038,977569,977376,980038]}
        i = 0
        for x in years:
            for y in range(1,13):
                sea = m2s[y]
                hh = 0
                for h in ["nh","sh"]:
                    #print "Year %d Month %d Season %d Hemi %s" % (x,y,sea,h)
                    #print "\t",lower_thres_season[h][sea-1],upper_thres_season[h][sea-1]
                    fixed[i,hh,lower,0] = lower_thres_season[h][sea-1]
                    fixed[i,hh,upper,0] = upper_thres_season[h][sea-1]
                    fixed[i,hh,lower,1] = lower_thres_season[h][sea-1]
                    fixed[i,hh,upper,1] = upper_thres_season[h][sea-1]
                    
                    #fixed[sea-1,hh,lower,1] = lower_thres_season[h][sea-1]
                    #fixed[sea-1,hh,upper,1] = upper_thres_season[h][sea-1]
                    fixed[sea-1,hh,lower,2] = lower_thres_season[h][sea-1]
                    fixed[sea-1,hh,upper,2] = upper_thres_season[h][sea-1]
                    fixed[i,hh,lower,3] = lower_thres_season[h][sea-1]
                    fixed[i,hh,upper,3] = upper_thres_season[h][sea-1]
                    hh += 1
                i += 1

    # Create out_path if it doesn't exist.
    pick_dirs = cfg.mcms_directories_base_defs.define_dirs()
    msg = pick_dirs.make_pick("standard")
    if verbose: print msg
    dirs_set = pick_dirs.selection
    fig_dir = [x for x in dirs_set if x.startswith("figs")]
    hemis = ["nh","sh"]
    dirs_set = ["%s/%s/%s/" % (x,y,z) for x in fig_dir for y in hemis for z in seasons]
    need_dirs = ["%s%s" % (out_path,x) for x in dirs_set]
    for nd in need_dirs:
        if not os.path.exists(nd):
            os.makedirs(nd)
            if verbose: print "\tDirectory %s Created." % (nd)

    start_year = int(super_years[0])
    end_year = int(super_years[1])+1
    header = "mcms_%s_%04d" % (model,start_year)
    in_file = "%s%s%s" % (out_path,header,tail)
    # Set definitions and instantiate read_mcms w/out a template
    what_do = {"model" : model,
                "in_file" : in_file,
                "out_file" : "",
                "just_center_table" : True,
                "detail_tracks" : 0,
                "as_tracks" : 0,
                "start_time" : "YYYY MM DD HH SEASON",
                "end_time" : "YYYY MM DD HH SEASON",
                "places" : ["GLOBAL"],
                "include_att" : False,
                "include_atts" : False,
                "save_output" : False,
                "overwrite" : False
                }
    # Pass in model definitions, if sf_file available this is simple.
    if model in ["nra","nra2"]:
        # For the NCAR/NCEP Reanalysis 1 and 2 these values are provided
        # and nothing need be done.
        pass
    else:
        # Provide values
        what_do["tropical_end"] = row_end[tropical_n_alt]
        what_do["tropical_start"] = row_start[tropical_s_alt]
        what_do["maxID"] = maxid
        what_do["land_gridids"] = land_gridids
    
    # Shortcut to keep parameter list shorter.
    specifics = {'over_write_years' : super_years,
                 'over_write_out_path' : out_path,
                 'shared_path' : shared_path,
                 'over_write_slp_path' : slp_path,
                 'model' : model,
                 'make_plots' : make_plots,
                 'zmean' : zmean,
                 'land_sea' : land_sea,
                 'fig_format' : fig_format,
                 'skip_to_plots' : skip_to_plots,
                 'alter_files' : alter_files,
                 'continuous' : continuous,
                 'super_uci_stamps' : super_uci_stamps,
                 'super_time_data' : super_time_data,
                 'super_lookup' : super_lookup,
                 'the_calendar' : the_calendar,
                 'jd_fake' : jd_fake,
                 'break_fname' : break_fname,
                 'month_sort_order' : month_sort_order,
                 'timestep' : timestep,
                 'use_other_bounds' : use_other_bounds,
                 'out_path_alt' : out_path_alt,
                 'model_alt' : model_alt,
                 'clim_tag_alt' : clim_tag_alt,
                 'ls_tag_alt' : ls_tag_alt,
                 "slp_pools" : slp_pools,
                 "used_pool" : used_pool,
                 "fixed" : fixed,
                 "use_fixed" : use_fixed,
                 "basin" : basin,
                 "basin_name" : basin_name,
                 "which_hemi" : which_hemi
                 }

    msg = main(defs_set,what_do,**specifics)
    print msg
