#!/usr/bin/env python -tt
"""This module runs all the setup routines to pre-calculate things for
the MCMS project.

Options/Arguments:

Returns/Creates:

Examples:

Notes:

Author: Mike Bauer  <mbauer@giss.nasa.gov>

Log:
    2009/04  MB - File created.
    2010/04  MB - Fixed NCAR Reanalysis grids as nodes/edges. Other fixes.
    2010/07  MB - Moved to v5.
    2011/01  MB - Updated, fixed bug with gdict.
    2012/05  MB - Store data files as gzip to speed read/write.
"""
from __future__ import division
# Alter python 2 division where division of two *integer* values gives an
#   *integer* result (i.e., 1/2 = 0 but 1.0/2 = 0.5 or 1/2.0 = 0.5) to the
#   more rational python 3+ version where all division returns floats
#   (i.e., 1/2 = 0.5). Adds a new integer division operator so that 1//2 = 0

def main(argv):

    # Standard library imports
    import sys
    import os
    import cPickle
    import gzip
    import math

    # Third party module imports
    import numpy
    import netCDF4 as NetCDF

    # MCMS module imports
    from util import setup_center_finder
    from util import setup_tracking
    from util import setup_density
    from util import defs
    from util import g2l
    from util import ij2grid
    from util import grid2ij
    from util import first_last_lons
    from util import organize_data
    from util import shift_gridid
    from util import inflate_gdict
    from util import ParseRunFile
    from util import InjestSetup
    from util import FileExists
    from util import ParseRunFile
    from util import where_y
    from util import PrintColumns
    from util import PullData
    from util import Save_NetCDF


    # Define the base run_file template
    here_path = os.path.abspath(os.path.dirname(__file__))
    run_file_template = "%s/cfg/mcms_setup_template.py" % (here_path)
    if not FileExists(run_file_template,__name__): return 1
   
    # --------------------------------------------------------------------------
    # Parse Request
    # --------------------------------------------------------------------------
    rf = ParseRunFile(run_file_template,argv,here_path)
    if rf == 1:
        return rf

    # --------------------------------------------------------------------------
    # Execute the request
    # --------------------------------------------------------------------------
    if rf.save_plot:
        from util import plotmap
        from util import plotmap_polar 
    # --------------------------------------------------------------------------
    # Start basic definitions of what is to be done.
    # --------------------------------------------------------------------------
 
    # This can be used during the initial setup to limit what is being done.
    skip_time_dat = 1
    skip_cf_dat = 1
    skip_tf_dat = 1
    skip_df_dat = 1
    skip_s_dat = 0
    limit_plots = 1 # skips plots associated with skip_X_dat if triggered
    
    # Redirect I/O to file
    keep_log = 1
    if skip_time_dat+skip_cf_dat+skip_tf_dat+skip_df_dat+skip_s_dat != 0:
        keep_log = 0
    if keep_log:
        log_file = "%ssetup_logfile.txt" % rf.shared_path
        print("\tLog File sent to %s" % log_file)
        sys.stdout = open(log_file, 'w')
    print "\n%s\n%s\n%s\n" % ("*"*80,'Running Setup'.center(80,"*"),"*"*80)
    print "Using:"
    msg = "\tModel:\t\t%s (a.k.a. %s)\n\tOut_Path:\t%s\n\t"
    msg += "Shared_Dir:\t%s\n\tSLP_Path:\t%s"
    print msg % (rf.model_def,rf.model,rf.out_path,rf.shared_path,rf.slp_path)
    msg = "\tno_topo:\t%s\n\tno_mask:\t%s"
    print msg % (bool(rf.no_topo),bool(rf.no_mask))
    
    # Fetch and update definitions. These are held in mcms_defs.py
    defs = defs(**rf.defs_set)

    # --------------------------------------------------------------------------
    # Process the SLP source data
    # --------------------------------------------------------------------------
    # See "Note A" in docs/mcms_setup_notes.txt

    point_registered = 0
    pixel_registered = 0
    if defs.register == 'point':
        point_registered = 1
    if defs.register == 'pixel':
        pixel_registered = 1

    # Dimension & Variable Names: 
    #   These can be changed if needed in mcms_ncvars_base_defs.py
    dim_lat = rf.nc_vars["dim_lat"]
    dim_lon = rf.nc_vars["dim_lon"]
    dim_time = rf.nc_vars["dim_time"]
    var_lat = rf.nc_vars["var_lat"]
    var_lon = rf.nc_vars["var_lon"]
    var_time = rf.nc_vars["var_time"]
    var_slp = rf.nc_vars["var_slp"]
    var_topo = rf.nc_vars["var_topo"]
    var_land_sea_mask = rf.nc_vars["var_land_sea_mask"]
    jd_fake = rf.nc_vars["jd_fake"]
    the_calendar = rf.nc_vars["calendar"]
    faux_grids = rf.nc_vars["faux_grids"]
    polar_cap = rf.nc_vars["polar_cap"]
    eq_grid_on_edge = rf.nc_vars["eq_grid_on_edge"]

    # Assess the slp_path. Get a list of files and from the file
    # names get the time frame that the data cover; if this won't
    # work allow user to manually set years.
    print "\n\tSetup Data Organization and Time References..."
    if not skip_time_dat:
        msg = organize_data(rf.slp_path,rf.nc_vars['break_fname'],
            rf.nc_vars['var_slp'],rf.nc_vars['month_sort_order'],
            rf.nc_vars['calendar'],rf.nc_vars['var_time'],
            rf.nc_vars['jd_fake'],rf.shared_path,rf.blob,
            base_name='time_dat',verbose=1)
    else:
        print ("\t\tSkipped time_dat")
    fnc_out,inputs = InjestSetup("%stime_dat.pz" % (rf.shared_path),4)
    super_uci_stamps = fnc_out[inputs.index("super_uci_stamps")]
    super_tsteps = fnc_out[inputs.index("super_tsteps")]
    super_time_data = fnc_out[inputs.index("super_time_data")]
    super_years = fnc_out[inputs.index("super_years")]
    timestep = fnc_out[inputs.index("timestep")]
    super_lookup = fnc_out[inputs.index("super_lookup")]
    del fnc_out

    start_year = super_years[0]
    end_year = super_years[-1]
    found_years = range(start_year,end_year)
    file_seperater = rf.nc_vars['file_seperater']
    if rf.model == "nra":
        # Observational inconsistencies and/or lack of obs warrant
        #   skipping the 1948-1957 time frame in NCEP/NCAR Reanalysis 1
        #   output. Comment this out to override. See
        #       Kalnay et al.,The NCEP/NCAR 40-year reanalysis project,
        #           Bull. Amer. Meteor. Soc., 77, 437-470, 1996
        print "Skipping 1948-1957 in NCEP/NCAR Reanalysis 1"
        start_year = 1958
        super_years =[1958,end_year]

    # Test for extra files if requested.
    if not os.path.exists(rf.shared_path):
        os.makedirs(rf.shared_path)
        print "Directory %s Created." % (rf.shared_path)
        if rf.no_topo and rf.no_mask:
            print "WARNING: Proceeding without topography or land_sea mask."
        elif not rf.no_topo and rf.no_mask:
            print "WARNING: Proceeding without land_sea mask."
            msg = "Copy topography file into %s and re-run setup."
            print  msg % (rf.shared_path)
            return 1
        elif rf.no_topo and not rf.no_mask:
            print "WARNING: Proceeding without topography."
            msg = "Copy land_sea mask file into %s and re-run setup."
            print msg % (rf.shared_path)
            return 1
        else:
            msg = "Copy the topography and land_sea mask files into %s"
            msg += " and re-run setup."
            print msg % (rf.shared_path)
            msg = "This step can be skipped by setting no_topo and"
            msg += " no_mask to 1 and re-running."
            print msg
            return 1
    else:
        topo_file = "%s%s_%s.nc" % (rf.shared_path,rf.model,var_topo)
        # If no specific land/sea mask available the topo_file can be
        #  used to separate land as all grids with topography > sea_level.
        #  Do this by setting var_land_sea_mask = var_topo.
        mask_file = "%s%s_%s.nc" % (rf.shared_path,rf.model,var_land_sea_mask)
        if rf.no_topo and rf.no_mask:
            print "WARNING: Proceeding without topography or land_sea mask."
        elif not rf.no_topo and rf.no_mask:
            print "WARNING: Proceeding without land_sea mask."
            if not os.path.exists(topo_file):
                print "ERROR: %s not found." % (topo_file)
                return 1
        elif rf.no_topo and not rf.no_mask:
            print "WARNING: Proceeding without topography."
            if not os.path.exists(mask_file):
                print "ERROR: %s not found." % (mask_file)
                return 1
        else:
            if not os.path.exists(topo_file):
                print "ERROR: %s not found." % (topo_file)
                return 1
            if not os.path.exists(mask_file):
                print "ERROR: %s not found." % (mask_file)
                return 1

    # Open test file
    test_file = ""
    if not rf.no_topo:
        test_file = topo_file
    else:
        if not rf.no_mask:
            test_file = mask_file
        else:
            msg = "ERROR: No netcdf file in shared_path to pull lon/lat from, provide a test_file." 
            print(msg)
            return 1

    nc_in = NetCDF.Dataset(test_file,'r',format='NETCDF3_CLASSIC')
    # Pull in Latitude
    lat = nc_in.variables[var_lat]
    if rf.model == "gfdl":
        # GFDL lats look to need rounding.
        lat = [round(x,0) for x in list(lat[:])]
    if rf.model == "ccsm":
        # CCSM lats look to need rounding.
        lat = [round(x,3) for x in list(lat[:])]
    lats = numpy.array(lat[:],dtype=float,copy=1)    
    if lats[0] > lats[-1]:
        lat_flip = 1
        lats = lats[::-1]
    else:
        lat_flip = 0
    dy =  abs(lats[20]-lats[21])
    # Test for Gaussian grids (i.e., dy not uniform)
    gaussian_grid = 0
    dyp = abs(lats[0]-lats[1])
    g_off = abs(dy - dyp)
    if g_off > 0.01 and not polar_cap:
        gaussian_grid = 1
        # Alter polar points so looks like regular grid
        lats[0] = lats[0]-g_off
        lats[-1] = lats[-1]+g_off
    jm = len(lats)
    dlat = math.radians(dy)
    start_lat = lats[0]
    dlat_sq = dlat*dlat
    two_dlat = 2.0*dlat
    dy = round(dy,3)
    interval = round(dy*0.5,3)
    edges = dict([(round(i+interval,3),1) for i in lats])
    edges.update([(round(i-interval,3),1) for i in lats])
    edges = edges.keys()
    edges.sort()
    # Sometimes due to rounding the extra edges sneak in that
    #   really are duplicates.
    lat_edges = []
    for i in edges:
        if i == edges[0]:
            lat_edges.append(i)
        elif i == edges[-1]:
            lat_edges.append(i)
        else:
            if abs(abs(lat_edges[-1] - i) - dy) < 0.1:
                lat_edges.append(i)
    #for i in xrange(len(lat_edges)):
    #    print "%03i --------- %4.2f" % (i,lat_edges[i])
    #    if i <= jm-1:
    #        print "     * %4.2f" % lats[i]
    if faux_grids and point_registered:
        defs.faux_grids = faux_grids
        if lat_flip:
            lats = lats[::-1]
        if faux_grids == 1:
            edges = dict([(round(i+interval,3),1) for i in lats])
            edges.update([(round(i-interval,3),1) for i in lats])
            edges = edges.keys()
            edges.sort()
            # Sometimes due to rounding the extra edges sneak in that
            #   really are duplicates.
            lat_edges = []     
            for i in edges:
                if i == edges[0]:
                    lat_edges.append(i)
                elif i == edges[1]:
                    lat_edges.append(i)
                elif i == edges[-1]:
                    lat_edges.append(i)
                elif i == edges[-2]:
                    lat_edges.append(i)
                else:
                    #print lat_edges[-1],i,abs(lat_edges[-1] - i) 
                    if abs(abs(lat_edges[-1] - i) - dy) < 0.1:
                        lat_edges.append(i)
            if rf.model == "merrac":
                # Set the EQ edge to zero (rounding issue)
                #print [edges.index(x) for x in edges if abs(x) < 0.2]
                lat_edges[72] = 0
            edges = lat_edges  
        elif faux_grids == 2:
            interval_half = round(dy*0.25,2)
            # Ensure that the starting latitude is -90.0
            edges = dict([((round(i+interval,3),1),(round(i,3),1))[i == lats[0]]
                for i in lats[0:-2]])
            edges.update([((round(i-interval,3),1),(round(i,3),1))[i == lats[-1]]
                for i in lats[1:]])
            # Shift half-width polar grid-center
            lats[0] = lats[0]-interval_half
            lats[-1] = lats[-1]+interval_half
            # Sometimes due to rounding the extra edges sneak in that
            #   really are duplicates.
            edges = edges.keys()
            edges.sort()
            #edges.reverse()
            lat_edges = []     
            for i in edges:
                if i == edges[0]:
                    lat_edges.append(i)
                elif i == edges[1]:
                    lat_edges.append(i)
                elif i == edges[-1]:
                    lat_edges.append(i)
                elif i == edges[-2]:
                    lat_edges.append(i)
                else:
                    #print lat_edges[-1],i,abs(lat_edges[-1] - i) 
                    if abs(abs(lat_edges[-1] - i) - dy) < 0.1:
                        lat_edges.append(i)
            edges = lat_edges
        else:
            sys.exit("ERROR: Add interpolation option to faux_grid.")
        if lat_flip:
            lats = lats[::-1]
    else:
        print "ERROR: pick can't use faux_grid."
        return 1
    if not point_registered:
        defs.faux_grids = 0
    #for i in xrange(len(lat_edges)):
    #    print "%03i --------- %4.2f" % (i,lat_edges[i])
    #    if i <= jm-1:
    #        print "     * %4.2f" % lats[i]
    print "\n\tLatitude Information:"
    print "\t\tPoint Registered: %s" % (bool(point_registered))
    print "\t\tFaux Grids: %d" % faux_grids
    print "\t\tGaussian Grid: %s" % (bool(gaussian_grid))
    print "\t\tLatitude Flipped: %s" % (bool(lat_flip))
    print "\t\tLatitude Spacing: %g" % (dy)
    print "\t\tLatitudes (jm = %d):" % (jm)
    PrintColumns(list(lats),indent_tag="\t\t\t",fmt="% 8.3f",cols=4,width=15)
    print "\t\tLatitudes Edges (%d):" % (len(lat_edges))
    PrintColumns(lat_edges,indent_tag="\t\t\t",fmt="% 8.3f",cols=4,width=15)
    plot_lats = lats[:]
    if faux_grids:
        # Reset polar most latitudes to poles for plotting
        lats[0] = -90.0
        lats[-1] = 90.0

    # Pull in Longitude
    lon = nc_in.variables[var_lon]
    if rf.model == "ccsm":
        # CCSM lats look to need rounding.
        lon = [round(x,3) for x in list(lon[:])]
    lons = numpy.array(lon[:],dtype=float,copy=1)
    # If lons in +-180 format shift to 360
    if numpy.any(numpy.less(lons,0.0)):
        tmp = numpy.where(lons < 0.0,numpy.add(lons,360.0),lons)
        lons = tmp
    dx = round(abs(lons[10]-lons[11]),3)
    for lon_shift in range(len(lons)):
        left_edge = lons[lon_shift] - dx*0.5
        #if left_edge < 0.0:
        #    left_edge += 360.0
        right_edge = lons[lon_shift] + dx*0.5
        #if right_edge > 360.0:
        #    right_edge -= 360.0
        # Test where prime meridian is located
        if left_edge <= 0.0 < right_edge:
            break
    if lon_shift:
        lons = numpy.roll(lons,lon_shift)
    im = len(lons)
    # Some of our methods (e.g., see util/inflate_gdict.py) need to be adapted
    #   for non-even number of longitudes.
    if im%2 != 0:
        msg = "\nError: Some of our methods (see docs/working_with_grids.pdf"
        msg += " and util/inflate_gdict.py) need to be adapted to non-even"
        msg += " number of longitudes"
        sys.exit(msg)
    halfway = int(im/2)
    dlon = math.radians(dx)
    start_lon = lons[0]
    dlon_sq = dlon*dlon
    interval = round(dx*0.5,3)
    edges = dict([(round(i+interval,3),1) for i in lons])
    edges.update([(round(i-interval,3),1) for i in lons])
    edges = edges.keys()
    edges.sort()
    # Sometimes due to rounding the extra edges sneak in that
    #   really are duplicates.
    lon_edges = []     
    for i in edges:
        if i == edges[0]:
            lon_edges.append(i)
        elif i == edges[1]:
            lon_edges.append(i)
        elif i == edges[-1]:
            lon_edges.append(i)
        elif i == edges[-2]:
            lon_edges.append(i)
        else:
            #print lon_edges[-1],i,abs(lon_edges[-1] - i) 
            if abs(abs(lon_edges[-1] - i) - dx) < 0.1:
                lon_edges.append(i)
    print "\n\tLongitude Information:"
    print "\t\tLongitude Rotated: %s (%d)" % (bool(lon_shift),lon_shift)
    print "\t\tLongitude Spacing: %g degrees" % (dx)
    print "\t\tLongitudes (im = %d):" % (im)
    PrintColumns(list(lons),indent_tag="\t\t\t",fmt="% 8.3f",cols=4,width=15)
    print "\t\tLongitude Edges (%d):" % (len(lon_edges))
    PrintColumns(lon_edges,indent_tag="\t\t\t",fmt="% 8.3f",cols=4,width=15)

    # Maximum number of gridIDs in model grid
    maxid = jm*im
    model_flag = "_%s_" % (rf.model.upper())
    # Set some boundaries (latitude row)
    eq_grid,eq_grid_on_edge = where_y([round(x,3) for x in lats],0.0,round(dy*0.5,3),edges=lat_edges)
    if eq_grid_on_edge:
        # When this happens the eq_grid by default is the grid in the southern
        #   so the eq_grid+1 denotes the northern hemisphere.
        # Where normally
        #   SH lats[:eq_grid]
        #   NH lats[eq_grid+1:]
        # Now
        #   SH lats[:eq_grid+1]
        #   NH lats[eq_grid+1:]
        if lats[eq_grid] > 0:
            eq_grid = eq_grid - 1
    tropical_s,tmp = where_y([round(x,3) for x in lats],-1*defs.tropical_boundary,dy*0.5,edges=lat_edges)
    tropical_n,tmp  = where_y([round(x,3) for x in lats],defs.tropical_boundary,dy*0.5,edges=lat_edges)
    bot = ij2grid(tropical_s,0,im,jm)
    mid =  ij2grid(eq_grid,0,im,jm)
    top = ij2grid(tropical_n,im-1,im,jm)
    msg = "\n\tTropical_boundary:\t%d degrees\n\tTropical_n:\t\t%d %d "
    msg += "(row,gridid)\n\tEQ_grid:\t\t%d (on_edge %s) %d (row,gridid)\n\tTropical_s:"
    msg += "\t\t%d %d (row,gridid)"
    print msg % (defs.tropical_boundary,tropical_n,top,eq_grid,bool(eq_grid_on_edge),mid,
        tropical_s,bot)
    tropical_s_alt,tmp  = where_y(list(lats),-1*defs.tropical_boundary_alt,dy*0.5,edges=lat_edges)
    tropical_n_alt,tmp  = where_y(list(lats),defs.tropical_boundary_alt,dy*0.5,edges=lat_edges)
    bot_alt = ij2grid(tropical_s_alt,0,im,jm)
    top_alt = ij2grid(tropical_n_alt,im-1,im,jm)
    msg = "\n\tTropical_boundary Alt:\t%d degrees\n\tTropical_n_alt:\t\t%d %d "
    msg += "(row,gridid)\n\tEQ_grid:\t\t%d %d (row,gridid)\n\tTropical_s_alt:"
    msg += "\t\t%d %d (row,gridid)"
    print msg % (defs.tropical_boundary_alt,tropical_n_alt,top_alt,eq_grid,mid,
        tropical_s_alt,bot_alt)
    
    polar_s,tmp  = where_y(list(lats),-1*defs.polar_boundary,dy*0.5,edges=lat_edges)
    polar_n,tmp  = where_y(list(lats),defs.polar_boundary,dy*0.5,edges=lat_edges)
    pbot = ij2grid(polar_s,0,im,jm)
    ptop = ij2grid(polar_n,im-1,im,jm)
    msg = "\n\tPolar_boundary:\t%d degrees\n\tPolar_n:\t\t%d %d "
    msg += "(row,gridid)\n\tPolar_s:""\t\t%d %d (row,gridid)"
    print msg % (defs.polar_boundary,polar_n,ptop,polar_s,pbot)

    
    # Define High-Resolution models (dx <= 1 degree)
    high_res = 0
    if dx <= 1.0:
        high_res = 1
    print "\tHigh Resolution: %s" % bool(high_res)

    # Get gridids for 1st and last lon of each lat row.
    row_start,row_end = first_last_lons(jm,im)
    print "\n\tRow_start:"
    PrintColumns(row_start,indent_tag="\t\t",fmt="% 6d",cols=10,width=10)
    
    print "\tRow_end:"
    PrintColumns(row_end,indent_tag="\t\t",fmt="% 6d",cols=10,width=10)

    #tropic_rows = range(tropical_s,tropical_n+1)
    # Leave two extra rows for comfort
    tropic_rows = range(tropical_s+2,tropical_n-1)
    tropic_rows = [] # skip 
    row_start_index = row_start.index

    # Prevent centers from being found in at the pole (reduces a lot of problems).
    no_poles = []
    if rf.polar_exclusion:
        sh_pex,tmp  = where_y(list(lats),rf.polar_exclusion[0],dy*0.5,edges=lat_edges)
        nh_pex,tmp  = where_y(list(lats),rf.polar_exclusion[1],dy*0.5,edges=lat_edges)
        no_poles = [x for x in xrange(row_start[0],row_end[sh_pex]+1)]
        no_poles.extend([x for x in xrange(row_start[nh_pex],row_end[jm-1]+1)])
        no_poles = sorted(no_poles)

    polar = [x for x in xrange(0,pbot)]
    polar.extend([x for x in xrange(ptop,maxid)])
    polar = sorted(polar)

    # Set up some default values for the next set of routines
    defs_grid = {"im" : im, "jm" : jm, "maxid" : maxid, "lats" : lats,
                 "lons" : lons, "dx" : dx, "dy" : dy, "dlon" : dlon,
                 "dlat" : dlat, "dlon_sq" : dlon_sq, "dlat_sq" : dlat_sq,
                 "two_dlat" : two_dlat, "start_lat" : start_lat,
                 "start_lon" : start_lon, "model_flag" : model_flag,
                 "eq_grid" : eq_grid, "tropical_s" : tropical_s,
                 "tropical_n" : tropical_n, "bot" : bot,"top" : top,
                 "mid" : mid, "row_start" : row_start, "row_end" : row_end,
                 "timestep" : timestep, "tropical_s_alt" : tropical_s_alt,
                 "tropical_n_alt" : tropical_n_alt, "bot_alt" : bot_alt,
                 "top_alt" : top_alt, "lon_shift" : lon_shift,
                 "lat_flip" : lat_flip, "faux_grids" : faux_grids,
                 "polar_s" : polar_s, "polar_n" : polar_n, "pbot" : pbot,
                 "ptop" : ptop, "eq_grid_on_edge" : eq_grid_on_edge,
                 "high_res" : high_res,"halfway" : halfway
                 }

    # --------------------------------------------------------------------------
    # Create various things needed for center_finder and others
    # --------------------------------------------------------------------------
    # Returns:
    #  use_all_lons - Latitude row where whole thing fits in critical_radius.
    #  search_radius - Defines Regional_Low radius by critical_radius or wavenumber.
    #  regional_nys - Maximum number of latitude rows to check based on critical_radius.
    #  rdict - Lookup table for all grids fit within search_radius by row.
    #  ldict - Lookup table for Concavity/Laplacian Test by row.
    #  ijdict - Lookup table for i,j,lon,lat of a gridid.
    # Stores: cf_dat.p
    print "\n\tSetup Center_Finder..."
    if not skip_cf_dat:
        fnc_out = setup_center_finder(defs_grid,defs,rf)
        # Save to File
        cf_file = "%scf_dat.pz" % (rf.shared_path)
        cPickle.dump(fnc_out,gzip.open(cf_file, "wb",-1))
        print "\tMade: %s\n" % (cf_file)
        # Write the API to file for later use
        cf_file_api = "%scf_dat.py" % (rf.shared_path)
        f = open(cf_file_api,'w')
        msg = """
inputs = ("use_all_lons","search_radius","regional_nys","rdict","ldict","ijdict","lapp_cutoff","hpg_cutoff")
        """
        f.write(msg)
        msg = """
#
# Example usage:
#
#   cf_file = "%scf_dat.pz" % (rf.shared_path)
#   fnc_out,inputs = InjestSetup(cf_file,1) 
#   use_all_lons = fnc_out[inputs.index("use_all_lons")]
#   search_radius = fnc_out[inputs.index("search_radius")]
#   regional_nys = fnc_out[inputs.index("regional_nys")]
#   rdict = fnc_out[inputs.index("rdict")]
#   ldict = fnc_out[inputs.index("ldict")]
#   ijdict = fnc_out[inputs.index("ijdict")]
#   lapp_cutoff = fnc_out[inputs.index("lapp_cutoff")]
#   hpg_cutoff = fnc_out[inputs.index("hpg_cutoff")]
#   del fnc_out,inputs        
            """
        f.write(msg)
        f.close()
        del fnc_out
    else:
        print ("\t\tSkipped cf_dat")
    fnc_out,inputs = InjestSetup("%scf_dat.pz" % (rf.shared_path),1)
    # Keep Wanted values
    if rf.save_plot: rdict = fnc_out[inputs.index("rdict")]
    ijdict = fnc_out[inputs.index("ijdict")]
    # Save memory
    del fnc_out
    if rf.save_plot and not (limit_plots and skip_cf_dat):
        # Select the longitude or latitude to center upon.
        c_lon = 180.0
        c_lat = 45.0
        #tmp = [abs(c_lon-lons[x]) for x in range(im)]
        #center_lon = tmp.index(min(tmp))
        tmp = [abs(c_lat-plot_lats[x]) for x in range(jm)]
        center_lat = tmp.index(min(tmp))
        c_lat_op = -1*c_lat
        tmp = [abs(c_lat_op-plot_lats[x]) for x in range(jm)]
        center_lat_op = tmp.index(min(tmp))
        bl =  [15.,-15.]
        hnames = ["nh","sh"]
        blank = numpy.zeros(im*jm)
        blank_cuml = numpy.zeros(im*jm)
        # Shift rdict 180 degrees longitude
        drow = 0
        dcol = halfway        
        c_halo = numpy.array([x for y in xrange(3) for x in xrange(im)],
                    dtype=numpy.int32)
        for j in range(jm):
            for pnt in rdict[j]:
                # Just values along this latitude
                alt_pnt = shift_gridid(pnt,dcol,drow,c_halo,halfway,im,jm)  
                if alt_pnt >= row_start[j] and alt_pnt <= row_end[j]: blank[alt_pnt] = 1.0
                blank_cuml[alt_pnt] += 1.0
        # Note these plots may seem offset in longitude due to how lons
        #   breaks with a plot centered on grids. For example, a c_lon = 0.0
        #   means the search radius is evaluated in the middle of the 1st
        #   longitude grid. As this grid is not always centered on 0.0 a plot
        #   centered on 0.0 will appear to have more search radius on the
        #   one side of the 0/360 break (or 180/-180 or c_lon = 180.0) than
        #   on the other side. This a result of using grid centers for the
        #   search radius rather than direct lon,lat and a distance calculation.
        #   On the otherhand the lookup tables in rdict at orders of magnitude
        #   faster than doing these calculations every time.
        #
        #   A similar problem occurs in latitude so the polar most grids may
        #   look offset.
        
        # This plot shows the net result of counting a rdict search at each
        #   latitude at c_lon. That is, counts are only once per grid. This
        #   just shows the width (in grids) of the search radius over all
        #   latitudes is the same (based on zonal wavenumber).
        fmt = "%s%s_regional_radius_one_longitude_all_latitudes%s"
        pname = fmt % (rf.shared_path,rf.model,rf.fig_format)
        plot = plotmap(missing=-1.0,color_scheme="gray_r",
            nocolorbar='nocolorbar',lon_0=0.0)
        plot.create_fig()
        plot.add_field(lons,plot_lats,blank,ptype='imshow')
        title = "Regional Radius At Each Latitude"
        plot.finish(pname,title=title)
        print "\t\tMade figure %s" % (pname)

        # This shows that due to the contraction of grid spacing with latitude
        #   more grids are counted overall at low latitudes because the same
        #   search radius because outside of the latitude of c_lon the same
        #   search radius is longer at low latitudes because the same wavenumber
        #   feature there is larger in an absolute sense. Also note that the
        #   counts are higher in the same longitude as c_lon for the same
        #   reason.
        pname = pname.replace("latitudes","latitudes_cuml")
        plot = plotmap(missing=0.0,color_scheme="jet",lon_0=0.0)
        plot.create_fig()
        plot.add_field(lons,plot_lats,blank_cuml,ptype='imshow')
        title = "Regional Radius At Each Latitude (Cumulative)"
        plot.finish(pname,title=title)
        print "\t\tMade figure %s" % (pname)
       
        # Next we show the same thing but using an equal area polar plot.
        blank.shape = (jm,im)
        for h in [0,1]:
            ppname = pname.replace("_cuml","_cuml_%s" % hnames[h])
            fplot = plotmap_polar(missing=0,color_scheme="jet",
                hemi=hnames[h],mproj='laea',bounding_lat=bl[h],
                lon_0=180.0,cints=[0.0,blank_cuml.max()],discrete=10)
            # Give only data from the correct hemisphere.
            if hnames[h] is "nh":
                zlats = numpy.array([x for x in plot_lats if x >= 0])
                if eq_grid_on_edge:
                    zval = blank_cuml[eq_grid+1:,:]
                else:
                    zval = blank_cuml[eq_grid:,:]
            else:
                zlats = numpy.array([x for x in plot_lats if x <= 0])
                if eq_grid_on_edge:
                    zval = blank_cuml[:eq_grid+1,:]
                else:
                    zval = blank_cuml[:eq_grid+1,:]
            fplot.create_fig()
            fplot.add_pcolor(lons,zlats,zval)
            fplot.add_extras(circles=[bl[h],2*bl[h]],meridians=[0,180])
            if title:
                fplot.finish(ppname,title=title)
            else:
                fplot.finish(ppname)
            print "\t\tMade figure %s" % (ppname)

        # Here we use c_lat to show what rdict looks like at a specific
        #   latitude.
        blank = numpy.zeros(im*jm)
        blank_one = numpy.zeros(im*jm)
        # NH 
        xlist = [shift_gridid(xx,dcol,0,c_halo,halfway,im,jm)
            for xx in rdict[center_lat] for dcol in xrange(im)]
        # SH
        xlist.extend([shift_gridid(xx,dcol,0,c_halo,halfway,im,jm) 
            for xx in rdict[center_lat_op] for dcol in xrange(im)])      
        for pnt in xlist:
            blank[pnt] = 1.0
        for pnt in [shift_gridid(xx,halfway,0,c_halo,halfway,im,jm) 
                for xx in rdict[center_lat]]:
            blank_one[pnt] = 1.0
        for pnt in [shift_gridid(xx,halfway,0,c_halo,halfway,im,jm) 
                for xx in rdict[center_lat_op]]:
            blank_one[pnt] = 1.0

        # Note: the poleward and equatorward ends are not exactly
        # symmetrical because the fixed length search radius can
        # result in differing numbers of latitudes being reached as
        # of latitude spacing changes with latitude.
        fmt = "%s%s_regional_radius_all_longitudes_one_latitude%s"
        pname = fmt % (rf.shared_path,rf.model,rf.fig_format)
        title = u"Regional Search at %5.2f\u00b0N/S" % c_lat
        plot = plotmap(missing=-1.0,color_scheme="gray_r",
            nocolorbar='nocolorbar',lon_0=c_lon)
        plot.create_fig()
        plot.add_field(lons,plot_lats,blank,ptype='imshow')
        plot.finish(pname,title=title)
        print "\t\tMade figure %s" % (pname)
        
        fmt = "%s%s_regional_radius_example%s"
        pname = fmt % (rf.shared_path,rf.model,rf.fig_format)
        title = u"Regional Search at %5.2f\u00b0N/S" % c_lat
        plot = plotmap(missing=-1.0,color_scheme="gray_r",
            nocolorbar='nocolorbar',lon_0=c_lon)
        plot.create_fig()
        plot.add_field(lons,plot_lats,blank_one,ptype='imshow')
        plot.finish(pname,title=title)
        print "\t\tMade figure %s" % (pname)
        # Again from a polar plot viewpoint
        blank.shape = (jm,im)
        for h in [0,1]:
            fmt = "%s%s_regional_radius_all_longitudes_one_latitude_%s%s"
            pname = fmt % (rf.shared_path,rf.model,hnames[h],rf.fig_format)
            title = u"Regional Search at %5.2f\u00b0%s" % (c_lat,
                hnames[h][0].upper())
            fplot = plotmap_polar(missing=-1,color_scheme="gray_r",
                hemi=hnames[h],mproj='laea',bounding_lat=bl[h],
                lon_0=c_lon,nocolorbar='nocolorbar',cints=[0.0,1.0],
                discrete=2)
            # Give only data from the correct hemisphere.
            if hnames[h] is "nh":
                zlats = numpy.array([x for x in plot_lats if x >= 0])
                if eq_grid_on_edge:
                    zval = blank[eq_grid+1:,:]
                else:
                    zval = blank[eq_grid:,:]
            else:
                zlats = numpy.array([x for x in plot_lats if x <= 0])
                if eq_grid_on_edge:
                    zval = blank[:eq_grid+1,:]
                else:
                    zval = blank[:eq_grid+1,:]                   
            fplot.create_fig()
            fplot.add_pcolor(lons,zlats,zval)
            fplot.add_extras(circles=[bl[h],2*bl[h]],meridians=[0,180])
            if title:
                fplot.finish(pname,title=title)
            else:
                fplot.finish(pname)
            print "\t\tMade figure %s" % (pname)
            fmt = "%s%s_regional_radius_example_%s%s"
            pname = fmt % (rf.shared_path,rf.model,hnames[h],rf.fig_format)
            title = u"Regional Search at %5.2f\u00b0%s" % (c_lat,
                hnames[h][0].upper())
            fplot = plotmap_polar(missing=-1,color_scheme="gray_r",
                hemi=hnames[h],mproj='laea',bounding_lat=bl[h],
                lon_0=c_lon,nocolorbar='nocolorbar',cints=[0.0,1.0],
                discrete=2)
            # Give only data from the correct hemisphere.
            if hnames[h] is "nh":
                zlats = numpy.array([x for x in plot_lats if x >= 0])
                if eq_grid_on_edge:
                    zval = blank_one[eq_grid+1:,:]
                else:
                    zval = blank_one[eq_grid:,:]
            else:
                zlats = numpy.array([x for x in plot_lats if x <= 0])
                if eq_grid_on_edge:
                    zval = blank_one[:eq_grid+1,:]
                else:
                    zval = blank_one[:eq_grid+1,:]                   
            fplot.create_fig()
            fplot.add_pcolor(lons,zlats,zval)
            fplot.add_extras(circles=[bl[h],2*bl[h]],meridians=[0,180])
            if title:
                fplot.finish(pname,title=title)
            else:
                fplot.finish(pname)
            print "\t\tMade figure %s" % (pname)

    # Purpose: Create various things needed for tracking and others.
    # Returns:
    #   tdict - Lookup table for grids within a great circle radius of (travel_distance).
    # Stores: tf_dat.p
    print "\n\tSetup Track_Finder..."
    tf_file = "%stf_dat.pz" % (rf.shared_path)
    if not skip_tf_dat:
        args = (defs,ijdict,defs_grid,rf)
        fnc_out = setup_tracking(*args)
        # Save to File
        cPickle.dump(fnc_out,gzip.open(tf_file, "wb",-1))
        print "\t\tMade: %s\n" % (tf_file)
        # Write the API to file for later use
        tf_file_api = "%stf_dat.py" % (rf.shared_path)
        f = open(tf_file_api,'w')
        msg = """
inputs = ("tdict")
"""
        f.write(msg)  
        msg = """
#
# Example usage:
#
#   tf_file = "%stf_dat.pz" % (rf.shared_path)
#   tdict,inputs = InjestSetup(tf_file,2)   
        """
        f.write(msg)
        f.close()
        del fnc_out
    else:
        print ("\t\tSkipped tf_dat")
    tdict,inputs = InjestSetup(tf_file,2)

    # 
    # Purpuse: Create various things needed for center/track density calculations.
    # Returns:
    #   ddict - Lookup table for grids within a great circle radius of (c_size).
    # Stores: df_dat.p
    print "\n\tSetup Density..."
    df_file = "%sdf_dat.pz" % (rf.shared_path)
    if not skip_df_dat:
        args = (defs,ijdict,defs_grid,rf)
        ddict = setup_density(*args)
        cPickle.dump(ddict,gzip.open(df_file, "wb",-1))
        print "\t\tMade: %s\n" % (df_file)
        # Write the API to file for later use
        df_file_api = "%sdf_dat.py" % (rf.shared_path)
        f = open(df_file_api,'w')
        msg = """
inputs = ("ddict")
"""
        f.write(msg)
        msg = """
#
# Example usage:
#
#   df_file = "%sdf_dat.pz" % (rf.shared_path)
#   ddict,inputs = InjestSetup(df_file,3)   
        """
        f.write(msg)
        f.close()
        del ddict   
    else:
        print ("\t\tSkipped df_dat")
    ddict,inputs = InjestSetup(df_file,3)
    del ddict

    # Purpose: Locate grids for land-sea mask
    # Returns: lists of gridids used for optional screens.
    # Stores:
    if rf.model.find("merran") != -1:
        # tmp problem with merra extra files
        lon_shift1 = lon_shift
        lon_shift = 270    
    if not skip_s_dat:
        debug_land = 0
        debug_trouble = 0
        gdict = inflate_gdict(im,jm)
        if not rf.no_mask:
            print "\n\tMaking Land Mask"
            if mask_file == topo_file:
                if no_topo:
                    # Topography screen disabled, but want
                    # to use topo field to make mask.
                    if rf.model == "ccsm":
                        tfile = NetCDF.Dataset(topo_file,'r',format='NETCDF4_CLASSIC')
                    else:
                        tfile = NetCDF.Dataset(topo_file,'r',format='NETCDF3_CLASSIC')
                    topo = tfile.variables[var_topo]
                    topo = numpy.array(topo[:],dtype=float,copy=1)
                    if len(topo.shape) > 2:
                        # Assuming arranged as time,lat,long
                        topo = topo[0,:,:]
                    if lat_flip:
                        tmp = topo[::-1,:]
                        topo = numpy.array(tmp) # ensures contiguous
                    if lon_shift:
                        tmp = numpy.roll(topo,lon_shift,axis=1)
                        topo = numpy.array(tmp)
                    tfile.close()
                    topo.shape = im*jm
                    # Use topo file to make land-sea mask
                    land_gridids = [x for x in xrange(maxid) if topo[x] > 0.0]
                else:
                    # Mask created below
                    pass
            else:
                # Use provided mask file. Note it is assumed that land grids
                #   have a value of > 0. If not please make exception based
                #   on if pick:
                if rf.model == "ccsm":
                    tfile = NetCDF.Dataset(mask_file,'r',format='NETCDF4_CLASSIC')
                else:
                    tfile = NetCDF.Dataset(mask_file,'r',format='NETCDF3_CLASSIC')
                mask = tfile.variables[var_land_sea_mask]
                mask = numpy.array(mask[:],dtype=float,copy=1)
                # Remove time dimension
                if len(mask.shape) > 2:
                    # Assuming arranged as time,lat,long
                    mask = mask[0,:,:]
                if lat_flip:
                    if rf.model.find("nra") != -1 or rf.model.find("test") != -1:
                        # For some reason NRA topo has the opposite
                        #   orientation of the other files.
                        pass
                    else:
                        tmp = mask[::-1,:]
                        mask = numpy.array(tmp) # ensures contiguous
                if lon_shift:
                    tmp = numpy.roll(mask,lon_shift,axis=1)
                    mask = numpy.array(tmp)
                tfile.close()
                mask.shape = im*jm
                if rf.model.find("merran") != -1:
                    # Read also the LANDICE as need for Greenland and Antarctica
                    var_land_sea_mask1 = "FRLANDICE"
                    mask_file1 = mask_file.replace(var_land_sea_mask,var_land_sea_mask1)
                    tfile = NetCDF.Dataset(mask_file1,'r',format='NETCDF3_CLASSIC')
                    mask1 = tfile.variables[var_land_sea_mask1]
                    mask1 = numpy.array(mask1[:],dtype=float,copy=1)
                    # Remove time dimension
                    if len(mask1.shape) > 2:
                        # Assuming arranged as time,lat,long
                        mask1 = mask1[0,:,:]
                    if lat_flip:
                        tmp = mask1[::-1,:]
                        mask1 = numpy.array(tmp) # ensures contiguous
                    if lon_shift:
                        tmp = numpy.roll(mask1,lon_shift,axis=1)
                        mask1 = numpy.array(tmp)
                    tfile.close()
                    mask1.shape = im*jm 
                    # Combine FRLAND and FRLANDICE 
                    for x in xrange(maxid):
                         mask[x] = mask[x] + mask1[x]
                    # Read also the FRLAKE 
                    var_land_sea_mask1 = "FRLAKE"
                    mask_file1 = mask_file.replace(var_land_sea_mask,var_land_sea_mask1)
                    tfile = NetCDF.Dataset(mask_file1,'r',format='NETCDF3_CLASSIC')
                    mask1 = tfile.variables[var_land_sea_mask1]
                    mask1 = numpy.array(mask1[:],dtype=float,copy=1)
                    # Remove time dimension
                    if len(mask1.shape) > 2:
                        # Assuming arranged as time,lat,long
                        mask1 = mask1[0,:,:]
                    if lat_flip:
                        tmp = mask1[::-1,:]
                        mask1 = numpy.array(tmp) # ensures contiguous
                    if lon_shift:
                        tmp = numpy.roll(mask1,lon_shift,axis=1)
                        mask1 = numpy.array(tmp)
                    tfile.close()
                    mask1.shape = im*jm 
                    # Combine
                    for x in xrange(maxid):
                         mask[x] = mask[x] + mask1[x]

                # Make land-sea mask
                if rf.model.find("merra") != -1:
                    land_gridids = [x for x in xrange(maxid) if mask[x] > 0.0]
                else:
                    land_gridids = [x for x in xrange(maxid) if mask[x] > 0.0]
                
                if rf.model.find("nra") != -1 or rf.model.find("test") != -1:
                    # The NCEP Reanalysis shows a land/sea screen of ocean
                    #    for iceshelfs and <=0 topography which means for
                    #   the ross and ronne-filcher ice shelfs in Antarctica
                    #   an odd situation of land sea mask or topo mask occurs.
                    ice_shelf_tweeks = [365, 366, 367, 368, 369, 370, 371, 499, 500,
                            501, 502, 503, 504, 505, 506, 507, 508, 509, 510, 511,
                            512, 513, 514, 515, 641, 642, 643, 644, 645, 646,
                            647, 648, 649, 650, 651, 652, 653, 654, 655, 656,
                            657, 688, 689, 690, 691, 692, 693, 694, 695,
                            696, 697, 698, 699, 702, 703, 704, 705, 786, 787,
                            788, 789, 790, 791, 792, 793, 794, 795, 796, 797,
                            798, 799, 800, 801, 803, 804, 837, 838, 839, 840,
                            841, 842, 843, 844, 845, 846, 847, 848, 849, 850,
                            984, 1111, 1112, 1118, 1119, 1120, 1121, 1122, 1123,
                            1128, 1266, 1267, 1272, 1415, 1416, 1559, 1560, 1705]
                    land_gridids.extend(ice_shelf_tweeks)
                land_tweaks = []
                if rf.model.find("gissm") != -1:
                    # Tweak the land mask around Antarctica
                    land_tweaks = [440,441,443,444,445,448,449,450,456,455,456,457,458,459,460,490]
                    land_tweaks.extend([360,361,364,365,392,393,417,429,430,431])
                    land_tweaks.extend([321,332,333,334,336,337,338,339,354,355])
                    land_tweaks.extend([279,280])
                    land_gridids.extend(land_tweaks)
                land_gridids.sort()
                if debug_land:
                    land_map = numpy.zeros(im*jm)
                    for x in land_gridids:
                         land_map[x] = 1
                    for x in land_tweaks:
                        land_map[x] = 2
                    pname = "%s%s_landmask.nc" % (rf.shared_path,rf.model)
                    save_it = Save_NetCDF(land_map,lons,plot_lats,pname,0.0)
                    pname = "%s%s_mask.nc" % (rf.shared_path,rf.model)
                    save_it = Save_NetCDF(mask,lons,plot_lats,pname,-1.0)     
                           
        # Purpose: Locate grids that fail the topography screen.
        # Returns: lists of gridids used for optional screens.
        # Stores:
        if not rf.no_topo:
            print "\n\tMaking Problematic Mask"
            if rf.model == "ccsm":
                tfile = NetCDF.Dataset(topo_file,'r',format='NETCDF4_CLASSIC')
            else:
                tfile = NetCDF.Dataset(topo_file,'r',format='NETCDF3_CLASSIC')
            topo = tfile.variables[var_topo]
            topo = numpy.array(topo[:],dtype=float,copy=1)
            # Remove time dimension
            if len(topo.shape) > 2:
                # Assuming arranged as time,lat,long
                topo = topo[0,:,:]
            if lat_flip:
                tmp = topo[::-1,:]
                topo = numpy.array(tmp) # ensures contiguous
            if lon_shift:
                tmp = numpy.roll(topo,lon_shift,axis=1)
                topo = numpy.array(tmp)
            #if 'add_offset' in tfile.variables[var_topo].ncattrs():
            #    offset = getattr(tfile.variables[var_topo],'add_offset')
            #else:
            #    offset = 0.0
            #if 'scale_factor' in tfile.variables[var_topo].ncattrs():
            #    scale_factor = getattr(tfile.variables[var_topo],'scale_factor')
            #else:
            #    scale_factor = 1.0
            ## Apply offset and scale_factor
            #tmp = numpy.multiply(numpy.add(numpy.array(topo[:],dtype=float,copy=1)
            #                               ,offset),scale_factor)
            #topo = numpy.array(tmp)
            tfile.close()
            topo.shape = im*jm
            
            if mask_file == topo_file:
                # Use topo file to make land-sea mask
                land_gridids = [x for x in xrange(maxid) if topo[x] > 0.0]
            
            # Troubled Grids: SLP away from sea level is a derived quantity.
            #   Certain conditions seem to allow for erroneous SLP values.
            #   These can hamper cyclone detection by mimicking or
            #   obscuring real cyclones. Here we create an optional list
            #   of grids that the scheme used to warn itself that the
            #   SLP values in these locations may not be reliable.
            #   Generally, we are skeptical of SLP values over high
            #   elevation (>= 1000m) or steep topography (average local
            #   relief > 150m + maximum local coastal relief > 500m) and
            #   in some cases land bound grids with sub-level elevations
            #   (deep basins). We likewise add the polemost rows of grids to
            #   this list (in many models the SLPs are uniform value for all
            #   longitudes in the polar grids).
            #
            #   Note centers in these grids are not discarded out of hand,
            #   but rather, undergo extra tests during the analysis.
           
            # If just_topo > 0, thus just topo >= just_topo is used to
            #   make troubled_centers. That is, the standard high topo screen.
            just_topo = 0
            #just_topo = 1000.0 # Raible 2008

            # List all land or near land grids
            troubled_centers = []
            if debug_trouble:
                relief_field = numpy.multiply(numpy.ones(im*jm),10000.0)
                max_relief = numpy.multiply(numpy.ones(im*jm),10000.0)
                min_relief = numpy.multiply(numpy.ones(im*jm),10000.0)
            # Set negative topography values to 0
            topo = numpy.where(topo < 0.0,0.0,topo)
            if just_topo:
                for gridid in xrange(maxid):
                    row_guess = [x for x in row_start if x <= gridid]
                    row_guess = row_start_index(row_guess[-1])
                    if row_guess in tropic_rows: continue                
                    if topo[gridid] >= 1000: troubled_centers.append(gridid)
            else:
                for gridid in xrange(maxid):
                    # Skip deep tropics
                    row_guess = [x for x in row_start if x <= gridid]
                    row_guess = row_start_index(row_guess[-1])
                    if row_guess in tropic_rows: continue
                    # Check hemisphere
                    if eq_grid_on_edge:
                        nh = (0 if row_guess <= eq_grid else 1)
                    else:
                        nh = (0 if row_guess < eq_grid else 1)
                    # Skip grids where all 8 neighbors are ocean
                    near_8 = gdict[gridid]
                    land = [x for x in near_8 if x in land_gridids]
                    if len(land) < 1: continue
                    if rf.model.find("merra") != -1 or rf.model.find("gissm") != -1:
                        # Note the abs added with MERRA
                        relief = [abs(topo[x]-topo[gridid]) for x in gdict[gridid]]
                    else:
                        relief = [topo[x]-topo[gridid] for x in gdict[gridid]]
                    ave_relief = sum(relief)/9.0
                    relief = sorted(relief)
                    if rf.model.find("gissm") != -1:
                        # The coarse resolution of this model requires some
                        #   modifications
                        if topo[gridid] >= 1000:
                            troubled_centers.append(gridid)
                        elif ave_relief >= 450:
                            troubled_centers.append(gridid)
                    else:
                        if ave_relief >= 150:
                            troubled_centers.append(gridid)
                        elif topo[gridid] >= 1000:
                            troubled_centers.append(gridid)
                        elif gridid not in land_gridids and relief[-1] >= 400:
                            troubled_centers.append(gridid)
                    if debug_trouble:
                        relief_field[gridid] = ave_relief
                        max_relief[gridid] = relief[-1]
                        min_relief[gridid] = relief[0]
            
            # Add gridids that you specifically want screened, for
            #  example if you find you have a hotspot of center
            #  activity that you feel is erroneous due to say SLP
            #  reduction errors that are not caught by the above
            #  screen. Or if you want to discard all centers from
            #  a certain locale.
            grids_of_interest = []
            # # Add polar most rows
            # grids_of_interest.extend([x for x in
            #     xrange(row_start[0],row_end[0]+1)])
            # grids_of_interest.extend([x for x in
            #         xrange(row_start[jm-1],row_end[jm-1]+1)])
            # Add all of land grids from antarctica (land south of 60S)
            if not just_topo:
                for j in xrange(jm):
                    if lats[j] >= -60.0: break
                antarctic_land = [x for x in land_gridids if x <= row_end[j]]
                grids_of_interest.extend(antarctic_land)
                for gridid in grids_of_interest:
                    if gridid not in troubled_centers:
                            troubled_centers.append(gridid)
            
                if rf.model == "nra":
                    # The NRA seems to have issues with sub-sea level grids
                    # along the antarctic ice shelves so these are added to
                    # the troubled grids.
                    ice_shelf_tweeks = [365, 366, 367, 368, 369, 370, 371, 499, 500, 501,
                            502, 503, 504, 505, 506, 507, 508, 509, 510, 511,
                            512, 513, 514, 515, 641, 642, 643, 644, 645, 646,
                            647, 648, 649, 650, 651, 652, 653, 654, 655, 656,
                            657, 688, 689, 690, 691, 692, 693, 694, 695,
                            696, 697, 698, 699, 702, 703, 704, 705, 786, 787,
                            788, 789, 790, 791, 792, 793, 794, 795, 796, 797,
                            798, 799, 800, 801, 803, 804, 837, 838, 839, 840,
                            841, 842, 843, 844, 845, 846, 847, 848, 849, 850,
                            984, 1111, 1112, 1118, 1119, 1120, 1121, 1122, 1123,
                            1128, 1266, 1267, 1272, 1415, 1416, 1559, 1560, 1705]
                    for gridid in ice_shelf_tweeks:
                        if gridid not in troubled_centers:
                            troubled_centers.append(gridid)
            
            # Make sure no duplicates etc.
            troubled_centers = list(set(troubled_centers))
            troubled_centers.sort()
            
            if debug_trouble:
                pname = "%s%s_relief_ave.nc" % (rf.shared_path,rf.model)
                save_it = Save_NetCDF(relief_field,lons,plot_lats,pname,10000.0)
                pname = "%s%s_relief_max.nc" % (rf.shared_path,rf.model)    
                save_it = Save_NetCDF(max_relief,lons,plot_lats,pname,10000.0)
                pname = "%s%s_relief_min.nc" % (rf.shared_path,rf.model)                  
                save_it = Save_NetCDF(min_relief,lons,plot_lats,pname,10000.0)  
                pname = "%s%s_troubled.nc" % (rf.shared_path,rf.model) 
                troub_map = numpy.zeros(im*jm)
                troub_map[troubled_centers] = 1 
                #for x in troubled_centers: troub_map[x] = 1                             
                save_it = Save_NetCDF(troub_map,lons,plot_lats,pname,0.0)  

            save_topo = 0
            if save_topo:
                pname = "%s%s_topo.nc" % (rf.shared_path,rf.model)
                save_it = Save_NetCDF(topo,lons,plot_lats,pname,-10000.0)
            # Save for fast plotting below
            pname = "%s%s_tmp_topo.nc" % (rf.shared_path,rf.model)
            save_it = Save_NetCDF(topo,lons,plot_lats,pname,-10000.0)
                
        if rf.model.find("merran") != -1:
            # tmp problem with merra extra files
            lon_shift = lon_shift1

        # Store troubled_centers, land_gridids by hemisphere
        tmp = [[],[]]
        tmp[0] = [x for x in troubled_centers if x >= row_start[eq_grid]]
        tmp[1] = [x for x in troubled_centers if x <= row_start[eq_grid]]
        troubled_centers = tmp
        tmp = [[],[]]
        tmp[0] = [x for x in land_gridids if x >= row_start[eq_grid]]
        tmp[1] = [x for x in land_gridids if x <= row_start[eq_grid]]
        land_gridids = tmp

        # 
        # Setup up various screens that are used to limit operations to certain
        #   geographic locations for speed or appropriatness (e.g., smoothing).
        #

        # Limit center finding to places likely to host cyclones (for speed purposes)
        #
        # Exclude sub-Saharan Africa, the Arabian peninsula, and the Indian 
        #   subcontinent. Including the Red Sea, Persian Gulf and Indian Ocean as well.
        trop_cols_n = [i for i in xrange(im) if (330 <= lons[i]) or (lons[i] <= 103)]
        trop_oceans = [i for i in xrange(im) if (345 <= lons[i]) or (lons[i] <= 103)]
        # Add central-southern Mexico
        trop_cols_n_mex = [i for i in xrange(im) if 245 <= lons[i] <= 275]
        # Add parts of central South America
        trop_cols_s = [i for i in xrange(im) if 270 <= lons[i] <= 295]
        # Add northern Australia
        trop_cols_s_aus = [i for i in xrange(im) if 110 <= lons[i] <= 160]
        # Add northwestern South Africa 
        trop_cols_sa = [i for i in xrange(im) if 10 <= lons[i] <= 20]

        # Special screen for tropical land grids
        # N Hemi
        tropical_land_gridids = [[],[]]
        tropical_land_gridids[0] = [x for x in xrange(row_start[eq_grid],top_alt+1)
                if x in land_gridids[0] and x%im in trop_cols_n]
        # For Mexico don't extend as far poleward as elsewhere 
        tropical_land_gridids[0].extend([x for x in xrange(row_start[eq_grid],
            top_alt-int(5.0/dy)*im) if x in land_gridids[0]
            and x%im in trop_cols_n_mex])
        # Include the Pacific ocean along Central America too
        trow = (top//im)+int(5.0/dy)       
        tropical_land_gridids[0].extend([row*im + col for col in trop_cols_n_mex 
            for row in xrange(eq_grid,trow)]) 
        # Include the Red Sea, Persian Gulf and Indian Ocean as well 
        trow = top_alt//im
        tropical_land_gridids[0].extend([row*im + col for col in trop_oceans 
            for row in xrange(eq_grid,trow)]) 
        tropical_land_gridids[0] = list(set(tropical_land_gridids[0]))
        # S Hemi
        tropical_land_gridids[1] = [x for x in xrange(bot_alt,row_start[eq_grid])
                if x in land_gridids[1] and x%im in trop_cols_s]
        tropical_land_gridids[1].extend([x for x in xrange(bot_alt+int(5.0/dy)*im,bot)
            if x in land_gridids[1] and x%im in trop_cols_s_aus])
        tropical_land_gridids[1].extend([x for x in xrange(bot_alt,bot)
            if x in land_gridids[1] and x%im in trop_cols_sa])
        tropical_land_gridids[1] = list(set(tropical_land_gridids[1]))

        # Special Mask used to determine where smoothing or special sampling
        #   is done. Use troubled centers and add high latitude.
        smask = [[],[]]
        smask[0] = troubled_centers[0][:]
        smask[0].extend(range(row_start[(troubled_centers[0][-1]//im)-int(5.0/dy)],maxid))
        smask[0] = list(set(smask[0]))        
        smask[1] = troubled_centers[1][:]
        smask[1].extend(range(0,troubled_centers[1][0]+int(20.0/dy)*im))
        smask[1] = list(set(smask[1]))

    # ---------------------------------
    # For Center_Finder, and others
    # ---------------------------------
    
    # Purpose: Store all the setup stuff I just imported.
    # Returns:
    # Stores: s_dat.p

    print "\n\tStoring Setup Values..."
    if not skip_s_dat:
        fnc_out = (im,jm,maxid,lats,lons,timestep,dx,dy,dlon,dlat,start_lat,
            start_lon,dlon_sq,dlat_sq,two_dlat,model_flag,eq_grid,tropical_n,
            tropical_s,bot,mid,top,row_start,row_end,tropical_n_alt,tropical_s_alt,
            bot_alt,top_alt,lon_shift,lat_flip,the_calendar,found_years,super_years,
            dim_lat,dim_lon,dim_time,var_lat,var_lon,var_time,var_slp,var_topo,
            var_land_sea_mask,file_seperater,rf.no_topo,rf.no_mask,rf.slp_path,
            rf.model,rf.out_path,rf.shared_path,lat_edges,lon_edges,land_gridids,
            troubled_centers,faux_grids,polar_s,polar_n,pbot,ptop,eq_grid_on_edge,
            high_res,halfway,no_poles,polar,smask,tropical_land_gridids)
        # Save full version (includes land_gridids and troubled_centers)
        sf_file = "%ss_dat.pz" % (rf.shared_path)
        cPickle.dump(fnc_out,gzip.open(sf_file, "wb",-1))
        print "\t\tMade: %s\n" % (sf_file)
        # Write the API to file for later use
        sf_file_api = "%ss_dat.py" % (rf.shared_path)
        f = open(sf_file_api,'w')
        msg = """
inputs = ("im","jm","maxid","lats","lons","timestep","dx","dy","dlon",
    "dlat","start_lat","start_lon","dlon_sq","dlat_sq","two_dlat",
    "model_flag","eq_grid","tropical_n","tropical_s","bot","mid","top",
    "row_start","row_end","tropical_n_alt","tropical_s_alt","bot_alt",
    "top_alt","lon_shift","lat_flip","the_calendar","found_years",
    "super_years","dim_lat","dim_lon","dim_time","var_lat","var_lon",
    "var_time","var_slp","var_topo","var_land_sea_mask",
    "file_seperater","no_topo","no_mask","slp_path","model","out_path",
    "shared_path","lat_edges","lon_edges","land_gridids",
    "troubled_centers","faux_grids","polar_s","polar_n","pbot","ptop",
    "eq_grid_on_edge","high_res","halfway","no_poles","polar","smask",
    "tropical_land_gridids")
    """
        f.write(msg) 
        msg = """
#
# Example usage:
#
#   sf_file = "%ss_dat.pz" % (rf.shared_path)
#   fnc_out,inputs = InjestSetup(sf_file,0)        
#   im = fnc_out[inputs.index("im")]
#   jm = fnc_out[inputs.index("jm")]
#   maxid = fnc_out[inputs.index("maxid")]
#   lons = fnc_out[inputs.index("lons")]
#   lats = fnc_out[inputs.index("lats")]
#   timestep = fnc_out[inputs.index("timestep")]
#   dx = fnc_out[inputs.index("dx")]
#   dy = fnc_out[inputs.index("dy")]
#   dlon = fnc_out[inputs.index("dlon")]
#   dlat = fnc_out[inputs.index("dlat")]
#   start_lat = fnc_out[inputs.index("start_lat")]
#   start_lon = fnc_out[inputs.index("start_lon")]
#   dlon_sq = fnc_out[inputs.index("dlon_sq")]
#   dlat_sq = fnc_out[inputs.index("dlat_sq")]
#   two_dlat = fnc_out[inputs.index("two_dlat")]
#   model_flag = fnc_out[inputs.index("model_flag")]
#   eq_grid = fnc_out[inputs.index("eq_grid")]
#   tropical_n = fnc_out[inputs.index("tropical_n")]
#   tropical_s = fnc_out[inputs.index("tropical_s")]
#   bot = fnc_out[inputs.index("bot")]
#   mid = fnc_out[inputs.index("mid")]
#   top = fnc_out[inputs.index("top")]
#   row_start = fnc_out[inputs.index("row_start")]
#   row_end = fnc_out[inputs.index("row_end")]
#   tropical_n_alt = fnc_out[inputs.index("tropical_n_alt")]
#   tropical_s_alt = fnc_out[inputs.index("tropical_s_alt")]
#   bot_alt = fnc_out[inputs.index("bot_alt")]
#   top_alt = fnc_out[inputs.index("top_alt")]
#   lon_shift = fnc_out[inputs.index("lon_shift")]
#   lat_flip = fnc_out[inputs.index("lat_flip")]
#   the_calendar = fnc_out[inputs.index("the_calendar")]
#   found_years = fnc_out[inputs.index("found_years")]
#   super_years = fnc_out[inputs.index("super_years")]
#   dim_lat = fnc_out[inputs.index("dim_lat")]
#   dim_lon = fnc_out[inputs.index("dim_lon")]
#   dim_time = fnc_out[inputs.index("dim_time")]
#   var_lat = fnc_out[inputs.index("var_lat")]
#   var_lon = fnc_out[inputs.index("var_lon")]
#   var_time = fnc_out[inputs.index("var_time")]
#   var_slp = fnc_out[inputs.index("var_slp")]
#   var_topo = fnc_out[inputs.index("var_topo")]
#   var_land_sea_mask = fnc_out[inputs.index("var_land_sea_mask")]
#   file_seperater = fnc_out[inputs.index("file_seperater")]
#   no_topo = fnc_out[inputs.index("no_topo")]
#   no_mask = fnc_out[inputs.index("no_mask")]
#   slp_path = fnc_out[inputs.index("slp_path")]
#   model = fnc_out[inputs.index("model")]
#   out_path = fnc_out[inputs.index("out_path")]
#   shared_path = fnc_out[inputs.index("shared_path")]
#   lat_edges = fnc_out[inputs.index("lat_edges")]
#   lon_edges = fnc_out[inputs.index("lon_edges")]
#   land_gridids = fnc_out[inputs.index("land_gridids")]
#   troubled_centers = fnc_out[inputs.index("troubled_centers")]
#   faux_grids = fnc_out[inputs.index("faux_grids")]
#   polar_s = fnc_out[inputs.index("polar_s")]
#   polar_n = fnc_out[inputs.index("polar_n")]
#   pbot = fnc_out[inputs.index("pbot")]
#   ptop = fnc_out[inputs.index("ptop")]
#   eq_grid_on_edge = fnc_out[inputs.index("eq_grid_on_edge")]
#   row_start_index = row_start.index
#   high_res = fnc_out[inputs.index("high_res")]
#   halfway = fnc_out[inputs.index("halfway")]
#   no_poles = fnc_out[inputs.index("no_poles")] 
#   polar = fnc_out[inputs.index("polar")]
#   smask = fnc_out[inputs.index("smask")]
#   tropical_land_gridids = fnc_out[inputs.index("tropical_land_gridids")]
#   del fnc_out,inputs
"""
        f.write(msg)             
        f.close()
        # Save a "fast" version of s_dat with empty land_gridids, troubled_centers, smask,tropical_land_gridids
        fnc_out = (im,jm,maxid,lats,lons,timestep,dx,dy,dlon,dlat,start_lat,
            start_lon,dlon_sq,dlat_sq,two_dlat,model_flag,eq_grid,tropical_n,
            tropical_s,bot,mid,top,row_start,row_end,tropical_n_alt,tropical_s_alt,
            bot_alt,top_alt,lon_shift,lat_flip,the_calendar,found_years,super_years,
            dim_lat,dim_lon,dim_time,var_lat,var_lon,var_time,var_slp,var_topo,
            var_land_sea_mask,file_seperater,rf.no_topo,rf.no_mask,rf.slp_path,
            rf.model,rf.out_path,rf.shared_path,lat_edges,lon_edges,[],
            [],faux_grids,polar_s,polar_n,pbot,ptop,eq_grid_on_edge,high_res,
            halfway,no_poles,polar,[],[])
        # Save full version (includes land_gridids and troubled_centers)
        sf_file = "%ss_datf.pz" % (rf.shared_path)
        cPickle.dump(fnc_out,gzip.open(sf_file, "wb",-1))
        print "\t\tMade: %s\n" % (sf_file)
        # Write the API to file for later use
        sf_file_api = "%ss_datf.py" % (rf.shared_path)
        f = open(sf_file_api,'w')
        msg = """
inputs = ("im","jm","maxid","lats","lons","timestep","dx","dy","dlon",
    "dlat","start_lat","start_lon","dlon_sq","dlat_sq","two_dlat",
    "model_flag","eq_grid","tropical_n","tropical_s","bot","mid","top",
    "row_start","row_end","tropical_n_alt","tropical_s_alt","bot_alt",
    "top_alt","lon_shift","lat_flip","the_calendar","found_years",
    "super_years","dim_lat","dim_lon","dim_time","var_lat","var_lon",
    "var_time","var_slp","var_topo","var_land_sea_mask",
    "file_seperater","no_topo","no_mask","slp_path","model","out_path",
    "shared_path","lat_edges","lon_edges","land_gridids",
    "troubled_centers","faux_grids","polar_s","polar_n","pbot","ptop",
    "eq_grid_on_edge","high_res","halfway","no_poles","polar","smask",
    "tropical_land_gridids")
    """
        f.write(msg) 
        msg = """
#
# Example usage:
#
#   sf_file = "%ss_dat.pz" % (rf.shared_path)
#   fnc_out,inputs = InjestSetup(sf_file,0)        
#   im = fnc_out[inputs.index("im")]
#   jm = fnc_out[inputs.index("jm")]
#   maxid = fnc_out[inputs.index("maxid")]
#   lons = fnc_out[inputs.index("lons")]
#   lats = fnc_out[inputs.index("lats")]
#   timestep = fnc_out[inputs.index("timestep")]
#   dx = fnc_out[inputs.index("dx")]
#   dy = fnc_out[inputs.index("dy")]
#   dlon = fnc_out[inputs.index("dlon")]
#   dlat = fnc_out[inputs.index("dlat")]
#   start_lat = fnc_out[inputs.index("start_lat")]
#   start_lon = fnc_out[inputs.index("start_lon")]
#   dlon_sq = fnc_out[inputs.index("dlon_sq")]
#   dlat_sq = fnc_out[inputs.index("dlat_sq")]
#   two_dlat = fnc_out[inputs.index("two_dlat")]
#   model_flag = fnc_out[inputs.index("model_flag")]
#   eq_grid = fnc_out[inputs.index("eq_grid")]
#   tropical_n = fnc_out[inputs.index("tropical_n")]
#   tropical_s = fnc_out[inputs.index("tropical_s")]
#   bot = fnc_out[inputs.index("bot")]
#   mid = fnc_out[inputs.index("mid")]
#   top = fnc_out[inputs.index("top")]
#   row_start = fnc_out[inputs.index("row_start")]
#   row_end = fnc_out[inputs.index("row_end")]
#   tropical_n_alt = fnc_out[inputs.index("tropical_n_alt")]
#   tropical_s_alt = fnc_out[inputs.index("tropical_s_alt")]
#   bot_alt = fnc_out[inputs.index("bot_alt")]
#   top_alt = fnc_out[inputs.index("top_alt")]
#   lon_shift = fnc_out[inputs.index("lon_shift")]
#   lat_flip = fnc_out[inputs.index("lat_flip")]
#   the_calendar = fnc_out[inputs.index("the_calendar")]
#   found_years = fnc_out[inputs.index("found_years")]
#   super_years = fnc_out[inputs.index("super_years")]
#   dim_lat = fnc_out[inputs.index("dim_lat")]
#   dim_lon = fnc_out[inputs.index("dim_lon")]
#   dim_time = fnc_out[inputs.index("dim_time")]
#   var_lat = fnc_out[inputs.index("var_lat")]
#   var_lon = fnc_out[inputs.index("var_lon")]
#   var_time = fnc_out[inputs.index("var_time")]
#   var_slp = fnc_out[inputs.index("var_slp")]
#   var_topo = fnc_out[inputs.index("var_topo")]
#   var_land_sea_mask = fnc_out[inputs.index("var_land_sea_mask")]
#   file_seperater = fnc_out[inputs.index("file_seperater")]
#   no_topo = fnc_out[inputs.index("no_topo")]
#   no_mask = fnc_out[inputs.index("no_mask")]
#   slp_path = fnc_out[inputs.index("slp_path")]
#   model = fnc_out[inputs.index("model")]
#   out_path = fnc_out[inputs.index("out_path")]
#   shared_path = fnc_out[inputs.index("shared_path")]
#   lat_edges = fnc_out[inputs.index("lat_edges")]
#   lon_edges = fnc_out[inputs.index("lon_edges")]
#   land_gridids = fnc_out[inputs.index("land_gridids")]
#   troubled_centers = fnc_out[inputs.index("troubled_centers")]
#   faux_grids = fnc_out[inputs.index("faux_grids")]
#   polar_s = fnc_out[inputs.index("polar_s")]
#   polar_n = fnc_out[inputs.index("polar_n")]
#   pbot = fnc_out[inputs.index("pbot")]
#   ptop = fnc_out[inputs.index("ptop")]
#   eq_grid_on_edge = fnc_out[inputs.index("eq_grid_on_edge")]
#   row_start_index = row_start.index
#   high_res = fnc_out[inputs.index("high_res")]   
#   halfway = fnc_out[inputs.index("halfway")]        
#   no_poles = fnc_out[inputs.index("no_poles")] 
#   polar = fnc_out[inputs.index("polar")]
#   smask = fnc_out[inputs.index("smask")] 
#   tropical_land_gridids = fnc_out[inputs.index("tropical_land_gridids")]
#   del fnc_out,inputs
"""
        f.write(msg) 
        f.close()
        del fnc_out
    else:
        print ("\t\tSkipped s_dat")
    sf_file = "%ss_dat.pz" % (rf.shared_path)
    fnc_out,inputs = InjestSetup(sf_file,0)
    im = fnc_out[inputs.index("im")]
    jm = fnc_out[inputs.index("jm")]
    maxid = fnc_out[inputs.index("maxid")]
    lons = fnc_out[inputs.index("lons")]
    lats = fnc_out[inputs.index("lats")]
    timestep = fnc_out[inputs.index("timestep")]
    dx = fnc_out[inputs.index("dx")]
    dy = fnc_out[inputs.index("dy")]
    dlon = fnc_out[inputs.index("dlon")]
    dlat = fnc_out[inputs.index("dlat")]
    start_lat = fnc_out[inputs.index("start_lat")]
    start_lon = fnc_out[inputs.index("start_lon")]
    dlon_sq = fnc_out[inputs.index("dlon_sq")]
    dlat_sq = fnc_out[inputs.index("dlat_sq")]
    two_dlat = fnc_out[inputs.index("two_dlat")]
    model_flag = fnc_out[inputs.index("model_flag")]
    eq_grid = fnc_out[inputs.index("eq_grid")]
    tropical_n = fnc_out[inputs.index("tropical_n")]
    tropical_s = fnc_out[inputs.index("tropical_s")]
    bot = fnc_out[inputs.index("bot")]
    mid = fnc_out[inputs.index("mid")]
    top = fnc_out[inputs.index("top")]
    row_start = fnc_out[inputs.index("row_start")]
    row_end = fnc_out[inputs.index("row_end")]
    tropical_n_alt = fnc_out[inputs.index("tropical_n_alt")]
    tropical_s_alt = fnc_out[inputs.index("tropical_s_alt")]
    bot_alt = fnc_out[inputs.index("bot_alt")]
    top_alt = fnc_out[inputs.index("top_alt")]
    lon_shift = fnc_out[inputs.index("lon_shift")]
    lat_flip = fnc_out[inputs.index("lat_flip")]
    the_calendar = fnc_out[inputs.index("the_calendar")]
    found_years = fnc_out[inputs.index("found_years")]
    super_years = fnc_out[inputs.index("super_years")]
    dim_lat = fnc_out[inputs.index("dim_lat")]
    dim_lon = fnc_out[inputs.index("dim_lon")]
    dim_time = fnc_out[inputs.index("dim_time")]
    var_lat = fnc_out[inputs.index("var_lat")]
    var_lon = fnc_out[inputs.index("var_lon")]
    var_time = fnc_out[inputs.index("var_time")]
    var_slp = fnc_out[inputs.index("var_slp")]
    var_topo = fnc_out[inputs.index("var_topo")]
    var_land_sea_mask = fnc_out[inputs.index("var_land_sea_mask")]
    file_seperater = fnc_out[inputs.index("file_seperater")]
    no_topo = fnc_out[inputs.index("no_topo")]
    no_mask = fnc_out[inputs.index("no_mask")]
    slp_path = fnc_out[inputs.index("slp_path")]
    model = fnc_out[inputs.index("model")]
    out_path = fnc_out[inputs.index("out_path")]
    shared_path = fnc_out[inputs.index("shared_path")]
    lat_edges = fnc_out[inputs.index("lat_edges")]
    lon_edges = fnc_out[inputs.index("lon_edges")]
    land_gridids = fnc_out[inputs.index("land_gridids")]
    troubled_centers = fnc_out[inputs.index("troubled_centers")]
    faux_grids = fnc_out[inputs.index("faux_grids")]
    polar_s = fnc_out[inputs.index("polar_s")]
    polar_n = fnc_out[inputs.index("polar_n")]
    pbot = fnc_out[inputs.index("pbot")]
    ptop = fnc_out[inputs.index("ptop")]
    eq_grid_on_edge = fnc_out[inputs.index("eq_grid_on_edge")]
    row_start_index = row_start.index
    high_res = fnc_out[inputs.index("high_res")]
    halfway = fnc_out[inputs.index("halfway")]
    no_poles = fnc_out[inputs.index("no_poles")] 
    polar = fnc_out[inputs.index("polar")]
    smask = fnc_out[inputs.index("smask")]
    tropical_land_gridids = fnc_out[inputs.index("tropical_land_gridids")]
    # Drop the remainder
    del fnc_out,inputs

    if rf.save_plot and not (limit_plots and skip_s_dat):
        if skip_s_dat:
            # Read in specially saved topo data...
            pname = "%s%s_tmp_topo.nc" % (rf.shared_path,rf.model)
            topo = PullData(pname,0,"comp","",0,0,1,0)
        pname = "%s%s_topo%s" % (rf.shared_path,rf.model,rf.fig_format)
        plot = plotmap(clevs=[-500,4000,200],cints=[-500.0,4000.0],
            color_scheme='bone_r',lon_0=0.0)
        center_loc = []
        plot.create_fig()
        plot.add_field(lons,plot_lats,topo,ptype='pcolor',draw_lon_lat=False)
        plot.finish(pname,title="Topography")

        if smask:
            pname = "%s%s_smask%s" % (rf.shared_path,rf.model,rf.fig_format)
            plot = plotmap(missing=-1.0,color_scheme="gray_r",
                nocolorbar='nocolorbar',lon_0=0.0)
            plot.create_fig()
            blank = numpy.zeros(im*jm)
            tmp = smask[0][:]
            tmp.extend(smask[1])
            blank[tmp] = 1
            plot.add_field(lons,plot_lats,blank,ptype='pcolor',draw_lon_lat=False)                   
            title = "Smooth Mask (Excluded)"
            plot.finish(pname,title=title)
            print "\t\tMade figure %s" % (pname)
        
        if tropical_land_gridids:
            pname = "%s%s_tropical_land_mask%s" % (rf.shared_path,rf.model,rf.fig_format)
            plot = plotmap(missing=-1.0,color_scheme="gray_r",
                nocolorbar='nocolorbar',lon_0=0.0)
            plot.create_fig()
            blank = numpy.zeros(im*jm)
            tmp = tropical_land_gridids[0][:]
            tmp.extend(tropical_land_gridids[1])
            blank[tmp] = 1
            plot.add_field(lons,plot_lats,blank,ptype='pcolor',draw_lon_lat=False)
            title = "Tropical Land Mask"
            plot.finish(pname,title=title)
            print "\t\tMade figure %s" % (pname)

        if land_gridids:
            pname = "%s%s_land_mask%s" % (rf.shared_path,rf.model,rf.fig_format)
            plot = plotmap(missing=-1.0,color_scheme="gray_r",
                nocolorbar='nocolorbar',lon_0=0.0)
            plot.create_fig()
            blank = numpy.zeros(im*jm)
            tmp = land_gridids[0][:]
            tmp.extend(land_gridids[1])
            blank[tmp] = 1
            plot.add_field(lons,plot_lats,blank,ptype='pcolor',draw_lon_lat=False)                  
            title = "Land Mask"
            plot.finish(pname,title=title)
            print "\t\tMade figure %s" % (pname)
        
        if troubled_centers:
            pname = "%s%s_troubled_grids%s" % (rf.shared_path,rf.model,rf.fig_format)
            plot = plotmap(missing=-1.0,color_scheme="gray_r",
                nocolorbar='nocolorbar',lon_0=0.0)
            plot.create_fig()
            blank = numpy.zeros(im*jm)
            tmp = troubled_centers[0][:]
            tmp.extend(troubled_centers[1])
            blank[tmp] = 1
            plot.add_field(lons,plot_lats,blank,ptype='pcolor',draw_lon_lat=False)                                   
            title = "Troubled Grids"
            plot.finish(pname,title=title)
            print "\t\tMade figure %s" % (pname)

        print "\n\tTesting Pull_Data..."
        # Produce an example plot of the SLP field. Good idea to check
        # correctness with other software as this is how the data will
        # be read and used. Look for upside down or longitude shifts
        # in output.
        fname = super_lookup[super_uci_stamps[0]]
        tsteps = super_time_data[fname]['tsteps']
        date_stamps = super_time_data[fname]['date_stamps']
        tfile = "%s%s.nc" % (rf.slp_path,fname)
        slp,times,the_time_units = PullData(tfile,int(super_years[0]),
            var_slp,var_time,lat_flip,lon_shift,defs.read_scale)
        # Plot an example to see if okay.
        plot = plotmap(clevs=[960,1040,2],lon_0=0,
                cints=[960.0,1040.0],color_scheme="jet")
        for step in range(tsteps):
            msg = "State at %s UTC" % (date_stamps[step])
            pname = "%s%s_example_slp_%s%s" % (rf.shared_path,rf.model,
                    date_stamps[step].replace(" ",""),rf.fig_format)
            plot.create_fig()
            slp_step = slp[step,:,:].copy()
            slp_step.shape = jm*im
            plot.add_field(lons,plot_lats,slp_step,ptype='pcolor')
            #plot.add_field(lons,plot_lats,slp_step,ptype='contour',draw_lon_lat=False)
            plot.finish(pname,title=msg)
            print "\t\tMade figure: %s" % (pname)
            # only make a single plot, for more comment break
            break
        save_slp = 0
        if save_slp:
            pname = "%s%s_slp.nc" % (rf.shared_path,rf.model)
            save_it = Save_NetCDF(slp_step,lons,plot_lats,pname,-1.0)  
        del slp
    print "\n%s\n%s\n%s" % ("*"*80,'Setup Complete'.center(80,"*"),"*"*80)  

    return 0

#---Start of main code block.
if __name__=='__main__':
    import sys
    sys.exit( main(sys.argv) )

