#!/usr/bin/env python -tt
""" This module extracts cyclone tracks from input lists of centers
    from a series of sea level pressure fields. The is module contains
    the main logic of this process.

    Options/Arguments:
        defs_set -- directory of options.
        imports -- list of modules to import.
        start_tstep -- starting timestep.
        end_tstep -- ending timestep.
        out_path -- path to directory for storing results.
        center_file -- output of center_finder.
        dumped_file -- discards file from refine_centers and center_finder.
        shared_path -- path to directory storing info about the data source.
        slp_path -- path to the data.
        year_start -- first year of data.
        year_end -- last year of data.
        model -- designator of the source of the data.
        get_var -- name of the variable to be extracted from the data.

    Returns/Creates:
        tracks_save -- ASCII file of candidate tracks found and kept.
        dumped_centers_file -- ASCII file of candidate centers found and
                               discarded as untrackable.

    Examples:

    Notes: See docs/mcms_tracking_notes.txt

    Memory Use: Depending on resolution and options expect at least 300MB per
            instance and up to 2 GB.

    Run Time: With the 60 Year NCEP Reanalysis I takes roughly 1 hour and uses
              ~2 GB of memory. This is on a Mac Pro with 2.26 GHz processors,
              12 GB of RAM, and a 3 disk RAID.

    Author: Mike Bauer  <mbauer@giss.nasa.gov>

    Log:
        2006/12  MB - File created.
        lost track of dates of updates and fixes... sorry
        2008/10  MB - Added input checks, docstring.
        2010/06  MB - Updated to v5
        2012/06  MB - Update to initial release format
"""
from __future__ import division
# Alter python 2 division where division of two *integer* values gives an
#   *integer* result (i.e., 1/2 = 0 but 1.0/2 = 0.5 or 1/2.0 = 0.5) to the
#   more rational python 3+ version where all division returns floats
#   (i.e., 1/2 = 0.5). Adds a new integer division operator so that 1//2 = 0

# Standard library imports
import sys
import os

# Third party module imports

# MCMS module imports

# Globals

def find_tracks(tf_runfile):
    """Supervisor for the generation of statistics."""
    # --------------------------------------------------------------------------
    # Setup Section
    # --------------------------------------------------------------------------

    # Standard library imports
    import math
    #import cPickle
    import copy

    # Third party module imports
    import numpy

    # MCMS module imports
    from util import defs
    from util import InjestSetup
    from util import print_center
    from util import acquire_centers
    from util import find_dissimilarity_scores
    from util import refactor_tracks
    from util import track_reports_and_plots

#FIX work in the write_mcms.form_uci

    # Import run_file
    __import__(tf_runfile)
    tf = sys.modules[tf_runfile]

    # Conditional modules
    if tf.save_stats:
        from util import setup_track_stats

    # Pre-bind for speed
    #cos = math.cos
    sin = math.sin
    d2r = math.radians

    # Fetch definitions and impose those set in cf.defs_set.
    defs = defs(**tf.defs_set)

    # Get some definitions. Note must have run setup_vx.py already!
    if defs.troubled_filter:
        fnc_out,inputs = InjestSetup("%ss_dat.pz" % (tf.shared_path),0)
    else:
        fnc_out,inputs = InjestSetup("%ss_datf.pz" % (tf.shared_path),0)
    # Keep Wanted values
    im = fnc_out[inputs.index("im")]
    jm = fnc_out[inputs.index("jm")]
    maxid = fnc_out[inputs.index("maxid")]
    lons = fnc_out[inputs.index("lons")]
    lats = fnc_out[inputs.index("lats")]
    timestep = fnc_out[inputs.index("timestep")]
    dx = fnc_out[inputs.index("dx")]
    dy = fnc_out[inputs.index("dy")]
    dlon = fnc_out[inputs.index("dlon")]
    dlat = fnc_out[inputs.index("dlat")]
    start_lat = fnc_out[inputs.index("start_lat")]
    start_lon = fnc_out[inputs.index("start_lon")]
    dlon_sq = fnc_out[inputs.index("dlon_sq")]
    dlat_sq = fnc_out[inputs.index("dlat_sq")]
    two_dlat = fnc_out[inputs.index("two_dlat")]
    eq_grid = fnc_out[inputs.index("eq_grid")]
    tropical_n = fnc_out[inputs.index("tropical_n")]
    tropical_s = fnc_out[inputs.index("tropical_s")]
    bot = fnc_out[inputs.index("bot")]
    mid = fnc_out[inputs.index("mid")]
    top = fnc_out[inputs.index("top")]
    row_start = fnc_out[inputs.index("row_start")]
    row_end = fnc_out[inputs.index("row_end")]
    tropical_n_alt = fnc_out[inputs.index("tropical_n_alt")]
    tropical_s_alt = fnc_out[inputs.index("tropical_s_alt")]
    bot_alt = fnc_out[inputs.index("bot_alt")]
    top_alt = fnc_out[inputs.index("top_alt")]
    lon_shift = fnc_out[inputs.index("lon_shift")]
    lat_flip = fnc_out[inputs.index("lat_flip")]
    the_calendar = fnc_out[inputs.index("the_calendar")]
    found_years = fnc_out[inputs.index("found_years")]
    dim_lat = fnc_out[inputs.index("dim_lat")]
    dim_lon = fnc_out[inputs.index("dim_lon")]
    dim_time = fnc_out[inputs.index("dim_time")]
    var_lat = fnc_out[inputs.index("var_lat")]
    var_lon = fnc_out[inputs.index("var_lon")]
    var_time = fnc_out[inputs.index("var_time")]
    var_slp = fnc_out[inputs.index("var_slp")]
    var_topo = fnc_out[inputs.index("var_topo")]
    var_land_sea_mask = fnc_out[inputs.index("var_land_sea_mask")]
    file_seperater = fnc_out[inputs.index("file_seperater")]
    no_topo = fnc_out[inputs.index("no_topo")]
    no_mask = fnc_out[inputs.index("no_mask")]
    pbot = fnc_out[inputs.index("pbot")]
    ptop = fnc_out[inputs.index("ptop")]
    faux_grids = fnc_out[inputs.index("faux_grids")]
    eq_grid_on_edge = fnc_out[inputs.index("eq_grid_on_edge")]
    row_start_index = row_start.index
    halfway = fnc_out[inputs.index("halfway")]
    high_res = fnc_out[inputs.index("high_res")]
    if faux_grids and (tf.save_plots or tf.save_stats):
        # Reset polar most latitudes to poles for plotting
        lats[0] = -90.0
        lats[-1] = 90.0
    if defs.troubled_filter:
        troubled_centers = fnc_out[inputs.index("troubled_centers")]
        # Remove hemisphere difference
        tmp = troubled_centers[0]
        tmp.extend(troubled_centers[1])
        troubled_centers = tmp
        if tf.use_hotspot:
            troubled_centers.extend([x for x in tf.screen_hot_spots
                if x not in troubled_centers])
    else:
        troubled_centers = []
    # Drop the remainder
    del fnc_out,inputs
    if not hasattr(tf, 'timestep'):
        tf.timestep = timestep
    tf.im = im
    tf.jm = jm
    tf.pbot = pbot
    tf.ptop = ptop
    tf.halfway = halfway
    tf.nseasons = len(tf.seasons)
    tf.lons = lons
    tf.lats = lats
    tf.land_gridids = []
    tf.maxid = maxid
    tf.tropical_end = row_end[tropical_n_alt]
    tf.tropical_start = row_start[tropical_s_alt]
    if not hasattr(tf, 'high_res'):
        tf.high_res = high_res

    # Get some definitions. Note must have run setup_vx.py already!
    tdict,inputs = InjestSetup("%stf_dat.pz" % (tf.shared_path),2)

    # Define some things
    tf.inv_accuracy = 1.0/defs.accuracy
    tf.maxdp = defs.maxdp*timestep
    tf.travel_distance = defs.travel_distance*timestep
    tf.might_be_long = int((24*7)/timestep)

    # Shortest allowable track length (based on time elapsed between pnts, such
    # that 1 timestep spans 2 pnts).
    tf.min_trk_cnt = int(defs.age_limit/timestep) + 1

    # Gives a bergero of 1 at 6 hPa per timestep at 60 degrees
    tf.sin60 = math.sin(d2r(60.0))/float(timestep)

    # Skip high latitude for angle, separation, area calculations
    #   If non-zero the value is the abs latitude above which no values retained.
    #skip_high_lat = 85
    tf.skip_high_lat = 89

    # Limit bearing stats and StheC to centers with at least this must separation
    #   as likely to be large angle changes when centers are say 10 km apart due
    #   to a stalled/blocked cyclone.
    #min_sep = 222.0
    tf.min_sep = 111.0

    # Summary Stats: Checks on operation to report and catch oddities.
    tf.known_flags = {5 : "Failed trackable center",
                   6 : "Failed track lifetime filter",
                   7 : "Failed track travel filter",
                   8 : "Failed track minimum SLP filter",
                   10: "Failed extratropical track filter"}
    tf.flag_files = {0 : "passed",
                  5 : "trackable",
                  6 : "lifetime",
                  7 : "travel",
                  8 : "minslp",
                  10 : "tropical"}
    tf.nflags = 11
    tf.total_time_steps = 0
    tf.flags_used = [5,6,7,8,10] # used flags
    tf.super_total_centers_read = 0

    track_stats_inputs = track_stats = ""
    if tf.save_stats:
        track_stats_inputs,track_stats = setup_track_stats(tf,troubled_centers)

    #---------------------------------------------------------------------------
    # Read all centers into memory
    #---------------------------------------------------------------------------
    centers,tf.ids = acquire_centers(tf)

    tf.super_total_centers_read = len(centers)

    readin_centers = []
    if tf.save_stats:
        readin_centers = track_stats[track_stats_inputs.index("readin_centers")]
        for center in centers:
            readin_centers[center[tf.ids['GridID']]] += 1

    #---------------------------------------------------------------------------
    # Finding Dissimilarity Scores
    #---------------------------------------------------------------------------
    dead_tracks,hits = find_dissimilarity_scores(tf,defs,centers,tdict,
            track_stats_inputs,track_stats)
    if dead_tracks == 1:
        return 1

    #--------------------------------------------------------------------------
    # Refactor Tracks (update usi and apply post-tracking filters
    #---------------------------------------------------------------------------
    fnc_out = refactor_tracks(tf,defs,dead_tracks,hits,troubled_centers)

    #-------------------------------------------------------------------------
    # Making Final Report
    #-------------------------------------------------------------------------
    if tf.save_stats == 0:
        return
    track_reports_and_plots(tf,fnc_out,readin_centers)

    return

def main(argv):

    # MCMS module imports
    from extras import ProgressBarAlt
    #from util import PrintColumns
    from util import FileExists
    from util import ParseRunFile
    from util import rewrite

    # --------------------------------------------------------------------------
    # Parse Request
    # --------------------------------------------------------------------------

    # Define the base run_file template
    here_path = os.path.abspath(os.path.dirname(__file__))
    run_file_template = "%s/cfg/mcms_track_finder_template.py" % (here_path)
    if not FileExists(run_file_template,__name__): return 1

    # --------------------------------------------------------------------------
    # Parse Request
    # --------------------------------------------------------------------------
    rf = ParseRunFile(run_file_template,argv,here_path)
    if rf == 1:
        return rf

    # --------------------------------------------------------------------------
    # Execute the request
    # --------------------------------------------------------------------------
    find_tracks(rf.getthis)

    if rf.save_stats == 2:
        return

    # Rewrite tracks
    print "\tRe-writing yearly files..."
    rewrite(rf.tracks_file,rf.years)

    # Append dumped
    if FileExists(rf.dumped_centers_file,__name__,0):
        rewrite(rf.dumped_centers_file,rf.years,action="a",reorder=True)

    if rf.tempFile:
        # Ensure temp files removed
        if FileExists(rf.tempFile,__name__,0):
            os.remove(rf.tempFile)

#---Start of main code block
if __name__=='__main__':
    sys.exit( main(sys.argv) )
