#!/usr/bin/env python -tt
"""
Default run_file template for mcms_track_finder

Input:
^^^^^
None

Output:
^^^^^^
Various python data structures.

Examples:
^^^^^^^^

Notes/Warnings:
^^^^^^^^^^^^^^

Author: Mike Bauer  <mbauer@giss.nasa.gov>

Log:
2011/01 MB - File created
"""

# Standard library imports
import sys
import os
import fnmatch 

# MCMS module imports: Need to have mcms in your PYTHONPATH environment variable!
from cfg import define_source
from cfg import define_figure
from cfg import define_dirs
from cfg import define_defs
from cfg import define_vars
from cfg import ensure_path
from cfg import define_season
from cfg import define_m2s
from util import FileExists
from util import organize_data
from util import query_yes_no
from util import InjestSetup
from util import Is_It_Available

# Note: the following comment precedes all options that MUST be set by the user
# REQUIRED

# Note: the following string MUST be replaced by the user
# REPLACE

# ----------------------------------------------------------------------------
#                               Main Settings Set
# ----------------------------------------------------------------------------
verbose = 0

dotline = "%s" % ("-"*40)
msg = '\n%s\n%40s\n%s\n' % (dotline,"Main Settings Set",dotline)
if verbose: print msg

# ----------------------------------------------------------------------------
# Pick data source see files in /cfg
#   See cfg/mcms_source_base_defs.py
# ----------------------------------------------------------------------------
pick_source = define_source()
msg = pick_source.make_pick("ccsm")
if verbose: print msg
model = pick_source.selection
model_def = pick_source.selections[model][0]

# ----------------------------------------------------------------------------
# Directory to read temporary pick specific files.
#   Because the setup files make by mcms_setup.py are stored in this directory
#   this has to be manually entered.
# ----------------------------------------------------------------------------    
shared_path = "/Users/mbauer/Scratchf/output/%s_files/" % model
shared_path = ensure_path("",shared_path)
msg = "SHARED PATH: %s"
if verbose: print msg % shared_path

# ----------------------------------------------------------------------------
# Overwrite the output_path defined in mcms_setup.py
# ----------------------------------------------------------------------------
ensemble = ""
experiments = {"e12k" : "PSL-12k",
            "e14k" : "PSL-14k"}
blobs = {"e12k" : "cam_b30.12_2kaDVTn_T85.cam2.h1",
         "e14k" : 'cam_b30.14_37kaDVT_T85.cam2.h1'}
key = 'e14k'
experiment = experiments[key]
blob = blobs[key]
if ensemble:
    subdir = "%s/%s/" % (experiment,ensemble)
else:
    if experiment:
        subdir = "%s/" % (experiment)
    else:
        subdir = ""
over_write_out_path = "/Users/mbauer/Scratchf/output/%s/%s" % (model,subdir)
#over_write_out_path = ""
if over_write_out_path:
    over_write_out_path = ensure_path(subdir,over_write_out_path,check_exists=0)

over_write_slp_path = "/Users/mbauer/Scratchf/%s/%s" % (model,subdir)
over_write_slp_path = ""
if over_write_slp_path:
    over_write_slp_path = ensure_path(subdir,over_write_slp_path)

# --------------------------------------------------------------------------
# Define seasons used to collect tracking statistics
# --------------------------------------------------------------------------
pick_season = define_season()
msg = pick_season.make_pick('3mo')
if verbose: print msg
seasons = pick_season.selection

pick_m2s = define_m2s()
msg = pick_m2s.make_pick('3mo')
if verbose: print msg
m2s = pick_m2s.selection

# --------------------------------------------------------------------------
# Names associated with the netcdf files for the source 
#   See cfg/mcms_ncvars_base_defs.py
# --------------------------------------------------------------------------
source_vars = define_vars()
msg = source_vars.make_pick(model)
if verbose: print msg
nc_vars = source_vars.selection
break_fname = nc_vars['break_fname']
month_sort_order = nc_vars['month_sort_order']
the_calendar = nc_vars['calendar']
jd_fake = nc_vars['jd_fake']
get_time_dat = "time_dat" 

# Get some definitions. Note must have run mcms_setup.py already!
fnc_out,inputs = InjestSetup("%ss_datf.pz" % (shared_path),0)
msg = "OUTPUT PATH (Modified): %s"
msg2 = "SLP PATH (Modified): %s"
if not over_write_out_path:
    over_write_out_path = fnc_out[inputs.index("out_path")]
    msg = "OUTPUT PATH (Default): %s"
if not over_write_slp_path:
    over_write_slp_path = fnc_out[inputs.index("slp_path")]
    msg2 = "SLP PATH (Default): %s"
out_path = over_write_out_path
slp_path = over_write_slp_path
ensure_path(subdir,out_path,check_exists=0)
ensure_path(subdir,slp_path,check_exists=1)
if verbose: print msg % out_path
if verbose: print msg2 % slp_path
# Create needed directories if they don't exist.
pick_dirs = define_dirs()
msg = pick_dirs.make_pick("standard")
dirs_set = pick_dirs.selection
need_dirs = ["%s%s" % (out_path,x) for x in dirs_set]
need_dirs.append("%s%s" % (out_path,"figs/maps/global/"))
need_dirs.extend(["%sfigs/maps/global/%s" % (out_path,s) for s in seasons])
need_dirs.append("%s%s" % (out_path,"figs/pdfs/global/"))
need_dirs.extend(["%sfigs/pdfs/global/%s" % (out_path,s) for s in seasons])
for ndir in need_dirs:
    while ndir.find("//") != -1:
        ndir = ndir.replace("//","/") 
    if not os.path.exists(ndir):
        try:
            os.mkdir(ndir)
            if verbose:
                print "\tMade: %s" % ndir
        except:
            sys.exit("ERROR: creating %s" % ndir)

# --------------------------------------------------------------------------
# Read in the date/time info for these files
# --------------------------------------------------------------------------
# Name for new time_dat file
if ensemble:
    get_time_dat  = "time_dat_%s_%s" % (experiment,ensemble)
else:
    if experiment:
        get_time_dat = "time_dat_%s" % (experiment)
    else:
        get_time_dat = "time_dat" 
# Test to see if already get_time_dat defined
gf = shared_path+get_time_dat+".pz"
if not FileExists(gf,__name__,verbose=False):
    msg = organize_data(slp_path,break_fname,nc_vars['var_slp'],
            month_sort_order,the_calendar,nc_vars['var_time'],jd_fake,
            shared_path,blob,base_name=get_time_dat,verbose=0)    
    print msg 
fnc_out,inputs = InjestSetup(gf,4)
super_time_data = fnc_out[inputs.index("super_time_data")]

# The default behavior is to run over all the
# years found by mcms_setup.py. Here you can
# elect to override this behavior.
over_write_years = []  
#over_write_years = [539,539]
msg = "SOURCE YEARS (Modified): %s"
if not over_write_years:
    over_write_years = fnc_out[inputs.index("super_years")]
    over_write_years = [int(over_write_years[0]),int(over_write_years[1])]
    msg = "SOURCE YEARS (Default): %s"
super_years = over_write_years
if verbose: print msg % repr(super_years)
start_year = int(super_years[0])
end_year = int(super_years[1])
years = range(start_year,end_year+1)

fnames = super_time_data.keys()
if month_sort_order:
    fnames = sorted(fnames,key = lambda fnames: (break_fname(fnames,blob)[0],
                            month_sort_order[break_fname(fnames,blob)[1]]))
else:
    # Annual Files or numerical sort order
    fnames = sorted(fnames,key = lambda fnames: (break_fname(fnames,blob)[0],
                            break_fname(fnames,blob)[1]))
uci_stamps = []
tsteps = 0
for fn in fnames:
    f_year =  break_fname(fn,blob)[0]
    if f_year in years:
        uci_stamps.extend(super_time_data[fn]['uci_stamps'])
        tsteps += super_time_data[fn]['tsteps']
del super_time_data
del fnc_out
uci_stamps = sorted(uci_stamps)
# Very first tstep is not SLP!
if uci_stamps[0] == "%04d010100" % start_year:
    uci_stamps = uci_stamps[1:]
    tsteps -= 1

# --------------------------------------------------------------------------
# Alter default behavior found in mcms_defs.py
# --------------------------------------------------------------------------
# Pick data source see defs files in /cfg
pick_defs = define_defs()
msg = pick_defs.make_pick(model)
if verbose: print msg
defs_set = pick_defs.selection

# --------------------------------------------------------------------------
# Define some files and important values
# --------------------------------------------------------------------------
clim_tag = "%04d_%04d" % (start_year,end_year)
header = "mcms_%s_%s_" % (model,clim_tag)
centers_file = "%s%scenters.txt" % (out_path,header)
dumped_centers_file = "%s%sdumped_centers.txt" % (out_path,header)
tracks_file = centers_file.replace("centers","tracks")
if verbose:
    print "CENTERS_FILE: %s" % centers_file
    print "DUMPED_CENTERS_FILE: %s" % dumped_centers_file
    print "TRACKS_FILE: %s" % tracks_file
# ----------------------------------------------------------------------------
#                           Extra Settings Set
# ----------------------------------------------------------------------------
msg = '\n%s\n%40s\n%s\n' % (dotline,"Extra Settings Set",dotline)
if verbose: print msg

# What sort of figures (png faster, eps/pdf better quality)
pick_figure = define_figure()
msg = pick_figure.make_pick("png")
if verbose: print msg
fig_format  = pick_figure.selection

# Store tracking stats and make plots
# save_stats = 0 save tracks to file, no stats or plots
#               1 save tracks to file, make stats and plots
#               2 to not save tracks to file... just stats and plots
save_stats = 1
if save_stats == 2:
    print "\t\tWarning not Saving Tracks to File!"
msg = 'save_stats = %d' % save_stats
if verbose: print msg

# Save track finding plots (requires matplotlib, also doubles or more memory
#   footprint). Need this on to save filter specfic netcdf files.
#       save_plots = 0 skip
#                 = 1 plots and netcdf
#                 = 2 just netcdfs
save_plots = 2
msg = 'save_plots = %d' % save_plots
if verbose: print msg

# Halt program on error or just warn?
exit_on_error = 1
msg = 'exit_on_error = %d' % exit_on_error
if verbose: print msg

#
# Override values from mcms_setup.py
#

# Match RAIBLE et. all 2008
#defs_set['max_cyclone_speed']= 42.0
#defs_set['age_limit'] = 72.0
high_res = 1

# ----------------------------------------------------------------------------
# Check for graphics programs
#   The psrose plotting program from the GMT (The Generic Mapping Tools,
#   http://gmt.soest.hawaii.edu/) project provides a much better wind
#   rose plot than that from the matplotlib plotting package that is generally
#   used by the MCMS project.
# ----------------------------------------------------------------------------
psrose = Is_It_Available("psrose")
if not psrose:
    msg = "\n\t\tWarning: GMT (The Generic Mapping Tools) not found. "
    msg += "Some plots will not be made."
    print msg
    try:
        import matplotlib
    except ImportError:
        print "\n\t\tWarning: matplotlib not available. Disabling plotting altogether."
        save_plots = 0

