#!/usr/bin/env python -tt
"""
Default run_file template for mcms_center_finder

Input:
^^^^^
None

Output:
^^^^^^
Various python data structures.

Examples:
^^^^^^^^

Notes/Warnings:
^^^^^^^^^^^^^^

Author: Mike Bauer  <mbauer@giss.nasa.gov>

Log:
2011/01 MB - File created
"""

# Standard library imports
import sys
import os
import fnmatch

# MCMS module imports: Need to have mcms in your PYTHONPATH environment variable!
from cfg import define_source
from cfg import define_figure
from cfg import define_dirs
from cfg import define_defs
from cfg import define_vars
from cfg import ensure_path
from util import FileExists
from util import organize_data
from util import query_yes_no
from util import InjestSetup
import output_root
import data_root

# ----------------------------------------------------------------------------
#                               Main Settings Set
# ----------------------------------------------------------------------------
verbose = 0

dotline = "%s" % ("-"*40)
msg = '\n%s\n%40s\n%s\n' % (dotline,"Main Settings Set",dotline)
if verbose: print msg

# ----------------------------------------------------------------------------
# Pick data source see files in /cfg
#   See cfg/mcms_source_base_defs.py
# ----------------------------------------------------------------------------
pick_source = define_source()
msg = pick_source.make_pick("merrac")
if verbose: print msg
model = pick_source.selection
model_def = pick_source.selections[model][0]

# Set the base directories
if hasattr(output_root,'%s_root' % model):
    oroot = getattr(output_root,'%s_root' % model)
else:
    oroot = output_root.root
if hasattr(data_root,'%s_root' % model):
    droot = getattr(data_root,'%s_root' % model)
else:
    droot = data_root.root

# ----------------------------------------------------------------------------
# Full path to the root directory where pick specific output will be stored.
#   If multiple data sets under same model (say GCM runs) add a subdirectory
#   to the model root. Can be worked with ensembles and experiments withint the
#   same model.# ----------------------------------------------------------------------------
#
#ensembles = {"control_run" : [""],
#    "Scenario_A1B_run" : ["E3abaoM20","E3abboM20","E3abcoM20","E3abdoM20","E3abeoM20"],
#    "Scenario_A1B_iceMelt_run" : ["E3FIBaoM20","E3FIBboM20","E3FIBcoM20","E3FIBdoM20","E3FIBeoM20"]}
#experiment = "Scenario_A1B_iceMelt_run"
#ensemble = ensembles[experiment][0]
ensemble = ""
experiment = ""
blob = ""
if ensemble:
    subdir = "%s/%s/" % (experiment,ensemble)
else:
    if experiment:
        subdir = "%s/" % (experiment)
    else:
        subdir = ""

# Full path to the root directory where pick specific output will be stored.
#   This must be provided in order to set shared path. Note this can be over
#   rode with over_write_out_path for storing the output while the shared_path
#   defined with out_path here is preserved.
out_path = "%s%s%s/" % (oroot,model,subdir)
out_path = ensure_path(subdir,out_path,check_exists=0)

# ----------------------------------------------------------------------------
# Full path to the directory where the SLP data files are stored.
#    The default behavior is to read SLP data from the directory slp_path
#    defined in mcms_setup.py. Here you can elect to override this behavior.
# ----------------------------------------------------------------------------
slp_path = "%s%s/%s" % (droot,model,subdir)
slp_path = ensure_path(subdir,slp_path)

# ----------------------------------------------------------------------------
# Directory to be created for storing temporary pick specific files.
#   Because the setup files make by mcms_setup.py are stored in this directory
#   this has to be manually entered.
# ----------------------------------------------------------------------------
if subdir:
    shared_path = "%s%s_files/" % (oroot,model)
else:
    shared_path = "%s_files/" % (out_path.replace("%s/" % model,model))
shared_path = ensure_path(subdir,shared_path)
msg = "SHARED PATH: %s"
if verbose: print msg % shared_path

# --------------------------------------------------------------------------
# Names associated with the netcdf files for the source
#   See cfg/mcms_ncvars_base_defs.py
# --------------------------------------------------------------------------
source_vars = define_vars()
msg = source_vars.make_pick(model)
if verbose: print msg
nc_vars = source_vars.selection
break_fname = nc_vars['break_fname']
month_sort_order = nc_vars['month_sort_order']
the_calendar = nc_vars['calendar']
jd_fake = nc_vars['jd_fake']
get_time_dat = "time_dat"

# --------------------------------------------------------------------------
#  Settings to modify or speedup the execution from the default.
# --------------------------------------------------------------------------
# Fast way to do a single year ... by splitting the year into pieces &
#   work on each subset with a processor. This is generally for debugging
#   or testing something.
fast_run = 0
fast_year = 2009
if verbose and fast_run: print "Fast run mode for %d" % fast_year
# For Merrac skip these hours (rather than every 3). Again for speed
#   or to match other datasets.
#skip_hours = ["03","09","15","21"]
skip_hours = ["00","06","12","18"]

# Fast way to run a climatology, but by disabling the tracking and creation
#   of standard diagnostics (and also saving discarded centers) the execution
#   time is greatly reduced.
express = 1
if verbose and express: print "Express mode in use."
# The default behavior is to run over all the years found by mcms_setup.py.
# Here you can elect to override this behavior.
over_write_years = []
#over_write_years = [2007,2011]
#over_write_years = [2011,2011]
#over_write_years = [1979,1989]
if fast_run:
    over_write_years = [fast_year,fast_year]

# Get some definitions. Note must have run mcms_setup.py already!
fnc_out,inputs = InjestSetup("%ss_datf.pz" % (shared_path),0)
msg = "OUTPUT PATH (Default): %s"
if out_path != fnc_out[inputs.index("out_path")]:
    msg = "OUTPUT PATH (Modified): %s"
if verbose: print msg % out_path
# Create needed directories if they don't exist.
pick_dirs = define_dirs()
msg = pick_dirs.make_pick("standard")
dirs_set = pick_dirs.selection
need_dirs = ["%s%s" % (out_path,x) for x in dirs_set]
need_dirs.append("%s%s" % (out_path,"figs/maps/global/"))
need_dirs.append("%s%s" % (out_path,"figs/maps/global/annual/"))
need_dirs.append("%s%s" % (out_path,"figs/pdfs/global/"))
need_dirs.append("%s%s" % (out_path,"figs/pdfs/global/annual/"))
for ndir in need_dirs:
    if not os.path.exists(ndir):
        try:
            os.mkdir(ndir)
            if verbose:
                print "\tMade: %s" % ndir
        except:
            sys.exit("ERROR: creating %s" % ndir)
msg = "SLP PATH (Default): %s"
check_time = 0
if slp_path != fnc_out[inputs.index("slp_path")]:
    msg = "SLP PATH (Modified): %s"
    check_time = 1
if not os.path.exists(slp_path):
    sys.exit("ERROR: slp_path not found.")
if verbose: print msg % slp_path
msg = "SOURCE YEARS (Modified): %s"
if not over_write_years:
    over_write_years = fnc_out[inputs.index("super_years")]
    over_write_years = [int(over_write_years[0]),int(over_write_years[1])]
    msg = "SOURCE YEARS (Default): %s"
super_years = over_write_years
if verbose: print msg % repr(super_years)
if fast_run:
    tail = "_centers.txt"
    just_center_table = 1
    detail_tracks = ""
    include_att = 0
    include_atts = 0
    row_end = fnc_out[inputs.index("row_end")]
    row_start = fnc_out[inputs.index("row_start")]
    tropical_n_alt = fnc_out[inputs.index("tropical_n_alt")]
    tropical_s_alt = fnc_out[inputs.index("tropical_s_alt")]
    tropical_end = row_end[tropical_n_alt]
    tropical_start = row_start[tropical_s_alt]
    maxid = fnc_out[inputs.index("maxid")]
    land_gridids = [] #fnc_out[inputs.index("land_gridids")]
    flons = fnc_out[inputs.index("lons")]
    flats = fnc_out[inputs.index("lats")]
del fnc_out

# If command line python mcms_xxx.py rf_file.py -s
#   suppress check_time
for file in os.listdir('.'):
    if fnmatch.fnmatch(file, 'skip*'):
        check_time = 0
        os.remove(file)
if check_time:
    # Evoke mcms_organize_data.py because the time_dat.p file created by
    #   mcms_setup.py may not correctly represent the data

    # If command line python mcms_xxx.py rf_file.py -n
    #   suppress query_yes_no for time_dat
    no_ask = 0
    for file in os.listdir('.'):
        if fnmatch.fnmatch(file, 'no_ask*'):
            no_ask = 1
            os.remove(file)

    # Name for new time_dat file
    if ensemble:
        get_time_dat = "time_dat_%s_%s" % (experiment,ensemble)
    else:
        if experiment:
            get_time_dat = "time_dat_%s" % (experiment)
        else:
            get_time_dat = "time_dat"

    # Test to see if already get_time_dat defined
    gf = shared_path+get_time_dat+".pz"
    over = 1
    if FileExists(gf,__name__,verbose=False) and not no_ask:
        # Warn about overwriting
        msg = "\n\tWarning: Overwrite [%s]?"
        print msg % gf
        answer = query_yes_no("\t\tContinue?")
        if not answer: over = 0
    if over:
        # Run mcms_organize_data... create time_dat.p

        ## Names associated with the netcdf files for the source
        #source_vars = define_vars()
        #msg = source_vars.make_pick(model)
        #nc_vars = source_vars.selection

        # Assume that mcms_ncvars_base_defs.py correctly defines
        #   file_seperater
        #   month_sort_order
        #   break_fname
        #   calendar
        #   jd_fake
        # If not put the new ones here and pass along
        #break_fname = nc_vars['break_fname']
        #month_sort_order = nc_vars['month_sort_order']
        #the_calendar = nc_vars['calendar']
        #jd_fake = nc_vars['jd_fake']
        msg = organize_data(slp_path,break_fname,nc_vars['var_slp'],
                month_sort_order,the_calendar,nc_vars['var_time'],jd_fake,
                shared_path,blob,base_name=get_time_dat,verbose=0)
        print msg

# --------------------------------------------------------------------------
# Alter default behavior found in mcms_defs.py
# --------------------------------------------------------------------------
# Pick data source see defs files in /cfg
pick_defs = define_defs()
msg = pick_defs.make_pick(model)
if verbose: print msg
defs_set = pick_defs.selection

# --------------------------------------------------------------------------
# Define some files and important values
# --------------------------------------------------------------------------
centers_file = "centers.txt"
dumped_centers_file = "dumped_centers.txt"
start_year = int(super_years[0])
end_year = int(super_years[1])
nyears = end_year-start_year+1
if verbose:
    print "CENTERS_FILE: %s" % centers_file
    print "DUMPED_CENTERS_FILE: %s" % dumped_centers_file

# ----------------------------------------------------------------------------
#                           Extra Settings Set
# ----------------------------------------------------------------------------
msg = '\n%s\n%40s\n%s\n' % (dotline,"Extra Settings Set",dotline)
if verbose: print msg

# What sort of figures (png faster, eps/pdf better quality)
pick_figure = define_figure()
msg = pick_figure.make_pick("png")
#msg = pick_figure.make_pick("pdf")
if verbose: print msg
fig_format  = pick_figure.selection

# Set for debugging as multiprocessing doesn't always report why
#   a program hangs.
single_processor = 0
if fast_run:
    single_processor = 0
msg = "Warning Running Single Processor Mode!"
if verbose and single_processor: print msg

# Manually set the number of processors for multiprocessing.
processor_cnt = 0
if verbose and processors: print "Manual processor count: %d" % processor_cnt

# Save debugging data for post analysis
save_stats = 1
if express:
   save_stats = 0
if fast_run:
    save_stats = 0
msg = 'save_stats = %d' % save_stats
if verbose: print msg

# Save center finding plots (requires matplotlib, also doubles or more memory
#   footprint)
save_plots = 1
if fast_run:
    save_plots = 0
if express:
   save_plots = 0
msg = 'save_plots = %d' % save_plots
if verbose: print msg

# Halt program on error or just warn?
exit_on_error = 1
msg = 'exit_on_error = %d' % exit_on_error
if verbose: print msg

# Plot map on error (requires matplotlib, also doubles or more memory
#   footprint)
plot_on_error = 0
if fast_run:
    plot_on_error = 0
if express:
   plot_on_error = 0
msg = 'plot_on_error = %d' % plot_on_error
if verbose: print msg

# Create progress files which can used by python util/done.py out_path
#   to check the progress of a run if you cannot use the progress_bar
#   (e.g., remote logging or background process).
progress = 1
if fast_run:
    progress = 1
if progress:
    file_list = os.listdir(out_path)
    file_list.sort()
    pfiles = [x for x in file_list if x.find("progress") != -1]
    for p in pfiles:
        os.remove(out_path+p)

# Special Limit center finding by hemisphere (for speed)
limit_nh = 0
limit_sh = 0
if limit_nh or limit_sh:
    print "Warning Using Hemisphere Limit!"

# Apply a weak 2D Gaussian filter to smooth the SLP field before center finding.
smooth = 0

# Screen tropical land grids in initial center finding.
screen_tropical_land = 1

# Special discard centers if in troubled rather than treat differently.
drop_troubled = 0

#
# Override values from mcms_setup.py
#
no_poles = []

#high_res = 1

# Screen high pressure minima
#defs_set["plim_filter"] = 102000
#defs_set["keep_discards"] = 0
if fast_run:
    defs_set["keep_discards"] = 0
if express:
    defs_set["keep_discards"] = 0

# Threshold for Laplacian Test (Default 0.15)
lapp_cutoff = None

# Threshold for the peak horizontal pressure gradient (Default 0.07)
hpg_cutoff = None

use_projection = 1
#projs = ['laea','stere','aeqd']
#pmap = projs[0]
#mproj_sh = 'sp'+pmap
#mproj_nh = 'np'+pmap
if use_projection:
    no_poles = []

# Pure candidates
pure_candidates = 0

# Use hotspot file
#  use_hotspot =
#              0 ignore
#              1 add to troubled (screen)
#              2 outright discard
#              3 discard centers if in troubled_hotspots, otherwise add
#                   hot_spots, (halo and troubled_halo to troubled if use_halo).
# Include the 8 surrounding grids for each hotspot as insurance.
# use_halo =
#              0 ignore
#              1 add to troubled
use_hotspot = 3
use_halo = 1
if use_hotspot:
    import cPickle
    import gzip
    hfile = "%s%s_hotspots.pz" % (shared_path,model)
    in_hot_spots,in_halo,in_troubled_hot_spots,in_troubled_halo = cPickle.load(
            gzip.open(hfile))
    discard_hot_spots = []
    screen_hot_spots = []
    if use_hotspot == 1:
        # Everything added to screen
        screen_hot_spots = in_hot_spots
        screen_hot_spots.extend(in_troubled_hot_spots)
        if use_halo:
            screen_hot_spots.extend(in_halo)
            screen_hot_spots.extend(in_troubled_halo)
    elif use_hotspot == 2:
        # Everything added to discard
        discard_hot_spots = in_hot_spots
        discard_hot_spots.extend(in_troubled_hot_spots)
        if use_halo:
            discard_hot_spots.extend(in_halo)
            discard_hot_spots.extend(in_troubled_halo)
    elif use_hotspot == 3:
        # Troubled hot_spot to discard, else to screen
        screen_hot_spots = in_hot_spots
        discard_hot_spots = in_troubled_hot_spots
        if use_halo:
            screen_hot_spots.extend(in_halo)
            screen_hot_spots.extend(in_troubled_halo)