#!/usr/bin/env python -tt
"""
Default run_file template for mcms_track_finder

Create from a template via:

    python mcms_track_finder.py -c rf_files/rf_model_tf.py

Input:
^^^^^
None

Output:
^^^^^^
Various python data structures.

Examples:
^^^^^^^^

Notes/Warnings:
^^^^^^^^^^^^^^

Author: Mike Bauer  <mbauer@giss.nasa.gov>

Log:
2011/01 MB - File created
2012/10 MB - Updated
"""
from __future__ import division

# Standard library imports
import sys
import os
import fnmatch

# MCMS module imports: Need to have mcms in your PYTHONPATH environment variable!
from cfg import define_source
from cfg import define_figure
from cfg import define_dirs
from cfg import define_defs
from cfg import define_vars
from cfg import ensure_path
from cfg import define_season
from cfg import define_m2s
from util import FileExists
from util import organize_data
from util import query_yes_no
from util import InjestSetup
from util import Is_It_Available

# Modify these to set the paths for the data and MCMS output
import output_root
import data_root

# Quit on error else just send message to logfile?
def do_this(do_msg,do_msg2,do_date_stamp):
    print "\n\n"
    print do_msg
    print do_msg2
    print "\t\tDate_Stamp:"+do_date_stamp
    print

verbose = 0
dotline = "%s" % ("-"*40)
msg = '\n%s\n%40s\n%s\n' % (dotline,"Main Settings Set",dotline)
if verbose: print msg

# ----------------------------------------------------------------------------
#                               Main Settings Set
# ----------------------------------------------------------------------------
verbose = 0

dotline = "%s" % ("-"*40)
msg = '\n%s\n%40s\n%s\n' % (dotline,"Main Settings Set",dotline)
if verbose: print msg

# ----------------------------------------------------------------------------
# Pick data source see files in /cfg
#   See cfg/mcms_source_base_defs.py
# ----------------------------------------------------------------------------
pick_source = define_source()
msg = pick_source.make_pick("merrac")
if verbose: print msg
model = pick_source.selection
model_def = pick_source.selections[model][0]

# ----------------------------------------------------------------------------
# Import the base directories for the data and MCMS output
#   See rf_files/output_root.py
#       rf_files/data_root.py
# ----------------------------------------------------------------------------
if hasattr(output_root,'%s_root' % model):
    oroot = getattr(output_root,'%s_root' % model)
else:
    oroot = output_root.root
if hasattr(data_root,'%s_root' % model):
    droot = getattr(data_root,'%s_root' % model)
else:
    droot = data_root.root
# Can manually override these here
#oroot = ""
#droot = ""

# ----------------------------------------------------------------------------
# Full path to the root directory where pick specific output will be stored.
#   If multiple data sets under same model (say GCM runs) add a subdirectory
#   to the model root. Can be worked with ensembles and experiments within the
#   same model.
#
# Example:
#   ensembles = {"control_run" : [""],
#       "Scenario_A1B_run" : ["E3abaoM20","E3abboM20","E3abcoM20","E3abdoM20"],
#       "Scenario_A1B_iceMelt_run" : ["E3FIBaoM20","E3FIBboM20"]}
#   blobs = {"control_run" : "gissm_20",
#           "Scenario_A1B_run" : "gissm_20",
#           "Scenario_A1B_iceMelt_run" : "gissm_20"}
#   experiment = "Scenario_A1B_iceMelt_run"
#   ensemble = ensembles[experiment][0]
#   blob = blobs[experiment]
#
# Blob represents the sometimes long descriptive name some GCMs add to their
# output files (e.g., for the file naming pattern
#   cam_b30.14_37kaDVT_T85.cam2.h1.0531-01-01-PSL.nc
# the blob would be
#   cam_b30.14_37kaDVT_T85.cam2.h1
# which tells mcms that the changeable part of the file name pattern is
#   .0531-01-01-PSL.nc
# ----------------------------------------------------------------------------
ensemble = ""
experiment = ""
blob = ""
if ensemble:
    subdir = "%s/%s/" % (experiment,ensemble)
else:
    if experiment:
        subdir = "%s/" % (experiment)
    else:
        subdir = ""
out_path = "%s%s%s/" % (oroot,model,subdir)
out_path = ensure_path(subdir,out_path,check_exists=0)

# ----------------------------------------------------------------------------
# Full path to the directory where the SLP data files are stored.
# ----------------------------------------------------------------------------
slp_path = "%s%s/%s" % (droot,model,subdir)
slp_path = ensure_path(subdir,slp_path)

# ----------------------------------------------------------------------------
# Directory to be created for storing temporary model specific files.
# ----------------------------------------------------------------------------
if subdir:
    shared_path = "%s%s_files/" % (oroot,model)
else:
    shared_path = "%s_files/" % (out_path.replace("%s/" % model,model))
shared_path = ensure_path(subdir,shared_path)

# --------------------------------------------------------------------------
# Define seasons used to collect tracking statistics
#    See cfg/mcms_season_base_defs.py and cfg/mcms_m2s_base_defs.py
#  Example:
#   msg = pick_season.make_pick("3mo") for ["annual","djf","mam","jja","son"]
#  Note: use the same option for pick_season and pick_m2s
# --------------------------------------------------------------------------
pick_season = define_season()
msg = pick_season.make_pick("3mo")
if verbose: print msg
seasons = pick_season.selection

pick_m2s = define_m2s()
msg = pick_m2s.make_pick("3mo")
if verbose: print msg
m2s = pick_m2s.selection

# --------------------------------------------------------------------------
# Over write the default of doing all years located by mcms_setup.py
# Example:
#   over_write_years = [2000,2010]
# --------------------------------------------------------------------------
over_write_years = []
#over_write_years = [1979,1980]

# --------------------------------------------------------------------------
# Skip some hours in the dataset. If skip_hours was used during center finding
#   this must be reflected here. The also requires that timestep is altered
#   from the original. keep_hours is the inverse of skip_hours, with the
#   assumption that skip_hours+keep_hours = all hours
# Example:
#   skip_hours = ["03","09","15","21"]
#   keep_hours = ["00","06","12","18"]
#   timestep = 6
#   scale_tsteps = len(keep_hours)/(len(skip_hours) + len(keep_hours))
# --------------------------------------------------------------------------
skip_hours = ["03","09","15","21"]
keep_hours = ["00","06","12","18"]
timestep = 6
scale_tsteps = len(keep_hours)/(len(skip_hours) + len(keep_hours))
if verbose and skip_hours:
    print "Skipping Hours: %s" % skip_hours
    print "Keeping Hours: %s" % keep_hours
    print "New Timestep: %d" % timestep

# --------------------------------------------------------------------------
# Names associated with the netcdf files for the source
#   See cfg/mcms_ncvars_base_defs.py
# --------------------------------------------------------------------------
source_vars = define_vars()
msg = source_vars.make_pick(model)
if verbose: print msg
nc_vars = source_vars.selection
break_fname = nc_vars['break_fname']
month_sort_order = nc_vars['month_sort_order']
the_calendar = nc_vars['calendar']
jd_fake = nc_vars['jd_fake']
get_time_dat = "time_dat"

# from util import organize_data
# get_time_dat = "time_dat"
# gf = shared_path+get_time_dat+".pz"
# msg = organize_data(slp_path,break_fname,nc_vars['var_slp'],
#     month_sort_order,the_calendar,nc_vars['var_time'],jd_fake,
#     shared_path,blob,base_name=get_time_dat,verbose=0)
# print msg

# --------------------------------------------------------------------------
# Get some definitions. Note must have run mcms_setup.py already!
# --------------------------------------------------------------------------
fnc_out,inputs = InjestSetup("%ss_datf.pz" % (shared_path),0)
msg = "OUTPUT PATH (Default): %s"
if out_path != fnc_out[inputs.index("out_path")]:
    msg = "OUTPUT PATH (Modified): %s"
if verbose: print msg % out_path
if verbose: print "SHARED PATH: %s" % shared_path

# --------------------------------------------------------------------------
# Create needed directories if they don't exist.
# --------------------------------------------------------------------------
pick_dirs = define_dirs()
msg = pick_dirs.make_pick("standard")
dirs_set = pick_dirs.selection
need_dirs = ["%s%s" % (out_path,x) for x in dirs_set]
need_dirs.append("%s%s" % (out_path,"figs/maps/global/"))
need_dirs.append("%s%s" % (out_path,"figs/maps/global/annual/"))
need_dirs.append("%s%s" % (out_path,"figs/pdfs/global/"))
need_dirs.append("%s%s" % (out_path,"figs/pdfs/global/annual/"))
for ndir in need_dirs:
    if not os.path.exists(ndir):
        try:
            os.mkdir(ndir)
            if verbose:
                print "\tMade: %s" % ndir
        except:
            sys.exit("ERROR: creating %s" % ndir)

msg = "SOURCE YEARS (Modified): %s"
if not over_write_years:
    over_write_years = fnc_out[inputs.index("super_years")]
    over_write_years = [int(over_write_years[0]),int(over_write_years[1])]
    msg = "SOURCE YEARS (Default): %s"
super_years = over_write_years

# Delete the rest
del fnc_out

# --------------------------------------------------------------------------
# Alter default behavior found in mcms_defs.py
# --------------------------------------------------------------------------
# Pick data source see defs files in /cfg
pick_defs = define_defs()
msg = pick_defs.make_pick(model)
if verbose: print msg
defs_set = pick_defs.selection

# --------------------------------------------------------------------------
# Define some files and important values
# --------------------------------------------------------------------------
start_year = int(super_years[0])
end_year = int(super_years[1])
years = range(super_years[0],super_years[1]+1)
clim_tag = "%04d_%04d" % (start_year,end_year)
header = "mcms_%s_%s_" % (model,clim_tag)
centers_file = "%s%scenters.txt" % (out_path,header)
dumped_centers_file = "%s%sdumped_centers.txt" % (out_path,header)
tracks_file = centers_file.replace("centers","tracks")
if verbose:
    print "CENTERS_FILE: %s" % centers_file
    print "DUMPED_CENTERS_FILE: %s" % dumped_centers_file
    print "TRACKS_FILE: %s" % tracks_file

# --------------------------------------------------------------------------
# Read in the date/time info for these files
# --------------------------------------------------------------------------
# Name for new time_dat file
if ensemble:
    get_time_dat  = "time_dat_%s_%s" % (experiment,ensemble)
else:
    if experiment:
        get_time_dat = "time_dat_%s" % (experiment)
    else:
        get_time_dat = "time_dat"
# Test to see if already get_time_dat defined
gf = shared_path+get_time_dat+".pz"
fnc_out,inputs = InjestSetup(gf,4)
super_time_data = fnc_out[inputs.index("super_time_data")]
fnames = super_time_data.keys()
fnames = sorted(fnames)
if month_sort_order:
    fnames = sorted(fnames,key = lambda fnames: (break_fname(fnames)[0],
                            month_sort_order[break_fname(fnames)[1]]))
else:
    # Annual Files
    fnames = sorted(fnames,key = lambda fnames: (break_fname(fnames)[0],
                            break_fname(fnames)[1]))
years = range(super_years[0],super_years[-1]+1)
uci_stamps = []
adates = []
tsteps = 0
for fn in fnames:
    if break_fname(fn)[0] in years:
        if skip_hours:
            for ii in xrange(len(super_time_data[fn]['uci_stamps'])):
                if super_time_data[fn]['uci_stamps'][ii][-2:] in keep_hours:
                    uci_stamps.append(super_time_data[fn]['uci_stamps'][ii])
                    adates.append(super_time_data[fn]['adates'][ii])
                    #tsteps += 1
        else:
            uci_stamps.extend(super_time_data[fn]['uci_stamps'])
            adates.extend(super_time_data[fn]['adates'])
        tsteps += super_time_data[fn]['tsteps']
del super_time_data
del fnc_out
if scale_tsteps:
    tsteps = int(round((tsteps*scale_tsteps),0))

# ----------------------------------------------------------------------------
#                           Extra Settings Set
# ----------------------------------------------------------------------------
msg = '\n%s\n%40s\n%s\n' % (dotline,"Extra Settings Set",dotline)
if verbose: print msg

# What sort of figures (png faster, eps/pdf better quality)
pick_figure = define_figure()
msg = pick_figure.make_pick("png")
if verbose: print msg
fig_format  = pick_figure.selection

# Store tracking stats and make plots
# save_stats = 0 save tracks to file, no stats or plots
#               1 save tracks to file, make stats and plots
#               2 to not save tracks to file... just stats and plots
save_stats = 1
if save_stats == 2:
    print "\t\tWarning not Saving Tracks to File!"
    defs_set["keep_discards"] = 0
msg = 'save_stats = %d' % save_stats
if verbose: print msg

# Save track finding plots (requires matplotlib, also doubles or more memory
#   footprint). Need this on to save filter specfic netcdf files.
#       save_plots = 0 skip
#                 = 1 plots and netcdf
#                 = 2 just netcdfs
save_plots = 2
msg = 'save_plots = %d' % save_plots
if verbose: print msg

# Halt program on error or just warn?
exit_on_error = 1
msg = 'exit_on_error = %d' % exit_on_error
if verbose: print msg

#
# Override values from mcms_setup.py
#
#defs_set["keep_discards"] = 0

# Match RAIBLE et. all 2008
#defs_set['max_cyclone_speed']= 42.0
#defs_set['age_limit'] = 72.0

# Use hotspot file: After a mcms center finding is run it might be that some
#   locations exhibit unrealistically high center counts (hotspots). Once
#   identified the option exists to re-run center counting and either treating
#   these hotspots (and also 8 neighbors) to the extra checks for troubled centers
#   or simple discard them outright. The concern here is that if noise causes a
#   persistent error in the SLP field in a location that hotspot might mask out
#   all nearby legitimate cyclone centers via the regional minimum tests. Also
#   latter with tracking a hotspot can force passing tracks always vier over
#   this spot.
#
#  use_hotspot =
#              0 ignore
#              1 add to troubled (screen)
# Include the 8 surrounding grids for each hotspot as insurance.
# use_halo =
#              0 ignore
#              1 add to troubled
use_hotspot = 1
use_halo = 1
if use_hotspot:
    import cPickle
    import gzip
    hfile = "%s%s_hotspots.pz" % (shared_path,model)
    in_hot_spots,in_halo,in_troubled_hot_spots,in_troubled_halo = cPickle.load(
            gzip.open(hfile))
    discard_hot_spots = []
    screen_hot_spots = []
    if use_hotspot == 1:
        # Everything added to screen
        screen_hot_spots = in_hot_spots
        screen_hot_spots.extend(in_troubled_hot_spots)
        if use_halo:
            screen_hot_spots.extend(in_halo)
            screen_hot_spots.extend(in_troubled_halo)

# ----------------------------------------------------------------------------
# Check for graphics programs
#   The psrose plotting program from the GMT (The Generic Mapping Tools,
#   http://gmt.soest.hawaii.edu/) project provides a much better wind
#   rose plot than that from the matplotlib plotting package that is generally
#   used by the MCMS project.
# ----------------------------------------------------------------------------
psrose = Is_It_Available("psrose")
if not psrose:
    msg = "\n\t\tWarning: GMT (The Generic Mapping Tools) not found. "
    msg += "Some plots will not be made."
    print msg
    try:
        import matplotlib
    except ImportError:
        print "\n\t\tWarning: matplotlib not available. Disabling plotting altogether."
        save_plots = 0

