#!/usr/bin/env python -tt
"""
Create from a template via:

 	python mcms_dbase_reformer.py -c rf_files/rf_model_rf.py

Input:
^^^^^
None

Output:
^^^^^^
Various python data structures.

Examples:
^^^^^^^^

Notes/Warnings:
^^^^^^^^^^^^^^

    MCMS data is sometimes stored in annual files. Assuming that you have 
    sufficient computer memory you can simply concatenate all the files into a 
    single file simpler processing. For example,
        
    find . -name \*tracks.txt\* -print0 | xargs -0 cat > mcms_nra2_1979-2012_tracks.txt

Author: Mike Bauer  <mbauer@giss.nasa.gov>

Log:
2012/10 MB - File created
"""

# Standard library imports
import sys
import os

# MCMS module imports: Need to have mcms in your PYTHONPATH environment variable!
from cfg import define_source
from cfg import ensure_path

# Modify these to set the paths for the data and MCMS output
import output_root

verbose = 0
dotline = "%s" % ("-"*40)
msg = '\n%s\n%40s\n%s\n' % (dotline,"Main Settings Set",dotline)
if verbose: print msg

# ----------------------------------------------------------------------------
#           Settings concerning the identity of the dataset being read
# ----------------------------------------------------------------------------
 
# ----------------------------------------------------------------------------
# Pick data source see files in /cfg
#   See cfg/mcms_source_base_defs.py
# ----------------------------------------------------------------------------
pick_source = define_source()
msg = pick_source.make_pick("nra2")
if verbose: print msg
model = pick_source.selection
model_def = pick_source.selections[model][0]
if verbose: print "Source/Model: %s (%s)" % (model,model_def)

# ----------------------------------------------------------------------------
# Import the base directories for the data and MCMS output
#	See rf_files/output_root.py
#		rf_files/data_root.py
# ----------------------------------------------------------------------------
if hasattr(output_root,'%s_root' % model):
    oroot = getattr(output_root,'%s_root' % model)
else:
    oroot = output_root.root
# Can manually override these here
#oroot = ""

# ----------------------------------------------------------------------------
# Full path to the root directory where pick specific output will be stored.
#   If multiple data sets under same model (say GCM runs) add a subdirectory
#   to the model root. Can be worked with ensembles and experiments withint the
#   same model.
#
# Example:
#	ensembles = {"control_run" : [""],
#    	"Scenario_A1B_run" : ["E3abaoM20","E3abboM20","E3abcoM20","E3abdoM20"],
#    	"Scenario_A1B_iceMelt_run" : ["E3FIBaoM20","E3FIBboM20"]}
#	blobs = {"control_run" : "gissm_20",
#     	    "Scenario_A1B_run" : "gissm_20",
#     	    "Scenario_A1B_iceMelt_run" : "gissm_20"}
#	experiment = "Scenario_A1B_iceMelt_run" 
#	ensemble = ensembles[experiment][0]
#	blob = blobs[experiment]
# ----------------------------------------------------------------------------
ensemble = ""
experiment = ""
blob = ""
if ensemble:
    subdir = "%s/%s/" % (experiment,ensemble)
else:
    if experiment:
        subdir = "%s/" % (experiment)
    else:
        subdir = ""
out_path = "%s%s%s/" % (oroot,model,subdir)
out_path = ensure_path(subdir,out_path,check_exists=0)
print "Output PATH: %s" % out_path

# ----------------------------------------------------------------------------
# Directory to be created for storing temporary model specific files.
# ----------------------------------------------------------------------------    
if subdir:
    shared_path = "%s%s_files/" % (oroot,model)
else:
    shared_path = "%s_files/" % (out_path.replace("%s/" % model,model))
shared_path = ensure_path(subdir,shared_path)
print "Shared PATH: %s" % shared_path

# ----------------------------------------------------------------------------
# Type of MCMS dataset beginning read (Centers, Tracks or Attributed)
# ----------------------------------------------------------------------------
file_types = {0 : ("Unknown",""),
              1 : ("Centers","centers.txt"),
              2 : ("Tracks","tracks.txt"),
              3 : ("Attributed","att.txt")}
file_type = 3
if verbose: print "File Type: %s" % file_types[file_type]

# ----------------------------------------------------------------------------
#           Settings that govern how MCMS reads and organizes the data
# ----------------------------------------------------------------------------
 
# start_time and end_time:
#   Allows us to tell MCMS to return a temporal subset of the data. As it stands
#   now this is a limited, but useful facility. One way around some of the 
#   limitations is to apply these settings in sequence, with the output of one
#   subset acting as the input for the next. This will be explained below.
#
#   The default setting is "YYYY MM DD HH SEASON" which tells MCMS to pull data
#   for all dates and times from the dataset being read.
#
#   YYYY      The first/last year to be extracted. 
#   MM       "               month                 "
#   DD       "               day                   "
#   HH       "               hour                  "
#   SEASON   "               Season                "
#
#   For example, start_time = "2006 MM DD HH SEASON" limits the returned data 
#   to or after the year 2006. If end_time = "2006 MM DD HH SEASON" then the
#   returned data is limited to just the year 2006.
#   
#   The same thing can be done with MM, DD and HH. Thus, "YYYY 02 DD HH SEASON" 
#   returns just February data while "YYYY MM DD 00 SEASON" only returns 0 UTC 
#   data. Likewise, setting MM for start_time to 02 and 03 in end_time returns 
#   February and March data.
#
#   These can be used together in a limited way, such as start_time = 
#   "2006 02 14 06 SEASON" which limits the record to just 6 UTC on 
#   February, 14th 2006 no matter how end_time is set. This is a limitation of
#   the current system in that if more than one of "YYYY MM DD HH SEASON" is
#   non-default then the results are for that time combination only rather than 
#   a range. Thus start_time = "2006 02 DD HH SEASON" returns data from only
#   0 UTC February 1st, 2006 (the first data from February 2006).
#
#   The workaround for this is to do the subsets sequentially. For example, 
#   subset the data by a year range then by March and finally by 0 UTC to get
#   the 0 UTC March data over a year range.
#
#   SEASON works differently as well. For example, it must be paired with 
#   "YYYY MM DD HH". Thus, if you want a seasonal subset for something other than
#   all the data you have to sequentially subset it first by "YYYY MM DD HH" as
#   described above and then by SEASON. The supported values for SEASON are 
#   ["DJF", "MAM", "JJA", "SON", "NDJFMA", "MJJASO"]. 
start_time = "YYYY MM DD HH SEASON"
end_time = "YYYY MM DD HH SEASON"

start_time = "1979 MM DD HH SEASON"
end_time = "1980 MM DD HH SEASON"

# places: 
#   Allows us to tell MCMS to return a spatial subset of the data. This works
#   much like start_time/end_time:
#
#   The default setting is "GLOBAL" which tells MCMS to pull data from all 
#   locations in the dataset being read. Other supported options are
#   ("NH", "SH", "LAND", "SEA"). 
#   
#   As with start_time/end_time these can be used sequentially to extract 
#   something like the Northern Hemisphere ocean only centers ("NH" then "SEA").
#
#   It is also possible to provide a custom list of GridIDs such that places
#   is set to python list of GridIDs (say just the north Atlantic region) in
#   the following form places = [GridID,GridID....]
places = ["GLOBAL"]

# A non-default start_time/end_time can be paired with a non-default places.

# save_output:
#   Tells MCMS to store the retrieved data in a new file rather than return it
#   in memory to the calling function. This is used when the user wishes to 
#   create an actual subset file rather than simple extract one and pass it
#   to another routine for further processing. For example, this is used when
#   applying a sequence of non-default start_time/end_time/places to create
#   a special subset for the final analysis. The default is save_output = "".
#   The assumption is that the save_output includes path information. 
#save_output = "%smcms_nra2_1979_1980_tracks.txt" % out_path
save_output = "%smcms_nra2_1979_1980_att.txt" % out_path
#save_output = ""

# overwrite:
#   When save_output is evoked setting this determines whether MCMS overwrites
#   any existing files with the same path/name as save_output. The default is
#   False which raises a warning when a name collision occurs and stops the
#   program.
overwrite = True

# just_center_table: 
#   Tells MCMS that we only want basic center-data (see center_fields in
#   read_mcms.py) and to ignore other aspects of the MCMS dataset (see att_fields
#   and atts_fields in read_mcms.py). This reduces the memory requirements and
#   increases the speed a given operation when used.
just_center_table = 0

# use_tracks:
#   Tells MCMS whether we want the output organized by centers or tracks.
#   Both are sorted time sequentially, but when the data is organized by tracks
#   all the centers making up that track are grouped together and the date-time
#   of the center at the start of the track determines how it is sorted by time.
#   Whereas, with center-based data each center is time sorted by its occurrence.
#   
#   The default is centers based.
#
# use_tracks = 0 centers based
#            = 1 track based
use_tracks = 1

# use_att:
#   Tells MCMS that we want to use spatial attributed data somehow. This is
#   specific to the use of ATT gridIDs (see att_fields in read_mcms.py)
#   rather than referring to say cyclone area/size 'Area_ATT'.
use_att = 1

# use_atts:
#   Tells MCMS that we want to use spatial attributed data somehow. This is
#   specific to the use of ATTS gridIDs (see atts_fields in read_mcms.py)
#   rather than referring to say cyclone area/size 'Area_ATTS'.
use_atts = 0

if use_att or use_atts:
    just_center_table = 0

if use_tracks and use_atts:
    # Note when MCMS data is reformatted into the track-base format there is 
    # the problem of how to store the ATTS data, which depends on time but also
    # links multiple centers/tracks. This is only an issue when using the 
    # make_subset() routine in mcms_reformat.py, which is meant to simply create
    # a subset of the original mcms data without modifying the format. When 
    # use_tracks is used with make_subset() the resulting files are not the same
    # and the default MCMS datasets which are purely time sequential, rather than
    # track sequential. Again, this works just fine when you want to output the 
    # data for some purpose other than those provided by MCMS and you only 
    # request just_center_table or use_att data. However, until we determine a 
    # workable method for dealing with the ATTS issue the option to store ATTS 
    # data in the track-base format is not supported. Note that MCMS provides 
    # many ways of working with track data and keeping ATT/ATTS properly 
    # associated, but this is done in memory rather than a static file.
    sys.exit("\tuse_atts and use_tracks are incompatible")

if use_atts and not use_att:
    sys.exit("\nuse_atts requires the use of use_att")

# Set use_att = 1 and use_atts = 0 to get ATT-only rather than ATT+ATTS output.

# ----------------------------------------------------------------------------
#                         Final Checks and Settings
# ----------------------------------------------------------------------------
if use_tracks and file_type == 1:
    sys.exit("\nCenter files lack track information for use_tracks")
if (use_att or use_atts) and file_type != 3:
    sys.exit("\nuse_att(s) requires an 'att.txt' type file.")

# ----------------------------------------------------------------------------
#                           Extra Settings Set
# ----------------------------------------------------------------------------
msg = '\n%s\n%40s\n%s\n' % (dotline,"Extra Settings Set",dotline)
if verbose: print msg

# Set for debugging as multiprocessing doesn't always report why a program hangs.
single_processor = 0
msg = "Warning Running Single Processor Mode!"
if verbose and single_processor: print msg



