#!/usr/bin/env python -tt
"""
Default run_file template for mcms_setup.py

Input:
^^^^^
None

Output:
^^^^^^
Various python data structures.

Examples:
^^^^^^^^

Notes/Warnings:
^^^^^^^^^^^^^^

Author: Mike Bauer  <mbauer@giss.nasa.gov>

Log:
2011/01 MB - File created
"""

# Standard library imports
import sys
import os

# MCMS module imports: Need to have mcms in your PYTHONPATH environment variable!
from cfg import define_source
from cfg import define_figure
from cfg import define_dirs
from cfg import define_defs
from cfg import define_vars
from cfg import ensure_path
import output_root
import data_root

# ----------------------------------------------------------------------------
#                               Main Settings Set
# ----------------------------------------------------------------------------
verbose = 0

dotline = "%s" % ("-"*40)
msg = '\n%s\n%40s\n%s\n' % (dotline,"Main Settings Set",dotline)
if verbose: print msg

# ----------------------------------------------------------------------------
# Pick data source see files in /cfg
#   See cfg/mcms_source_base_defs.py
# ----------------------------------------------------------------------------
pick_source = define_source()
msg = pick_source.make_pick("nra2")
if verbose: print msg
model = pick_source.selection
model_def = pick_source.selections[model][0]

if hasattr(output_root,'%s_root' % model):
    oroot = getattr(output_root,'%s_root' % model)
else:
    oroot = output_root.root
if hasattr(data_root,'%s_root' % model):
    droot = getattr(data_root,'%s_root' % model)
else:
    droot = data_root.root

# ----------------------------------------------------------------------------
# Full path to the root directory where pick specific output will be stored.
#   If multiple data sets under same model (say GCM runs) add a subdirectory
#   to the model root
# ----------------------------------------------------------------------------
#experiments = {"e12k" : "PSL-12k",
#            "e14k" : "PSL-14k"}
#blobs = {"e12k" : "cam_b30.12_2kaDVTn_T85.cam2.h1",
#         "e14k" : 'cam_b30.14_37kaDVT_T85.cam2.h1'}
#experiment = experiments['e14k']
#blob = blobs['e14k']
ensemble = ""
experiment = ""
blob = ""
if ensemble:
    subdir = "%s/%s/" % (experiment,ensemble)
else:
    if experiment:
        subdir = "%s/" % (experiment)
    else:
        subdir = ""
#out_path = "/Users/mbauer/Scratchf/output/%s/%s/" % (model,subdir)
out_path = "%s%s/%s/" % (oroot,model,subdir)
out_path = ensure_path(subdir,out_path,check_exists=0)

# Create needed directories if they don't exist.
#   See cfg/mcms_directories_base_defs.py
pick_dirs = define_dirs()
msg = pick_dirs.make_pick("standard")
dirs_set = pick_dirs.selection
need_dirs = ["%s%s" % (out_path,x) for x in dirs_set]
if subdir:
    op = out_path.replace("/%s" % subdir,"")
    need_dirs.insert(0,op)
for ndir in need_dirs:
    while ndir.find("//") != -1:
        ndir = ndir.replace("//","/")
    if not os.path.exists(ndir):
        try:
            os.mkdir(ndir)
            if verbose:
                print "\tMade: %s" % ndir
        except:
            sys.exit("ERROR: creating %s" % ndir)
if verbose: print "OUT_PATH: %s" % out_path

# ----------------------------------------------------------------------------
# Full path to the directory where the SLP data files are stored.
# ----------------------------------------------------------------------------
#slp_path = "/Users/mbauer/Scratchf/data/%s/%s/" % (model,subdir)
slp_path = "%s%s/%s/" % (droot,model,subdir)
slp_path = ensure_path(subdir,slp_path)
msg = "SLP_PATH: %s" % slp_path
if verbose: print msg

# ----------------------------------------------------------------------------
# Directory to be created for storing temporary pick specific files.
# ----------------------------------------------------------------------------    
if subdir:
    shared_path = "%s%s_files/" % (oroot,model)
else:
    shared_path = "%s_files/" % out_path[:-1]
shared_path = ensure_path(subdir,shared_path)
msg = "SHARED PATH: %s"
if verbose: print msg % shared_path

# --------------------------------------------------------------------------
# Define the definitions to be read in.
#
# Hows how to alter a parameter in defs w/out having to alter the file
# itself. Here I use setup_all as a flag to alter the defs so that all data
# that can be pre-calculated and saved are, rather than the default which is
# to read those data from file.
# --------------------------------------------------------------------------

# If set to 1, then topography information will not be used to determine
#  potentially troublesome locations for SLP dependent analysis. For
#  example, regions of very high or steep topography can result in erroneous
#  SLP values that either mimic or obscure cyclones. Generally, the
#  results are better with no_topo set to 0.
no_topo = 0

# If set to 1, then the land_sea mask is not used to separate results
#  that occur over land or ocean grids. This info is not required for
#  full analysis. Note: the topography field can be used for this in
#  a pinch.
no_mask = 0

# Select wavenumber for regional screens.
wavenumbers = [4.0,8.0,13.0,18.0,26.0,52.0]

# This value is used by center_finder_vX.py to screen for regional minima
#  status. Generally, using too low wavenumbers means overly screening
#  centers and too large values have little effect other than to
#  make the analysis run longer. Good rule of thumb is 8-26.
wavenumber = wavenumbers[4]

# Set the latitude equivalent *radius* in degrees for center/track density
# calculations.
c_size = 5

# Prevent centers from being found in at the pole (reduces a lot of problems).
#   The default is to screen centers poleward of -86 and 89. Change this
#   below if wanted (use degrees). Setting to -90,90 disables the screen.
polar_exclusion = [-86.0,89.0]
polar_exclusion = [-90.0,90.0]

# --------------------------------------------------------------------------
# Alter default behavior found in util/defs.py
#   See cfg/mcms_defs_base_defs.py
# --------------------------------------------------------------------------
pick_defs = define_defs()
msg = pick_defs.make_pick(model)
if verbose: print msg
defs_set = pick_defs.selection
defs_set.update({'tropical_boundary':15, "tropical_boundary_alt":30,
    "critical_radius":0.0,"wavenumber":wavenumber,"use_gcd":True})
# match RAIBLE et. all 2007
#defs_set = {'max_cyclone_speed': 42.0,'age_limit':72.0}

# --------------------------------------------------------------------------
# Names associated with the netcdf files for the source 
#   See cfg/mcms_ncvars_base_defs.py
# --------------------------------------------------------------------------
source_vars = define_vars()
msg = source_vars.make_pick(model)
if verbose: print msg
nc_vars = source_vars.selection

# --------------------------------------------------------------------------
#                           Extra Settings Set
# --------------------------------------------------------------------------
msg = '\n%s\n%40s\n%s\n' % (dotline,"Extra Settings Set",dotline)
if verbose: print msg

# Save center finding plots (requires matplotlib, 2x memory footprint)
save_plot = 1
msg = 'save_plot = %d' % save_plot
if verbose: print msg

# --------------------------------------------------------------------------
# What sort of figures (png faster, eps/pdf better quality)
#   See cfg/mcms_figure_base_defs.py
#   'pdf' and 'eps' much slower than 'png', but higher quality
# --------------------------------------------------------------------------
pick_figure = define_figure()
#msg = pick_figure.make_pick("pdf")
#msg = pick_figure.make_pick("eps")
msg = pick_figure.make_pick("png")
if verbose: print msg
fig_format  = pick_figure.selection

# Halt program on error or just warn?
exit_on_error = 1
msg = 'exit_on_error = %d' % exit_on_error
if verbose: print msg
print
