#!/usr/bin/env python -tt
"""This function reads in progress files from mcms software and
returns a progress bar.

Options/Arguments:
    start-- directory to search for progress files.

Returns:
    progress -- % of total job completed. As well as the modification time of each file.

Examples:

       # For short report
       pyf done.py /Volumes/scratch/output/test
       # For long report add any character after path
       pyf done.py /Volumes/scratch/output/test 1

Notes:

Author: Mike Bauer  <mbauer@giss.nasa.gov>

Log:
    2008/11  MB - File created.
    2010/06  MB - Updated to v5.
    2011/01  MB - Updated again
"""

from __future__ import division
# Alter python 2 division where division of two *integer* values gives an
#   *integer* result (i.e., 1/2 = 0 but 1.0/2 = 0.5 or 1/2.0 = 0.5) to the
#   more rational python 3+ version where all division returns floats
#   (i.e., 1/2 = 0.5). Adds a new integer division operator so that 1//2 = 0

# Standard library imports
import sys
import os
import time
import datetime
import os.path
import commands

# Third party module imports
#import extras.progress_bar
from extras import ProgressBarAlt
from extras import ProgressBar

# MCMS module imports
from print_columns import PrintColumns

def get_osx_creation_time(filename):
    """Accepts a filename as a string. Gets the OS X creation date/time by parsing "mdls" output.

    Returns file creation date as a float; float is creation date as seconds-since-epoch.
    """
    status, output = commands.getstatusoutput('/usr/bin/mdls -name kMDItemFSCreationDate "%s"' % (filename))
    if status != 0:
        print('Error getting OS X metadata for %s. Error was %d. Error text was: <%s>.' %
              (filename, status, output))
        sys.exit(3)
    datestring = output.split('=')[1].strip()
    datestring_split = datestring.split(' ')
    datestr = datestring_split[0]
    timestr = datestring_split[1]
    # At present, we're ignoring timezone.
    tzstr = datestring_split[2]

    date_split = datestr.split('-')
    year = int(date_split[0])
    month = int(date_split[1])
    day = int(date_split[2])

    time_split = timestr.split(':')
    hour = int(time_split[0])
    minute = int(time_split[1])
    second = int(time_split[2])

    # convert to "seconds since epoch" to be compatible with os.path.getctime and os.path.getmtime.
    return time.mktime([year, month, day, hour, minute, second, 0, 0, -1])

def fractSec(s,text=1):
   if text:
       years, s = divmod(s, 31556952)
       min, s = divmod(s, 60)
       h, min = divmod(min, 60)
       d, h = divmod(h, 24)
       elapsed_time = "%02d Days %02d Hours %02d Minutes %02d Seconds " % (d,h,min,int(s))
       # estimated_completion
       return elapsed_time
   else:
#       return s # per s
       return (divmod(s, 60)[0]) # per min
#        return divmod(s, 3600)[0] # per hour

if __name__=='__main__':

    # Check for progress files in directory args[1]
    if len(sys.argv) < 2:
        help_msg = """
For Short Report:

        python ~/msms/src/util/done.py full_path_to_output_directory
        
        e.g.,
        
            python ~/msms/src/util/done.py /Scratchf/output/nra2/

For Long Report:

        python ~/msms/src/util/done.py full_path_to_output_directory 1
        
        e.g.,
        
            python ~/msms/src/util/done.py /Scratchf/output/nra2/ 1

        """
        sys.exit(help_msg)
        #print "using done.txt"
        #tf = open("done.txt","r")
        #source = tf.readline()
        #tf.close()
    else:
        source = sys.argv[1]
       
    long_fmt = 0
    pbar = 0
    if len(sys.argv) == 3:
        if sys.argv[2] == "l":
            long_fmt = 1
        else:
            pbar = 1
    if source.endswith("/"):
        pass
    else:
        source = source.replace ( "\n", "" )
        source = source + "/"

    print "Checking....",source

    # List progress files
    file_list = os.listdir(source)
    file_list.sort()
    pfiles = [x for x in file_list if x.find("progress") != -1]
    if not pfiles:
        sys.exit("No progress files found")

    # Read progress files (1st line gives start and end timesteps for that file)
    print "\nProgress Report\n%s\n" % ("="*115)
    start_tstep = 0
    storage = {}
    complete = {}
    for pf in pfiles:
        read_pf = open(source+pf,"r")
        start = 1
        val = 0
        for line in read_pf:
            if start == 1:
                a_tail = line.rstrip('\n')
                start = 2
                continue
            elif start == 2:
                end_tstep = int(line)-1
                start = 0
                continue
            else:
                val = int(line)
        if not val:
            print "\t%s is just starting up..." % pf
            continue
        prog = val/end_tstep*100.0
        tmp = pf.split("_")
        if pf.find("contours") != -1:
            year = int(tmp[5])
        elif pf.find("center") != -1:
            year = int(tmp[2])
        else:
            import sys; sys.exit("Tweak for %s" % pf)
        syear = str(year)
        dat_file = source+"mcms_"+tmp[1]+"_"+syear+"_"+a_tail+".txt"
        msg = ''
        if prog < 100:
            if val < 5:
                print "\t%s is just starting up..." % dat_file
                continue
            if long_fmt:
                (mode, ino, dev, nlink, uid, gid, size, atime, mtime, ctime) = os.stat(dat_file)
                atime = get_osx_creation_time(dat_file)
                fnc = fractSec(mtime-atime)
                fsize = os.path.getsize(dat_file)
                msg = "\t%s is %0.3f%% complete. Start: %d End: %d Now: %d\n" % (pf,prog,start_tstep,end_tstep,val)
                msg = msg + "\t\tAssociated Att File:\n"
                msg = msg +  "\t\t\tFile Created  : %s\n" % time.ctime(atime)
                msg = msg + "\t\t\tLast Modified : %s\n" % time.ctime(mtime)
                msg = msg + "\t\t\tElapsed Time  : %s\n" % fnc
                if (mtime-atime):
                    step_per_second = val/mtime-atime
                    seconds_to_finish = (end_tstep-val)/step_per_second
                    fnc = fractSec(seconds_to_finish)
                    msg = msg + "\t\t\tFrame Rate (per minute): %f\n" % (val/fractSec(mtime-atime,text=0))
                    msg = msg + "\t\t\tRemaining Time: %s or %s\n" % (fnc,time.ctime(mtime+seconds_to_finish))
                msg = msg + "\t\t\tFile Size     : %0.1f kb" % fsize/1000.0
            if pbar:
                print "\tYear %04d" % year,
                p = ProgressBar(end_tstep)
                p.fill_char = '-'
                p.update_time(val)
                print p
            else:
                msg = "\t%s is %7.3f%% complete. Start: % 5d End: % 5d Now: % 5d" % (pf,prog,start_tstep,end_tstep,val)
            storage[year] = msg
        else:
                complete[year] = 1
    if complete:
        print "\tComplete:"
        PrintColumns(complete.keys(),indent_tag="\t\t",fmt="%s ",cols=5,width=5)
        print
    if not pbar:
        for pf in sorted(storage.keys()):
            print storage[pf]
    print "\n%s\n" % ("="*115)

