#!/usr/bin/env python -tt
"""This module pre-calculates search information

Options/Arguments:

Returns/Creates:

Examples:

Notes:

Author: Mike Bauer  <mbauer@giss.nasa.gov>

Log:
    2012/06  MB - File created, as stand alone module.
"""
from __future__ import division
# Alter python 2 division where division of two *integer* values gives an
#   *integer* result (i.e., 1/2 = 0 but 1.0/2 = 0.5 or 1/2.0 = 0.5) to the
#   more rational python 3+ version where all division returns floats
#   (i.e., 1/2 = 0.5). Adds a new integer division operator so that 1//2 = 0

# Standard modules

# Third party modules

# MCMS modules
from g2l import g2l
from gcd import gcd
from grid2ij import grid2ij

def fill_search(defs_grid,defs,rf,regional_nys,search_radius,sr_extra,gdict,
        ijdict,use_all_lons,verbose):       
    """
    Fill a dictionary of grids falling with search_radius
    """
    # --------------------------------------------------------------------------
    # Setup Section
    # --------------------------------------------------------------------------


    # Pre-bind for speed

    jm = defs_grid["jm"]
    im = defs_grid["im"]
    start_lat = defs_grid["start_lat"]
    start_lon = defs_grid["start_lon"]
    row_end = defs_grid["row_end"]
    row_start = defs_grid["row_start"]
    dlon = defs_grid["dlon"]
    dlat = defs_grid["dlat"]
    maxid = defs_grid["maxid"]
    eq_grid_on_edge = defs_grid["eq_grid_on_edge"]

    extra_verbose = 0
    if verbose or extra_verbose:
        from print_columns import PrintColumns
        from print_gdict import PrintGdict
        print "\n\tSetup rdict\n"
    
    rdict = {}
    for gridid in row_start:
        """
        Define a gridid for every grid point on the entire
        model grid and then makes a dictionary with all the
        points within a great circle radius of (critical_radius).
        
        Here we find the distances from the grid center and it's 4 corners to
        bound the distance between centers found somewhere inside Grid A with
        another found somewhere within Grid B.
        
        Note if grid in the 1st or last row treat as at the pole and use polar
        cap for screen.
        """

        bylat = []
        bylat.append(int(gridid))
        # Find candidate grid centers by latitude
        goingy = 0
        polarity = 1
        check_here = gridid
        col,row,lon,lat = ijdict[gridid]

        # Corners and crosses of a grid (p4)
        #    p0   p1   p2
        #    p3   p4   p5
        #    p6   p7   p8
        #
        # p3_lon,p4_lon,p5_lon
        p_x = g2l(col,start_lon,start_lat,dlon,dlat,jm=jm,lat_lon_flag="lon",
            edge_flag=True,center_flag="center",flag_360=True,
            faux_grids=defs.faux_grids)
        # p1_lat,p4_lat,p5_lat
        p_y = g2l(row,start_lon,start_lat,dlon,dlat,jm=jm,lat_lon_flag="lat",
            edge_flag=True,center_flag="center",faux_grids=defs.faux_grids)
        p = [0]*9
        p[0] = (p_x[0],p_y[0])
        p[1] = (p_x[1],p_y[0])
        p[2] = (p_x[2],p_y[0])
        p[3] = (p_x[0],p_y[1])
        p[4] = (p_x[1],p_y[1])
        p[5] = (p_x[2],p_y[1])
        p[6] = (p_x[0],p_y[2])
        p[7] = (p_x[1],p_y[2])
        p[8] = (p_x[2],p_y[2])

        srad = int(search_radius[row]) #+ sr_extra[row]
        
        # Search "up" to see how many rows to extend
        #   the search poleward. Use gdict
        while goingy < regional_nys[row]:
            # Just walk poleward (like upc above)
            if extra_verbose: 
                print "\tup %s" % ("-"*50)
                PrintGdict(gdict[check_here])
            if polarity == 1:
                takeit = 1 # use upc
            else:
                takeit = 7 # use dnc
            upc = gdict[check_here][takeit]
            bylat.append(int(upc))
            check_here = upc # shift search
            goingy += 1
            if check_here > row_start[-1] - 1 and polarity > 0:
                # NPole search that need to down south after wrap over.
                polarity = -1

        # Search "down" to see how many rows to extend
        #   the search poleward. Use gdict
        goingy = 0
        polarity = -1
        check_here = gridid
        while goingy < regional_nys[row]:
            # Just walk poleward (like dnc above)
            if extra_verbose: 
                print "\tdown %s" % ("-"*50)
                PrintGdict(gdict[check_here])
            if polarity == 1:
                takeit = 1 # use upc
            else:
                takeit = 7 # use dnc
            dnc = gdict[check_here][takeit]
            bylat.append(int(dnc))
            if check_here < im and dnc < im and polarity < 0:
                # SPole search that need to down north after wrap over.
                polarity = 1
            check_here = dnc # shift search
            goingy += 1
            
        bylat = list(set(bylat))
        
        # Double check that all bylat are actually reachable
        nodrop = []
        for eachone in sorted(bylat):
            if eachone in gdict[gridid]:
                if extra_verbose:
                    print "\tNo Need to Compare Neighbor",eachone
                nodrop.append(eachone)
                continue
            nextcol,nextrow = grid2ij(eachone,im,jm)
            q_x = g2l(nextcol,start_lon,start_lat,dlon,dlat,jm=jm,lat_lon_flag="lon",
                edge_flag=True,center_flag="center",flag_360=True,
                faux_grids=defs.faux_grids)
            q_y = g2l(nextrow,start_lon,start_lat,dlon,dlat,jm=jm,lat_lon_flag="lat",
                edge_flag=True,center_flag="center",faux_grids=defs.faux_grids)
            q = [0]*9
            q[0] = (q_x[0],q_y[0])
            q[1] = (q_x[1],q_y[0])
            q[2] = (q_x[2],q_y[0])
            q[3] = (q_x[0],q_y[1])
            q[4] = (q_x[1],q_y[1])
            q[5] = (q_x[2],q_y[1])
            q[6] = (q_x[0],q_y[2])
            q[7] = (q_x[1],q_y[2])
            q[8] = (q_x[2],q_y[2])
            if extra_verbose: 
                print "\tTesting ",eachone
                print "\t\tPval"
                PrintLbox(p)
                print "\t\tQval"
                PrintLbox(q)
                print
            # See if any p to q combo withing search_radius
            can_reach = 0
            for pick in xrange(9):
                pval = p[pick]
                for qick in xrange(9):
                    qval = q[qick]
                    distx = gcd(pval[0],pval[1],qval[0],qval[1])
                    if extra_verbose: 
                        pfmt = "\t\tpval(%d) to qval(%d) DistX: %d"
                        print pfmt % (pick,qick,int(distx)),
                    if int(distx) <= srad:
                        nodrop.append(eachone)
                        can_reach = 1
                        if extra_verbose: print "Inside\n"
                        break
                    if extra_verbose and not can_reach: print "Too Far"
                if can_reach: break
        bylat = [x for x in bylat if x in nodrop]
        if verbose:
            print "\n\tBy_Lat (%d):" % len(bylat)
            PrintColumns(bylat,indent_tag="\t\t",fmt="%5d ",cols=10,width=10)
            print       
        use_polar_cap = 0
        if row in [0,jm-1]:
            # Use polar cap: all longitudes for by_lat rows.
            use_polar_cap = 1
        
        # Now search by longitude for each bylat
        bylon = []
        for eachone in sorted(bylat):
            # Which latitude row is center in?
            for rowe in row_end:
                if eachone <= rowe:
                    nrow = row_end.index(rowe)
                    break
            if extra_verbose:
                print "\tchecking ",eachone," in row",nrow," search_radius",
                print search_radius[row],"+",sr_extra[row]
                print
            if use_polar_cap:
                bylon.extend((x for x
                    in xrange(row_start[nrow],row_end[nrow]+1)))
                bylon = list(set(bylon))
                if extra_verbose: print "\t\tPolar Cap\n"   
            elif nrow in use_all_lons:
                bylon.extend((x for x
                    in xrange(row_start[nrow],row_end[nrow]+1)))
                bylon = list(set(bylon))
                if extra_verbose: print "\t\tall Lons\n"
            else:
                # Check along "plus" longitude at this latitude
                nextgrid = eachone + 1
                polarity = 1
                newadds = 1
                distx = 0.0
                if extra_verbose: print "\t\tnextgrid+",nextgrid
                drop = 1
                while int(distx) <= int(search_radius[row]) + sr_extra[row]:
                    if nextgrid > row_end[nrow]:
                        nextgrid -= im*polarity
                        if extra_verbose: print "\t\tnew nextgrid+ polarity",nextgrid
                    nextcol,nextrow = grid2ij(nextgrid,im,jm)
                    q_x = g2l(nextcol,start_lon,start_lat,dlon,dlat,jm=jm,lat_lon_flag="lon",
                        edge_flag=True,center_flag="center",flag_360=True,
                        faux_grids=defs.faux_grids)
                    q_y = g2l(nextrow,start_lon,start_lat,dlon,dlat,jm=jm,lat_lon_flag="lat",
                        edge_flag=True,center_flag="center",faux_grids=defs.faux_grids)
                    q = [0]*9
                    q[0] = (q_x[0],q_y[0])
                    q[1] = (q_x[1],q_y[0])
                    q[2] = (q_x[2],q_y[0])
                    q[3] = (q_x[0],q_y[1])
                    q[4] = (q_x[1],q_y[1])
                    q[5] = (q_x[2],q_y[1])
                    q[6] = (q_x[0],q_y[2])
                    q[7] = (q_x[1],q_y[2])
                    q[8] = (q_x[2],q_y[2])
                    if extra_verbose: 
                        print "\tTesting ",nextgrid
                        print "\t\tPval"
                        PrintLbox(p)
                        print "\t\tQval"
                        PrintLbox(q)
                        print
                    # See if any p to q combo withing search_radius
                    can_reach = 0
                    for pick in xrange(9):
                        pval = p[pick]
                        for qick in xrange(9):
                            qval = q[qick]
                            distx = gcd(pval[0],pval[1],qval[0],qval[1])
                            if extra_verbose: 
                                pfmt = "\t\tpval(%d) to qval(%d) DistX: %d"
                                print pfmt % (pick,qick,int(distx)),
                            if int(distx) <= srad:
                                nodrop.append(eachone)
                                can_reach = 1
                                if extra_verbose: print "Inside\n"
                                break
                            if extra_verbose and not can_reach: print "Too Far"
                        if can_reach: break                
                    bylon.append(int(nextgrid))
                    if nextgrid not in bylon:
                        bylon.append(int(nextgrid))
                    newadds += 1
                    nextgrid += 1
                    if extra_verbose: print "\t\tnextgrid+",nextgrid," newadds",newadds
                    if newadds >= im: # stop inf loops
                        distx = search_radius[row]*2
                        drop = 0
                # drop bylon added that dropped out of while loop
                if drop and bylon: bylon.pop()
                bylon = list(set(bylon))
                if extra_verbose:
                    print "\t\tBy_Lon+ (%d):" % len(bylon)
                    PrintColumns(bylon,indent_tag="\t\t\t",fmt="%5d ",cols=10,width=10)
                    print
                    
                # Check along "minus" longitude at this latitude
                nextgrid = eachone  - 1
                polarity = -1
                if newadds < im:
                    # "plus" longitude did not span all longitudes
                    distx = 0.0
                    newadds = 1
                    drop = 1
                    if extra_verbose: print "\t\tnextgrid-",nextgrid
                    while int(distx) <= int(search_radius[row]) + sr_extra[row]:
                        if nextgrid < row_start[nrow]:
                            nextgrid -= im*polarity
                            if extra_verbose: print "\t\t\tnew nextgrid- polarity",nextgrid
                        nextcol,nextrow = grid2ij(nextgrid,im,jm)
                        q_x = g2l(nextcol,start_lon,start_lat,dlon,dlat,jm=jm,lat_lon_flag="lon",
                            edge_flag=True,center_flag="center",flag_360=True,
                            faux_grids=defs.faux_grids)
                        q_y = g2l(nextrow,start_lon,start_lat,dlon,dlat,jm=jm,lat_lon_flag="lat",
                            edge_flag=True,center_flag="center",faux_grids=defs.faux_grids)
                        q = [0]*9
                        q[0] = (q_x[0],q_y[0])
                        q[1] = (q_x[1],q_y[0])
                        q[2] = (q_x[2],q_y[0])
                        q[3] = (q_x[0],q_y[1])
                        q[4] = (q_x[1],q_y[1])
                        q[5] = (q_x[2],q_y[1])
                        q[6] = (q_x[0],q_y[2])
                        q[7] = (q_x[1],q_y[2])
                        q[8] = (q_x[2],q_y[2])
                        if extra_verbose: 
                            print "\tTesting ",nextgrid
                            print "\t\tPval"
                            PrintLbox(p)
                            print "\t\tQval"
                            PrintLbox(q)
                            print
                        # See if any p to q combo withing search_radius
                        can_reach = 0
                        for pick in xrange(9):
                            pval = p[pick]
                            for qick in xrange(9):
                                qval = q[qick]
                                distx = gcd(pval[0],pval[1],qval[0],qval[1])
                                if extra_verbose: 
                                    pfmt = "\t\tpval(%d) to qval(%d) DistX: %d"
                                    print pfmt % (pick,qick,int(distx)),
                                if int(distx) <= srad:
                                    nodrop.append(eachone)
                                    can_reach = 1
                                    if extra_verbose: print "Inside\n"
                                    break
                                if extra_verbose and not can_reach: print "Too Far"
                            if can_reach: break
                        bylon.append(int(nextgrid))
                        if nextgrid not in bylon:
                            bylon.append(int(nextgrid))
                        newadds += 1
                        nextgrid -= 1
                        if extra_verbose: print "\t\tnextgrid-",nextgrid," newadds",newadds
                        if newadds >= im: # stop inf loops
                            distx = search_radius[row]*2
                            drop = 0
                    # drop bylon added that dropped out of while loop
                    if drop and bylon: bylon.pop()
                    bylon = list(set(bylon))
                    if extra_verbose:
                        print "\t\tBy_Lon- (%d):" % len(bylon)
                        PrintColumns(bylon,indent_tag="\t\t\t",fmt="%5d ",cols=10,width=10)
                        print
                else:
                    if extra_verbose: print "By_Lon- skipped\n"
        if verbose:
            print "\t\tBy_Lon (%d):" % len(bylon)
            PrintColumns(bylon,indent_tag="\t\t\t",fmt="%5d ",cols=10,width=10)
            print

        # Merge center list
        regional = set(bylat)
        regional.update(bylon)
        rdict[row] = list(regional)
        if verbose:
            print "\trdict (%d):" % len(rdict[row])
            PrintColumns(rdict[row],indent_tag="\t\t",fmt="%5d ",cols=10,width=10)
            print

        verbose1 = 0
        flat_map = 1
        polar_map = 0
        sep_alts = 0
        if verbose1:
            from shift_gridid import shift_gridid
            import numpy
            from plot_map import plotmap
            from plot_map import plotmap_polar
            from print_columns import PrintColumns

            rvals = rdict[row]
            print "\n\trdict[%d] (len %d):" % (row,len(rvals))
            if polar_map:
                r_locs = []
                for cgridid in rvals:
                    if cgridid == gridid: continue
                    r_locs.append((ijdict[cgridid][2],ijdict[cgridid][3]))
            
            # Use shift_gridid to move to center of map grid
            halfway = int(im/2)
            drow = 0
            dcol = halfway
            alt_rvals = rdict[row]
            c_halo = numpy.array([x for y in xrange(3) for x in xrange(im)],
                    dtype=numpy.int32)
            alt_gridid = shift_gridid(gridid,dcol,drow,c_halo,halfway,im,jm)  
            print "\n\talt_rdict[%d] (len %d):" % (row,len(alt_rvals))
            tmp_r = []
            for cgridid in alt_rvals:
                tmp_r.append(shift_gridid(cgridid,dcol,drow,c_halo,halfway,im,jm))
            alt_rvals = tmp_r
            if polar_map:
                alt_r_locs = []
                for cgridid in alt_rvals:
                    if cgridid == alt_gridid: continue
                    if sep_alts:
                        alt_r_locs.append((ijdict[cgridid][2],ijdict[cgridid][3]))
                    else:
                        r_locs.append((ijdict[cgridid][2],ijdict[cgridid][3]))
            # Make a plot of this rdict entry for debugging.
            rf.fig_format = ".png"
            blank = numpy.zeros(im*jm)
            alt_blank = numpy.zeros(im*jm)
            lons = defs_grid["lons"]
            lats = defs_grid["lats"]
            bl =  [15.,-15.]
            hnames = ["nh","sh"]
            i,j,lon,lat = ijdict[gridid]
            if defs_grid["faux_grids"]:
                # Reset polar most latitudes to poles for plotting
                lats[0] = -90.0
                lats[-1] = 90.0
                if j == 0: lat = -90.0
                if j == jm-1: lat = 90.0
            if lat >= 0:
                h = 0
            else:
                h = 1
            for pnt in rvals: blank[pnt] = 1.0
            if sep_alts:
                for pnt in alt_rvals: alt_blank[pnt] = 1.0
            else:
                for pnt in alt_rvals: blank[pnt] = 0.5
            blank.shape = (jm,im)
            alt_blank.shape = (jm,im)
            fmt = "%srdict_%05d%s"
            pname = fmt % (rf.shared_path,gridid,rf.fig_format)
            if sep_alts:
                alt_fmt = "%salt_rdict_%05d%s"
                alt_pname = alt_fmt % (rf.shared_path,alt_gridid,rf.fig_format)
            tfmt = u"Regional Search at %5.2f\u00b0%s"
            title = tfmt % (lat,hnames[h][0].upper())
            if flat_map:
                plot = plotmap(missing=-1.0,color_scheme="gray_r",
                    nocolorbar='nocolorbar',lon_0=180.0)
                plot.create_fig()
                plot.add_field(lons,lats,blank,ptype='imshow')
                plot.finish(pname,title=title)
                print "\tMade figure %s" % (pname)
                blank.shape = (jm,im)
                if sep_alts:
                    plot.create_fig()
                    plot.add_field(lons,lats,alt_blank,ptype='imshow')
                    plot.finish(alt_pname,title=title)
                    print "\tMade figure %s" % (alt_pname)
                    blank.shape = (jm,im)
                    alt_blank.shape = (jm,im)
            if polar_map:
                # Give only data from the correct hemisphere.
                if h == 0:
                    zlats = numpy.array([x for x in lats if x >= 0])
                    if eq_grid_on_edge:
                        zval = blank[defs_grid["eq_grid"]+1:,:]
                    else:
                        zval = blank[defs_grid["eq_grid"]:,:]
                    lon0 = 180.0
                else:
                    zlats = numpy.array([x for x in lats if x <= 0])
                    if eq_grid_on_edge:
                        zval = blank[:defs_grid["eq_grid"]+1,:]
                    else:      
                        zval = blank[:defs_grid["eq_grid"]+1,:]
                    lon0 = 180.0
                pname = pname.replace(rf.fig_format,"_polar"+rf.fig_format)
                fplot = plotmap_polar(missing=-1,color_scheme="gray_r",
                    hemi=hnames[h],mproj='laea',bounding_lat=bl[h],
                    lon_0=lon0,cints=[0.0,1.0],nocolorbar='nocolorbar')
                fplot.create_fig()
                fplot.add_pcolor(lons,zlats,zval)
                # Put gray dots on included grids in search_radius
                fplot.add_pnts(r_locs,marker=",",msize=0.2,mfc='0.4',mec='0.4')
                # Put red x at grid center
                fplot.add_pnts([(lon,lat),],marker="x",msize=5,mfc='r',
                    mec='r',lw=5)
                fplot.add_extras(circles=[bl[h],2*bl[h]],meridians=[0,180])
                fplot.finish(pname,title=title)
                print "\t\t\tMade %s" % (pname)
                if sep_alts:
                    if h == 0:
                        zlats = numpy.array([x for x in lats if x >= 0])
                        if eq_grid_on_edge:
                            zval = alt_blank[defs_grid["eq_grid"]+1:,:]
                        else:
                            zval = alt_blank[defs_grid["eq_grid"]:,:]
                        lon0 = 180.0
                    else:
                        zlats = numpy.array([x for x in lats if x <= 0])
                        if eq_grid_on_edge:
                            zval = alt_blank[:defs_grid["eq_grid"]+1,:]
                        else:      
                            zval = alt_blank[:defs_grid["eq_grid"]+1,:]
                        lon0 = 180.0
                    alt_pname = alt_pname.replace(rf.fig_format,"_polar"+rf.fig_format)
                    fplot.create_fig()
                    fplot.add_pcolor(lons,zlats,zval)
                    # Put gray dots on included grids in search_radius
                    fplot.add_pnts(alt_r_locs,marker=",",msize=0.2,mfc='0.4',mec='0.4')
                    # Put red x at grid center
                    fplot.add_pnts([(lon,lat),],marker="x",msize=5,mfc='r',
                        mec='r',lw=5)
                    fplot.add_extras(circles=[bl[h],2*bl[h]],meridians=[0,180])
                    fplot.finish(alt_pname,title=title)
                    print "\t\t\tMade %s" % (alt_pname)
                
    return rdict 

