#!/usr/bin/env python -tt
"""
Utility functions for MCMS project.

Author: Mike Bauer  <mbauer@giss.nasa.gov>

Log:
2010/08 MB - File created.
"""

from __future__ import division
# Alter python 2 division where division of two *integer* values gives an
#   *integer* result (i.e., 1/2 = 0 but 1.0/2 = 0.5 or 1/2.0 = 0.5) to the
#   more rational python 3+ version where all division returns floats
#   (i.e., 1/2 = 0.5). Adds a new integer division operator so that 1//2 = 0

from __future__ import print_function
# Alter python 2 to use python 3 print as a function method. This makes
#   it easier to use this code on python 2 or 3 installations.

# Standard library imports
from math import ceil

# Third party module imports

# ----------------------------------------------------------------------------
class HistogramToScreen():
    """
    Draw a simple histogram (horizontal or vertical) to screen.

    Input:
    ^^^^^
    data - List or numpy array (int or float)
        Data to histogram.
    bins - Int or List
        If bins is a integer then it gives the number of equal-width
            bins for histogram using data.min(), data.max().
        If bins is a list it defines the bin edges, including the
            rightmost edge.
        See documentation for numpy.histogram!

    Output:
    ^^^^^^
    msg - String
        Histogram

    Examples:
    ^^^^^^^^
    These are written in doctest format, and show how expected use and output.

    Notes/Warnings:
    ^^^^^^^^^^^^^^

    From documentation for numpy.histogram!

        All but the last (righthand-most) bin is half-open.  In other words,
            if 'bins' is [1, 2, 3, 4] then the first bin is [1, 2]
            (including 1, but excluding 2) and the second is [2, 3].
            The last bin, however, is [3, 4], which *includes* 4.

    So, if you want 4 values to fall in their own bin then bins
        should be [1,2,3,4,5] in the above example.

    Based on http://pyinsci.blogspot.com/2009/10/ascii-histograms.html

    Author: Mike Bauer  <mbauer@giss.nasa.gov>


    Log:
        2010/08 MB - File created.
    """

    def __init__(self,data,bins):
        self.h = histogram(data,bins=bins)
        if isinstance(bins,int):
            self.nbins = bins
        else:
            self.nbins = len(bins)

        #print(self.h)
        #print(self.nbins)
        #print("\n")

    def horizontal(self,height=4,character ='|'):
        """
        Draw a horizontal histogram to screen.

        Input:
        ^^^^^
        height - Integer
            Maximum length of the histogram bars. Default = 4
        character - String
            Character to construct the bars with. Default '|'

        Output:
        ^^^^^^
        msg - String
            Histogram

        Examples:
        ^^^^^^^^
        These are written in doctest format, and show how expected use and output.

            from numpy.random import normal
            d = normal(size=1000)
            h = HistogramToScreen(d,25)
            histo = h.horizontal(5,'|')
            print(histo)


            101              ||
                           ||||
                          ||||||||
                        |||||||||||
                     |||||||||||||||
            -3.28                         2.93


        Notes/Warnings:
        ^^^^^^^^^^^^^^

        Based on http://pyinsci.blogspot.com/2009/10/ascii-histograms.html

        Author: Mike Bauer  <mbauer@giss.nasa.gov>

        Log:
            2010/08 MB - File created.
            2010/10 MB - Fixed offset bug on top count line.
        """
        his = """"""
        # Create a list of bars
        max_cnt = self.h[0].max()
        # Divide each bin count by max_cnt to normalize
        #   then multiply by height to scale to max bar length.
        #   Also convert bars to a list
        bars = list((self.h[0]/max_cnt)*height)
        # Now round and convert to integers for display
        bars = [int(round(x,1)) for x in bars]
        for l in reversed(range(1,height+1)):
            line = ""
            if l == height:
                # histogram top count
                line = '%s ' % max_cnt
            else:
                # add leading spaces
                line = ' '*(len(str(max_cnt))+1)
            for c in bars:
                if c >= ceil(l):
                    line += character
                else:
                    line += ' '
            line += '\n'
            his += line
        msg_right = '%.2f' % self.h[1][0]
        msg_mid = ' '*(self.nbins)
        msg_left = '%.2f \n' % self.h[1][-1]
        his += msg_right+msg_mid+msg_left

        return his

    def vertical(self,height=20,character ='|'):
        """
        Draw a vertical histogram to screen.

        Input:
        ^^^^^
        height - Integer
            Maximum length of the histogram bars. Default = 20
        character - String
            Character to construct the bars with. Default '|'

        Output:
        ^^^^^^
        msg - String
            Histogram

        Examples:
        ^^^^^^^^
        These are written in doctest format, and show how expected use and output.

            from numpy.random import normal
            d = normal(size=1000)
            h = HistogramToScreen(d,10)
            histo = h.vertical(15,'*')
            print(histo)
                                             245
            -2.7     1.60% :*
            -2.1     4.60% :***
            -1.5    11.00% :*******
            -0.88   20.60% :*************
            -0.27   24.50% :***************
             0.33   19.70% :************
             0.93   10.10% :******
             1.5     6.30% :****
             2.1     1.40% :*
             2.7     0.20% :

        Notes/Warnings:
        ^^^^^^^^^^^^^^

        Based on http://pyinsci.blogspot.com/2009/10/ascii-histograms.html

        Author: Mike Bauer  <mbauer@giss.nasa.gov>

        Log:
            2010/08 MB - File created.
        """
        his = """"""
        # Create rounded bin labels
        labels = ['% .2g' % round(self.h[1][n],2)
                for n in xrange(len(self.h[1])-1)]
        lxl = [len(l) for l in labels]
        long_lxl = max(lxl)
        # Find percent of total count in each bin
        total_cnts = sum(self.h[0])
        frac = list(100.0*(self.h[0]/total_cnts))
        labels = ["%s%s  %6.2f%% :" % (labels[i],' '*(long_lxl-lxl[i]),
            round(frac[i],2)) for i in xrange(len(frac))]
        lxl = [len(l) for l in labels]
        long_lxl = max(lxl)
        # Create a list of bars
        max_cnt = self.h[0].max()
        # Divide each bin count by max_cnt to normalize
        #   then multiply by height to scale to max bar length.
        #   Also convert bars to a list
        bars = list((self.h[0]/max_cnt)*height)
        # Now round and convert to integers for display
        bars = [int(round(x,0)) for x in bars]
        # Note due to numpy.histogram the last label is not
        #   used or needed.
        #print("Cnts (%d)   : %s" % (len(self.h[0]),self.h[0]))
        #print("Bins* (%d)  : %s" % (len(self.h[1]),self.h[1]))
        #print("Labels (%d) : %s" % (len(labels),labels))
        #print("Bars  (%d)  : %s" % (len(bars),bars))
        # Draw a line showing the maximum count
        left_buffer = ' '*(max(bars)+2+long_lxl)
        label = '%s\n' % max_cnt
        his = left_buffer+label
        # Draw each bar
        for i,c in enumerate(bars):
            his += "%s%s\n" % (labels[i],character*c)

        return his


