#!/usr/bin/env python -tt
"""
Utility functions for MCMS project.

Author: Mike Bauer  <mbauer@giss.nasa.gov>

Log:
2010/08 MB - File created.
"""

from __future__ import division
# Alter python 2 division where division of two *integer* values gives an
#   *integer* result (i.e., 1/2 = 0 but 1.0/2 = 0.5 or 1/2.0 = 0.5) to the
#   more rational python 3+ version where all division returns floats
#   (i.e., 1/2 = 0.5). Adds a new integer division operator so that 1//2 = 0

from __future__ import print_function
# Alter python 2 to use python 3 print as a function method. This makes
#   it easier to use this code on python 2 or 3 installations.

# Standard library imports
import sys
import os
from math import ceil
import gzip

# Third party module imports
import pprint
import numpy
from numpy import histogram
import netCDF4 as NetCDF
import textwrap
import datetime
import cPickle

# ----------------------------------------------------------------------------
def ParseRunFile(run_file_template,argv,in_path):
    """
    Reads and parses the command line input.

    Input:
    ^^^^^
    in_string - String
       Long string to be broken.

    Output:
    ^^^^^^
    out_string - String
       A reformatted version of in_string.

    Examples:
    ^^^^^^^^
    These are written in doctest format, and show how expected use and output.

    Notes/Warnings:
    ^^^^^^^^^^^^^^

    Author: Mike Bauer  <mbauer@giss.nasa.gov>

    Log:
        2011/01 MB - File created.
    """
    # ----------------------------------------------------------------------------
    from optparse import OptionParser
    import shutil
    import tempfile
    msg_need_run_file = "\nPlease provide a run_file name.\n"
    parser = OptionParser(usage="usage: %prog [options] run_file")
    parser.add_option("-v", "--verbose",
            action="store_true", dest="verbose", default=False,
            help="Report what is done [default = False]")
    parser.add_option("-c", "--create",
            action="store_true", dest="create_run_file", default=False,
            help="Create a New Run File and Quit [default = False]")
    parser.add_option("-n", "--no_ask",
            action="store_true", dest="no_ask", default=False,
            help="Suppress File Overwrite Query [default = False]")
    
    (options, args) = parser.parse_args()
    if len(argv) < 2:
        parser.print_help()
        return 1
    # Deal w/ run_file
    if not args:
        print(msg_need_run_file)
        parser.print_help()
        return 1
    run_file = args[0]
    if options.no_ask:
        # create temporary file
        tempFile = tempfile.NamedTemporaryFile(suffix=".txt",prefix="no_ask",dir=in_path,delete=False)
        tempFile.seek(0)
        #print (tempFile.name)  
    # Assume that all RunFiles are in the subdirectory rf_files
    try:
        # Add path to this file to run_file if run_file does not
        #   contain a path already
        if not os.path.split(run_file)[0]:
            #print("\n\tAdded current path to run_file")
            run_file = in_path+"/"+run_file
        elif os.path.split(run_file)[0] != in_path:
            run_file = in_path+"/"+run_file
        # Add .py to run_file if not present
        basename, extension = os.path.splitext(run_file)
        if extension != ".py":
            #print("\n\tAdded .py extension to run_file")
            run_file = basename+".py"
        # Check is run_file exists, if asked to create a new run_file then
        #   warn about overwriting, else create it, otherwise warn that
        #   run_file not found
        if FileExists(run_file,__name__,verbose=False):
            if options.create_run_file:
                # Warn about overwriting
                msg = "\n\tWarning: Overwrite run_file [%s]?"
                print(msg % run_file,end="")
                answer = query_yes_no("\tContinue?")
                if not answer:
                    return 1
                # Create new run_file from template
                shutil.copyfile(run_file_template,run_file)
                msg = "\n\trun_file [%s] created."
                print(msg % run_file)
                return 1
        else:
            if options.create_run_file:
                # Create new run_file from template
                shutil.copyfile(run_file_template,run_file)
                msg = "\n\trun_file [%s] created."
                print(msg % run_file)
                return 1
            else:
                FileExists(run_file,__name__)
                print("Error in ParseRunFile1")
                return 1
        # Import run_file
        # Assume that all RunFiles are in the subdirectory rf_files
        getthis = "rf_files."+basename.split("/")[-1] # should be 'rf_model_cf' etc
        __import__(getthis)
        rf = sys.modules[getthis]
        rf.getthis = getthis 
    except:
        print("Error in ParseRunFile2")
        return 1
    return rf

def WrapList(in_list,indent,width):
    in_string = ", ".join([x for x in in_list])
    breakup = textwrap.wrap(in_string,width)
    out_string =  indent.join([x for x in breakup])
    return out_string

def WrapHeader(in_string,starter,width=70):
    """
    Breaks a long string to a fixed width (for header info).

    Input:
    ^^^^^
    in_string - String
       Long string to be broken.
    starter - Integer
       Accommodation size for any extra space to be added before in_string.
       That is, for a label such as 'File Description: '. Can be Zero.
    width - Integer
       Length of each new line of text (default 70 characters).
    Output:
    ^^^^^^
    out_string - String
       A reformated version of in_string.

 file_description = ("It was a dark and stormy night; the rain fell in torrents"+
            " -- except at occasional intervals, when it was checked by a" +
            " violent gust of wind which swept up the streets (for it is in" +
            " London that our scene lies), rattling along the housetops, and" +
            " fiercely agitating the scanty flame of the lamps that struggled" +
            " against the darkness.)")


    Examples:
    ^^^^^^^^
    These are written in doctest format, and show how expected use and output.

    Notes/Warnings:
    ^^^^^^^^^^^^^^

    Author: Mike Bauer  <mbauer@giss.nasa.gov>

    Log:
        2010/10 MB - File created.
    """
    # ----------------------------------------------------------------------------
    in_string = " "*starter+in_string
    breakup = textwrap.wrap(in_string,width)
    breakup[0] = breakup[0].lstrip()
    breakup = ["#"+x for x in breakup]
    out_string = "\n".join([x for x in breakup])
    out_string = out_string[1:]
    out_string = out_string.replace("#","#\t")

    return out_string

def FixWarning(fix_message,caller):
    """
    Print a message about something to fix and where that is.

    Input:
    ^^^^^
    fix_message - String
        Message to output about what needs fixing.
    caller - String
        Name of the module that message refers to.

    Output:
    ^^^^^^
    msg - String
        Message for stdout

    Examples:
    ^^^^^^^^
    These are written in doctest format, and show how expected use and output.
    >>> FixWarning("Might not need all this as omni file",
    ...     FixWarning.__name__) # doctest: +NORMALIZE_WHITESPACE
    '*FIX MESSAGE* from module FixWarning: Might not need all this as omni file'

    Notes/Warnings:
    ^^^^^^^^^^^^^^

    Author: Mike Bauer  <mbauer@giss.nasa.gov>

    Log:
        2010/08 MB - File created.
    """
    msg = "*FIX MESSAGE* from module %s: %s" % (caller,fix_message)
    return msg

def PullData(pd_in_path,pd_read_year,pd_var_data,pd_var_time,pd_lat_flip,
        pd_lon_shift,pd_scale,pd_amount="all"):
    """
    Read in a netcdf data file.

    This function takes an input directory (in_path) and finds all
       the available files to be read. Then the requested file is
       read and put into a single array.

    Input:
    ^^^^^
    pd_in_path - String
        Full path the directory hosting the data to be read. IF pd_in_path
        is a netcdf file (.nc) then rather than search pd_in_path for a
        file with pd_read_year, pd_var_data etc. it just opens that specific
        file.
    pd_read_year - Integer
        Specific year of data requested... assumes datafiles are named
            and separated by year.
    pd_var_data - String
        Name of the netcdf data variable to be extracted.
    pd_var_time - String
        Name of the netcdf time variable
    lat_flip - Boolean
        Force data and latitude to be reverse order from data file.
    lon_shift - Integer
        Force longitude to shift by lon_shift so 1st index contains the
            the Greenwich Meridian.
    pd_scale - Float
        Scales the variable as desired (can be 1.0 to do nothing).
    pd_amount = Integer
        If set to 'all', the default, then the whole time series of data
            is returned.
        If set to 'info', then everything is the same as the default except only
            a single time step of the main variable is returned.
        If set to an integer value, then only main variable data for that time
            step is returned.
        'info' and integer values can be used when the data set is very large
        and reading the whole thing into memory is slow or not possible.

    Output:
    ^^^^^^
    pd_data - Numpy array
        Array of the data requested by pd_var_data.
    pd_times - Numpy array
        Array of the time data.
    pd_time_units - String
        Time units for the data file.

    Examples:
    ^^^^^^^^
    These are written in doctest format, and show how expected use and output.

    >>> x,y,z = PullData("/Users/mbauer/Work/sysnopsis/refactored/src/test/",
    ...        2009,"mslp","time",1,0,0.01)
    >>> print(x.shape,y.shape,z) #doctest: +NORMALIZE_WHITESPACE
    (1460, 73, 144) (1460,) hours since 1800-1-1 00:00:0.0
    >>> print(x[0,0,0],x[0,0,-1],
    ...     x[0,-1,0],x[0,-1,-1]) #doctest: +NORMALIZE_WHITESPACE
    997.5 997.5 1019.2 1019.2

    Notes/Warnings:
    ^^^^^^^^^^^^^^

    Author: Mike Bauer  <mbauer@giss.nasa.gov>

    Log:
        2008/01  MB - File created.
        2008/10  MB - Added input checks, docstring.
        2008/10  MB - Fixed error where file_list was unordered...
        2009/11  MB - Updated to version 4.
        2010/08  MB - Starting to update to version 5.
        2010/08  MB - Moved to mcms_utilities.
        2012/05  MB - Added pd_amount to allow for large file reads
    """
    if pd_in_path.find(".nc") != -1:
        # pd_in_path is a specific file rather than a directory
        found_file  = pd_in_path
        # Open file to read, use the netcdf 3 format
        pd_in_path = ""
    else:
        import fnmatch
        pattern = '*%04d*.nc' % int(pd_read_year)
        matches = fnmatch.filter(os.listdir(pd_in_path), pattern)
        if not matches:
            msg = "WARNING: Cannot file for year %d in directory %s"
            print(msg)
            sys.exit(1)
        elif len(matches) > 1:
            msg = "WARNING: Multiple matches for year %d in directory %s"
            print(msg)
            print("Matches",matches)
            sys.exit(1)
        found_file = matches[0]
        #print ("\nFound File:",found_file)

    # Open file to read, use the netcdf 3 format
    nc_in = NetCDF.Dataset(pd_in_path+found_file,'r',format='NETCDF3_CLASSIC')

    if isinstance(pd_amount,int):
        # Only return data
        #
        # Pull slp_var, assume has dimensions of [time,lat,lon]
        if pd_var_time:
            got_data = numpy.multiply(numpy.array(nc_in.variables[pd_var_data][pd_amount,:,:],
                dtype=numpy.float32,copy=1),pd_scale)
        else:
            # No time dim for this file
            got_data = numpy.multiply(numpy.array(nc_in.variables[pd_var_data][:,:],
                dtype=numpy.float32,copy=1),pd_scale)
        if pd_lat_flip:
            got_data = got_data[::-1,:]
        if pd_lon_shift:
            got_data = numpy.roll(got_data,pd_lon_shift,axis=1)
        nc_in.close()
        # Add fake time dimention
        got_data.shape = (1,got_data.shape[0],got_data.shape[1])
        return got_data
    else:        
        # Pull var_time
        times = nc_in.variables[pd_var_time][:]
        the_time_units = nc_in.variables[pd_var_time].units
        times = times.astype(numpy.float32)
        if pd_amount == 'info':
            nc_in.close()     
            return times,the_time_units
        # Pull slp_var, assume has dimensions of [time,lat,lon]
        got_data = numpy.multiply(numpy.array(nc_in.variables[pd_var_data][:],
            dtype=numpy.float32,copy=1),pd_scale)
        if pd_lat_flip:
            got_data = got_data[:,::-1,:]
        if pd_lon_shift:
            got_data = numpy.roll(got_data,pd_lon_shift,axis=2)
        nc_in.close()
        return got_data,times,the_time_units

#def CompletedCallback(step):
#    """
#    Reports the process of a multiprocessing session.

#    Input:
#    ^^^^^
#    step - String, Integer or Float (not List etc).
#        Return value from the function being called by

#    Output:
#    ^^^^^^
#    results - List
#        Appends step the global list results
#    msg - String
#        Message sent to sys.stdout

#    Examples:
#    ^^^^^^^^
#    These are written in doctest format, and show how expected use and output.
#
#    >>> CompletedCallback(1979) #doctest: +NORMALIZE_WHITESPACE
#    Completed 1979

#    >>> CompletedCallback("Giant Mouse") #doctest: +NORMALIZE_WHITESPACE
#    Completed Giant Mouse

#    Notes/Warnings:
#    ^^^^^^^^^^^^^^
#    Here is an example program to give an idea of the usage.
#
#    import multiprocessing as mp

#    def dummyTask(step):
#        # Do something
#        return(step)

#    if __name__ == '__main__':

#        PROCESSORS =  16
#        print("Master processid = %s" % os.getpid())
#        pool = mp.Pool(PROCESSORS)
#        start_year = 1979
#        end_year = 2009
#        results = []
#        for loop_year in xrange(start_year,end_year+1):
#            pool.apply_async(dummyTask,(loop_year,),callback=completedCallback)
#        pool.close()
#        pool.join()
#
#    This returns a stream of messaged to sys.stdout like

#    Completed 1979
#    Completed 1980
#    ....

#    and results becomes.
#    print results
#    > [1979, 1980, 1981, 1982, 1983, 1984, 1985, 1986, 1987, 1988, 1989, 1990,
#    1991, 1992, 1993, 1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002,
#    2003, 2004, 2005, 2006, 2007, 2008, 2009]


#    Author: Mike Bauer  <mbauer@giss.nasa.gov>

#    Log:
#        2010/08 MB - File created.
#    """
#
#    # Assumes the calling program has a global list named results
#    #   see example above.
#    global results
#
#    print("\tCompleted %s" % str(step),file=sys.stdout)
#
#    results.append(step)
#
#    return 0

#def CompletedCallbackPbar(step):
#    """
#    Reports the process of a multiprocessing session.

#    Same as CompletedCallback except intended for use with
#    global defined ProgressBar instead of printing a string.

#    """
#
#    # Assumes the calling program has global ProgressBar defined.
#    global results
#    global mp_pbar
#    results.append(step)
#    mp_pbar.update_time(len(results))
#    print(mp_pbar)
#
#    return 0

def Is_It_Available(program):
    """
    Is some program available on this system?

    Simple test to see if a program is available for use by MCMS.

    Input:
    ^^^^^
    program - String
        The name of the program for the existence/availability test.

    Output:
    ^^^^^^
    result - Boolean
        True/False if program is found on the system.

    Examples:
    ^^^^^^^^
    These are written in doctest format, and show how expected use and output.

    >>> Is_It_Available("python")
    True

    >>> Is_It_Available("bython")
    False

    Notes/Warnings:
    ^^^^^^^^^^^^^^
    Pretty much a duplicate of the recipe suggested under the question
    'Test if executable exists in Python?' at http://stackoverflow.com

    Author: Mike Bauer  <mbauer@giss.nasa.gov>

    Log:
        2009/11 MB - File created.
    """
    def is_exe(fpath):
        return os.path.exists(fpath) and os.access(fpath, os.X_OK)
    fpath, fname = os.path.split(program)
    if fpath:
        if is_exe(program):
            return True
    else:
        for path in os.environ["PATH"].split(os.pathsep):
            exe_file = os.path.join(path, program)
            if is_exe(exe_file):
                return True
    return False

def query_yes_no(question, default="yes"):
    """Ask a yes/no question via raw_input() and return their answer.

    "question" is a string that is presented to the user.
    "default" is the presumed answer if the user just hits <Enter>.
        It must be "yes" (the default), "no" or None (meaning
        an answer is required of the user).

    The "answer" return value is one of "yes" or "no".
    """
    valid = {"yes":True,   "y":True,  "ye":True,
             "no":False,     "n":False}
    if default == None:
        prompt = " [y/n] "
    elif default == "yes":
        prompt = " [Y/n] "
    elif default == "no":
        prompt = " [y/N] "
    else:
        raise ValueError("invalid default answer: '%s'" % default)

    while True:
        sys.stdout.write(question + prompt)
        choice = raw_input().lower()
        if default is not None and choice == '':
            return valid[default]
        elif choice in valid:
            return valid[choice]
        else:
            sys.stdout.write("Please respond with 'yes' or 'no' "\
                             "(or 'y' or 'n').\n")

def FileExists(check_this,calling_routine,verbose=True):
    """
    Is some file available on this system?

    Input:
    ^^^^^
    check_this - String
        The name of the file for the existence/availability test.
    calling_routine - String
        The name of the routine calling FileExists

    Output:
    ^^^^^^
    result - Boolean
        True/False if program is found on the system.

    Examples:
    ^^^^^^^^
    These are written in doctest format, and show how expected use and output.

    >>> FileExists("test.dat",__name__)
    True

    >>> FileExists("test.txt",__name__)
    Sorry, the file test.nc cannot be found by __main__

    Notes/Warnings:
    ^^^^^^^^^^^^^^
    Author: Mike Bauer  <mbauer@giss.nasa.gov>

    Log:
        2009/11 MB - File created.
    """
    if os.path.exists(check_this):
        return True
    else:
        msg = "Sorry, the file %s cannot be found by %s"
        if verbose: print(msg % (check_this,calling_routine),file=sys.stderr)
        return False

def PrintColumns(the_list,indent_tag,fmt,cols,width,sort_me=1):
    """

    Prints a nicely formated list.

    Input:
    ^^^^^
    the _list - List
        List of values to print
    indent_tag - String
        String containing the tabs wanted in the formatted output
    fmt - String
        Format statement for the individual elements of the list
    cols - Integer
        The number of columns to create
    width - Integer
        The interacts with cols and fmt to set spacing of the output.
    sort_me - Boolean
        The default is for a copy ofthe_list to be sorted before printing.

    Output:
    ^^^^^^
    None - String
        Message sent to sys.stdout

    Examples:
    ^^^^^^^^
    These are written in doctest format, and show how expected use and output.

    #>>> PrintColumns(list(range(100,120)),indent_tag="\t",fmt="% 3d",cols=5,width=5) # doctest: +NORMALIZE_WHITESPACE
       100 101 102 103 104 105 106
       107 108 109 110 111 112 113
       114 115 116 117 118 119

    Notes/Warnings:
    ^^^^^^^^^^^^^^

    Author: Mike Bauer  <mbauer@giss.nasa.gov>

    Log:
        2010/08 MB - File created.
    """

    line = 0
    if not the_list:
        print(indent_tag+"   None")
        return 1
    if sort_me:
        a_list = sorted(the_list)
    else:
        a_list = the_list
    wide = width*cols
    end = len(a_list)
    i = 0
    for g in a_list:
        i += 1
        tag = ""
        if not line:
            tag = indent_tag
        if i == end:
            print(tag+fmt % (g),end="\n")
        elif line <= wide:
            print(tag+fmt % (g),end="")
            line += width
        else:
            print(tag+fmt % (g),end="\n")
            line = 0
    return 0

class HistogramToScreen():
    """
    Draw a simple histogram (horizontal or vertical) to screen.

    Input:
    ^^^^^
    data - List or numpy array (int or float)
        Data to histogram.
    bins - Int or List
        If bins is a integer then it gives the number of equal-width
            bins for histogram using data.min(), data.max().
        If bins is a list it defines the bin edges, including the
            rightmost edge.
        See documentation for numpy.histogram!

    Output:
    ^^^^^^
    msg - String
        Histogram

    Examples:
    ^^^^^^^^
    These are written in doctest format, and show how expected use and output.

    Notes/Warnings:
    ^^^^^^^^^^^^^^

    From documentation for numpy.histogram!

        All but the last (righthand-most) bin is half-open.  In other words,
            if 'bins' is [1, 2, 3, 4] then the first bin is [1, 2]
            (including 1, but excluding 2) and the second is [2, 3].
            The last bin, however, is [3, 4], which *includes* 4.

    So, if you want 4 values to fall in their own bin then bins
        should be [1,2,3,4,5] in the above example.

    Based on http://pyinsci.blogspot.com/2009/10/ascii-histograms.html

    Author: Mike Bauer  <mbauer@giss.nasa.gov>


    Log:
        2010/08 MB - File created.
    """

    def __init__(self,data,bins):
        self.h = histogram(data,bins=bins)
        if isinstance(bins,int):
            self.nbins = bins
        else:
            self.nbins = len(bins)

        #print(self.h)
        #print(self.nbins)
        #print("\n")

    def horizontal(self,height=4,character ='|'):
        """
        Draw a horizontal histogram to screen.

        Input:
        ^^^^^
        height - Integer
            Maximum length of the histogram bars. Default = 4
        character - String
            Character to construct the bars with. Default '|'

        Output:
        ^^^^^^
        msg - String
            Histogram

        Examples:
        ^^^^^^^^
        These are written in doctest format, and show how expected use and output.

            from numpy.random import normal
            d = normal(size=1000)
            h = HistogramToScreen(d,25)
            histo = h.horizontal(5,'|')
            print(histo)


            101              ||
                           ||||
                          ||||||||
                        |||||||||||
                     |||||||||||||||
            -3.28                         2.93


        Notes/Warnings:
        ^^^^^^^^^^^^^^

        Based on http://pyinsci.blogspot.com/2009/10/ascii-histograms.html

        Author: Mike Bauer  <mbauer@giss.nasa.gov>

        Log:
            2010/08 MB - File created.
            2010/10 MB - Fixed offset bug on top count line.
        """
        his = """"""
        # Create a list of bars
        max_cnt = self.h[0].max()
        # Divide each bin count by max_cnt to normalize
        #   then multiply by height to scale to max bar length.
        #   Also convert bars to a list
        bars = list((self.h[0]/max_cnt)*height)
        # Now round and convert to integers for display
        bars = [int(round(x,1)) for x in bars]
        for l in reversed(range(1,height+1)):
            line = ""
            if l == height:
                # histogram top count
                line = '%s ' % max_cnt
            else:
                # add leading spaces
                line = ' '*(len(str(max_cnt))+1)
            for c in bars:
                if c >= ceil(l):
                    line += character
                else:
                    line += ' '
            line += '\n'
            his += line
        msg_right = '%.2f' % self.h[1][0]
        msg_mid = ' '*(self.nbins)
        msg_left = '%.2f \n' % self.h[1][-1]
        his += msg_right+msg_mid+msg_left

        return his

    def vertical(self,height=20,character ='|'):
        """
        Draw a vertical histogram to screen.

        Input:
        ^^^^^
        height - Integer
            Maximum length of the histogram bars. Default = 20
        character - String
            Character to construct the bars with. Default '|'

        Output:
        ^^^^^^
        msg - String
            Histogram

        Examples:
        ^^^^^^^^
        These are written in doctest format, and show how expected use and output.

            from numpy.random import normal
            d = normal(size=1000)
            h = HistogramToScreen(d,10)
            histo = h.vertical(15,'*')
            print(histo)
                                             245
            -2.7     1.60% :*
            -2.1     4.60% :***
            -1.5    11.00% :*******
            -0.88   20.60% :*************
            -0.27   24.50% :***************
             0.33   19.70% :************
             0.93   10.10% :******
             1.5     6.30% :****
             2.1     1.40% :*
             2.7     0.20% :

        Notes/Warnings:
        ^^^^^^^^^^^^^^

        Based on http://pyinsci.blogspot.com/2009/10/ascii-histograms.html

        Author: Mike Bauer  <mbauer@giss.nasa.gov>

        Log:
            2010/08 MB - File created.
        """
        his = """"""
        # Create rounded bin labels
        labels = ['% .2g' % round(self.h[1][n],2)
                for n in xrange(len(self.h[1])-1)]
        lxl = [len(l) for l in labels]
        long_lxl = max(lxl)
        # Find percent of total count in each bin
        total_cnts = sum(self.h[0])
        frac = list(100.0*(self.h[0]/total_cnts))
        labels = ["%s%s  %6.2f%% :" % (labels[i],' '*(long_lxl-lxl[i]),
            round(frac[i],2)) for i in xrange(len(frac))]
        lxl = [len(l) for l in labels]
        long_lxl = max(lxl)
        # Create a list of bars
        max_cnt = self.h[0].max()
        # Divide each bin count by max_cnt to normalize
        #   then multiply by height to scale to max bar length.
        #   Also convert bars to a list
        bars = list((self.h[0]/max_cnt)*height)
        # Now round and convert to integers for display
        bars = [int(round(x,0)) for x in bars]
        # Note due to numpy.histogram the last label is not
        #   used or needed.
        #print("Cnts (%d)   : %s" % (len(self.h[0]),self.h[0]))
        #print("Bins* (%d)  : %s" % (len(self.h[1]),self.h[1]))
        #print("Labels (%d) : %s" % (len(labels),labels))
        #print("Bars  (%d)  : %s" % (len(bars),bars))
        # Draw a line showing the maximum count
        left_buffer = ' '*(max(bars)+2+long_lxl)
        label = '%s\n' % max_cnt
        his = left_buffer+label
        # Draw each bar
        for i,c in enumerate(bars):
            his += "%s%s\n" % (labels[i],character*c)

        return his

class CreateFileHeader():
    """
    Creates a file header.

        Input:
        ^^^^^
        height - Integer
            Maximum length of the histogram bars. Default = 20
        character - String
            Character to construct the bars with. Default '|'

        Output:
        ^^^^^^
        msg - String
            Histogram

    Examples:
    ^^^^^^^^
    These are written in doctest format, and show how expected use and output.


    Notes/Warnings:
    ^^^^^^^^^^^^^^


    Author: Mike Bauer  <mbauer@giss.nasa.gov>

    Log:
        2010/10 MB - File created.
    """

    # Format statements
    file_header = {}
    file_header['long_line'] = "-"*80
    # Title: What is this file called (could be the file name)
    file_header['title'] = "%s"
    # File Description: Describe what is in this file.
    file_header['file_description'] = "%s: %s"
    # Origin: What code created this file?
    file_header['origin'] = "%s: %s"
    # Source: What dataset does this file come from (can be 'self')?
    file_header['source'] = "%s: %s"
    # Created: When was this dataset created?
    file_header['created'] = "%s: %s"

    header_table = {'Title' : '#{0:s}\n#\n',
            'File Description' : '{1:s}\n',
            'Origin' : '{2:s}\n',
            'Source' : '{3:s}\n',
            'Created' : '{4:s}\n',
            'Format' : '{5:s}'
            }
    header_info = ['Title','File Description','Origin',
            'Source','Created','Format']

    def __init__(self,):
        header_fmt =  "".join(['%s' % (CreateFileHeader.header_table[x])
            for x in CreateFileHeader.header_info]) + "\n"
        for hf in CreateFileHeader.header_info:
            if hf == 'Title': continue
            header_fmt = header_fmt.replace(CreateFileHeader.header_table[hf],
                    "# %s: %s" % (hf,CreateFileHeader.header_table[hf]))
        header_fmt += "{6:80s}\n"
        self.header_fmt = header_fmt

    def ctime(self,):
        now = datetime.datetime.now()
        self.created = now.strftime("%Y-%m-%d %H:%M")

    def fill(self,title,description,origin,source,created,fmt):

        if title:
            self.title = title
        else:
            self.title = ""

        if description:
            self.description = WrapHeader(description,
                    len(CreateFileHeader.header_info[1]))
        else:
            self.description = ""

        if origin:
            self.origin = WrapHeader(origin,
                    len(CreateFileHeader.header_info[2]))
        else:
            self.origin = ""

        if source:
            self.source = WrapHeader(source,
                    len(CreateFileHeader.header_info[3]))
        else:
            self.source = ""

        if created:
            self.created = created
        else:
            self.ctime()

        if fmt:
            self.fmt = WrapHeader(fmt,
                    len(CreateFileHeader.header_info[5]))
        else:
            self.fmt = fmt

        self.header_string = self.header_fmt.format(self.title.center(80),
                self.description,self.origin,self.source,self.created,
                self.fmt,"# "+"-"*78)

def InjestSetup(fname,which_set):
    """
    Read setup file and return requested values.

    Input:
    ^^^^^
    fname - String
        Full path to requested setup file.
    which_set - Integer
        Which setup file to be read
        0 - Base Setup File
        1 - Center Finding Setup File
        2 - Track Finding Setup File
        3 - Attribution Finding Setup File

    Output:
    ^^^^^^
    sendback - List of names (Strings), filled values
        Names and values requested.

    Examples:
    ^^^^^^^^

    Notes/Warnings:
    ^^^^^^^^^^^^^^

    Author: Mike Bauer  <mbauer@giss.nasa.gov>

    Log:
        2011/01 MB - File created.
    """

    # See recipe inline_1 in inline.py
    # Get some definitions. Note must have run setup_vx.py already!
    try:
        fnc_out = cPickle.load(gzip.open(fname))
    except:
        sys.exit("\n\tWARNING: Error reading or finding %s." % (fname))

    # Parse fnc_out
    inputs = []
    if which_set == 0:
        # Base Setup file
        # Import the API written by mcms_setup.py for this file.   
        f = os.path.split(fname)[0]
        sys.path.append(f)
        import s_dat
        inputs = s_dat.inputs[:]
    elif which_set == 1:
        # Import the API written by mcms_setup.py for this file.   
        f = os.path.split(fname)[0]
        sys.path.append(f) 
        import cf_dat
        inputs = cf_dat.inputs[:]
    elif which_set == 2:
        f = os.path.split(fname)[0]
        sys.path.append(f) 
        import tf_dat
        inputs = tf_dat.inputs[:]
    elif which_set == 3:
        f = os.path.split(fname)[0]
        sys.path.append(f) 
        import df_dat
        inputs = df_dat.inputs[:]
    elif which_set == 4:
        f = os.path.split(fname)[0]
        sys.path.append(f) 
        import time_dat
        inputs = time_dat.inputs[:]
    else:
        sys.exit("Warning unknowned which_set in InjestSetup")
    return fnc_out,inputs

#---Start of main code block.

if __name__ == '__main__':

    results = []

    # Run doc tests
    #   if all tests pass there is not output
    #   python util/mcms_utilites.py -v returns verbose test results.
    #import doctest
    #doctest.testmod()
    #import sys; sys.exit("Stop Here")

    #import progress_bar
    #results = []
    #years = range(1979,2010)
    #mp_pbar= progress_bar.ProgressBar(len(years),"Processing Data...")
    #mp_pbar.fill_char = '-'
    #for year in years:
    #    #CompletedCallback(year)
    #    CompletedCallbackPbar(year)
    #print(results)


    from numpy.random import normal
    d = normal(size=1000)
    h = HistogramToScreen(d,10)
    histo = h.horizontal(15,'|')
    print(histo)
    print()
    #h = HistogramToScreen(d,10)
    histo = h.vertical(15,'*')
    print(histo)
    import sys; sys.exit("Stop Here")

    ## Test array of missing (-1) and good values >= 0
    ##   where the values range from 1 to 10
    #test_map = [-1]*4
    #for i in xrange(1,11):
    #    test_map.extend([i]*i)
    #print("test_map (%d): "  % (len(test_map)))
    #PrintColumns(test_map,indent_tag="\t",fmt="% 3d",cols=5,width=5)
    ## Set the bins so as to [-1....11] so that 9 and 10
    ##   counts are not mixed.
    #bins = range(-1,12)
    #h = HistogramToScreen(test_map,bins)
    #print(h.vertical(15,'*'))

    in_path = os.getcwd()+"/src/test/"
    read_year = 2009
    var_slp = "mslp"
    var_time = "time"
    lat_flip = 1
    lon_shift = 0
    scale = 0.01
    slp,times,time_units = PullData(in_path,read_year,var_slp,
            var_time,lat_flip,lon_shift,scale)
    print(in_path,read_year,var_slp,var_time,lat_flip,lon_shift,scale)
    #print(slp[0,0,0],slp[0,0,-1],slp[0,-1,0],slp[0,-1,-1])
