#!/usr/bin/env python -tt
"""
Utility functions for MCMS project that help with reading in the source data.

Author: Mike Bauer  <mbauer@giss.nasa.gov>

Log:
2012/05 MB - File created.
"""

from __future__ import division
# Alter python 2 division where division of two *integer* values gives an
#   *integer* result (i.e., 1/2 = 0 but 1.0/2 = 0.5 or 1/2.0 = 0.5) to the
#   more rational python 3+ version where all division returns floats
#   (i.e., 1/2 = 0.5). Adds a new integer division operator so that 1//2 = 0

from __future__ import print_function
# Alter python 2 to use python 3 print as a function method. This makes
#   it easier to use this code on python 2 or 3 installations.

# Standard library imports
import sys
import os
import gzip
import cPickle

# Third party module imports
import netcdftime

# MCMS module imports: Need to have mcms in your PYTHONPATH environment variable!
from print_columns import PrintColumns
from pull_data import PullData

# ----------------------------------------------------------------------------
def file_listing(path_to,cfunc,svar,mkey,blob):
    filelist = []
    yearlist = []
    monthlist = []
    daylist = []
    fmap = {}
    temp = os.listdir(path_to)
    for a in temp:
        # Ignore subdirectories of path_to
        if os.path.isfile(path_to+'/'+a):
            # Parse File
            basename, extension = os.path.splitext(a)
            if extension != ".nc": continue
            if basename.find(svar) == -1: continue
            if blob:
                bb = cfunc(basename,blob)
            else:
                bb = cfunc(basename)
            filelist.append(basename)
            yearlist.append(bb[0])
            if bb[1]:
                monthlist.append(bb[1])
            if bb[2]:
                daylist.append(bb[2])
            fmap[basename] = bb
    yearlist = sorted(list(set(yearlist)))
    if monthlist:
        if mkey:
            monthlist = sorted(list(set(monthlist)),key = lambda x: mkey[x])
        else:
            monthlist = sorted(list(set(monthlist)))
    if daylist:
        daylist = sorted(list(set(daylist)))
    if mkey:
        filelist = sorted(filelist,
                key = lambda filelist: (cfunc(filelist)[0],mkey[cfunc(filelist)[1]]))
    return(filelist,yearlist,monthlist,daylist,fmap)

def organize_data(target_folder,bfunc,var_slp,mon_key,the_calendar,var_time,
        jd_fake,shared_path,blob,base_name='time_dat',verbose=0):
    """Examine the target directory for possible data files. Attempt to
    extract information (mostly time) from these files, store and return.

    Input:
    ^^^^^

        target_folder - String
            Full path to the folder containing the data.
        bfunc - Function
            Python function to parse filename into year,month,day
        var_slp - String
            Name of SLP variable in file name
        mon_key - Dictionary
            Key for sorting the order of files with month names.
        the_calendar - String
            Name of the calendar type for datetime (default = 'standard')
        var_time - String
            Name of time variable in file name
        jd_fake - Integer
            If 1, then use time steps rather than Julian dates. This happens
            with model runs that don't work with Julian dates (e.g., paelo runs).
        shared_path - String
            Full path to where MCMS stores model setup data.
        blob - String
            Prefix string to file name: example "cam_b30.12_2kaDVTn_T85.cam2.h1"
            for a file "cam_b30.12_2kaDVTn_T85.cam2.h1.0301-01-01-PSL.nc"
        base_name - String
            Base name for the data storage... Default ['time_dat']
    """
    # Test if target_folder exits
    if not os.path.exists(target_folder):
       msg = "Warning %s: Folder %s not found"
       sys.exit(msg % (__name__,target_folder))
       return 1
    if verbose:
        print("\tReading Data Folder: %s" % target_folder)

    # Retrieve the list of files from target_folder that contain the
    #   search_pattern.
    fnc_out = file_listing(target_folder,bfunc,var_slp,mon_key,blob)
    found_files,found_years,found_months,found_days,fname_map = fnc_out
    #print (target_folder)
    #print (found_files)
    #print (found_years)
    #print (found_months)
    #print (found_days)
    #for f in fname_map:
    #    print (f,fname_map[f])
    #    sys.exit("\nStop Here")
    #print (fname_map)
    if not found_files:
        msg = "Warning %s: No data files in folder %s"
        sys.exit(msg % (__name__,target_folder))
        return 1
    if verbose:
        print("\tFound Files:")
        PrintColumns(found_files,indent_tag="\t\t",
                fmt="%s ",cols=2,width=20,sort_me=0)
        print("\tFound Years:")
        PrintColumns(found_years,indent_tag="\t\t",
                fmt="%04d ",cols=2,width=20,sort_me=0)
        print("\tFound Months:")
        if found_months:
            PrintColumns(found_months,indent_tag="\t\t",
                fmt="%s ",cols=2,width=20,sort_me=0)
        else:
            print("\t\tNone")
        print("\tFound Days:")
        if found_days:
            PrintColumns(found_days,indent_tag="\t\t",
                    fmt="%s ",cols=2,width=20,sort_me=0)
        else:
            print("\t\tNone")

    # Open each file, extract time data, make uci_stamps,tsteps,
    start_year = found_years[0]
    end_year = found_years[-1]
    super_years = [start_year,end_year]
    super_time_data = {}
    super_lookup = {}
    super_tsteps = 0
    super_uci_stamps = []
    timestep = 0
    for fname in found_files:
        fname_ext = target_folder+fname+".nc"
        super_time_data[fname] = {}
        times,the_time_units = PullData(fname_ext,
                start_year,var_slp,var_time,"","","","info")
        start = the_time_units
        tsteps = len(times)
        cdftime = netcdftime.utime(start,calendar=the_calendar)
        get_datetime = cdftime.num2date
        dtimes = [get_datetime(times[step]) for step in xrange(0,tsteps)]
        #print ("\nfname",fname)
        #print ("start",start)
        #print ("tsteps",tsteps)
        #print ("dtimes[0],dtimes[-1]",dtimes[0],dtimes[-1])
        # Get Julian Days.. unless GCM uses non-standard calendar in which case
        #  enumerate with timestep and use uci_stamps for datetime things.
        if jd_fake:
            # Use timesteps rather than dates
            # examples '        0', '        1'
            # Note: want a number for the form_usi etc so not %010d.
            adates = ["%010d" % (x) for x in range(tsteps)]
        else:
            # Using regular date/times
            # The decimal parts of a Julian date:
            #   0.1 = 2.4 hours or 144 minutes or 8640 seconds
            #   0.01 = 0.24 hours or 14.4 minutes or 864 seconds
            #   0.001 = 0.024 hours or 1.44 minutes or 86.4 seconds
            #
            # This should be good for at least 3 hour timesteps
            #    [2447878.0, 2447878.125, 2447878.25, 2447878.375, 2447878.5, 2447878.625, 2447878.75, 2447878.875]
            # -> [2447878000, 2447878125, 2447878250, 2447878375, 2447878500, 2447878625, 2447878750, 2447878875]
            date2jd = netcdftime.JulianDayFromDate
            adates = [int(1000*date2jd(x,calendar='standard')) for x in dtimes]
        #print ("adates[0],adates[-1]",adates[0],adates[-1])
        # This will be an issue for data with timesteps less than an hour!
        uci_stamps = ['%04d%02d%02d%02d' % (d.year,d.month,d.day,d.hour)
            for d in dtimes]
        date_stamps = ["%04d %02d %02d %02d" % (d.year,d.month,d.day,d.hour)
            for d in dtimes]
        #print ("uci_stamps[0],uci_stamps[-1]",uci_stamps[0],uci_stamps[-1])
        #print ("date_stamps[0],date_stamps[-1]",date_stamps[0],date_stamps[-1])
        #if timestep: sys.exit("\nStop Here")
        if not timestep:
            # Find timestep
            syear = dtimes[0].year
            smonth = dtimes[0].month
            sday = dtimes[0].day
            shour = dtimes[0].hour
            for d in dtimes:
                dyear = d.year
                dmonth = d.month
                dday = d.day
                dhour = d.hour
                if dyear == syear:
                    if dmonth == smonth:
                        if dday == sday:
                            if dhour != shour:
                                timestep = dhour-shour
                                break

        # Store filename based mapping
        super_time_data[fname]['start'] = start
        super_time_data[fname]['tsteps'] = tsteps
        super_time_data[fname]['uci_stamps'] = uci_stamps
        super_time_data[fname]['jd_fake'] = jd_fake
        super_time_data[fname]['date_stamps'] = date_stamps
        super_time_data[fname]['adates'] = adates

        #print ("super_time_data[fname]['start']",super_time_data[fname]['start'])
        #print ("super_time_data[fname]['tsteps']",super_time_data[fname]['tsteps'])
        #print ("super_time_data[fname]['uci_stamps'][0],super_time_data[fname]['uci_stamps'][-1]",super_time_data[fname]['uci_stamps'][0],super_time_data[fname]['uci_stamps'][-1])
        #print ("super_time_data[fname]['jd_fake']",super_time_data[fname]['jd_fake'])
        #print ("super_time_data[fname]['date_stamps'][0],super_time_data[fname]['date_stamps'][-1]",super_time_data[fname]['date_stamps'][0],super_time_data[fname]['date_stamps'][-1])
        #print ("super_time_data[fname]['adates'][0],super_time_data[fname]['adates'][-1]",super_time_data[fname]['adates'][0],super_time_data[fname]['adates'][-1])

        # Store uci_stamp based mapping
        for uci in uci_stamps:
            super_lookup[uci] = fname
        super_tsteps += tsteps
        super_uci_stamps.extend(uci_stamps)

    if verbose:
        print ("\tTime Information:")
        print ("\t\tCalendar:\t%s" % (the_calendar))
        print ("\t\tYear Bounds: [%s,%s]" % (super_years[0],super_years[1]))
        print ("\t\tTime steps:\t%d" % (super_tsteps))
        print ("\t\tTime step:\t%d" % (timestep))

    # Store for later recall
    fnc_out = (super_uci_stamps,super_tsteps,super_time_data,super_years,
            timestep,super_lookup)
    time_file = "%s%s.pz" % (shared_path,base_name)
    cPickle.dump(fnc_out,gzip.open(time_file, "wb",-1))
    if verbose: print("\tMade: %s\n" % (time_file))
    # Write the API to file for later use
    time_file_api = "%s%s.py" % (shared_path,base_name)
    f = open(time_file_api,'w')
    msg = """
inputs = ("super_uci_stamps","super_tsteps","super_time_data","super_years","timestep","super_lookup")"""
    f.write(msg)
    msg = """
#
# Example usage:
#
#   time_file = "%stime_dat.pz"  % (rf.shared_path)
#   fnc_out,inputs = InjestSetup(time_file,4)
#   super_uci_stamps = fnc_out[inputs.index("super_uci_stamps")]
#   super_tsteps = fnc_out[inputs.index("super_tsteps")]
#   super_time_data = fnc_out[inputs.index("super_time_data")]
#   super_years = fnc_out[inputs.index("super_years")]
#   timestep = fnc_out[inputs.index("timestep")]
#   super_lookup = fnc_out[inputs.index("super_lookup")]
#   del fnc_out,inputs"""
    f.write(msg)
    f.close()
    del fnc_out

    #test = 1
    #if test:
    #    # Test
    #    InjestSetup = mcms_utilities.InjestSetup
    #    fnc_out,inputs = InjestSetup(time_file,4)
    #    super_uci_stamps = fnc_out[inputs.index("super_uci_stamps")]
    #    super_tsteps1 = fnc_out[inputs.index("super_tsteps")]
    #    super_time_data = fnc_out[inputs.index("super_time_data")]
    #    super_years1 = fnc_out[inputs.index("super_years")]
    #    print ("\n\tRe-Read Test:")
    #    print ("\n\tTime Information:")
    #    print ("\t\tYear Bounds: [%s,%s]" % (super_years1[0],super_years1[1]))
    #    print ("\t\tTime steps:\t%d" % (super_tsteps1))

    return "\tMade: %s\n" % (time_file)

#---Start of main code block.
if __name__ == '__main__':
    verb = 1
    ffolder = "/Volumes/Scratch/data/gissm/control_run/"
    shared_path = "/Volumes/Scratch/output/gissm_files/"

    # Would be gotten from s_dat.p
    dim_lat = 'lat'
    dim_lon = 'lon'
    dim_time = 'time'
    var_lat = 'lat'
    var_lon = 'lon'
    var_time = 'time'
    var_slp = 'slp'
    the_calendar = '365_day'
    jd_fake = 1

    # SLPDEC3781.E3oM20
    #def break_fname(fname):
    #    yyyy = int(fname[6:10])
    #    mm = fname[3:6]
    #    dd = ""
    #    return (yyyy,mm,dd)

    #slp.MAY.3794.nc
    def break_fname(fname):
        bf = fname.split(".")
        yyyy = int(bf[2])
        mm = bf[1]
        dd = ""
        # how to make name
        pattern = "slp.month.year.nc"
        return (yyyy,mm,dd,pattern)

    msg = organize_data(ffolder,break_fname,var_slp,
            month_sort_order,the_calendar,var_time,jd_fake,shared_path,
            verbose=verb)

