#!/usr/bin/env python -tt
"""
Utility functions for MCMS project.

Author: Mike Bauer  <mbauer@giss.nasa.gov>

Log:
2010/08 MB - File created.
"""

from __future__ import division
# Alter python 2 division where division of two *integer* values gives an
#   *integer* result (i.e., 1/2 = 0 but 1.0/2 = 0.5 or 1/2.0 = 0.5) to the
#   more rational python 3+ version where all division returns floats
#   (i.e., 1/2 = 0.5). Adds a new integer division operator so that 1//2 = 0

from __future__ import print_function
# Alter python 2 to use python 3 print as a function method. This makes
#   it easier to use this code on python 2 or 3 installations.

# Standard library imports
import sys
import os
from optparse import OptionParser
import shutil
import tempfile

# Third party module imports

# MCMS module imports
from file_exists import FileExists
from query_yes_no import query_yes_no

# ----------------------------------------------------------------------------
def ParseRunFile(run_file_template,argv,in_path):
    """
    Reads and parses the command line input.

    Input:
    ^^^^^
    in_string - String
       Long string to be broken.

    Output:
    ^^^^^^
    out_string - String
       A reformatted version of in_string.

    Examples:
    ^^^^^^^^
    These are written in doctest format, and show how expected use and output.

    Notes/Warnings:
    ^^^^^^^^^^^^^^

    Author: Mike Bauer  <mbauer@giss.nasa.gov>

    Log:
        2011/01 MB - File created.
    """
    # ----------------------------------------------------------------------------
    verbose = 0

    msg_need_run_file = "\nPlease provide a run_file name.\n"
    parser = OptionParser(usage="usage: %prog [options] run_file")
    parser.add_option("-v", "--verbose",
            action="store_true", dest="verbose", default=False,
            help="Report what is done [default = False]")
    parser.add_option("-c", "--create",
            action="store_true", dest="create_run_file", default=False,
            help="Create a New Run File and Quit [default = False]")
    parser.add_option("-n", "--no_ask",
            action="store_true", dest="no_ask", default=False,
            help="Suppress File Overwrite Query [default = False]")
    parser.add_option("-s", "--skip_time",
            action="store_true", dest="skip_time", default=False,
            help="Skip Rerun of time_dat.p [default = False]")
    (options, args) = parser.parse_args()

    if len(argv) < 2:
        parser.print_help()
        return 1
    # Deal w/ run_file
    if not args:
        print(msg_need_run_file)
        parser.print_help()
        return 1
    run_file = args[0]
    if options.no_ask:
        # create temporary file
        tempFile = tempfile.NamedTemporaryFile(suffix=".txt",prefix="no_ask",dir=in_path,delete=False)
        tempFile.seek(0)
        #print (tempFile.name) 
    if options.skip_time:
        # create temporary file
        tempFile1 = tempfile.NamedTemporaryFile(suffix=".txt",prefix="skip",dir=in_path,delete=False)
        tempFile1.seek(0)
        #print (tempFile1.name) 

    # Add path to this file to run_file if run_file does not
    #   contain a path already
    if verbose: print("\n\trun_file: %s" % run_file)
    if not os.path.split(run_file)[0]:
        print("\n\tAdded current path to run_file")
        run_file = in_path+"/"+run_file
    elif os.path.split(run_file)[0] != in_path:
        run_file = in_path+"/"+run_file
    # Add .py to run_file if not present
    basename, extension = os.path.splitext(run_file)
    if extension != ".py":
        print("\n\tAdded .py extension to run_file")
        run_file = basename+".py"
    if verbose: print("\trun_file: %s" % run_file)
    if verbose: print("\tbasename: %s" % basename)

    # Check is run_file exists, if asked to create a new run_file then
    #   warn about overwriting, else create it, otherwise warn that
    #   run_file not found
    if FileExists(run_file,__name__,verbose=False):
        if options.create_run_file:
            # Warn about overwriting
            msg = "\n\tWarning: Overwrite run_file [%s]?"
            print(msg % run_file,end="")
            answer = query_yes_no("\tContinue?")
            if not answer:
                return 1
            # Create new run_file from template
            shutil.copyfile(run_file_template,run_file)
            msg = "\n\trun_file [%s] created."
            print(msg % run_file)
            return 1
    else:
        if options.create_run_file:
            # Create new run_file from template
            shutil.copyfile(run_file_template,run_file)
            msg = "\n\trun_file [%s] created."
            print(msg % run_file)
            return 1
        else:
            FileExists(run_file,__name__)
            print("Error in ParseRunFile1")
            return 1
    # Import run_file
    if verbose: print("\tImporting rule_file")
    # Assume that all RunFiles are in the subdirectory rf_files
    getthis = "rf_files."+basename.split("/")[-1] # should be 'rf_model_cf' etc
    if verbose: print("\tgetthis: %s" % getthis)
    __import__(getthis)
    rf = sys.modules[getthis]
    if verbose: print("\trf: %s" % dir(rf))
    rf.getthis = getthis 
    if verbose: print("\trf.getthis: %s" % rf.getthis)
    rf.tempFile = ""
    if options.no_ask:
        rf.tempFile = tempFile.name 
    if options.skip_time:
        rf.tempFile1 = tempFile1.name      
    return rf
