#!/usr/bin/env python -tt
"""
Utility functions for MCMS project.

Author: Mike Bauer  <mbauer@giss.nasa.gov>

Log:
2010/08 MB - File created.
"""

# Standard library imports
import sys
import os

# Third party module imports
import pprint
import numpy
import netCDF4 as NetCDF

def PullData(pd_in_path,pd_read_year,pd_var_data,pd_var_time,pd_lat_flip,
        pd_lon_shift,pd_scale,pd_amount="all"):
    """
    Read in a netcdf data file.

    This function takes an input directory (in_path) and finds all
       the available files to be read. Then the requested file is
       read and put into a single array.

    Input:
    ^^^^^
    pd_in_path - String
        Full path the directory hosting the data to be read. IF pd_in_path
        is a netcdf file (.nc) then rather than search pd_in_path for a
        file with pd_read_year, pd_var_data etc. it just opens that specific
        file.
    pd_read_year - Integer
        Specific year of data requested... assumes datafiles are named
            and separated by year.
    pd_var_data - String
        Name of the netcdf data variable to be extracted.
    pd_var_time - String
        Name of the netcdf time variable
    lat_flip - Boolean
        Force data and latitude to be reverse order from data file.
    lon_shift - Integer
        Force longitude to shift by lon_shift so 1st index contains the
            the Greenwich Meridian.
    pd_scale - Float
        Scales the variable as desired (can be 1.0 to do nothing).
    pd_amount = Integer
        If set to 'all', the default, then the whole time series of data
            is returned.
        If set to 'info', then everything is the same as the default except only
            a single time step of the main variable is returned.
        If set to an integer value, then only main variable data for that time
            step is returned.
        'info' and integer values can be used when the data set is very large
        and reading the whole thing into memory is slow or not possible.

    Output:
    ^^^^^^
    pd_data - Numpy array
        Array of the data requested by pd_var_data.
    pd_times - Numpy array
        Array of the time data.
    pd_time_units - String
        Time units for the data file.

    Examples:
    ^^^^^^^^
    These are written in doctest format, and show how expected use and output.

    >>> x,y,z = PullData("/Users/mbauer/Work/sysnopsis/refactored/src/test/",
    ...        2009,"mslp","time",1,0,0.01)
    >>> print(x.shape,y.shape,z) #doctest: +NORMALIZE_WHITESPACE
    (1460, 73, 144) (1460,) hours since 1800-1-1 00:00:0.0
    >>> print(x[0,0,0],x[0,0,-1],
    ...     x[0,-1,0],x[0,-1,-1]) #doctest: +NORMALIZE_WHITESPACE
    997.5 997.5 1019.2 1019.2

    Notes/Warnings:
    ^^^^^^^^^^^^^^

    Author: Mike Bauer  <mbauer@giss.nasa.gov>

    Log:
        2008/01  MB - File created.
        2008/10  MB - Added input checks, docstring.
        2008/10  MB - Fixed error where file_list was unordered...
        2009/11  MB - Updated to version 4.
        2010/08  MB - Starting to update to version 5.
        2010/08  MB - Moved to mcms_utilities.
        2012/05  MB - Added pd_amount to allow for large file reads
    """
    if pd_in_path.find(".nc") != -1:
        # pd_in_path is a specific file rather than a directory
        found_file  = pd_in_path
        # Open file to read, use the netcdf 3 format
        pd_in_path = ""
    else:
        import fnmatch
        pattern = '*%04d*.nc' % int(pd_read_year)
        matches = fnmatch.filter(os.listdir(pd_in_path), pattern)
        if not matches:
            msg = "WARNING: Cannot file for year %d in directory %s"
            print(msg)
            sys.exit(1)
        elif len(matches) > 1:
            msg = "WARNING: Multiple matches for year %d in directory %s"
            print(msg)
            print("Matches",matches)
            sys.exit(1)
        found_file = matches[0]
        #print ("\nFound File:",found_file)

    # Open file to read, use the netcdf 3 format
    nc_in = NetCDF.Dataset(pd_in_path+found_file,'r',format='NETCDF3_CLASSIC')
    #print nc_in.dimensions
    #print nc_in.variables
    
    if isinstance(pd_amount,int):
        # Only return data
        #
        # Pull slp_var, assume has dimensions of [time,lat,lon]
        if pd_var_time:
            got_data = numpy.multiply(numpy.array(nc_in.variables[pd_var_data][pd_amount,:,:],
                dtype=numpy.float32,copy=1),pd_scale)
        else:
            # No time dim for this file
            got_data = numpy.multiply(numpy.array(nc_in.variables[pd_var_data][:,:],
                dtype=numpy.float32,copy=1),pd_scale)
        if pd_lat_flip:
            got_data = got_data[::-1,:]
        if pd_lon_shift:
            got_data = numpy.roll(got_data,pd_lon_shift,axis=1)
        nc_in.close()
        # Add fake time dimention
        got_data.shape = (1,got_data.shape[0],got_data.shape[1])
        return got_data
    else:        
        # Pull var_time
        times = nc_in.variables[pd_var_time][:]
        the_time_units = nc_in.variables[pd_var_time].units
        times = times.astype(numpy.float32)
        if pd_amount == 'info':
            nc_in.close()     
            return times,the_time_units
        # Pull slp_var, assume has dimensions of [time,lat,lon]
        got_data = numpy.multiply(numpy.array(nc_in.variables[pd_var_data][:],
            dtype=numpy.float32,copy=1),pd_scale)
        if pd_lat_flip:
            got_data = got_data[:,::-1,:]
        if pd_lon_shift:
            got_data = numpy.roll(got_data,pd_lon_shift,axis=2)
        nc_in.close()
        return got_data,times,the_time_units

