#!/usr/bin/env python -tt
"""This module shifts a gridID by a number of columns and rows.

Options/Arguments:

Inputs:
    start_gridid - Integer
        The starting gridID.
    dcol - Integer
        The number of columns to shift start_gridid.
    drow - Integer
        The number of rows to shift start_gridid.
    c_halo - Array
        Halo around array edges. 
    halfway - Integer
        The column that is 180 degrees longitude from the model's 1st column.
    im - Integer
        The number of longitude grids in the source grid.
    jm - Integer
        The number of latitude grids in the source grid.

Returns:
    i - Integer
        The longitude index of the desired gridID
    j - Integer
        The latitude index of the desired gridID

Examples:

Notes:

Author: Mike Bauer  <mbauer@giss.nasa.gov>

Log:
    2012/06  MB - File created, as stand alone module.
"""
import numpy as np
cimport numpy as np
DTYPE = np.int32
ctypedef np.int32_t DTYPE_i
cimport cython
@cython.boundscheck(False)
@cython.wraparound(False)
@cython.nonecheck(False)
#@cython.cdivision(True)
def shift_gridid(int start_gridid, int dcol, drow, 
        np.ndarray[DTYPE_i,ndim=1,mode='c'] c_halo, int halfway, int im, int jm):    
    cdef int start_col, start_row, end_col, end_row
    assert c_halo.dtype == DTYPE
    
    # Find row and column of start_gridid
    start_col = start_gridid%im
    start_row = start_gridid/im
    
    # Note there are combinations that return start_gridid for non-zero dcol,row
    #   That is, they wrap around the globe either directly along latitude, or
    #   by a combination of polar roll over and column sift. Similarly, a +/-
    #   combo of dcol and drow can return the same result.
    #
    #   For example (See docs/working_with_grids.pdf),
    #   im = 8
    #   jm = 8
    #   halfway = 4
    #
    #   start_gridid = 56
    #   dcol = +/-4 = abs(halfway)
    #   drow = 1   
    #   end_gridid = start_gridid    
    #
    #   start_gridid = 56
    #   dcol = +/-8 = abs(im)
    #   drow = 0   
    #   end_gridid = start_gridid
    
    # See util.inflate_gdict.py for example and test usage.

    # See what sort of sift asked for
    if 0 <= start_row+drow < jm:
        # No polar roll over
        end_row = start_row + drow
        end_col = start_col + dcol
        # See if column wrap around
        end_col = c_halo[end_col+im]
        return im*end_row+end_col
    elif start_row+drow < 0:
        # SPole roll over
        end_row = start_row - drow - 1
        end_col = start_col + dcol + halfway
        # See if column wrap around
        end_col = c_halo[end_col+im]
        return im*end_row+end_col
    elif start_row+drow >= jm:
        # NPole roll over 
        end_row = start_row - drow + 1
        end_col = start_col + dcol + halfway
        # See if column wrap around
        end_col = c_halo[end_col+im]
        return im*end_row+end_col

