#!/usr/bin/env python -tt
""" This module works with i/o for the mcms project.

    Options/Arguments:

    Author: Mike Bauer  <mbauer@giss.nasa.gov>

    Log:
    2011/01  MB - File created.
    2012/05  MB - Fixed dump_jd for fake_jd
"""

from __future__ import division
# Alter python 2 division where division of two *integer* values gives an
#   *integer* result (i.e., 1/2 = 0 but 1.0/2 = 0.5 or 1/2.0 = 0.5) to the
#   more rational python 3+ version where all division returns floats
#   (i.e., 1/2 = 0.5). Adds a new integer division operator so that 1//2 = 0

# Standard library imports
import sys
import os

# Third party module imports

# MCMS module imports
from read_mcms import Read_MCMS 

# Fetch some definitions.
center_table = Read_MCMS.center_fields
usi_fmt = Read_MCMS.usi_fmt
c_fmt = Read_MCMS.center_fmt

# Define some things.
default_flag = default_dissimilarity = default_intensity = 0
default_uci = default_usi = Read_MCMS.usi_fmt % (0,0,0,0,0,0)

center_fmt = " ".join([Read_MCMS.center_fmt[x] for x in Read_MCMS.center_fields]) + "\n"
#center_fmt = "%04d %02d %02d %02d %10d %05d %05d %06d %07d "
#center_fmt += "%07d %05d %02d %02d %04d %s %s\n"
center_string_size = len(
    center_fmt % (0,0,0,0,0,0,0,0,0,0,0,0,0,0,default_uci,default_uci))
uci_string_size = len(default_uci)
center_fmt_alt = " ".join(["%%(%s)%s" % (x,Read_MCMS.center_fmt[x].replace("%",""))
    for x in Read_MCMS.center_fields]) + "\n"
#center_fmt_alt = "%(YYYY)04d %(MM)02d %(DD)02d %(HH)02d %(JD)10d %(CoLat)05d "
#center_fmt_alt += "%(Lon)05d %(GridID)06d %(GridSLP)07d %(RegSLP)07d "
#center_fmt_alt += "%(GridLAP)05d %(Flags)02d %(Intensity)02d "
#center_fmt_alt += "%(Dissimilarity)04d %(UCI)s %(USI)s\n"

def print_center(print_this):
    offset = max([len(x) for x in print_this.keys()])+1
    for rfield in center_table:
        fmt = "\t%s%s: %s" % (rfield," "*(offset-len(rfield)),c_fmt[rfield])
        print fmt % (print_this[rfield])

def form_lon(dump_lon):
    return int(dump_lon*100)

def unform_lon(dump_lon):
    return dump_lon*0.01

def form_colat(dump_lat):
    return int((90.0-dump_lat)*100)

def unform_colat(dump_lat):
    return 90.0-dump_lat*0.01


def form_uci(d_datestamp,g_lat,g_lon):
    """Create a UCI/USI string
    d_datestamp: = '%4d%02d%02d%02d' % (d.year,d.month,d.day,d.hour)
    g_lat: float latitude  0-360 degree format
    g_lon: float longitude
    NOTE: Only use grid center lon,lat for UCI/USI are fractional lon,lat
        could (due to rounding) result in centers having the same value if
        both are near the same grid edge.
    """
#     return Read_MCMS.usi_fmt % (int(dump_datestamp[:4]),
#         int(dump_datestamp[4:6]),int(jd[6:8]),
#         int(dump_datestamp[8:10]),int(round((90.0-dump_lat)*100)),
#         int(round(dump_lon*100)))

    msg = Read_MCMS.usi_fmt % (int(d_datestamp[:4]),int(d_datestamp[4:6]),
        int(d_datestamp[6:8]),int(d_datestamp[8:10]),form_colat(g_lat),
        form_lon(g_lon))
    # Check msg is correct size:
    if len(msg) != uci_string_size:
        print "Warning form_uci: Wrong size msg"
        print "len(msg),uci_string_size",len(msg),uci_string_size
        print "msg",msg
        print "d_datestamp,g_lat,g_lon",d_datestamp,g_lat,g_lon
        print "form_colat(g_lat)",form_colat(g_lat)
        print "form_lon(g_lon)",form_lon(g_lon)
        return 1
    return msg
    
def form_center(dump_datestamp,dump_jd,dump_lat,dump_lon,dump_gridid,
        dump_gridslp,dump_regslp,dump_gridlap,dump_flag,dump_intensity,
        dump_dissimilarity,dump_glat,dump_glon,as_dict):
    """Create a center string
    dump_datestamp : string 'YYYYMMDDHH'
    dump_jd : string
                '0000000000' if jd_fake or '2444605620'
    dump_lat : float
                +-90.0 (converted to CoLat here)
    dump_lon : float
                0.0-360.0 (converted if +-180.0 format)
    dump_gridid : integer
                4545
    dump_gridslp: integer
            1004000
    dump_regslp: integer
            1000000
    dump_gridlap: integer
            200
    dump_flag: integer
            0
    dump_intensity: integer
            None - use default_intensity
    dump_dissimilarity: float
            None - use default_dissimilarity
    dump_glat: float
            None - use dump_lat to for UCI/USI. Not recommended if dump_lat
            is based on fractional center location as rounding can result in
            duplicate UCI entries.
            Float - use to form UCI/USI (usually this is the center latitude
            for the grid containing the center.
    dump_glon: float
            Same as with dump_glat
    as_dict: integer
            0 - return a string for writing to file
            1 - return a dictionary instead
    """
    if len(dump_datestamp) != 10:
        print "Warning: dump_datestamp wrong length"
        return 1
    if abs(dump_lat) > 90.0:
        print "Warning: dump_lat too big"
        return 1
    if dump_lon < 0:
        dump_lon = (dump_lon+360.0)%360.0

    # Use dump_glat/glon if provided.
    if dump_glat:
        if abs(dump_glat) > 90.0:
            print "Warning: dump_glat too big"
            return 1
        if dump_glon < 0:
            dump_glon = (dump_glon+360.0)%360.0
    else:
        dump_glat = dump_lat
        dump_glon = dump_lon

    if not dump_intensity:
        dump_intensity = default_intensity
    if not dump_dissimilarity:
        dump_dissimilarity = default_dissimilarity

    # Discards sometimes have negative dump_gridlap, which can make the
    #   string too long. In this case cut it shorter
    if dump_gridlap < 0.0:
        dump_gridlap = int(round(dump_gridlap*0.1,0))

    if as_dict:
        msg = dict(zip(center_table,
            [int(dump_datestamp[:4]),int(dump_datestamp[4:6]),
            int(dump_datestamp[6:8]),int(dump_datestamp[8:10]),
            int(dump_jd),form_colat(dump_lat),form_lon(dump_lon),dump_gridid,
            dump_gridslp,dump_regslp,dump_gridlap,dump_flag,dump_intensity,
            dump_dissimilarity,form_uci(dump_datestamp,dump_glat,dump_glon),
            default_usi]))
    else:
        msg = center_fmt % (int(dump_datestamp[:4]),int(dump_datestamp[4:6]),
        int(dump_datestamp[6:8]),int(dump_datestamp[8:10]),int(dump_jd),
        form_colat(dump_lat),form_lon(dump_lon),dump_gridid,dump_gridslp,
        dump_regslp,dump_gridlap,dump_flag,dump_intensity,
        dump_dissimilarity,form_uci(dump_datestamp,dump_glat,dump_glon),
        default_usi)
        # Check msg is correct size:
        if len(msg) != center_string_size:
            print "Warning form_center: Wrong size msg"
            print "len(msg),center_string_size",len(msg),center_string_size
            print "msg",msg
            return 1

    return msg

def form_center_alt(in_dict):
    """Create a center string
    """
    msg = center_fmt_alt % in_dict
    return msg

#---Start of main code block. This show how to use the above code
if __name__=='__main__':

    import numpy
    uci_stamp = "1980010100"
    date_stamp = "1980 01 01 00"
    date_stamp = "1980 11 13 18"
    uci_stamp = "1980111318"
    alon = 77.5
    alat = -52.5
    gridid = 9525
    jdate = 2444325500
    flag = 1
    cslp = 1019000
    clap = 200
    aslp = 1020701

    ct = form_center(uci_stamp,jdate,alat,alon,alat,alon,gridid,cslp,aslp,clap,flag)
    print ct

    ct = form_center(uci_stamp,jdate,alat,alon,alat,alon,gridid,cslp,aslp,
        clap,flag,as_dict=1)
    print_center(ct)

    tt = form_center_alt(ct)
    print tt

    import sys; sys.exit("Stop Here")

    # Check to be sure uci unique
    lats = numpy.arange(-90.0,90.01,0.1).tolist()
    lons = numpy.arange(0.0,360,0.1).tolist()
    print  len(lats)*len(lons)
    for lat in lats:
        ucis = {}
        for lon in lons:
            uci = form_uci(uci_stamp,lat,lon)
            ucis[uci] = 0
        if len(ucis) != len(lons):
            print lat,"error",len(ucis),len(lons)

#
#         return dict(zip(center_table,v))
#
#
#     v = [0,1,2,3,4,5,6,7,8,9,10,11,12,13,"uci",usi_template]
#     kept_centers = {}
#     kept_centers[10] = dict(zip(center_table,v))
#     print_center(kept_centers[10])

    #print util.read_mcms.Read_MCMS.center_fmt
    #print util.read_mcms.Read_MCMS.center_group_fmt
    #print util.read_mcms.Read_MCMS.usi_fmt
    #print util.read_mcms.Read_MCMS.usi_template
    #print util.read_mcms.Read_MCMS.usi_fmt % (0,0,0,0,0,0)
